#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2006 - 2007 by The WordForge Foundation
#                     www.wordforgefoundation.org
#
# Version 0.1 (29 December 2006)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details. 
#
# Developed by:
#       Aditi Barua(aditi@ankur.org.bd)

# This module is working on showing error messages in a pop up window


from PyQt4 import QtCore, QtGui
from WordForge.Editor.ui.Ui_Batch_Process import Ui_Dialog
from WordForge.Editor.modules.Operator import Operator
import WordForge.Editor.modules.World as World
from WordForge.translate.storage import xliff

class BatchProcess(QtGui.QDialog):

    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self.ui=None
        self.show()
        self.parent = parent
            
    def applyBatchProcess(self):     
  #this function gathers the user settings Batch processing window and builds a global dictionary containing the information.      
        dict_State_Process = {'from_state':None,'to_state':None,'from_state_q':None,'to_state_q':None,'to_approved':None,'from_approved':None}        
        dict_State_Process['from_state'] = str(self.ui.cbx_changeFrom_State.currentText())
        dict_State_Process['to_state'] = str(self.ui.cbx_changeTo_State.currentText())
        dict_State_Process['from_state_q'] = str(self.ui.cbx_changeFrom_StateQualifier.currentText())
        dict_State_Process['to_state_q'] = str(self.ui.cbx_changeTo_StateQualifier.currentText())
        dict_State_Process['from_approved'] = self.ui.chk_changeFrom_Approved.isChecked()
        dict_State_Process['to_approved'] = self.ui.chk_changeTo_Approved.isChecked()
        self.Batch_Dict=dict_State_Process
        messageBox = QtGui.QMessageBox
        title = self.trUtf8("Warning")
        msg = self.trUtf8("You might lose some translation process information, and this action cannot be undone.\nAre you sure you want to change all fuzzy messages to translated?")
        result = messageBox.warning(None, title, msg, 
                    messageBox.Yes | messageBox.Default,
                    messageBox.No | messageBox.Escape)
        if (result == messageBox.Yes):
            self.start_Batch_Process()  
        self.close() 
        
    def start_Batch_Process(self):
        #this function calls other functin which are resposible for different batch processing operations. 
        if self.Batch_Dict['from_state']!=self.Batch_Dict['to_state']:
            self.changeState()
        if self.Batch_Dict['from_state_q']!=self.Batch_Dict['to_state_q']:
            self.changestateQualifier(self.Batch_Dict['from_state_q'], self.Batch_Dict['to_state_q'])  
        if self.Batch_Dict['from_approved']!=self.Batch_Dict['to_approved']:
            self.changeApproved(self.Batch_Dict['from_approved'], self.Batch_Dict['to_approved'])  
        self.parent.operator.contentChanged(True)    
            
    def changeState(self):
        #this function changes the states of different strings of the current file accordin to the user settings in batch processing window
        for unit in self.parent.operator.__store__.units:
           if (not hasattr(unit, "x_editor_state")):
                unit.x_editor_state = self.parent.operator.__status__.unitState(unit)
        OverIn=1
        for unit in self.parent.operator.__store__.units:
            if (unit.source and not (isinstance(self.parent.operator.__store__, xliff.xlifffile) and ( "gettext-domain-header" in (unit.getrestype() or "")))):
                self.parent.dockOverview.ui.tableOverview.setSortingEnabled(False)
                if ((self.Batch_Dict["from_state"]=="fuzzy" and self.Batch_Dict["to_state"]=="translated") and  (unit.x_editor_state & World.fuzzy)):      
                    self.parent.operator.__status__.markFuzzy(unit, False)
                if ((self.Batch_Dict["from_state"]=="translated" and self.Batch_Dict["to_state"]=="fuzzy") and  (unit.x_editor_state & World.translated)):
                    self.parent.operator.__status__.markFuzzy(unit, True)              
                for row in range(self.parent.unitnum):
                    rowindex=self.parent.dockOverview.ui.tableOverview.item(row, 0)
                    rowindex=int(rowindex.text())
                    if OverIn==rowindex:
                        self.parent.dockOverview.markState(row, unit.x_editor_state)
                        self.parent.dockOverview.markError(row, unit.x_editor_state)
                        self.parent.dockOverview.ui.tableOverview.setSortingEnabled(True)
                        break
            OverIn=OverIn+1
        self.parent.operator.contentChanged(True)
#        if (self.Batch_Dict["from_state"]=="fuzzy" and self.Batch_Dict["to_state"]=="translated"):
#            self.parent.operator.filterFuzzy(False)
#        elif (self.Batch_Dict["from_state"]=="translated" and self.Batch_Dict["to_state"]=="fuzzy"):     
#            self.parent.operator.filterFuzzy(True)    

    def changestateQualifier(self, from_stateQualifier, to_stateQualifier):
        #this function changes the state-qualifier of different strings of the current file according to the user settings in batch processing window
        for unit in self.parent.operator.__store__.units:
            unit.changeStateQualifier(from_stateQualifier, to_stateQualifier)        
        
    def changeApproved(self,  fromApproved,  toApproved):  
      #this function changes the the value of the approved attribute of different strings of the current file according to the user settings in batch processing window  
        for unit in self.parent.operator.__store__.units:
            hasapproved, isapproved=unit.checkapproved()
            if hasapproved:
                if fromApproved == isapproved :
                    unit.markapproved(toApproved)

    def showDialog(self):
        """Make the dialog visible with batch Process infomation fill in, if any.
        """
        if (not self.ui):
            self.ui = Ui_Dialog()
            self.ui.setupUi(self)  
            StateList = ["translated", "fuzzy", "needs-translation"]
            self.ui.cbx_changeFrom_State.addItems(StateList)
            self.ui.cbx_changeTo_State.addItems(StateList)
            QualifierList = ["exact-match", "fuzzy-match", "id-match" , "needs-review-translation"]
            self.ui.cbx_changeFrom_StateQualifier.addItems(QualifierList)
            self.ui.cbx_changeTo_StateQualifier.addItems(QualifierList)
        
            #connect signals
            QtCore.QObject.connect(self.ui.buttonChange,QtCore.SIGNAL("clicked()"), self.applyBatchProcess)
            QtCore.QObject.connect(self.ui.buttonCancel,QtCore.SIGNAL("clicked()"), self.reject)

        self.setWindowTitle(self.tr("Batch Processing"))
        self.setModal(True)
        self.show()
#        StateList_For_Future_deb = ["needs-adaptation", "needs-l10n", "needs-review-adaptation", "needs-review-l10n" , "needs-review-translation","sign-off", "needs-translation"]
#        QualifierList_For_Future_deb = ["exact-match", "fuzzy-match", "id-match", "leveraged-glossary", "leveraged-inherited", "leveraged-repository", "leveraged-tm", "rejected-grammar", "rejected-inaccurate", "rejected-spelling", "leveraged-mt", "rejected-length" , "tm-suggestion", "mt-suggestion"]

if __name__ == "__main__":
    import os, sys
    app = QtGui.QApplication(sys.argv)
    Dialog = BatchProcess(None)
    Dialog.showDialog()
    sys.exit(Dialog.exec_())        

