#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2006 - 2007 by The WordForge Foundation
#                 www.wordforgefoundation.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details. 
#
# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)
#
# This module is working on Catalog Manager of translation files

from PyQt4 import QtCore, QtGui
from WordForge.Editor.ui.Ui_Catalog import Ui_Catalog
from WordForge.Editor.modules.CatalogSetting import CatalogSetting
from WordForge.Editor.modules import TMSetting
from WordForge.translate.storage import factory
import WordForge.Editor.modules.World as World
from WordForge.Editor.modules.FindInCatalog import FindInCatalog
from WordForge.Editor.modules.NewProject import newProject
from WordForge.Editor.ui.Ui_tmSetting import Ui_tmsetting
from WordForge import __version__
from lxml import etree
from WordForge.Editor.modules.StatisticsDialog import StatisticDialog
from WordForge.Editor.modules.globalClass import globalClass
from WordForge.Editor.modules.Preference import Preference
import WordForge.Editor.ui.images_rc
import os, sys, time

class Catalog(QtGui.QMainWindow):
    """
    The Catalog Manager which holds the toolviews.
    """
    def __init__(self, parent = None):
        QtGui.QMainWindow.__init__(self, parent)
        self.parent = parent
        self.ui = None
        self.Project = newProject(self)
    def lazyInit(self):
        """
        Initialize only at time of calling Catalog.
        """
        if (self.ui):
            return
        self.ui = Ui_Catalog()
        self.ui.setupUi(self)
        self.resize(720,400)
#        self.updateRecentProject()
        self.setWindowTitle(self.tr("%s Project Manager" % (World.settingApp)))
        self.ui.toolBar.toggleViewAction()
        self.ui.actionBuildTM.setEnabled(False)
        self.ui.toolBar.setWindowTitle("ToolBar View")
        self.ui.toolBar.setStatusTip("Toggle toolbar view")

        self.folderIcon = QtGui.QIcon(":/open.png")
        self.iconFile = QtGui.QIcon(":/iconfile.png")
        # set up table appearance and behavior
        self.headerLabels = [self.tr("Name"),
                            self.tr("Untranslated"),
                            self.tr("Fuzzy"),
                            self.tr("Translated"),
                            self.tr("Total"),
                            self.tr("Error"), 
                            self.tr("CVS/SVN Status"),
                            self.tr("Last Revision"),
                            self.tr("Last Translator")]
        
        headerItem = self.ui.treeCatalog.headerItem()
        headerItem.setTextAlignment(0, QtCore.Qt.AlignHCenter)
        headerItem.setTextAlignment(1, QtCore.Qt.AlignHCenter)
        headerItem.setTextAlignment(2, QtCore.Qt.AlignHCenter)
        headerItem.setTextAlignment(3, QtCore.Qt.AlignHCenter)
        headerItem.setTextAlignment(4, QtCore.Qt.AlignHCenter)
        headerItem.setTextAlignment(5, QtCore.Qt.AlignHCenter)
        headerItem.setTextAlignment(6, QtCore.Qt.AlignHCenter)
        headerItem.setTextAlignment(7, QtCore.Qt.AlignHCenter)
        headerItem.setTextAlignment(8, QtCore.Qt.AlignHCenter)
        
        self.ui.treeCatalog.setColumnCount(len(self.headerLabels))
        self.ui.treeCatalog.setHeaderLabels(self.headerLabels)
        self.ui.treeCatalog.setSortingEnabled(True)
        self.ui.treeCatalog.sortItems(0, QtCore.Qt.AscendingOrder)
        self.ui.treeCatalog.hideColumn(6)
        self.ui.treeCatalog.header().setResizeMode(QtGui.QHeaderView.Interactive)

        # Quit action
        self.connect(self.ui.actionQuit, QtCore.SIGNAL("triggered()"), QtCore.SLOT("close()"))

        # Reload action
        self.connect(self.ui.actionReload, QtCore.SIGNAL("triggered()"), self.refresh)

        # Stop action
        self.connect(self.ui.actionStop, QtCore.SIGNAL("triggered()"), self.stopUpdate)

        # Statistic action
        self.connect(self.ui.actionStatistics, QtCore.SIGNAL("triggered()"), self.showStatistic)

        # show hide toolbar
        self.connect(self.ui.action_Toolbar, QtCore.SIGNAL("toggled(bool)"), self.ui.toolBar.setVisible)

        # a new project of Catalog Manager
    
        self.connect(self.ui.actionNew, QtCore.SIGNAL("triggered()"), self.showProjectDialogNew)
        self.connect(self.ui.actionOpen, QtCore.SIGNAL("triggered()"), self.showProjectDialogOpen)
        self.connect(self.ui.actionDelete, QtCore.SIGNAL("triggered()"), self.showProjectDialogDelete)
#        self.connect(self.ui.actionSave, QtCore.SIGNAL("triggered()"), self.Project.saveProject)
#        self.connect(self.ui.actionSaveAs, QtCore.SIGNAL("triggered()"), self.Project.saveAsProject)
        self.connect(self.ui.actionClose, QtCore.SIGNAL("triggered()"), self.closeProject)
        self.connect(self.ui.actionProperties, QtCore.SIGNAL("triggered()"), self.showProjectDialogProperty)
        self.connect(self.Project, QtCore.SIGNAL("updateCatalog"), self.updateCatalog)
#        self.connect(self.Project, QtCore.SIGNAL("hasModified"), self.setModified)

        # catalog setting's checkboxes action.
        self.catSetting = CatalogSetting(self)
        self.connect(self.ui.actionPreferences, QtCore.SIGNAL("triggered()"), self.catSetting.show)
        self.connect(self.ui.actionBuildTM, QtCore.SIGNAL("triggered()"), self.emitBuildTM)
        self.connect(self.catSetting.ui.chbname, QtCore.SIGNAL("stateChanged(int)"), self.toggleHeaderItem)
        self.connect(self.catSetting.ui.chbfuzzy, QtCore.SIGNAL("stateChanged(int)"), self.toggleHeaderItem)
        self.connect(self.catSetting.ui.chblastrevision, QtCore.SIGNAL("stateChanged(int)"), self.toggleHeaderItem)
        self.connect(self.catSetting.ui.chbtranslator, QtCore.SIGNAL("stateChanged(int)"), self.toggleHeaderItem)
        self.connect(self.catSetting.ui.chbuntranslated, QtCore.SIGNAL("stateChanged(int)"), self.toggleHeaderItem)
        self.connect(self.catSetting.ui.chbtotal, QtCore.SIGNAL("stateChanged(int)"), self.toggleHeaderItem)
        self.connect(self.catSetting.ui.chbSVN, QtCore.SIGNAL("stateChanged(int)"), self.toggleHeaderItem)
        self.connect(self.catSetting.ui.chbtranslated, QtCore.SIGNAL("stateChanged(int)"), self.toggleHeaderItem)
#        self.connect(self.catSetting.ui.chbfileError, QtCore.SIGNAL("stateChanged(int)"), self.toggleHeaderItem)
        # catalog's error checking operation
        self.connect(self.ui.actionSearchProjectError, QtCore.SIGNAL("triggered()"), self.intProjectError)
    
        # Create Find String in Catalog
        self.findBar = FindInCatalog(self)
        self.addDockWidget(QtCore.Qt.BottomDockWidgetArea, self.findBar)

        self.connect(self.ui.actionFind_in_Files, QtCore.SIGNAL("triggered()"), self.findBar.showFind)
        # emit findfiles signal from FindInCatalog file
        self.connect(self.findBar, QtCore.SIGNAL("findNext"), self.findNext)
        self.connect(self.findBar, QtCore.SIGNAL("returnSignal"), self._setFocusOnCatalog)

        # progress bar
        self.progressBar = QtGui.QProgressBar()
        self.progressBar.setEnabled(True)
        self.progressBar.setProperty("value", QtCore.QVariant(0))
        self.progressBar.setOrientation(QtCore.Qt.Horizontal)
        self.progressBar.setObjectName("progressBar")
        self.progressBar.setVisible(False)
        self.ui.statusbar.addPermanentWidget(self.progressBar)

        # Help menu of aboutQt
        action = QtGui.QWhatsThis.createAction(self)
        action.setText(self.tr("&What's This?"))
        action.setStatusTip("Context sensitive help")
        action.setWhatsThis("<h3>Display context sensitive help</h3>In what's This? mode, the mouse cursor shows an arrow with a question mark, and you can click on the interface elements to get a short description of what they do and how to use them in a dialog, the feature can be accessed using the context help button in the titlebar.")
        self.ui.menuHelp.insertAction(self.ui.actionAbout , action)
        self.ui.menuHelp.insertSeparator(self.ui.actionAbout)

        self.connect(self.ui.actionAbout, QtCore.SIGNAL("triggered()"), self.showAboutDialog)
        self.connect(self.ui.actionAboutQt, QtCore.SIGNAL("triggered()"), QtGui.qApp, QtCore.SLOT("aboutQt()"))

        self.connect(self.ui.treeCatalog, QtCore.SIGNAL("itemDoubleClicked(QTreeWidgetItem *, int)"), self.emitOpenFile)
        self.connect(self.ui.treeCatalog.model(), QtCore.SIGNAL("layoutChanged()"), self.sort)
        self.connect(self.ui.treeCatalog, QtCore.SIGNAL("currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *)"), self.itemChanged)
        self.setupCheckbox()

        # timer..
        self.timer = QtCore.QTimer()
        self.connect(self.timer, QtCore.SIGNAL("timeout()"), self.updateStatistic)
        self.lastFoundNumber = 0

        # context menu of items
        self.menu = QtGui.QMenu()
        self.actionOpen = self.menu.addAction(QtGui.QIcon(":/open.png"), self.tr("Open"))
        self.actionFind = self.menu.addAction(QtGui.QIcon(":/find.png"), self.tr("Find"))
        self.actionShowStat = self.menu.addAction(QtGui.QIcon(":/statistic.png"), self.tr("Show statistic"))

        self.connect(self.actionOpen, QtCore.SIGNAL("triggered()"), self.emitOpenFile)
        self.connect(self.actionFind, QtCore.SIGNAL("triggered()"), self.findBar.showFind)
        self.connect(self.actionShowStat, QtCore.SIGNAL("triggered()"), self.showStatistic)

        # install custom menu event for treeCatalog
        self.ui.treeCatalog.contextMenuEvent = self.customContextMenuEvent

        self.reachedEnd = False
        self.currentFileOpen = None
        self.searchString = None
        # bool indicates progress bar need update as TM progresses.
        self.allowUpdate = False

        self.notFoundString = None
        self.fontBold = QtGui.QFont()
        self.fontBold.setBold(True)
        
        self.actionOpen.setEnabled(False)
        self.actionFind.setEnabled(False)
        self.actionShowStat.setEnabled(False)
        self.ui.actionProperties.setEnabled(False)
        
        #error checking 
        self.searchError=False
        self.languageok=True

    def intProjectError(self):
        # initializes error checking operation for current project
        messageBox = QtGui.QMessageBox
        msgerror = self.trUtf8("Do you want to change error checking options from preference?")
        ret = messageBox()
        ret.setTextFormat(QtCore.Qt.RichText)
        ret.setText(msgerror)
        YesButton = ret.addButton(self.trUtf8("Yes"),messageBox.YesRole)
        NoButton = ret.addButton(self.trUtf8("No"), messageBox.NoRole)
        ret.exec_()
        
        if (ret.clickedButton() == YesButton):
            self.emit(QtCore.SIGNAL("showPreferenceError"), 7)
        elif (ret.clickedButton() == NoButton):
            self.searchProjectError()
           
#        if self.hasfileError:
#            QtGui.QMessageBox.information(self,"File Error", "Some files in this project do not have the same language that has been specified for this project. Error checking operation cannot be done in these file.",  QtGui.QMessageBox.Ok)    
    
    def searchProjectError(self):
        #does the error checking operation for current project
        self.catSetting.ui.chbfileError.setChecked(True)
        self.searchError=True
        self.itemNumber = 0
        if self.fileItems:
            for item in self.fileItems:
                item.setText(1, "")
                item.setText(2, "")
                item.setText(3, "")
                item.setText(4, "")
                item.setText(5, "")
            self.timer.start(10)
            self.lenFileItem=len(self.fileItems)
       
    def itemChanged(self, curItem, preItem):
        """
        slot item changed, use to reset self.reachedEnd, self.currentFileOpen
        to mark the start of search circle.

        @param curItem: QTreeWidgetItem
        @param preItem: QTreeWidgetItem
        These two params are not used at the moment.
        """

        self.reachedEnd = False
        self.currentFileOpen = None

    def setSearchStatus(self, message):
        """
        Set search status so that we can determine the search is end at operator.

        @param message: a string tells about the status of search process.
        """
        if (message == "reachedEnd"):
            self.reachedEnd = True

    def findNext(self, searchString, searchOptions):
        """
        The search here is only determine if search string exist in filename,
        then it connects to search function in operator.

        @param searchString: text to search for; type string.
        @param searchOptions: source and/or target; type as integer defined in world.
        """
        if (not searchString):
            return

        # if it starts with last-not-found string, do not search.
        if (self.notFoundString) and (searchString[:len(self.notFoundString)] == self.notFoundString):
            self.findBar.setColorStatus("notFound")
            self.ui.statusbar.showMessage(self.tr("Search was not found."), 3000)
            return

        if (not self.reachedEnd) and (self.currentFileOpen) and (searchString == self.searchString):
            self.emit(QtCore.SIGNAL("searchNext"))
            return

        currentItem = self.ui.treeCatalog.currentItem()
        try:
            resumePoint = self.fileItems.index(currentItem)
        except:
            return False
        for item in self.fileItems[resumePoint:] + self.fileItems[:resumePoint]:
            filename = self.getFilename(item)
            if (filename == self.currentFileOpen) and (self.reachedEnd):
                continue
            # first find if it is in the file or not.
            try:
                store = factory.getobject(filename)
            except Exception, e:
                QtGui.QMessageBox.critical(self, 'Error', 'Error while trying to open file ' + filename + '\n' + str(e))
                continue

            if (not hasattr(store, "units")):
                continue
            found = -1
            for unit in store.units:
                searchableText = ""
                if (searchOptions & World.source):
                    searchableText += unit.source or ""
                if (searchOptions & World.target):
                    searchableText += unicode(unit.target)
                index = searchableText.lower().find(searchString.lower())
                if (index > -1):
                    found = index
                    break
            if (found >= 0):
                self.notFoundString = None
                self.currentFileOpen = filename
                self.searchString = searchString
                self.ui.treeCatalog.setCurrentItem(item)
                self.emit(QtCore.SIGNAL("openFile"), filename)
                filter = []
                if (searchOptions & World.source):
                    filter.append(World.source)
                if (searchOptions & World.target):
                    filter.append(World.target)
                self.emit(QtCore.SIGNAL("initSearch"), searchString, filter, False)
                self.emit(QtCore.SIGNAL("searchNext"))
                break
            self.reachedEnd = False
        else:
            self.notFoundString = searchString
            self.currentFileOpen = None
            self.searchString = None
            self.findBar.setColorStatus("notFound")
            self.ui.statusbar.showMessage(self.tr("Search was not found."), 3000)

    def _getStringFromUnit(self, unit, fields):
        """
        Return QString of unit's field; source, target.

        @param unit: class unit.
        @param fields: members of class unit such as source or target.
        """
        unitString = ""
        if (fields & World.source):
            unitString += unit.source
        if (fields & World.target):
            unitString += unit.target
        return unitString

    def showStatistic(self):
        """
        Show statistic message in a dialog.
        """
        item = self.ui.treeCatalog.currentItem()
        if (not item):
            return
        
        filename = self.getFilename(item)
        title = unicode(os.path.basename(filename))
        if os.path.isdir(filename):
            title = title + " (folder)"
        
        translated = int(item.text(1).__str__())
        fuzzy = int(item.text(2).__str__())
        untranslated = int(item.text(3).__str__())
        
        # need float value for percentage
        total = float(item.text(4).__str__())
        numOfFiles = self.getNumOfFile(item)
        
        if (total > 0):
            perTranslated = str((translated/total) * 100)
            perTranslated = perTranslated[0:perTranslated.find(".")+3] + "%"
            perFuzzy = str((fuzzy/total) * 100)
            perFuzzy = perFuzzy[0:perFuzzy.find(".")+3] + "%"
            perUntranslated = str((untranslated/total) * 100)
            perUntranslated = perUntranslated[0:perUntranslated.find(".")+3] + "%"
        else:
            perTranslated = "0%"
            perFuzzy = "0%"
            perUntranslated = "0%"
        
        self.statisticsDialog = StatisticDialog(self)
        
        self.lblStatistic = self.statisticsDialog.ui.lblStatistic
        self.lblStatistic.setText(title)
        
        self.lblNumberofFiles = self.statisticsDialog.ui.lblNumberofFiles
        self.lblNumberofFiles.setText(str(numOfFiles))
        
        self.lblTranslated = self.statisticsDialog.ui.lblTranslated
        self.lblTranslated.setText(str(translated))
        
        self.lblTransPercent = self.statisticsDialog.ui.lblTransPercent
        self.lblTransPercent.setText(perTranslated)
        
        self.lblFuzzy = self.statisticsDialog.ui.lblFuzzy
        self.lblFuzzy.setText(str(fuzzy))
        
        self.lblFuzzyPercent = self.statisticsDialog.ui.lblFuzzyPercent
        self.lblFuzzyPercent.setText(perFuzzy)
        
        self.lblUntranslated = self.statisticsDialog.ui.lblUntranslated
        self.lblUntranslated.setText(str(untranslated)) 
        
        self.lblUntranPercent = self.statisticsDialog.ui.lblUntranPercent
        self.lblUntranPercent.setText(perUntranslated) 
        
        self.lblTotal = self.statisticsDialog.ui.lblTotal
        self.lblTotal.setText(str(int(total)))
        
        self.statisticsDialog.show()

    def toggleHeaderItem(self):
        """
        Show or hide the header label of catalog main window.
        """
        if (isinstance(self.sender(), QtGui.QCheckBox)):
            text = self.sender().text()
            if text in self.headerLabels:
                checked = self.sender().isChecked()
                if (checked):
                    self.ui.treeCatalog.showColumn(self.headerLabels.index(text))
                    self.ui.treeCatalog.hideColumn(6)
                else:
                    self.ui.treeCatalog.hideColumn(self.headerLabels.index(text))
                    self.ui.treeCatalog.hideColumn(6)
                World.settings.setValue("Catalog." + text, QtCore.QVariant(checked))

    def updateProgress(self, value):
        """
        Set the value for the progress bar.
        self.allowUpdate is a condition whether to update or not, use to protect
        other class call.

        @param value: value to set, type as int.
        """
        if (not hasattr(self, "allowUpdate")) or (not self.allowUpdate):
           return
        if (not self.progressBar.isVisible()):
            self.progressBar.setVisible(True)
        elif (value == 100):
            self.allowUpdate = False
            self.progressBar.setVisible(False)
        self.progressBar.setValue(value)

    def showDialog(self):
        """
        Display the Catalog main window with configured information.
        """
        self.lazyInit()
        self.show()
        if (self.ui.treeCatalog.topLevelItemCount() == 0):
            # first look if there's current project previously opened.
            currentProject = World.settings.value("CurrentProject").toString()
            if (currentProject):
                self.openProject(currentProject)

    def updateCatalog(self, prjRoot):
        """
        Read data from "prjRoot" and display statistic of files in tree view.

        @param prjRoot: a project xml file, contains name, language, paths to project.
        @param includeSub: a boolean variable. If true, dive into sub folder.
        """

        if (prjRoot != None):
#            self.ui.actionSaveAs.setEnabled(True)
            self.actionOpen.setEnabled(True)
            self.actionFind.setEnabled(True)
            self.actionShowStat.setEnabled(True)
            self.ui.actionReload.setEnabled(True)
            self.ui.actionStop.setEnabled(True)
            self.ui.actionFind_in_Files.setEnabled(True)
            self.ui.actionStatistics.setEnabled(True)
            self.ui.actionProperties.setEnabled(True)
            self.ui.actionBuildTM.setEnabled(True)
            self.ui.actionClose.setEnabled(True)
        else:
            self.actionOpen.setEnabled(False)
            self.actionFind.setEnabled(False)
            self.actionShowStat.setEnabled(False)
#            self.ui.actionSaveAs.setEnabled(False)
            self.ui.actionReload.setEnabled(False)
            self.ui.actionStop.setEnabled(False)
            self.ui.actionFind_in_Files.setEnabled(False)
            self.ui.actionStatistics.setEnabled(False)
            self.ui.actionProperties.setEnabled(False)
            self.ui.actionBuildTM.setEnabled(False)
            self.ui.actionClose.setEnabled(False)
            self.progressBar.setVisible(False)
            self.ui.treeCatalog.clear()
            
            self.setWindowTitle(self.tr("%s Project Manager" % (World.settingApp)))
            self.fileItems = []
            return
        
        # read informations from prjRoot xml data
        self.prjRoot = prjRoot
        name = prjRoot.get("name")
        paths = []
        for child in prjRoot:
            if child.tag == "catalog":
                catalog = child.get("path")
                if catalog:
                    paths.append(catalog)
        includeSub = bool(prjRoot.get("includeSub") == "True") or False 
        
        if (not self.isVisible()):
            self.showDialog()
        title = unicode(self.tr("%s - %s  Project Manager")) % (unicode(name), World.settingApp)
        self.setWindowTitle(str(title))
        
        self.ui.treeCatalog.clear()
        self.fileItems = []
        
        # disconnect sorting to avoid calculation of self.fileItems in self.sort()
        # reconnect it after all numbers are calculated, after timer stopped.
        self.disconnect(self.ui.treeCatalog.model(), QtCore.SIGNAL("layoutChanged()"), self.sort)
        
        # sort the catalog path, so the adding function to ui is not duplicated.
        paths.sort()
        for path in paths:
            path = unicode(path)
            self.addCatalogFile(path, includeSub)
        self.ui.treeCatalog.resizeColumnToContents(0)
        self.sort()
        self.allowUpdate = True
##        self._setFocusOnCatalog()
        self.ui.treeCatalog.setFocus()
        self.itemNumber = 0
        self.timer.start(10)

    def keyReleaseEvent(self, event):
        """
        A subclass for keyReleaseEvent.
        """
        # press Enter key to open the file.
        if (event.key() == 16777220) and (self.ui.treeCatalog.currentItem()):
            self.emitOpenFile()

    def _setFocusOnCatalog(self):
        """
        Set focus on the first item in Catalog treeview.
        """
        try:
            if (self.fileItems):
                item = self.fileItems[0]
                self.ui.treeCatalog.setCurrentItem(item)
                self.ui.treeCatalog.setFocus()
        except:
            return False

    def addCatalogFile(self, path, includeSub):
        """
        add path to catalog tree view if it's file, if it's directory then
        dive into it and add files.

        @param path: file or directory path to set to the catalog tree view.
        @param includeSub: If it is True, dive into subfolder
        @param item: childItem in the folder
        """
        # do not add file that is aleady existed in tree.
        pathExisted = False
        for item in self.fileItems:
            itemData = item.data(0, QtCore.Qt.UserRole).toString().__str__()
            if (path == itemData):
                # skip
                pathExisted = True
                break
        
        if (not pathExisted):
            if os.path.isfile(path):
                parentItem = QtGui.QTreeWidgetItem()
                self.ui.treeCatalog.addTopLevelItem(parentItem)
                parentItem.setText(0, path)
                parentItem.setData(0, QtCore.Qt.UserRole, QtCore.QVariant(path))
                parentItem.setTextAlignment(1, QtCore.Qt.AlignRight)
                parentItem.setTextAlignment(2, QtCore.Qt.AlignRight)
                parentItem.setTextAlignment(3, QtCore.Qt.AlignRight)
                parentItem.setTextAlignment(4, QtCore.Qt.AlignRight)
                parentItem.setTextAlignment(5, QtCore.Qt.AlignRight)
                parentItem.setTextAlignment(7, QtCore.Qt.AlignRight)
                self.fileItems.append(parentItem)
                self.ui.treeCatalog.expandItem(parentItem)
            
        # do not recurve to its sub folder
        if pathExisted and os.path.isdir(path):
            return
        
        lastItem = None
        for root, dirs, files in os.walk(path):
            # try to find its parent in order to insert sub folder correctly.
            # add first root folder, it must shown in full path.
            if not lastItem:
                childItem = QtGui.QTreeWidgetItem()
                self.ui.treeCatalog.addTopLevelItem(childItem)
                childItem.setText(0, root)
            else:
                itemData = lastItem.data(0, QtCore.Qt.UserRole).toString().__str__()
                if os.path.dirname(root) == itemData:
                    parentItem = lastItem
                else:
                    parentItem = self.getParent(lastItem, root)
                # add sub folder to tree widget
                childItem = QtGui.QTreeWidgetItem(parentItem)
                childItem.setText(0, " " + os.path.basename(root))
                
            childItem.setData(0, QtCore.Qt.UserRole, QtCore.QVariant(root))
            childItem.setIcon(0, self.folderIcon)
            childItem.setFont(0, self.fontBold)
            childItem.setFont(1, self.fontBold)
            childItem.setFont(2, self.fontBold)
            childItem.setFont(3, self.fontBold)
            childItem.setFont(4, self.fontBold)
            childItem.setTextAlignment(1, QtCore.Qt.AlignRight)
            childItem.setTextAlignment(2, QtCore.Qt.AlignRight)
            childItem.setTextAlignment(3, QtCore.Qt.AlignRight)
            childItem.setTextAlignment(4, QtCore.Qt.AlignRight)
            childItem.setTextAlignment(5, QtCore.Qt.AlignRight)
            childItem.setTextAlignment(7, QtCore.Qt.AlignRight)
            self.fileItems.append(childItem)
            self.ui.treeCatalog.expandItem(childItem)
            lastItem = childItem
            
            for file in files:
                # add only these file types
                if (file.endswith(".po") or file.endswith(".pot") or file.endswith(".xlf") or file.endswith(".xliff")):
                    fileItem = QtGui.QTreeWidgetItem(childItem)
                    fileItem.setText(0, file)
                    fileItem.setData(0, QtCore.Qt.UserRole, QtCore.QVariant(root + os.path.sep + file))
                    fileItem.setIcon(0, self.iconFile)
                    fileItem.setTextAlignment(1, QtCore.Qt.AlignRight)
                    fileItem.setTextAlignment(2, QtCore.Qt.AlignRight)
                    fileItem.setTextAlignment(3, QtCore.Qt.AlignRight)
                    fileItem.setTextAlignment(4, QtCore.Qt.AlignRight)
                    fileItem.setTextAlignment(5, QtCore.Qt.AlignRight)
                    fileItem.setTextAlignment(7, QtCore.Qt.AlignRight)
                    self.fileItems.append(fileItem)
            if (not includeSub):
                break
            
    def getParent(self, item, path):
        if hasattr(item, "parent"):
            item = item.parent()
            if (item):
                itemData = item.data(0, QtCore.Qt.UserRole).toString().__str__()
                if os.path.dirname(path) == itemData:
                    return item
                else:
                    item = self.getParent(item, path)
        return item
            
    
    def updateFileStatus(self, filename, numUntran, numFuzzy, numTran):
        """
        Update status of a file.
        @param filename: the file whose status will be updated.
        """
        if (not hasattr(self, "fileItems")):
            return
        for item in self.fileItems:
            itemData = item.data(0, QtCore.Qt.UserRole).toString().__str__()
            if (filename == itemData):
                item.setText(1, (str(numUntran)).rjust(4))
                item.setText(2, (str(numFuzzy)).rjust(4))
                item.setText(3, (str(numTran)).rjust(4))
                item.setText(4, (str(numUntran + numFuzzy + numTran)).rjust(4))
                # skip
                break
    
    def ifFileExisted(self, path, item):
        """
        Get existed item in the view. If the item existed, it returns
        the item, otherwise returns False.

        @param path: file or directory path to get if it is existed.
        @param item: QTreeWidgetItem
        """
        if (not hasattr(item, "childCount")):
            return False
        for i in range(item.childCount()):
            it = item.child(i)
            if (hasattr(it, "text")) and (it.text(0) == os.path.basename(path)):
                return it
        return False

    def getStats(self, filename):
        """
        return a dictionary which consists of basename, translatedCount, fuzzyCount,
        untranslatedCount, totalCount, subVersionState, revisionDate, lastTranslator from the filename.
        return False when error.

        @param filename: path and file name whose status is returned.
        """
        numError=0
        if (not os.path.isfile(filename)):
            return False
        
        # read xliff or po file for information such as number of untranslated, translated, and fuzzy messages.
        # get file element if it is already existed in prjRoot
        fileElementExist = False
        
        for file in self.prjRoot:
            if (file.tag == "file"):
                if (filename == file.get("path")):
                    fileElementExist = True
                    break
        fileAlreadyRead = False
        self.hasfileError=False
        if fileElementExist:
            #start of Aditi's code for searching error in project 
            if self.searchError:
                filelanguage=self.parent.preference.checkfileLanguage(filename, "newproject")
                if ((filename in self.parent.preference.fileswithnolanguage) or (filename in self.parent.preference.fileswithdifflanguage)):
                    self.languageok=False      
            if self.searchError and self.languageok:     
                store = factory.getobject(filename)
                self.catalogStore=store
                self.projectFileName=filename
                searchError=self.searchError
                self.parent.checkingErrorFile(searchError, store)
                numError= self.parent.fileError
            else:
                if self.searchError and not self.languageok:
                    self.hasfileError=True  
            self.languageok=True
#            numError= len(globalClass.fileErrorList)

            #end of Aditi's code for searching error in project 
            # compare timestamp of filename and timestamp of file node of prj xml file.
            timestamp = os.lstat(filename).st_mtime
            timestampText = time.strftime("%Y-%b-%e %R %S", time.gmtime(timestamp))
            if (timestampText == file.get("timestamp")):
#                print "not read:",  filename
                fileAlreadyRead = True
                basename = os.path.basename(file.get("path"))
                numTranslated = 0
                numUntranslated = 0
                numFuzzy = 0
                for child in file:
                    if (child.tag == "translated"):
                        numTranslated = int(child.text)
                    if (child.tag == "untranslated"):
                        numUntranslated = int(child.text)
                    if (child.tag == "fuzzy"):
                        numFuzzy = int(child.text)
                    if (child.tag == "error"):
                        numError = int(child.text)    
                numTotal = numTranslated + numUntranslated + numFuzzy
                subVersionState = ""
                revisionDate = timestampText
                lastTranslator = file.get("lasttranslator")

        if (not fileAlreadyRead):
#            print "parsing:", filename
            try:
                store = factory.getobject(filename)
            except:
                return False
            basename = os.path.basename(filename)
            numTranslated = store.translated_unitcount()
            numFuzzy = store.fuzzy_unitcount()
            numUntranslated = store.untranslated_unitcount()
            numTotal = numTranslated + numUntranslated + numFuzzy
            subVersionState = ""
            
            numTotal = numTranslated + numUntranslated + numFuzzy
            subVersionState = ""
            timestamp = os.lstat(filename).st_mtime
            revisionDate = time.strftime("%Y-%b-%e %R %S", time.gmtime(timestamp))
    #        time.strftime("%Y-%b-%e %R %S", time.gmtime(ts))
            lastTranslator = ""
            if hasattr(store, "parseheader"):
                headerDic = store.parseheader()
#                try:
#                    revisionDate = str(headerDic["PO-Revision-Date"])
#                except:
#                    pass
                try:
                    lastTranslator = unicode(headerDic["Last-Translator"])
                    revisionDate = str(headerDic["PO-Revision-Date"])
                    
                except:
                    lastTranslator = "You"
                    pass        
        return {"basename":basename, "numUntranslated":numUntranslated, "numFuzzy":numFuzzy, 
                "numTranslated":numTranslated ,"numTotal":numTotal, 
                "subVersionState":subVersionState, "revisionDate":revisionDate, 
                "lastTranslator":lastTranslator, "error":numError}

    def countMessageNumber(self, item):
        """
        get number of untranslated, fuzzy, and translated units from item.

        @param item: treewidget item which has at least those four fields.
        @return dictionary of stats.
        """

        translated = 0
        fuzzy = 0
        untranslated = 0
        error = 0
        if item.text(1):
            translated = int(item.text(1))
        if item.text(2):
            fuzzy = int(item.text(2))
        if item.text(3):
            untranslated = int(item.text(3))
        if item.text(5):
            error = int(item.text(5))    

        for i in range(item.childCount()):
            child = item.child(i)
            stats = self.countMessageNumber(child)
            translated += stats["translated"]
            fuzzy += stats["fuzzy"]
            untranslated += stats["untranslated"]
            error += stats["error"]
        
        return {"untranslated":untranslated, "fuzzy":fuzzy, "translated":translated,"error":error }

    def getNumOfFile(self, item):
        """
        get number of files in the item, it will dive in to children.

        @param item: treewidget item.
        @return number of files.
        """
        if item.text(0):
            path = item.data(0, QtCore.Qt.UserRole).toString().__str__()
            if os.path.isfile(path):
                numOfFiles = 1
            else:
                numOfFiles = 0
        else:
            numOfFiles = 0
            
        for i in range(item.childCount()):
            child = item.child(i)
            numOfFiles += self.getNumOfFile(child)
            
        return numOfFiles
        
    def setupCheckbox(self):
        """
        Set checked or unchecked for the header label of Catalog main window.
        """

        # FIXME: from Jens
        #       you are doing the same thing again and again here 
        #       code duplication is evel ;-)
        #       please make a funktion with the name of the setting as parameter and
        #       which returns QtCore.Qt.Unchecked or QtCore.Qt.Checked
        #       than you can write like this:
        #           self.catSetting.ui.chbname.setCheckState(getCheckState("Catalog.Name"))

        value = World.settings.value("Catalog.Name")
        if (value.isValid()):
            if (value.toBool()):
                checkState = QtCore.Qt.Checked
            else:
                checkState = QtCore.Qt.Unchecked
        else:
            # on fresh start of program, if the value is not in setting yet
            # make it True by default.
            checkState = QtCore.Qt.Checked
        self.catSetting.ui.chbname.setCheckState(checkState)

        value = World.settings.value("Catalog.Translated")
        if (value.isValid()):
            if (value.toBool()):
                checkState = QtCore.Qt.Checked
            else:
                checkState = QtCore.Qt.Unchecked
        else:
            checkState = QtCore.Qt.Checked
        self.catSetting.ui.chbtranslated.setCheckState(checkState)

        value = World.settings.value("Catalog.Fuzzy")
        if (value.isValid()):
            if (value.toBool()):
                checkState = QtCore.Qt.Checked
            else:
                checkState = QtCore.Qt.Unchecked
        else:
            checkState = QtCore.Qt.Checked
        self.catSetting.ui.chbfuzzy.setCheckState(checkState)

        value = World.settings.value("Catalog.Untranslated")
        if (value.isValid()):
            if (value.toBool()):
                checkState = QtCore.Qt.Checked
            else:
                checkState = QtCore.Qt.Unchecked
        else:
            checkState = QtCore.Qt.Checked
        self.catSetting.ui.chbuntranslated.setCheckState(checkState)

        value = World.settings.value("Catalog.Total")
        if (value.isValid()):
            if (value.toBool()):
                checkState = QtCore.Qt.Checked
            else:
                checkState = QtCore.Qt.Unchecked
        else:
            checkState = QtCore.Qt.Checked
        self.catSetting.ui.chbtotal.setCheckState(checkState)

        value = World.settings.value("Catalog.CVS/SVN Status")
        if (value.isValid()):
            if (value.toBool()):
                checkState = QtCore.Qt.Checked
            else:
                checkState = QtCore.Qt.Unchecked
        else:
            checkState = QtCore.Qt.Checked
        self.catSetting.ui.chbSVN.setCheckState(checkState)
        self.catSetting.ui.chbSVN.setVisible(False)

        value = World.settings.value("Catalog.Last Revision")
        if (value.isValid()):
            if (value.toBool()):
                checkState = QtCore.Qt.Checked
            else:
                checkState = QtCore.Qt.Unchecked
        else:
            checkState = QtCore.Qt.Checked
        self.catSetting.ui.chblastrevision.setCheckState(checkState)

        value = World.settings.value("Catalog.Last Translator")

        if (value.isValid()):
            if (value.toBool()):
                checkState = QtCore.Qt.Checked
            else:
                checkState = QtCore.Qt.Unchecked
        else:
            checkState = QtCore.Qt.Checked
        self.catSetting.ui.chbtranslator.setCheckState(checkState)
        
        value = World.settings.value("Catalog.FileError")
        if (value.isValid()):
            if (value.toBool()):
                checkState = QtCore.Qt.Checked
            else:
                checkState = QtCore.Qt.Unchecked
        else:
            checkState = QtCore.Qt.Checked
        self.catSetting.ui.chbfileError.setCheckState(checkState)

    def emitOpenFile(self, item=None, col=None):
        """
        Send "openFile" signal with filename.

        @param item: type as treewidget item 
        @param col: type as int. 
        """
        if (not item):
            try:
                item = self.ui.treeCatalog.selectedItems()[0]
                
            except:
                return

        filename = self.getFilename(item)
        
        if (os.path.isfile(filename)): 
            self.emit(QtCore.SIGNAL("openFile"), filename)

    def getFilename(self, item):
        """
        return filename join from item.text(0) to its parent.

        @ param item: type as treewidget item. column must be 0 because
                                the filename is in the first col.
        """
        filename = item.data(0, QtCore.Qt.UserRole).toString().__str__()
#        if (not hasattr(item, "text")):
#            return None
#        filename = unicode(item.text(0))
#        if (item.parent()):
#            filename = os.path.join(self.getFilename(item.parent()) + os.path.sep + filename)
        return filename

    def refresh(self):
        """
        Re-read the files for catalog window.
        """
        self.stopUpdate()

        for item in self.fileItems:
            item.setText(1, "")
            item.setText(2, "")
            item.setText(3, "")
            item.setText(4, "")
            item.setText(5, "")
            item.setText(6, "")
            item.setText(7, "")
            item.setText(8, "")
#        # remove the file node so the catalog will re-read the files.
        if (self.prjRoot):
            for child in self.prjRoot:
                if (child.tag == "file"):
                    self.prjRoot.remove(child)
#        
        self.itemNumber = 0
        self.timer.start(10)

    def emitBuildTM(self):
        """
        Emit "buildTM" signal with catalog paths.
        """
        catPaths = []
        for i in range(self.ui.treeCatalog.topLevelItemCount()):
            topItem = self.ui.treeCatalog.topLevelItem(i)
            catPath = self.getFilename(topItem)
            catPaths.append(catPath)
        self.allowUpdate = True
        self.emit(QtCore.SIGNAL("buildTM"), catPaths)

    def updateStatistic(self):
        """
        Update statistic information once adding file to the Catalog tree.
        """
        if (len(self.fileItems) <= 0):
            self.timer.stop()
            self.itemNumber = 0
            return
        self.timer.stop()
        item = self.fileItems[self.itemNumber]
        path = self.getFilename(item)
        childStats = self.getStats(path)         
        if (childStats): 
            item.setText(1, (str(childStats["numUntranslated"])).rjust(4))
            item.setText(2, (str(childStats["numFuzzy"])).rjust(4))
            item.setText(3, (str(childStats["numTranslated"])).rjust(4))
            item.setText(4, (str(childStats["numTotal"])).rjust(4))
            item.setText(5, (str(childStats["error"])).rjust(4))
            item.setText(6, str(childStats["subVersionState"]))
            item.setText(7, str(childStats["revisionDate"]))
            item.setText(8, unicode(childStats["lastTranslator"]))
            
            # get file element if it is already existed in prjRoot
            fileElementExist = False
            for file in self.prjRoot:
                if (file.tag == "file"):
                    if (path == file.get("path")):
                        fileElementExist = True
                        break

            if fileElementExist:
                # use existed file element, update data to file and its children
                file.set("timestamp", str(childStats["revisionDate"]))
                file.set("lasttranslator", unicode(childStats["lastTranslator"]))
                file.set("fileError",  (str(childStats["error"])))
                child1 = file.find("translated")
                if hasattr(child1, "text"):
                    child1.text = str(childStats["numTranslated"])
                else:
                    child1 = etree.SubElement(file ,"translated")
                    child1.text = str(childStats["numTranslated"])
                
                
                child1 = file.find("untranslated")
                if hasattr(child1, "text"):
                    child1.text = str(childStats["numUntranslated"])
                else:
                    child1 = etree.SubElement(file ,"untranslated")
                    child1.text = str(childStats["numUntranslated"])

                child1 = file.find("fuzzy")
                if hasattr(child1, "text"):
                    child1.text = str(childStats["numFuzzy"])
                else:
                    child1 = etree.SubElement(file ,"fuzzy")
                    child1.text = str(childStats["numFuzzy"])
            
            else:
                # file element not exist, so it creates a new file element, and children
                file = etree.SubElement(self.prjRoot, "file", path=unicode(path), timestamp=str(childStats["revisionDate"]), lasttranslator=unicode("You"))
                child1 = etree.SubElement(file ,"translated")
                child1.text = str(childStats["numTranslated"])
                child2 = etree.SubElement(file, "untranslated")
                child2.text = str(childStats["numUntranslated"])
                child3 = etree.SubElement(file, "fuzzy")
                child3.text = str(childStats["numFuzzy"])
                
        self.itemNumber += 1
        self.allowUpdate = True
        perc = int((float(self.itemNumber) / len(self.fileItems)) * 100)
        self.updateProgress(perc)
        # start getting statistic
        self.timer.start(10)

         
        # end of reading number or loading each file for number.
        if (self.itemNumber == len(self.fileItems)):
            self.timer.stop()
            self.itemNumber = 0
            globalClass.searchProjectError=False
            self.searchError=False
            
            # calculation of messages in directory
            for item in self.fileItems:
                stats = self.countMessageNumber(item)
                translated = stats["translated"]
                fuzzy = stats["fuzzy"]
                untranslated = stats["untranslated"]
                error=stats["error"]
                total = translated + fuzzy + untranslated
                item.setText(1, str(translated))
                item.setText(2, str(fuzzy))
                item.setText(3, str(untranslated))
                item.setText(4, str(total))
                item.setText(5, str(error))
            # connect to self.sort which calculate the self.fileItems
            self.disconnect(self.ui.treeCatalog.model(), QtCore.SIGNAL("layoutChanged()"), self.sort)
            self.connect(self.ui.treeCatalog.model(), QtCore.SIGNAL("layoutChanged()"), self.sort)
            
            projectFilename = World.settings.value("CurrentProject").toString()
            self.Project.setPrjRoot(self.prjRoot)
            self.Project.saveProject()

    def stopUpdate(self):
        """
        Stop loading the file(s) into Catalog treeview.
        """
        self.timer.stop()
        self.updateProgress(100)
#    def updateRecentProject(self):
#        """
#        Open openRecentProject list in World, and update it in recent open menu.
#        Put current project on top of list.
#        """
#
#        projectList = World.settings.value("recentProjectList").toStringList()
#        projectFilename = World.settings.value("CurrentProject").toString()
#
##        print "now open:", projectFilename
#
#        if (projectFilename):
#            # remove from list if it's already existed.
#            projectFilenameIndex = projectList.indexOf(projectFilename)
#            if (projectFilenameIndex >= 0):
#                projectList.removeAt(projectFilenameIndex)
#            # then add current project at top
#            projectList.prepend(projectFilename)
#
#        # remove older recent files when reached max count.
#        while (projectList.count() > World.MaxRecentFiles + 1):
#            projectList.removeAt(World.MaxRecentFiles + 1)
#        World.settings.setValue("recentProjectList", QtCore.QVariant(projectList))
        
#        projectList = World.settings.value("recentProjectList").toStringList()
#        projectFilename = World.settings.value("CurrentProject").toString()
#        
#        if (projectFilename):
#            # remove from list if it's already existed.
#            projectFilenameIndex = projectList.indexOf(projectFilename)
#            if (projectFilenameIndex >= 0):
#                projectList.removeAt(projectFilenameIndex)
#            # then add current project at top
#            projectList.prepend(projectFilename)
#
#        # remove older recent files when reached max count.
#        while (projectList.count() > World.MaxRecentFiles + 1):
#            projectList.removeAt(World.MaxRecentFiles + 1)
#        World.settings.setValue("recentProjectList", QtCore.QVariant(projectList))
#
#        i = 0
#        self.ui.menuOpenRecentProject.clear()
#        for project in projectList:
#            i += 1
#            recentAction = QtGui.QAction(self)
            # FIXME: from Jens
            #       how should you translate dynamic content like str(i)?
            #       I think this is not possible
#            recentAction.setText(self.tr("&" + str(i) + ": ") + project)
#            recentAction.setData(QtCore.QVariant(project))
#            self.ui.menuOpenRecentProject.addAction(recentAction)
#            self.connect(recentAction, QtCore.SIGNAL("triggered()"), self.openRecentProject)
#
#        self.ui.menuOpenRecentProject.addSeparator()
#        clearAction = QtGui.QAction(self.tr("&Clear"), self)
#        self.ui.menuOpenRecentProject.addAction(clearAction)
#        self.connect(clearAction, QtCore.SIGNAL("triggered()"), self.clearRecentProject)

#        if (projectList):
#            self.ui.menuOpenRecentProject.setEnabled(True)
#        else:
#            self.ui.menuOpenRecentProject.setEnabled(False)

#    def clearRecentProject(self):
#        """
#        Slot to clear all path in open recent project list.
#        """
#        self.ui.menuOpenRecentProject.clear()
#        self.ui.menuOpenRecentProject.setEnabled(False)
#        World.settings.remove("recentProjectList")

#    def openRecentProject(self):
#        """
#        Get the file from the open recent list and call openProject().
#        """
#        action = self.sender()
#        filename = action.data().toString()
#
#        # A dialog ask to remove death file name.
#        if (not(os.path.isfile(filename))):
#            titled = unicode(self.tr("%s was not found.\nDo you want to remove it from list?")) % (unicode(filename))
#            ret = QtGui.QMessageBox.critical(self, self.tr("Error"), 
#                titled,
#                QtGui.QMessageBox.Yes | QtGui.QMessageBox.Default, 
#                QtGui.QMessageBox.No | QtGui.QMessageBox.Escape) 
#            if (ret == QtGui.QMessageBox.Yes):
#                projectList = World.settings.value("recentProjectList").toStringList()
#                filenameIndex = projectList.indexOf(filename)
#                if (filenameIndex >= 0):
#                    projectList.removeAt(filenameIndex)
#                World.settings.setValue("recentProjectList", QtCore.QVariant(projectList))
#                self.updateRecentProject()
#            return
#
#        self.openProject(filename)

    def customContextMenuEvent(self, e):
        """
        A subclass for customContextMenu event.
        """
        self.menu.exec_(e.globalPos())

    def sort(self):
        """
        Re-arrange the fileItems to suit the sorting order.
        """
        self.fileItems = []
        for i in range(self.ui.treeCatalog.topLevelItemCount()):
            item = self.ui.treeCatalog.topLevelItem(i)
            self.fileItems += self.getItems(item)
        
##            
##        for i in range(self.ui.treeCatalog.topLevelItemCount()):
##            item = self.ui.treeCatalog.topLevelItem(i)
##            self.sortItemsByType(item, i)

    def getItems(self, items):
        """
        Return a list of item in items.

        @param items: type as treewidget item.
        """
        item = [items]
        # children
        for i in range(items.childCount()):
            item += self.getItems(items.child(i))
        return item

    def showProjectDialogNew(self):
        self.Project.showDialog(World.projectModeNew)

    def showProjectDialogProperty(self):
        self.Project.showDialog(World.projectModeProperty)

    def showProjectDialogOpen(self):
        self.Project.showDialog(World.projectModeOpen)

    def showProjectDialogDelete(self):
        self.Project.showDialog(World.projectModeDelete)

    def openProject(self, filename = None):
        """
        Slot to call Project.openProject().
        """
        self.emit(QtCore.SIGNAL("projectOpened"), filename)
        self.lazyInit()
        self.Project.openProject(filename)
#        self.updateRecentProject()

    def closeProject(self):
        """
        Stop update to avoid crash and call Project.closeProject().
        """
        self.emit(QtCore.SIGNAL("projectOpened"), False)
        self.stopUpdate()
        self.Project.closeProject()
        self.setWindowTitle(self.tr("%s Project Manager" % (World.settingApp)))
        self.fileItems = []

    def showAboutDialog(self):
        """
        Display the about dialog.
        """
        from WordForge.Editor.modules.AboutEditor import AboutEditor
        if (not hasattr(self, "aboutDialog") ):
            self.aboutDialog = AboutEditor(self)
        self.aboutDialog.showDialog()

def main():
    # set the path for QT in order to find the icons
    if __name__ == "__main__":
        QtCore.QDir.setCurrent(os.path.join(sys.path[0], "../ui"))
        app = QtGui.QApplication(sys.argv)
        catalog = Catalog()
        catalog.showDialog()
        sys.exit(app.exec_())

if __name__ == "__main__":
    main()
