#!/usr/bin/python
# -*- coding: utf-8 -*-
#Copyright (c) 2006 - 2007 by The WordForge Foundation
#                       www.wordforgefoundation.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
# 
# See the LICENSE file for more details. 
#
# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)
# 
# This module is working on the main windows of Editor

import os.path
import sys
from PyQt4 import QtCore, QtGui
from WordForge.Editor.ui.Ui_CleanXLIFF import Ui_CleanXliff
from WordForge.Editor.modules import FileDialog
import WordForge.Editor.modules.World as World
from WordForge import __version__
from WordForge.Editor.modules import CleanXLIFF

class cleanXLIFF(QtGui.QDialog):
    """
    This module implementation with cleanXLIFF file.
    """
    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self.ui = None                
        
        
    def showDialog(self):
        """
        Dialog a Third language dialog for merging operation.
        """
        # lazy init
        if not self.ui:
            self.ui = Ui_CleanXliff()
            self.ui.setupUi(self)         
            self.setWindowTitle("Clean an XLIFF File")    
            
            self.connect(self.ui.btnInput, QtCore.SIGNAL("clicked()"), self.getSourceFile)
            self.connect(self.ui.btnOutput, QtCore.SIGNAL("clicked()"), self.getOutPutFile)
            self.connect(self.ui.btnOk, QtCore.SIGNAL("clicked()"), self.cleanOperation)
            self.connect(self.ui.btnCancel, QtCore.SIGNAL("clicked()"), self.reject)
            
        self.show()

    def cleanOperation(self):
        inputFile = str(self.ui.lineInput.text())
        outputFile = str(self.ui.lineOutput.text())
        cleanGlossary = self.ui.chkInternalGlossary.isChecked()
        cleanTM = self.ui.chkTranslationMemory.isChecked()
        cleanReference = self.ui.chkRererenceLanguage.isChecked()
                
        if cleanGlossary or cleanTM or cleanReference:
            if os.path.exists(inputFile) and outputFile:
                self.setCursor(QtCore.Qt.WaitCursor)        
                CleanXLIFF.cleanXliff(inputFile, outputFile,  cleanGlossary,  cleanTM,  cleanReference)
                self.setCursor(QtCore.Qt.ArrowCursor)
                QtGui.QMessageBox.information(self,"Clean an XLIFF File", "Cleaning Process is completed",  QtGui.QMessageBox.Ok)
        else:
            QtGui.QMessageBox.information(self,"Select action on XLIFF File", "Please select a checkbox to perform action.",  QtGui.QMessageBox.Ok)
        
    
    def getFileForGlossary(self):
        """
        Open the file dialog where you can choose both file and directory.
        
        @return filename or dirname
        """
        directory = World.settings.value("WorkingDir").toString()        
        filename = QtGui.QFileDialog.getOpenFileName(self, self.trUtf8("Open File"),
                                                 directory,
                                                 self.trUtf8("XLIFF Files (*.xlf)"));        
                                                 
        if filename:
            directory = os.path.dirname(unicode(filename))
            World.settings.setValue("WorkingDir", QtCore.QVariant(directory))
            return filename
        else:
            return False
    
    def getOutPutFile(self):
        """
        Open the file dialog where you can choose both file and directory.
        
        @return filename or dirname
        """
        directory = World.settings.value("WorkingDir").toString()        
        filename = QtGui.QFileDialog.getSaveFileName(self, self.trUtf8("Open File"),
                                                 directory,
                                                 self.trUtf8("XLIFF Files (*.xlf)"));        
                                                 
        if filename:
            directory = os.path.dirname(unicode(filename))
            World.settings.setValue("WorkingDir", QtCore.QVariant(directory))
            filename = str(filename)
            if not filename.endswith(".xlf"):
                filename = filename + ".xlf"
            self.ui.lineOutput.setText(filename)
    
    def getSourceFile(self):
        """
        Get path a file or a folder that contain third lanuage.
        """
        source = self.getFileForGlossary()
        head, tail = os.path.splitext(unicode(source))
        cleaned = head +'-cleaned.xlf'
        if source:
            self.ui.lineInput.setText(source)
            self.ui.lineOutput.setText(cleaned)
        
if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    clXliff = cleanXLIFF(None)
    clXliff.showDialog()
    sys.exit(clXliff.exec_())
