#!/usr/bin/python
# -*- coding: utf-8 -*-
#Copyright (c) 2006 - 2007 by The WordForge Foundation
#                       www.wordforgefoundation.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
# 
# See the LICENSE file for more details. 
# 
# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)
#       Chea Sovannoty(noty@khmeros.info)
#       Aditi Barua(aditi@ankur.org.bd)
#       Asheq Hamid(asheq@ankur.org.bd)
#       Md. Rezwan Shahid(rezwan@ankur.org.bd)
#       Goutom Roy(goutom@ankur.org.bd)
# This module is working on file conversion between formats. 

import WordForge.Editor.modules.World as World
from PyQt4 import QtCore, QtGui
import os, sys
import platform
class Converter(QtGui.QDialog):
    """
    file type that can be converted to xliff: po, csv, dtd, html, moz, oo, ts, txt.
    """
    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self.ui = None
        
        # file dialog type (input, output, or template), mode (open or save)
        self.type = None
        self.mode = None
        self.errorLevel = None
    
    def showDialog(self):
        """
        Make the dialog visible.
        """
        # lazy init 
        
        if (not self.ui):
            self.setWindowTitle(self.tr("Conversion Between Formats"))
            self.setModal(True)
            from WordForge.Editor.ui.Ui_Converter import Ui_ConverterDialog
            self.ui = Ui_ConverterDialog()
            self.ui.setupUi(self)
#            self.ui.lblDef.setAlignment(QtCore.Qt.AlignJustify)
            
            # get language for each local
            dict_languages = World.dict_languages
            self.locale=[]
            for k, v in dict_languages.iteritems():
                if (len(v)==3):
                    if v[0] not in self.locale:
                        self.locale.append(v[0])    
                    self.locale.append(v[2])
            self.locale.sort()

            # add item to converter lists
#            self.ui.cbxDirection.addItems(self.converters)
            self.ui.cbxDirection.addItems(World.converters)
            # show help, show options, set filters
            self.usage()
#          self.ui.cbxError.setCurrentIndex(2) # --errorlevel message as default
#          self.ui.cbxPsyco.setCurrentIndex(0) # --psyco None as default
            
            QtCore.QObject.connect(self.ui.btnOpenIn,QtCore.SIGNAL("clicked()"), self.input)
            QtCore.QObject.connect(self.ui.btnOpenOut,QtCore.SIGNAL("clicked()"), self.output)
            QtCore.QObject.connect(self.ui.cbxDirection, QtCore.SIGNAL("currentIndexChanged(const QString)"), self.usage)
            QtCore.QObject.connect(self.ui.btnOpenTemplate,QtCore.SIGNAL("clicked()"), self.template)
            QtCore.QObject.connect(self.ui.btnOpenExclude,QtCore.SIGNAL("clicked()"), self.exclude)
#          self.connect(self.ui.chkLog, QtCore.SIGNAL("stateChanged(int)"), self.enableLog) 
#          QtCore.QObject.connect(self.ui.btnOpenLog,QtCore.SIGNAL("clicked()"), self.log)  
            QtCore.QObject.connect(self.ui.btnConvert,QtCore.SIGNAL("clicked()"), self.convert)
            QtCore.QObject.connect(self.ui.btnCancel,QtCore.SIGNAL("clicked()"), self.reject)
        self.show()
        
    def getFileOrDiretory(self):
        """
        Open the file dialog where you can choose both file and directory.
        
        @return filename or dirname
        """
        converter = str(self.ui.cbxDirection.currentText())
        fileFilter = []
        # input filter
        if (self.type == World.inputType):
            filterDic = World.inputFilterDic[converter]
#            print "filterDic", filterDic  , converter   
            
        if (self.type == World.outputType):
            filterDic = World.outputFilterDic[converter]
            
        if (self.type == World.templateType):
            filterDic = World.templatedic[converter]
        
        if (self.type == World.tmType):
            filterDic = World.tm[converter]
        
        if (not filterDic):
            QtGui.QMessageBox.information(self, self.trUtf8("options"), self.trUtf8("No options for this conversion"))
            return
        for i in range(len(filterDic)):
            fileFilter.append(World.fileFilter[filterDic[i]])
        fileFilter.append(World.allFile)
#        print "filefilter", fileFilter    
        directory = World.settings.value(World.workingDir).toString()
        from WordForge.Editor.modules import FileDialog
        filenames = FileDialog.fileDialog().getExistingPath(
                self,
                directory,
                fileFilter,self.mode)
                
        if (filenames):
            directory = os.path.dirname(unicode(filenames[0]))
            World.settings.setValue(World.workingDir, QtCore.QVariant(directory))
            return filenames[0]
        
    def input(self):
        """
        Get input as a file or a folder
        """
        self.type = World.inputType
        self.mode = World.openMode
        input = self.getFileOrDiretory()
        if input:
            self.ui.lineIn.setText(input)
    
    def output(self):
        """
        Get output as a file or a folder
        """
        self.type = World.outputType
        self.mode = World.saveMode
        output = self.getFileOrDiretory()
        if output:
            self.ui.lineOut.setText(output)
#            dir = os.path.dirname(unicode(output))
#            logfile = os.path.join(dir, World.logFileOption)
#            self.ui.lineLog.setText(logfile)

    def template(self):
        """
        Get template as a file or a folder
        """
        self.type = World.templateType
        self.mode = World.openMode
        template = self.getFileOrDiretory()        
        if template:
            self.ui.lineTemplate.setText(template)

    def exclude(self):
        
        """
        Get exclude as a file or a folder
        """
        self.type = World.inputType
        self.mode = World.openMode
        exclude = self.getFileOrDiretory()
        if exclude:
            self.ui.lineExclude.setText(exclude)
            
    
    def getTM(self):
        self.type = World.tmType
        self.mode = World.openMode
        tm = self.getFileOrDiretory()        

    #TODO: check progress bar
    def reportprogress(self, filename, success):
        pass
        """shows that we are progressing..."""
        self.progressbar.amount += 1
        self.progressbar.show(filename)
    
    def usage(self):
        """     
        Show help and other options 
        """ 
        converter = str(self.ui.cbxDirection.currentText())   
#        print converter
        #show help
#        self.ui.lblDef.setPlainText(self.getHelp(World.converterdic[converter]))
        #new options for rc 0.6
        list_options_re_6 = { World.oo2xliff:0,World.xliff2oo:1,World.xliff2po: 2,World.po2xliff: 3,World.xliffupdate: 6 , World.oo2po:7 , World.po2oo:8,World.dtd2po: 9,World.po2dtd:10, World.html2po:11, World.po2html:12,World.po2moz:15,World.moz2po:16,World.po2csv:17,World.csv2po:18, World.po2pro:19,World.prop2po:20, World.ts2po:21,  World.po2ts:22,World.txt2po:23, World.po2txt:24, World.po2tmx: 25 ,World.pot2po:26,World.csv2tbx:28,World.odf2xliff:29, World.xliff2odf:30 }
        self.ui.stackedWidget.setCurrentIndex(list_options_re_6[converter]) 
        
        
        
        
        #set pot2po option 
        # show options
#        print self.ui.cbxDirection.currentIndex()
#        self.ui.stackedWidget.setCurrentIndex(self.ui.cbxDirection.currentIndex())
        # set input Filters
        inputFilter = World.inputFilterDic[converter]
        # set output Filters
        outputFilter = World.outputFilterDic[converter]
        # set template Filters
        templateFilter = World.templatedic[converter]        
        
        # set dialog components' usablity depending on each component        
        # template box usability
        templateDisbaledList = [World.csv2tbx, World.html2po, World.oo2po, World.oo2xliff, World.po2csv, World.nb2po, World.po2tmx, World.sxw2po, World.ts2po, World.txt2po, World.xliff2po, World.odf2xliff]
        if (converter in templateDisbaledList):
            self.ui.lineTemplate.setEnabled(False)
        else:
            self.ui.lineTemplate.setEnabled(True)
        
        # set components usuability for moodle2xliff
        if (converter == World.moodle2xliff):
            self.ui.lineExclude.setEnabled(False)
            self.ui.btnOpenExclude.setEnabled(False)
            self.errorLevel = False
#            self.ui.cbxError.setEnabled(False)
#            self.ui.cbxPsyco.setEnabled(False)
        else:
            self.ui.lineExclude.setEnabled(True)
            self.ui.btnOpenExclude.setEnabled(True)
            self.errorLevel = True
#            self.ui.cbxError.setEnabled(True)
#            self.ui.cbxPsyco.setEnabled(True)
        
        # add local language to Target and Source language box
        # for oo2po local box    
        if (converter == World.oo2po):
            self.ui.cbxSLangoo2po.addItems(self.locale)
            self.ui.cbxTLangoo2po.addItems(self.locale)
        # for cbxpo2moz Local box
        if (converter == World.po2moz):
            self.ui.cbxpo2mozLocale.addItems(self.locale)
            
        # for po2oo
        if (converter == World.po2oo):
            self.ui.cbxTLangpo2oo.addItems(self.locale)
            self.ui.cbxSLangpo2oo.addItems(self.locale)
           
#        if (converter == World.po2tmx):
#            self.ui.cbxSLangpo2tmx.addItems(self.locale)
#            self.ui.cbxTLangpo2tmx.addItems(self.locale)
        
        # for oo2xliff:
        if (converter == World.oo2xliff):
            self.ui.cbxSLangoo2xliff.addItems(self.locale)
            self.ui.cbxTLangoo2xliff.addItems(self.locale)
            self.ui.cbxTLangoo2xliff.setCurrentIndex(42)
            self.ui.cbxSLangoo2xliff.setCurrentIndex(17)
        if (converter == World.pot2po):
            self.ui.label.setText("Input (New version POT)")
            self.ui.label_2.setText("Output (New version PO)")
            self.ui.lblTemplate.setText("Template (Old version PO)")
        else:
            self.ui.label.setText("Input")
            self.ui.label_2.setText("Output")
            self.ui.lblTemplate.setText("Template")

#            QtCore.QObject.connect(self.ui.btnOpenTM,QtCore.SIGNAL("clicked()"), self.getTM)
        
        if (converter == World.xliff2oo):
            self.ui.cbxSLangxliff2oo.addItems(self.locale)
            self.ui.cbxTLangxliff2oo.addItems(self.locale)
            self.ui.cbxSLangxliff2oo.setCurrentIndex(17)
            self.ui.cbxTLangxliff2oo.setCurrentIndex(42)
         
        if (converter == World.csv2tbx):
            self.ui.cbxSLangcsv2tbx.addItems(self.locale)
            
        if (converter == World.xliffupdate):
            QtGui.QMessageBox.information(self, u"Information!", u"Please use XLIFF Update function in Tools->XLIFF Upgrade",  QtGui.QMessageBox.Ok)
            self.ui.cbxDirection.setCurrentIndex(0)
            

# convert in WF_File 
    def dircet_con(self, WF_convetor, cmd):
#        print str(WF_convetor)
        #cmd="WF_oo2xliff -i /home/io/Desktop/trunk/en-US.sdf -o /home/io/Desktop/trunk/en_US -l kh_KM"
        modcmdlist=[]
#        if "\ " in cmd:
#            modcmd=cmd.replace("\ ", "\/")
#            cmd=[]
#            modcmdlist=modcmd.split(' ')
#            del modcmdlist[0]
#            for i in modcmdlist:
#                if "\/" in i:
#                    i=i.replace( "\/", " ")
#                cmd.append(i) 
#        else:
#            cmd=cmd.split(' ')
        del cmd[0]
        WF_convetor.main(cmd)
        #WF_oo2xliff.main(cmd)

#this function is added to solve the issue 2817248, it just replaces the space of a path with backslash space if the system is not windows
    def replacement(self, filepath):
        #this function adjusts the filepath acording to the platform
        if platform.system() != "Windows":
            filepath_replace = filepath.replace ( ' ', '\ ' )
            return filepath_replace 
        else:
            return filepath
            
    def convert(self):
        """
        Convert between formats based on the tool choosen in cbxDirection.
        Create or replace the output (file or folder) if it exists.
        """
        converter = str(self.ui.cbxDirection.currentText())  
#        print World.odf2xliff
#        print "converter->"+converter
        converterName = World.converterdic[converter]
#        print "converter name->"+converterName
        
        #basic options
        input = str(self.ui.lineIn.text())
#        print "input file->"+input
        output = str(self.ui.lineOut.text())
#        print "output file->"+output
#        input=self.replacement(my_input)
#        output=self.replacement(my_output)
        #end of modification for the issue 2817248
        
        if (input is "" or output is ""):
            self.msgbox("Please Specify both input's and output's path","Path Error")
            return
            
    
        self.setCursor(QtCore.Qt.WaitCursor)
        cmd=[]
        World.inputOption=World.inputOption.replace(" ", "")
        World.outputOption=World.outputOption.replace(" ", "")
        World.templateOption=World.templateOption.replace(" ", "")  
        World.excludeFileOption=World.excludeFileOption.replace(" ", "")    
        World.columnOrderOption=World.columnOrderOption.replace(" ", "")
        World.duplicateOption=World.duplicateOption.replace(" ", "")
        World.potOption=World.potOption.replace(" ", "")
        World.languageOption=World.languageOption.replace(" ", "")    
        World.charsetOption=World.charsetOption.replace(" ", "")
        World.untaggedOption=World.untaggedOption.replace(" ", "")
        World.multiFile=World.multiFile.replace(" ", "")
        World.nonRecursiveInput=World.nonRecursiveInput.replace(" ", "")
        World.fuzzyOption=World.fuzzyOption.replace(" ", "")
        World.noFuzzyOption=World.noFuzzyOption.replace(" ", "")
        World.encodingOption=World.encodingOption.replace(" ", "")
        cmd.append(converterName)   
        cmd.append(World.inputOption)
        cmd.append(input)
        cmd.append(World.outputOption)
        cmd.append(output)
#        cmd = converterName + World.inputOption + input + World.outputOption + output

        # add template option if it's enable
        
        if (self.ui.lineTemplate.isEnabled()):
            template = str(self.ui.lineTemplate.text())
            if (template and os.path.exists(template)):
                cmd.append(World.templateOption)
                cmd.append(template)
#                cmd = cmd + World.templateOption + template
                
        # add exclude option if it's enabled
        if (self.ui.lineExclude.isEnabled()):
            exclude =  str(self.ui.lineExclude.text())
            if exclude:
                (path, file) = os.path.split(exclude)
                if file:
                    cmd.append(World.excludeFileOption)
                    cmd.append(file)
#                    cmd = cmd + World.excludeFileOption +  file
        


        # add error or psyco options if it's enabled
#        if (self.ui.cbxError.isEnabled()):
#            cmd = cmd + World.errorLevelOption
#        if (self.ui.cbxPsyco.isEnabled()):
#            cmd = cmd + World.psycoOption + str(self.ui.cbxPsyco.currentText())
#        
        # specific options for each converstion        
        # TODO: some converters have the same options, so it should the same components for those options
        
        #goutom start
        if (converter == World.odf2xliff):
#            if os.path.isfile(input) and  (not str(output).endswith(".xlf")):
#                output=output+".xlf" 
            cmd=[]    
#            cmd1= converterName + World.inputOption + input + World.outputOption + output 
            cmd.append(converterName)   
            cmd.append(World.inputOption)
            cmd.append(input)
            cmd.append(World.outputOption)
            cmd.append(output)
            if (self.ui.lineExclude.isEnabled()):
                exclude =  str(self.ui.lineExclude.text())
            if exclude:
                (path, file) = os.path.split(exclude)
                if file:
                    cmd.append(World.excludeFileOption)
                    cmd.append(file)
#            print "in convert", cmd
#            if (self.ui.chkPotxliff2po.isChecked()):
#                cmd.append(World.potOption)
#                cmd = cmd + World.potOption

            import WordForge.translate.convert.WF_odf2xliff as WF_odf2xlif
            self.dircet_con( WF_odf2xlif , cmd)
        elif (converter == World.xliff2odf):
            if os.path.exists(str(self.ui.lineTemplate.text())):
                cmd=[]
                cmd.append(converterName)
                cmd.append(World.inputOption)
                cmd.append(input)
                cmd.append(World.outputOption)
                cmd.append(output)
                cmd.append(World.templateOption)
                cmd.append(str(self.ui.lineTemplate.text()))
                if (self.ui.lineExclude.isEnabled()):
                    exclude =  str(self.ui.lineExclude.text())
                if exclude:
                    (path, file) = os.path.split(exclude)
                    if file:
                        cmd.append(World.excludeFileOption)
                        cmd.append(file)
            else:
                self.msgbox("You have to put Template and Tagets Language for Conversion.")

            import WordForge.translate.convert.WF_xliff2odf as WF_xliff2odf
            self.dircet_con( WF_xliff2odf , cmd)
            
#goutom end
        elif (converter == World.csv2po):
#            if str(self.ui.lineTemplate.text()) == "" :
#                self.msgbox( "You have to put Template for Conversion.")
#                return
            columnOrder=World.columnOrderOption + str(self.ui.cbxColOrdcsv2po.currentText())    
            cmd.append(columnOrder)
            duplicate=World.duplicateOption + str(self.ui.cbxDupStylecsv2po.currentText())    
            cmd.append(duplicate)
#            cmd = cmd + World.columnOrderOption + str(self.ui.cbxColOrdcsv2po.currentText()) + World.duplicateOption + str(self.ui.cbxDupStylecsv2po.currentText())
            if (str(self.ui.cbxCharsetcsv2po.currentText()) != World.charsetDefaultOption):
                charset=World.charsetOption + str(self.ui.cbxCharsetcsv2po.currentText())
                cmd.append(charset)   
#                cmd = cmd + World.charsetOption + str(self.ui.cbxCharsetcsv2po.currentText())

            import WordForge.translate.convert.WF_csv2po as WF_csv2po
            self.dircet_con( WF_csv2po , cmd)
    
        elif (converter == World.csv2tbx):
            columnOrder=World.columnOrderOption + str(self.ui.cbxColOrdcsv2tbx.currentText())
            cmd.append(columnOrder)   
#            cmd = cmd + World.columnOrderOption + str(self.ui.cbxColOrdcsv2tbx.currentText())
            if (str(self.ui.cbxCharsetcsv2tbx.currentText()) != World.charsetDefaultOption):
                charset=World.charsetOption + str(self.ui.cbxCharsetcsv2tbx.currentText())
                cmd.append(charset)   
#                cmd = cmd + World.charsetOption + str(self.ui.cbxCharsetcsv2tbx.currentText())

            lang=World.languageOption + str(self.ui.cbxSLangcsv2tbx.currentText()) 
            cmd.append(lang)

            import WordForge.translate.convert.WF_csv2tbx as WF_csv2tbx
            self.dircet_con( WF_csv2tbx , cmd)
        
        elif (converter == World.dtd2po):
            duplicate=World.duplicateOption + str(self.ui.cbxDupStyledtd2po.currentText())
            cmd.append(duplicate)
#            cmd = cmd + World.duplicateOption + str(self.ui.cbxDupStyledtd2po.currentText())
            if (self.ui.chkPotdtd2po.isChecked()):
                cmd.append(World.potOption)
#                cmd = cmd + World.potOption

            import WordForge.translate.convert.WF_dtd2po as WF_dtd2po
            self.dircet_con( WF_dtd2po, cmd)
        
        elif (converter == World.html2po):
            duplicate=World.duplicateOption + str(self.ui.cbxDupStylehtml2po.currentText())
            cmd.append(duplicate)
#            cmd = cmd + World.duplicateOption + str(self.ui.cbxDupStylehtml2po.currentText())
            if (self.ui.chkUntaggedhtml2po.isChecked()):
                
                cmd.append(World.untaggedOption)
#                cmd = cmd + World.untaggedOption
            if (self.ui.chkPothtml2po.isChecked()):
                cmd.append(World.potOption)
#                cmd = cmd + World.potOption

            import WordForge.translate.convert.WF_html2po as WF_html2po
            self.dircet_con( WF_html2po, cmd)
            
        elif (converter == World.moz2po):
            duplicate=World.duplicateOption + str(self.ui.cbxDupStylemoz2po.currentText())
            cmd.append(duplicate)
#            cmd = cmd + World.duplicateOption + str(self.ui.cbxDupStylemoz2po.currentText())
            if (self.ui.chkPotmoz2po.isChecked()):
                cmd.append(World.potOption)
#                cmd = cmd + World.potOption

            import WordForge.translate.convert.WF_moz2po as WF_moz2po
            self.dircet_con( WF_moz2po , cmd)
        
        elif (converter == World.nb2po):
            if (self.ui.chkPotnb2po.isChecked()):
                cmd.append(World.potOption)
#                cmd = cmd + World.potOption
    
            import WordForge.translate.convert.WF_nb2po as WF_nb2po
            self.dircet_con( WF_nb2po , cmd)
        
        elif (converter == World.oo2po):
            lang=World.languageOption + str(self.ui.cbxTLangoo2po.currentText()) 
            cmd.append(lang)
#            cmd = cmd + World.languageOption + str(self.ui.cbxTLangoo2po.currentText()) 
            duplicate=World.duplicateOption + str(self.ui.cbxDupStyleoo2po.currentText())
            cmd.append(duplicate)
            multiFile=World.multiFile + str(self.ui.cbxMultifileoo2po.currentText())
            cmd.append(multiFile)
#            cmd = cmd + World.duplicateOption + str(self.ui.cbxDupStyleoo2po.currentText()) + World.multiFile + str(self.ui.cbxMultifileoo2po.currentText())
            if (self.ui.chkRecurInoo2po.isChecked()):
                cmd.append(World.nonRecursiveInput)
#                cmd = cmd + World.nonRecursiveInput
            if (self.ui.chkPotoo2po.isChecked()):
                cmd.append(World.potOption)
#                cmd = cmd + World.potOption

            import WordForge.translate.convert.WF_oo2po as WF_oo2po
            self.dircet_con( WF_oo2po , cmd)
        
        elif (converter == World.oo2xliff):
            duplicate=World.duplicateOption + str(self.ui.cbxDupStyleoo2xliff.currentText())
            cmd.append(duplicate)
#            cmd = cmd + World.duplicateOption + str(self.ui.cbxDupStyleoo2xliff.currentText()) 
            if (self.ui.chkRecurInoo2xliff.isChecked()):
                cmd.append(World.nonRecursiveInput)
#                cmd = cmd + World.nonRecursiveInput
            if str(self.ui.cbxTLangoo2xliff.currentText()) == "" :
                self.msgbox("You have to put Targets Language for Conversion.")
                return 
            else:
                lang=World.languageOption + str(self.ui.cbxTLangoo2xliff.currentText())
                cmd.append(lang)
#                cmd=cmd+ World.languageOption + str(self.ui.cbxTLangoo2xliff.text())

                import WordForge.translate.convert.WF_oo2xliff as WF_oo2xliff
                self.dircet_con( WF_oo2xliff , cmd)
            
        elif (converter == World.po2csv):
            columnOrder=World.columnOrderOption + str(self.ui.cbxColOrdpo2csv.currentText())
            cmd.append(columnOrder)   
#            cmd = cmd + World.columnOrderOption + str(self.ui.cbxColOrdpo2csv.currentText())
            if (self.ui.chkPotpo2csv.isChecked()):
                cmd.append(World.potOption)
#                cmd = cmd + World.potOption

            import WordForge.translate.convert.WF_po2csv as WF_po2csv
            self.dircet_con( WF_po2csv , cmd)

        elif (converter == World.po2dtd):
            if (self.ui.chkFuzzypo2dtd.isChecked()):
                cmd.append(World.fuzzyOption)
#                cmd = cmd + World.fuzzyOption
            else:
                cmd.append(World.noFuzzyOption)
#                cmd = cmd + World.noFuzzyOption

            import WordForge.translate.convert.WF_po2dtd as WF_po2dtd
            self.dircet_con( WF_po2dtd , cmd)
        
        elif (converter == World.po2html): 
            if (self.ui.chbPo2htmlWrap.isChecked()):
                World.wrapOption=World.wrapOption.replace(" ", "")
                wrapOption=World.wrapOption + str(self.ui.spnWrappo2html.value())
                cmd.append(wrapOption)
#                cmd = cmd + World.wrapOption + str(self.ui.spnWrappo2html.value())
            if (self.ui.chkFuzzypo2html.isChecked()):
                cmd.append(World.fuzzyOption)
#                cmd = cmd + World.fuzzyOption

            import WordForge.translate.convert.WF_po2html as WF_po2html
            self.dircet_con( WF_po2html, cmd)
        
        elif (converter == World.po2moz):
            replace=converterName+World.localOption + str(self.ui.cbxpo2mozLocale.currentText())
            cmd=cmd.replace(converterName,replace, 1 )
            
            if (self.ui.chkPo2MozCloneXpi.isChecked()):
                World.closeXpiOption=World.closeXpiOption.replace(" ", "")
                cmd.append(World.closeXpiOption)
#                cmd = cmd + World.closeXpiOption
            if (self.ui.chkFuzzypo2moz.isChecked()):
                cmd.append(World.fuzzyOption)
#                cmd = cmd + World.fuzzyOption
            else:
                cmd.append(World.noFuzzyOption)
#                cmd = cmd + World.noFuzzyOption
   
            import WordForge.translate.convert.WF_po2moz as WF_po2moz
            self.dircet_con( WF_po2moz , cmd)
        
        elif (converter == World.po2nb):
            if (self.ui.chbPo2nbWrap.isChecked()):
                World.wrapOption=World.wrapOption.replace(" ", "")
                wrapOption=World.wrapOption + str(self.ui.spnWrapnb2po.value())
                cmd.append(wrapOption)
#                cmd = cmd + World.wrapOption + str(self.ui.spnWrapnb2po.value())

            import WordForge.translate.convert.WF_po2nb as WF_po2nb
            self.dircet_con( WF_po2nb , cmd)
        
        elif (converter == World.po2oo): 
            lang=World.languageOption + str(self.ui.cbxTLangpo2oo.currentText()) 
            cmd.append(lang)
            textTemplate=str(self.ui.lineTemplate.text())
            if not os.path.exists(textTemplate):
                self.msgbox("You have to put Template  for Conversion.")
                return
#            cmd = cmd + World.languageOption + str(self.ui.cbxTLangpo2oo.currentText()) 
##+ World.sourceLanguageOption + str(self.ui.cbxSLangpo2oo.currentText())
            multiFile=World.multiFile + str(self.ui.cbxMultifilepo2oo.currentText())
            cmd.append(multiFile)
#            cmd = cmd + World.multiFile + str(self.ui.cbxMultifilepo2oo.currentText())

            import WordForge.translate.convert.WF_po2oo as WF_po2oo
            self.dircet_con( WF_po2oo , cmd)

        elif (converter == World.po2pro):
            textTemplate=str(self.ui.lineTemplate.text())
            if not os.path.exists(textTemplate):
                self.msgbox("You have to put Template  for Conversion.")
                return
            else:
                cmd=cmd+World.templateOption+textTemplate + World.inputOption + input + World.outputOption + output

            import WordForge.translate.convert.WF_po2pro as WF_po2pro
            self.dircet_con(po2pro , cmd)

        elif (converter == World.po2tmx):
            #cmd = cmd + World.languageOption + str(self.ui.cbxTLangpo2tmx.currentText()) #+ World.sourceLanguageOption + str(self.ui.cbxSLangpo2tmx.text())
            #cmd = converterName+World.languageOption + str(self.ui.cbxTLangpo2tmx.text())+ World.inputOption +input+ World.outputOption + output
            if str(self.ui.cbxTLangpo2tmx.text()) == "" :
                self.msgbox("You have to put Tagets Language for Conversion.")
                return 
            else: 
               cmd =[]
               cmd.append(converterName)
               lang=World.languageOption + str(self.ui.cbxTLangpo2tmx.text())
               cmd.append(lang)
               cmd.append(World.inputOption)
               cmd.append(input)
               cmd.append(World.outputOption)
               cmd.append(output)
               
#                cmd = converterName+World.languageOption + str(self.ui.cbxTLangpo2tmx.text())+ World.inputOption +input+ World.outputOption + output 

            import WordForge.translate.convert.WF_po2tmx as WF_po2tmx
            self.dircet_con( WF_po2tmx, cmd)
                
        elif (converter == World.po2txt):
            encoding=World.encodingOption + str(self.ui.cbxEncodepo2txt.currentText())
            cmd.append(encoding)
#            cmd = cmd + World.encodingOption + str(self.ui.cbxEncodepo2txt.currentText())
            if (self.ui.chbPo2txtWrap.isChecked()):
                World.wrapOption=World.wrapOption.replace(" ", "")
                wrapOption=World.wrapOption + str(self.ui.spnWrappo2txt.value())
                cmd.append(wrapOption)
#                cmd = cmd + World.wrapOption + str(self.ui.spnWrappo2txt.value())
            if (self.ui.chkFuzzypo2txt.isChecked()):
                cmd.append(World.fuzzyOption)
#                cmd = cmd + World.fuzzyOption
            else:
                cmd.append(World.noFuzzyOption)
#                cmd = cmd + World.noFuzzyOption

            import WordForge.translate.convert.WF_po2txt as WF_po2txt
            self.dircet_con( WF_po2txt , cmd)
        
        elif (converter == World.po2xliff):
            if (self.ui.chkPotpo2xliff.isChecked()):
                cmd.append(World.potOption)        
#                cmd = cmd + World.potOption
            import WordForge.translate.convert.WF_po2xliff as WF_po2xliff
            self.dircet_con( WF_po2xliff, cmd)
                
        elif (converter == World.pot2po):
            World.similarityOption= World.similarityOption.replace(" ", "")
            similarity=World.similarityOption + str(self.ui.spnSimpot2po.value())
            cmd.append(similarity) 
#            cmd = cmd + World.similarityOption + str(self.ui.spnSimpot2po.value())
            if (self.ui.chkNoFuzzypot2po.isChecked()):
                World.noFuzzyMatching= World.noFuzzyMatching.replace(" ", "")
                cmd.append(World.noFuzzyMatching)        
#                cmd = cmd + World.noFuzzyMatching

            import WordForge.translate.convert.WF_pot2po as WF_pot2po
            self.dircet_con( WF_pot2po , cmd)

        elif (converter == World.prop2po):
            duplicate=World.duplicateOption + str(self.ui.cbxDupStyleprop2po.currentText())
            cmd.append(duplicate)
#            cmd = cmd + World.duplicateOption + str(self.ui.cbxDupStyleprop2po.currentText())
            if (self.ui.chkPotpro2po.isChecked()):
                cmd.append(World.potOption)
#                cmd = cmd + World.potOption

            import WordForge.translate.convert.WF_prop2po as WF_prop2po
            self.dircet_con( WF_prop2po, cmd)
                
        elif (converter == World.ts2po):
            if (self.ui.chkPotts2po.isChecked()):
                cmd.append(World.potOption)
#                cmd = cmd + World.potOption

            import WordForge.translate.convert.WF_ts2po as WF_ts2po
            self.dircet_con( WF_ts2po , cmd)
        
        elif (converter == World.txt2po):
            World.flavourOption= World.flavourOption.replace(" ", "")
            flavour=World.flavourOption + str(self.ui.cbxFlavortxt2po.currentText())
            cmd.append(flavour)
            encoding=World.encodingOption + str(self.ui.cbxEncodetxt2po.currentText())
            cmd.append(encoding)
            duplicate=World.duplicateOption + str(self.ui.cbxDuptxt2po.currentText())
            cmd.append(duplicate)
#            cmd = cmd + World.flavourOption + str(self.ui.cbxFlavortxt2po.currentText()) + World.encodingOption + str(self.ui.cbxEncodetxt2po.currentText()) + World.duplicateOption + str(self.ui.cbxDuptxt2po.currentText())
            if (self.ui.chkPottxt2po.isChecked()):
                cmd.append(World.potOption)
#                cmd = cmd + World.potOption

            import WordForge.translate.convert.WF_txt2po as WF_txt2po
            self.dircet_con( WF_txt2po, cmd)
        
        elif (converter == World.xliff2oo):
            if os.path.exists(str(self.ui.lineTemplate.text())):
                cmd=[]
                cmd.append(converterName)
                cmd.append(World.inputOption)
                cmd.append(input)
                cmd.append(World.outputOption)
                cmd.append(output)
                lang=World.languageOption + str(self.ui.cbxTLangxliff2oo.currentText())
                cmd.append(lang)
                cmd.append(World.templateOption)
                cmd.append(str(self.ui.lineTemplate.text()))
#                cmd = converterName + World.inputOption +input+ World.outputOption + output + World.languageOption + str(self.ui.cbxTLangxliff2oo.text())+World.templateOption+str(self.ui.lineTemplate.text())  
                #WF_xliff2oo -i test.xlf_m49 -o test.sdf  --language=km_KH -t en-US_m49.sdf  --errorlevel=message
            else:
                self.msgbox("You have to put Template and Tagets Language for Conversion.")
                return
                pass
            if (not str(self.ui.cbxFilterxliff2oo.currentText())=="none"):
                World.filerActionOption= World.filerActionOption.replace(" ", "")
                filerAction=World.filerActionOption + str(self.ui.cbxFilterxliff2oo.currentText())
                cmd.append(filerAction)
#                cmd = cmd + World.filerActionOption + str(self.ui.cbxFilterxliff2oo.currentText())
                
            if (self.ui.chkTimexliff2oo.isChecked()):
                World.keepTimeStampOption= World.keepTimeStampOption.replace(" ", "")
                cmd.append(World.keepTimeStampOption)
#                cmd = cmd + World.keepTimeStampOption
                
            if (self.ui.chkRecurOutxliff2oo.isChecked()):
                World.nonRecursiveOutputOption= World.nonRecursiveOutputOption.replace(" ", "")
                cmd.append(World.nonRecursiveOutputOption)
#                cmd = cmd + World.nonRecursiveOutputOption
                
            if (self.ui.chkRecurxliff2oo.isChecked()):
                World.nonRecursiveTemplateOption= World.nonRecursiveTemplateOption.replace(" ", "")
                cmd.append(World.nonRecursiveTemplateOption)
#                cmd = cmd + World.nonRecursiveTemplateOption

            if (self.ui.chkFuzzyxliff2oo.isChecked()):
                cmd.append(World.fuzzyOption)
#                cmd = cmd + World.fuzzyOption
            else:
                cmd.append(World.noFuzzyOption)
#                cmd = cmd + World.noFuzzyOption

            import WordForge.translate.convert.WF_xliff2oo as WF_xliff2oo
            self.dircet_con( WF_xliff2oo , cmd)
        
        elif (converter == World.xliff2po):
            if os.path.isfile(input) and  (not str(output).endswith(".po")):
                output=output+".po" 
            cmd=[]    
#            cmd1= converterName + World.inputOption + input + World.outputOption + output 
            cmd.append(converterName)   
            cmd.append(World.inputOption)
            cmd.append(input)
            cmd.append(World.outputOption)
            cmd.append(output)
#            print "in convert", cmd
            if (self.ui.chkPotxliff2po.isChecked()):
                cmd.append(World.potOption)
#                cmd = cmd + World.potOption

            import WordForge.translate.convert.WF_xliff2po as WF_xliff2po
            self.dircet_con( WF_xliff2po , cmd)
        # add errorLevel
        if (converter != World.xliff2oo) and (converter !=World.po2tmx) and (converter !=World.xliff2po) and (converter !=World.po2pro) and (converter != World.po2moz):
            if self.errorLevel:
                World.errorLevelOption=World.errorLevelOption.replace(" ", "")    
                cmd.append(World.errorLevelOption)
#                cmd = cmd + World.errorLevelOption    

#        from subprocess import Popen, PIPE, STDOUT
        
#        if (ostype == "Linux"):
#            print cmd
#            output = Popen(cmd, stdout=PIPE, stderr=STDOUT, shell=isinstance(cmd, basestring))
#        elif (ostype == "Windows"):
#        if (ostype == "Windows"):
#            import win32process
#            if hasattr(sys.stderr, 'fileno'):
#                childStdErr = sys.stderr
#            elif hasattr(sys.stderr, '_file') and hasattr(sys.stderr._file, 'fileno'):
#                childStdErr = childstderr = sys.stderr._file
#            else:
#                childStdErrPath = 'nul'
#                childStdErr = file(childStdErrPath, 'a')
#            new_cmd_for_window = sys.exec_prefix + "\\python.exe "+sys.exec_prefix+"\\Lib\\site-packages\\WordForge\\translate\\convert\\"+converterName+".py" 
#            cmd = cmd.replace(converterName, new_cmd_for_window, 1)
#            output = Popen(cmd, stdout=PIPE, stderr=childStdErr, shell=False , creationflags = win32process.CREATE_NO_WINDOW)
        homedir = os.path.expanduser('~')
        sep = os.path.sep
#        wordforgedir = homedir+sep+World.logDir
        wordforgedir = World.configDir
        if (not os.path.exists(wordforgedir)):
            os.makedirs(wordforgedir)
        logfile = wordforgedir + sep + World.logFile
        try:
            if (self.ui.chkLog.isChecked()):
                try:
                    flog = open(logfile, 'w')
                except IOError:
                    QtGui.QMessageBox.critical(self, self.tr("Report") , self.tr("Can't open logfile"))
                else:
                    msg = output.communicate()[0]
                    flog.write(msg)                
                    flog.close()        
        except :
            pass
       
    
        self.msgbox(  "Conversion completed.","Report")
        self.close()
        
    def msgbox(self, msg_str, tital="Aleat"):
        QtGui.QMessageBox.information(self, self.tr(tital) , self.tr(msg_str))
        self.setCursor(QtCore.Qt.ArrowCursor)
    
    def getHelp(self, command):
        """ get converter too help"""
        #old help
        
        from subprocess import Popen, PIPE, STDOUT
        import platform
        ostype = platform.system()
        if (ostype == "Linux"):
            try:
                cmd = command + World.helpOption
            except:
                return
        elif (ostype == "Windows"):
            try:
                cmd = sys.exec_prefix + "\\python.exe "+sys.exec_prefix+"\\Lib\\site-packages\\WordForge\\translate\\convert\\"+command+".py"  + World.helpOption
            except:
                return
        s = Popen(cmd, stdout=PIPE, stderr=STDOUT, shell=isinstance(cmd, basestring)).communicate()[0]   
        #new help
#        s=World.helpForAll[command]+"\n"+s
        return s

if __name__ == "__main__":
    import sys, os
    # set the path for QT in order to find the icons
    QtCore.QDir.setCurrent(os.path.join(sys.path[0], "..", "ui"))
    app = QtGui.QApplication(sys.argv)
    converter = Converter(None)
    converter.showDialog()
    sys.exit(app.exec_())

