#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2006 - 2007 by The WordForge Foundation
#                 www.wordforgefoundation.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details. 
#
# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)
#
# This module is working on the Dialog for merging third lanuage into XLIFF file.

from PyQt4 import QtCore, QtGui
from WordForge.Editor.ui.Ui_DialogThird import Ui_Thirdlanguage
from WordForge.Editor.modules import FileDialog
import WordForge.Editor.modules.World as World
import os, sys

class DialogThird(QtGui.QDialog):
    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self.ui = None
    
    def showDialog(self):
        """
        Dialog a Third language dialog for merging operation.
        """
        # lazy init
        if not self.ui:
            self.ui = Ui_Thirdlanguage()
            self.ui.setupUi(self)
#            code = ""
#            code =[]
#            import WordForge.translate.lang.data as data
#            for langCode, langInfo in data.languages.iteritems():
#                code.append(langCode)
                
#            code.sort()
#            self.ui.cbxTargetLang.addItems(code)
#            preferenceLangCode = World.settings.value("Code", QtCore.QVariant(""))            
            # get index for khmeros
#            INDEXDEFAULT = 0
#            for i in range(len(code)):
#                if (code[i] == preferenceLangCode.toString() ):
#                    INDEXDEFAULT = i
#                    break
#            self.ui.cbxTargetLang.setCurrentIndex(INDEXDEFAULT)
#            self.ui.cbxThirdLang.addItems(code)
            
            preferenceLangCode = World.settings.value("thirdLanguageCode").toString()
            self.ui.cbxThirdLang.setText(preferenceLangCode)
            self.ui.lineOriginal.setFocus()
            if (World.settings.value("multiThird", QtCore.QVariant(True)).toBool()):
                self.ui.chkmultilthird.setCheckState(QtCore.Qt.Checked)
            
            self.connect(self.ui.btnOriginal, QtCore.SIGNAL("clicked()"), self.originalPath)
            self.connect(self.ui.btnThird, QtCore.SIGNAL("clicked()"), self.thirdPath)
            self.connect(self.ui.btnMerged, QtCore.SIGNAL("clicked()"), self.mergedPath)
                    
            self.connect(self.ui.btnMerge, QtCore.SIGNAL("clicked()"), self.mergeOperation)
            self.connect(self.ui.btnCancel, QtCore.SIGNAL("clicked()"), self.reject)
            self.setWindowTitle(self.trUtf8("Merge Third Language"))
            
            self.connect(self.ui.lineOriginal, QtCore.SIGNAL("textChanged(QString)"), self.enableMergeButton)
            self.connect(self.ui.lineThird, QtCore.SIGNAL("textChanged(QString)"), self.enableMergeButton)
            self.connect(self.ui.lineMerged, QtCore.SIGNAL("textChanged(QString)"), self.enableMergeButton)
            
            self.connect(self.ui.chkmultilthird, QtCore.SIGNAL("stateChanged(int)"), self.ui.chkmultilthird.checkState)
            self.ui.btnMerge.setEnabled(False)
        self.show()
            
    def enableMergeButton(self):
        """
        Enable or disable ok button.
        """
        origin = str(self.ui.lineOriginal.text())
        third = str(self.ui.lineThird.text())
        merged = str(self.ui.lineMerged.text())
        if origin and third and merged:
            self.ui.btnMerge.setEnabled(True)
        else:
            self.ui.btnMerge.setEnabled(False)
            
    def getFileOrDiretory(self):
        """
        Open the file dialog where you can choose both file and directory.
        
        @return filename or dirname
        """
        fileFilters = World.fileFiltersXLIFF
        directory = World.settings.value("workingDir").toString()
        filenames = FileDialog.fileDialog().getExistingPath(
                self,
                directory,
                fileFilters)
        if (filenames):
            directory = os.path.dirname(unicode(filenames[0]))
            World.settings.setValue("workingDir", QtCore.QVariant(directory))
            return filenames[0]
        else:
            return False
            
    def originalPath(self):
        """
        Get path of a file or a folder that consider as the original file.
        """
        origin = self.getFileOrDiretory()
        if origin:
            self.ui.lineOriginal.setText(origin)
            head, tail = os.path.splitext(unicode(origin))
            merged = head +'-merged.xlf'
            self.ui.lineMerged.setText(merged)
            
    def thirdPath(self):
        """
        Get path a file or a folder that contain third lanuage.
        """
        third = self.getFileOrDiretory()
        if third:
            self.ui.lineThird.setText(third)
            
    def mergedPath(self):
        """
        Get path a file or a folder that contain merged file.
        """
        merged = self.getFileOrDiretory()
        if merged:
            self.ui.lineMerged.setText(merged)
            
    def mergeOperation(self):
        """
        Start merging operation
        """
        original = str(self.ui.lineOriginal.text())
        third = str(self.ui.lineThird.text())
        merged = str(self.ui.lineMerged.text())
#        targetLanguage = str(self.ui.cbxTargetLang.currentText())
        thirdLanguage = str(self.ui.cbxThirdLang.text())
        World.settings.setValue("thirdLanguageCode", QtCore.QVariant(thirdLanguage))
#        if (targetLanguage == thirdLanguage):
#            QtGui.QMessageBox.critical(self, self.trUtf8("Error") , self.trUtf8("Target lanuage and Third language cannot be the same!"))
#            return
            
        if (not os.path.exists(original)):
            msg = self.tr("%1 file name doesn't exist").arg(original)
            QtGui.QMessageBox.critical(self, self.tr("Error") , msg)
            return
        if (not os.path.exists(third)):
            msg = self.tr("%1 file name doesn't exist").arg(third)
            QtGui.QMessageBox.critical(self, self.tr("Error") , msg)
            return
            
        if (merged.endswith(".xlf") or merged.endswith(".xliff")):
            try:
                fout = open(merged, 'wb')
            except Exception, e:
                raise str(e)
        else:               
            msg = self.tr("%1 file name is not correct").arg(merged)
            QtGui.QMessageBox.critical(self, self.tr("Error") , msg)
            return
        
        self.setCursor(QtCore.Qt.WaitCursor)
        
        # Remove all existing third languages
        translist = []
        from WordForge.translate.storage import factory            
        from WordForge.translate.storage import lisa
        
        try:
            originalStore = factory.getobject(original)      
            thirdStore = factory.getobject(third)      
        except Exception,  e:            
            QtGui.QMessageBox.critical(self, self.trUtf8("Error"), str(e) )
        
        for unit in originalStore.units:
            if not self.ui.chkmultilthird.isChecked():                
                translist = unit.getalttrans()                            
                    
                if (translist):
                    for list in translist:                                     
#                        targetTag = list.xmlelement[0]
#                        xmlLang = lisa.getXMLlang(targetTag)                                   
                        alttranstype = list.alttranstype
                        if (alttranstype == "reference"):
                            unit.delalttrans(list)
                            
            unitThird = thirdStore.findunit(unit.source)
            if (unitThird):
                unit.addalttransWF(unitThird.target, "Reference Language", thirdLanguage,  None,  None, "reference")
            
        fout.write(str(originalStore))
        fout.close()
        
        QtGui.QMessageBox.information(self, self.trUtf8("Report") , self.trUtf8("Reference Language has been added to file(s)."))
        self.setCursor(QtCore.Qt.ArrowCursor)
        self.close()
        
        World.settings.setValue("multiThird", QtCore.QVariant(self.ui.chkmultilthird.checkState() == QtCore.Qt.Checked))        
        
if __name__ == "__main__":
    import sys, os
    # set the path for QT in order to find the icons
    QtCore.QDir.setCurrent(os.path.join(sys.path[0], "..", "ui"))
    app = QtGui.QApplication(sys.argv)
    thirdLanguage = DialogThird(None)
    thirdLanguage.showDialog()
    sys.exit(app.exec_())
    
