#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2006 - 2007 by The WordForge Foundation
#                 www.wordforgefoundation.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details. 
#
# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)
#       Aditi Barua(aditi@ankur.org.bd)
#       Asheq Hamid(asheq@ankur.org.bd)
#       Md. Rezwan Shahid(rezwan@ankur.org.bd)
#
# This module is working on the Dialog for merging third lanuage into XLIFF file.

from PyQt4 import QtCore, QtGui
from WordForge.Editor.ui.Ui_EmbedGlossary import Ui_EmbedGlossary
from WordForge.Editor.modules import FileDialog
import WordForge.Editor.modules.World as World
from WordForge.Editor.modules.InternalGlossary import *
import os, sys

class EmbedGlossaryDialog(QtGui.QDialog):
    def __init__(self, parent):
        if World.WF_debug:
            print "Enter __init__ in Class EmbedGlossaryDialog"
        self.parent=parent
        QtGui.QDialog.__init__(self, parent)
        self.ui = None
    
    def showDialog(self):
        """
        Dialog a Third language dialog for merging operation.
        """
        if World.WF_debug:
            print "Enter showdialog in Class EmbedGlossaryDialog"

        # lazy init
        if not self.ui:
            self.ui = Ui_EmbedGlossary()
            self.ui.setupUi(self)
            
            self.ui.lineGlossaryFile.setFocus()
            self.connect(self.ui.btnGlossaryFile, QtCore.SIGNAL("clicked()"), self.getGlossaryFile)
            self.connect(self.ui.btnSource, QtCore.SIGNAL("clicked()"), self.getSourceFile)
            self.connect(self.ui.btnTarget, QtCore.SIGNAL("clicked()"), self.getTarget)
            
            
            self.connect(self.ui.btnEmbed, QtCore.SIGNAL("clicked()"), self.embedOperation)
            self.connect(self.ui.btnCancel, QtCore.SIGNAL("clicked()"), self.reject)
            self.setWindowTitle(self.trUtf8("Embed a glossary into XLIFF source files"))
            
            
            self.connect(self.ui.lineGlossaryFile, QtCore.SIGNAL("textChanged(QString)"), self.enableEmbedButton)
            self.connect(self.ui.lineSource, QtCore.SIGNAL("textChanged(QString)"), self.enableEmbedButton)
            self.connect(self.ui.lineTarget, QtCore.SIGNAL("textChanged(QString)"), self.enableEmbedButton)
            
            self.ui.btnEmbed.setEnabled(False)
        self.show()
            
    def enableEmbedButton(self):
        """
        Enable or disable ok button.
        """
        if World.WF_debug:
            print "Enter enableEmbeddedButton in Class EmbedGlossaryDialog"

        glossary = str(self.ui.lineGlossaryFile.text())
        source = str(self.ui.lineSource.text())
        target = str(self.ui.lineTarget.text())
        if glossary and source and target:
            self.ui.btnEmbed.setEnabled(True)
        else:
            self.ui.btnEmbed.setEnabled(False)
            
    def getFileOrDiretory(self):
        """
        Open the file dialog where you can choose both file and directory.
        
        @return filename or dirname
        """
        if World.WF_debug:
            print "Enter getFileOrDirectory in Class EmbedGlossaryDialog"

        directory = World.settings.value("workingDir").toString()
        filenames = FileDialog.fileDialog().getExistingPath(
                self,
                directory,
                "XLIFF Files (*.xlf)")
        if (filenames):
            directory = os.path.dirname(unicode(filenames[0]))
            World.settings.setValue("workingDir", QtCore.QVariant(directory))
            return filenames[0]
        else:
            return False
    
#    def getFileForGlossary(self):
#        directory = World.settings.value("EnableWorkingDir").toString()        
#        filename = QtGui.QFileDialog.getOpenFileName(self, self.trUtf8("Open File"),
#                                                 directory,
#                                                 self.trUtf8("Glossary Files (*.xlf *.po *.tbx)"));                                                     
#        if filename:
#            directory = os.path.dirname(unicode(filename))
#            World.settings.setValue("EnableWorkingDir", QtCore.QVariant(directory))
#            return filename
#        else:
#            return False
            
    def getFileForGlossary(self):
        """
        Open the file dialog where you can choose both file and directory.
        
        @return filename or dirname
        """
        if World.WF_debug:
            print "Enter getFileForGlossary in Class EmbedGlossaryDialog"

        directory = World.settings.value("EnableWorkingDir").toString()        
        filename =  FileDialog.fileDialog().getExistingPath(
                self,
                directory,
                self.trUtf8("Glossary Files (*.xlf *.po *.tbx)"));        
                                                 
        if filename:
            directory = os.path.dirname(unicode(filename[0]))
            World.settings.setValue("EnableWorkingDir", QtCore.QVariant(directory))
            return filename[0]
        else:
            return False
        

    def getGlossaryFile(self):
        """
        Get path of a file or a folder that consider as the original file.
        """
        if World.WF_debug:
            print "Enter getGlossaryFile in Class EmbedGlossaryDialog"

        glossary = self.getFileForGlossary()
        if glossary:
            self.ui.lineGlossaryFile.setText(glossary)
        
            
    def getSourceFile(self):
        """
        Get path a file or a folder that contain third lanuage.
        """
        if World.WF_debug:
            print "Enter getSourceFile in Class EmbedGlossaryDialog"

        source = self.getFileOrDiretory()
        if source:
            self.ui.lineSource.setText(source)
            self.ui.lineTarget.setText(source)
            
    def getTarget(self):
        """
        Get path a file or a folder that contain merged file.
        """
        if World.WF_debug:
            print "Enter getTarget in Class EmbedGlossaryDialog"

        target = self.getFileOrDiretory()
        if target:
            self.ui.lineTarget.setText(target)
            
    def embedOperation(self):
        """
        Start merging operation
        """
        if World.WF_debug:
            print "Enter embedOperation in Class EmbedGlossaryDialog"

        glossary = str(self.ui.lineGlossaryFile.text())
        source = str(self.ui.lineSource.text())
        target = str(self.ui.lineTarget.text())
        glossarylanguage=''
        sourcelanguage=''
        glossarylanguage=self.parent.preference.checkfileLanguage(glossary, "")
        sourcelanguage=self.parent.preference.checkfileLanguage(source, "")  
        messageBox = QtGui.QMessageBox
        msgnolang= self.trUtf8("")
        if not sourcelanguage: 
            msgnolang= self.trUtf8("There is no translation language in the source file. Do you wish to continue?")
        elif not glossarylanguage and (not glossary.endswith(".tbx")):
            msgnolang= self.trUtf8("There is no translation language in the glossary file. Do you wish to continue?")
        elif glossary.endswith(".tbx"):    
            msgnolang= self.trUtf8("Some of the language codes in glossary file do not match with the language code in source file. Do you wish to continue?")            
        elif sourcelanguage!=glossarylanguage:
            msgnolang= self.trUtf8("The translation language in source file <b>(" + sourcelanguage+")</b> does not match The translation language in glossary file <b>(" + glossarylanguage+")</b> . Do you wish to continue?")
        if msgnolang:
            messageBox = QtGui.QMessageBox
            ret = messageBox()
            ret.setTextFormat(QtCore.Qt.RichText)
            ret.setText(msgnolang)
            YesButton = ret.addButton(self.trUtf8("Yes"),messageBox.YesRole)
            NoButton = ret.addButton(self.trUtf8("No"),messageBox.NoRole)
            ret.exec_()
            if (ret.clickedButton() == YesButton):
                continueoperation=True
            elif (ret.clickedButton() == NoButton):
                self.close()
                return
            
        Mycounts = [0, 0, 0]
        if (not os.path.exists(glossary)):
            msg = self.tr("%1 file name doesn't exist").arg(glossary)
            QtGui.QMessageBox.critical(self, self.tr("Error") , msg)
            return
        if (not os.path.exists(source)):
            msg = self.tr("%1 file name doesn't exist").arg(source)
            QtGui.QMessageBox.critical(self, self.tr("Error") , msg)
            return
        
        self.setCursor(QtCore.Qt.WaitCursor)        
        embedGlossary(glossary, source,  target,  Mycounts )
        self.setCursor(QtCore.Qt.ArrowCursor)
        if (Mycounts[1] > 0):
            QtGui.QMessageBox.information(self,"Embed Glossary", "The internal glossary ("+str(Mycounts[2])+" of "+str(Mycounts[0])+" terms) has been successfully embedded into the target file. "+str(Mycounts[1])+" fuzzy units have been ignored.",  QtGui.QMessageBox.Ok)
        else:
            QtGui.QMessageBox.information(self,"Embed Glossary", "The internal glossary ("+str(Mycounts[2])+" of "+str(Mycounts[0])+" terms) has been successfully embedded into the target file.",  QtGui.QMessageBox.Ok)
        self.close()
        
if __name__ == "__main__":
    import sys, os
    if World.WF_debug:
        print "Enter __name_ == __main__ in Class EmbedGlossaryDialog"

    # set the path for QT in order to find the icons
    QtCore.QDir.setCurrent(os.path.join(sys.path[0], "..", "ui"))
    app = QtGui.QApplication(sys.argv)
    embedGlossary = EmbedGlossaryDialog(None)
    embedGlossary.showDialog()
    sys.exit(app.exec_())
    

