#!/usr/bin/python
# -*- coding: utf-8 -*-
#Copyright (c) 2006 - 2007 by The WordForge Foundation
#                       www.wordforgefoundation.org

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
# 
# See the LICENSE file for more details. 
#
# Developed by:
#       Rezwan Shahid (rezwan@ankur.org.bd)
#       Aditi Barua (aditi@ankur.org.bd)
#       Asheq Hamid (asheq@ankur.org.bd)

# This module is working on the Error Checking features

from PyQt4 import QtCore, QtGui
from WordForge.Editor.ui.Ui_TableError import Ui_Form
from WordForge.Editor.modules.globalClass import globalClass
from WordForge.translate.filters import checks
from WordForge.translate.filters import decoration
from WordForge.Editor.modules.Operator import Operator
from WordForge.Editor.modules.Overview import OverviewDock
from WordForge.Editor.modules.Preference import Preference
from WordForge.Editor.modules.Catalog import Catalog
from WordForge.translate.filters import helpers
from WordForge.translate.storage import xliff
import WordForge.Editor.modules.World as World
from collections import defaultdict
import sys
import string
import re

class ErrorChecking (QtGui.QDockWidget):

#    This class does the Error Checking operations .

    def __init__(self, parent = None):
        QtGui.QDockWidget.__init__(self, parent)
        self.parent = parent
        self.setWindowTitle(self.tr("Error Window"))
        self.setObjectName("errorWindow")
        self.form = QtGui.QWidget(self)
        self.ui = Ui_Form()
        self.ui.setupUi(self.form) 
        self.setWidget(self.form)
#        This list holds the names of all the test functions that needs to be called after the translation.
        self.postTestList=[]
#        Initialization of checker objects of checks.py 
        self.standardChecker = checks.StandardChecker()
        self.normalState = QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable
        self.acronyms = []
        self.escapeDict = {}
        self.filePathList = []
        self.functions = []
        self.printf = []
        self.urlList = []
        self.variableMarkList = []
        self.xmlTags = []
        self.errorMessageList = []
        self.searchFileError=False
#        List of end punc which keeps the end puntuations of most languages.
        self.endPuncList = [u'.',u'!',u'?',u'…',u'։',u'؟',u'।',u'。',u'！',u'？',u'።', u':']
        self.removeXmlPrefix = re.compile(r'<[\ |\/|\?]*\w+[\ ]*')
        self.xmlAttributeMatchExpression = re.compile(r'[\ ]*\w+[\ ]*=[\ ]*[\'|\"][\ ]*.*[\'|\"][\ ]*')
        self.maxIndexLength=0
        self.blankIcon = QtGui.QIcon()        
#        Rafi's code starts
        self.applySettings()
        self.markError=0
        self.hasplural=True
        self.preCom=False
    
    def popUp(self, errorsWithInfo, onlyErrors):
        #shows the popup message while checking error in each string  
        self.errorMessageList = errorsWithInfo[:]


        for i in range(0, len(onlyErrors)):
            onlyErrors[i] = str(i+1) + ") " + onlyErrors[i]

        errorMessage = string.join(onlyErrors,'\n')

        if (not self.searchFileError) and (not globalClass.searchProjectError):
#            self.currentItem = self.parent.dockOverview.ui.tableOverview.item(self.parent.dockOverview.ui.tableOverview.currentRow(), 0) 
            from WordForge.Editor.modules.ErrorMessage import ErrorMessage
            self.errorUi = ErrorMessage(self)
            self.errorUi.setWindowTitle(self.tr("Error Message"))
            self.errorUi.ui.errorText.setText(QtGui.QApplication.translate("Dialog", errorMessage, None, QtGui.QApplication.UnicodeUTF8))
            self.rowSelected=self.ui.errorTable.currentRow()
            self.connect(self.errorUi.ui.ignoreButton, QtCore.SIGNAL("clicked()"), self.ignoreChecking)
            self.connect(self.errorUi.ui.goBackButton, QtCore.SIGNAL("clicked()"), self.goBackToMessage)
            
#        self.connect(self.ui.errorTable, QtCore.SIGNAL("cellDoubleClicked(int, int)"), self.jumpToMessage)
    
    
    def ignoreChecking(self):   
       # ignores to review the error in a string 
        self.errorUi.close()
        self.ui.errorTable.setSortingEnabled(True)
        self.ui.errorTable.selectRow(self.rowSelected)  
        
    def applySettings(self):
        """
        set color and font to the TM table.
        """
        if World.WF_debug:
            print "Enter applySettings in Class ErrorChecking"
            
        ErrorCheckingColor = World.settings.value("errorCheckingColor")
        if (ErrorCheckingColor.isValid()):
            colorObj = QtGui.QColor(ErrorCheckingColor.toString())
            palette = self.ui.errorTable.palette()
            palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(QtGui.QPalette.Text), colorObj)
            palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(QtGui.QPalette.Text), colorObj)
            self.ui.errorTable.setPalette(palette)

        font = World.settings.value("errorTableFont")
        if (font.isValid()):
            fontObj = QtGui.QFont()
            if (fontObj.fromString(font.toString())):
              self.ui.errorTable.setFont(fontObj)
              
        self.ui.errorTable.resizeRowsToContents()
        self.viewSetting()
    
    def viewSetting(self):
        # settings and properties of the error table 
        if World.WF_debug:
            print "Enter viewSetting in Class ErrorChecking"
        self.ui.errorTable.clear()
        self.headerLabels = [self.tr("Index"), self.tr("Error Name"), self.tr("Error Message")]
        self.ui.errorTable.setColumnCount(len(self.headerLabels))
        self.ui.errorTable.setHorizontalHeaderLabels(self.headerLabels)

        self.ui.errorTable.resizeColumnToContents(0)
        self.ui.errorTable.resizeColumnToContents(1)
        self.ui.errorTable.horizontalHeader().setResizeMode(2, QtGui.QHeaderView.Stretch)

        self.ui.errorTable.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        self.ui.errorTable.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        self.ui.errorTable.verticalHeader().hide()
        self.ui.errorTable.setRowCount(0)
        self.ui.errorTable.setEnabled(True)
        
    def CountUnitNumber(self, unitNumber):
        # counts the maximun length of digits in the index of every message in error table
        self.unitNumber=int(unitNumber)
        strunitNumber=str(unitNumber)
        self.maxIndexLength = len(strunitNumber)
        
    def fillTable(self, noteList = []):
        """
        fill each found unit into table
        @param candidates:list of pofile object
        """
#        print "filltable", noteList
        self.currentRow=self.ui.errorTable.currentRow()  
        self.ui.errorTable.setSortingEnabled(False)
        errorMessage = []
#        errorMessage = self.errorMessageList
#        if not errorMessage:
        errorMessage=noteList
        row = self.ui.errorTable.rowCount()
        columns = self.ui.errorTable.columnCount()
        table = self.ui.errorTable
        if errorMessage:
            rowIndex = errorMessage[0]
            rowIndex = str(rowIndex).rjust(self.maxIndexLength)
    #        removes the error messages with the same index from the table

            self.removeFromTable(rowIndex)
        if (not errorMessage):
            self.errorUi.close()
            return 
        
        pos = 0
        maxIndexLength = 0
#        Finding out the maximum Index length
#        for index in errorMessage:
#            if(pos%columns == 0):
#                if (len(index)>maxIndexLength):
#                    maxIndexLength = len(index)   
#            pos = pos+1
#        maxIndexLength=self.maxIndexLength
        col = 0
        for message in errorMessage:
            if not message:
                break
            if(col%columns == 0):
                row = self.ui.errorTable.rowCount()
                table.setRowCount(row + 1)
                col = 0
                message = str(message).rjust(self.maxIndexLength)
                

            item = QtGui.QTableWidgetItem(message)
            item.setFlags(self.normalState)
            table.setItem(row, col, item)
            col = col + 1
        currentItem = self.parent.dockOverview.ui.tableOverview.item(self.parent.dockOverview.ui.tableOverview.currentRow(), 0) 
        currentItem=currentItem.text()
        currentIndex = str(currentItem).rjust(self.maxIndexLength)   
        self.ui.errorTable.setSortingEnabled(True)
        self.ui.errorTable.sortItems(0)  
        rownum = self.ui.errorTable.rowCount()
        row=0
        for row in range(rownum):
            index=table.item(row, 0)
            index=index.text()
            index=str(index)
            if index==currentIndex:
                index=int(index)
                self.ui.errorTable.selectRow(row)    
                break

    def removeFromTable(self, rowIndex):
        # removes an erromessage from the error table. 
        row = self.ui.errorTable.rowCount()
        for r in range(0, row):
#            this has been done to make sure that the table is searched from the bottom so that after deleting a row,  it does not get out of the range
            rowIndex = str(rowIndex).rjust(self.maxIndexLength)    
            reverse = (row-r)-1 
            if self.ui.errorTable.item(reverse, 0)==None:
                return 
            strRev=self.ui.errorTable.item(reverse, 0).text()
            if len(strRev)<self.maxIndexLength:
                strRev = str(strRev).rjust(self.maxIndexLength)    
            else:
                strRev=self.ui.errorTable.item(reverse, 0).text()
            strRev=int(strRev)
            rowIndex=int(rowIndex)    
            if (strRev == rowIndex):
                self.ui.errorTable.removeRow(reverse)

    def goBackToMessage(self):
#        goes back to the row in overview for which popup error message is shown
        from WordForge.Editor.modules.Overview import OverviewDock
        prevIndex=int(self.currentRowItem)
        for row in range(self.unitNumber):
            itemInOverView = int(self.parent.dockOverview.ui.tableOverview.item(row, 0).text())
            if prevIndex==itemInOverView:
                self.parent.dockOverview.ui.tableOverview.selectRow(row)
                break
#        self.parent.dockOverview.ui.tableOverview.selectRow(self.index-1)
        self.errorUi.close()
        self.ui.errorTable.setSortingEnabled(True)    
        
    def jumpToMessage(self, a, b):
        # if user double clicks on a row in the error table, its corresponding original message in the overview will be selected. 
        item = self.ui.errorTable.item(self.ui.errorTable.currentRow(), 0)
        item=int(item.text())
        for row in range(self.unitNumber):
            itemInOverView = int(self.parent.dockOverview.ui.tableOverview.item(row, 0).text())
            if item==itemInOverView:
                self.parent.dockOverview.ui.tableOverview.selectRow(row)
                break

    def closeEvent(self, event):
#   Uncheck the menu options in maineditor.  
        self.parent.uncheckErrorAction()
        globalClass.errorCheckEnableFlag = 0
        
    def emptyTarget(self, index):
        #when the focus is changed in overview , this function checks that if the previously focused message has translation or not
        prevIndex=int(index)
        for row in range(self.unitNumber):
            itemInOverView = int(self.parent.dockOverview.ui.tableOverview.item(row, 0).text())
            if prevIndex==itemInOverView:
                target = self.parent.dockOverview.ui.tableOverview.item(row, 2).text()
                if target=="":
                    return True
                    break
        return False    
                
    def addNoteToProjectXlf(self, testAndMsg):
       # adds error note in the xliff files of current proejct after error checking operation has been finished. 
        units=[]
        notlist=[]
        projectFileName= self.parent.Catalog.projectFileName
        if (projectFileName.endswith(".xlf") or projectFileName.endswith(".xliff")):
            isXliff=True
        else:
            isXliff=False
        if isXliff:
            storeUnit= self.parent.Catalog.catalogStore.units
            for unit in storeUnit:
                unitid = unit.getid()
                if unitid==globalClass.unitid:    
                    if testAndMsg:
                        notlist=unit.removenotes()
                        for k, v in testAndMsg.iteritems():
                                unit.addErrornote(v, "error", k)
                        break
                    else:
                        notlist=unit.removenotes()
            self.parent.Catalog.catalogStore.savefile(projectFileName)            
        
    def addNoteToXlf(self, testAndMsg):
        #code for adding error messages into note tag
        units=[]
        notlist=[]
        units=self.parent.dockOverview.units
        for unit in units:
            unitid = unit.getid()
            if self.searchFileError or globalClass.searchProjectError:
                if unitid==globalClass.unitid:    
                    if testAndMsg:
                        notlist=unit.removenotes()
                        for k, v in testAndMsg.iteritems():
                                unit.addErrornote(v, "error", k)
                        break
                    else:
                        notlist=unit.removenotes()
            elif unitid==globalClass.lastUnitId:
                if testAndMsg:
                    notlist=unit.removenotes()
                    for k, v in testAndMsg.iteritems():
                            unit.addErrornote(v, "error", k)
                    break
                else:
                    notlist=unit.removenotes()
                    
    def glossaryMatch(self, source):
        self.glsExpression= self.parent.dockTUview.errGlsExpression
        try:
            if (str(source))[0]  in World.regularExpressionCharacters:
                source.prepend('n')
            if (str(source))[-1]  in World.regularExpressionCharacters:
                source.append('n')
        except:
            pass
        Qsource = QtCore.QString()
        Qsource.append(source)	
        glsIndex = Qsource.indexOf(self.glsExpression)
        length1=0
        while (glsIndex >= 0):
            length1 = self.glsExpression.indexIn(source, length1)
            length = self.glsExpression.matchedLength()
            length1 += length
            matchedWord = self.glsExpression.capturedTexts()[0]
            glsIndex = Qsource.indexOf(self.glsExpression, glsIndex + length)
            self.parent.operator.lookupTerm(matchedWord)

    def preCompute(self, source):
#        This functions checks for the test cases that needs to be called, parameter source is the source on which checking will be applied.
#        print "preCompute", globalClass.globalerrorlist, source
        self.preCom=True
        self.searchFileError=self.parent.ui.actionSearchErrors.isChecked()
        if not globalClass.searchProjectError:
            self.currentRowItem = self.parent.dockOverview.ui.tableOverview.item(self.parent.dockOverview.ui.tableOverview.currentRow(), 0) 
            self.currentRowItem=self.currentRowItem.text()
        hasState=True
        AcceleratorType=''
        globalClass.preComputeFlag = 0
#        By default added acronyms test.
        globalClass.globalerrorlist.append('acronyms')

        for index in range (0,len(globalClass.globalerrorlist)):
            test = globalClass.globalerrorlist[index]
#            print "pre", source, test
#            This test needs to be adjusted with the preference values.
            if test == 'accelerators':
                AcceleratorType=globalClass.endPuncDict['accelerators']
                self.AcceleratorMark=globalClass.endPuncDict[AcceleratorType]
                hasState = self.hasAccelerator(source, AcceleratorType)
                
            elif test == 'acronyms':    
                hasState = False
#            elif test == 'acronyms':
#                #gets the list of acronyms
#                self.acronyms = []
#                self.acronyms = self.standardChecker.WF_hasAcronyms(unicode(source))
#                if self.acronyms:
#                    hasState = True
#                else:
#                    hasState = False


            elif test == 'brackets':
                self.bracketDict = {}
                self.bracketDict = self.hasBracket(unicode(source))
                
                if self.bracketDict:
                    hasState = True
                else:
                    hasState = False
            elif test == 'doublequoting':
                hasState = self.hasCharacter(unicode(source), 'doublequoting')
            elif test == 'puncspacing':
                hasState = True
            elif test == 'doublespacing':
                hasState = self.hasCharacter(unicode(source), 'doublespacing')
            elif test == 'emails':
                hasState = decoration.getemails(unicode(source))
            elif test == 'EndpunctuationEnable':
                hasState = decoration.puncend(unicode(source), self.endPuncList)
            elif test == 'endwhitespace':
                hasState = decoration.spaceend(unicode(source))
            elif test == 'escapes':
                self.escapeDict = {}
                self.escapeDict = self.hasEscapes(source)
                if self.escapeDict:
                    hasState = True
                else:
                    hasState = False
            elif test == 'filepaths':
                self.filepath = []
                self.filepath = self.hasFilePath(unicode(source))
                if self.filepath:
                    hasState = True
                else:
                    hasState = False
            elif test == 'functions':
                self.functions = [] 
                tempFunctions = self.hasFunction(unicode(source), 'functions')
                if tempFunctions:
                    for each in tempFunctions:
                        if self.isFunction(each):
                            self.functions.append(each)
                    
                if self.functions:
                    
                    hasState = True
                else:
                    hasState = False
            elif test == 'newlines':
                hasState = self.hasCharacter(unicode(source), 'newlines')
            elif test == 'options':
                options = self.standardChecker.WF_hasOptions(unicode(source))
                if options:
                    hasState = True
                else:
                    hasState = False
            elif test == 'printf':
                self.printf = self.hasFunction(unicode(source), 'printf')
                if self.printf:
                    hasState = True
                else:
                    hasState = False
                    
            elif test == 'sentencecount':     
                hasState = True
            elif test == 'singlequoting':
                hasState = self.hasCharacter(unicode(source), 'singlequoting')
            elif test == 'startwhitespace':
                if unicode(source)[0] == u' ':
                    hasState = True
                else:
                    hasState = False
            elif test == 'tabs':
                hasState = self.hasCharacter(unicode(source), 'tabs')
            elif test == 'urls':
                self.urlList = self.hasUrl(unicode(source))
                if self.urlList:
                    hasState = True
                else:
                    hasState = False
            elif test == 'variables':
                hasState, self.variableMarkList = self.hasVariables(unicode(source), globalClass.endPuncDict['variables'])

            elif test == 'doublewords':
                hasState = True
            elif test == 'xmltagsAtt':
                self.xmltagsAtt = self.hasXmlAttr(unicode(source))
                if self.xmltagsAtt:
                    hasState = True
                else:
                    hasState = False    
                    
            elif test == 'xmltags':
                self.duplicate, self.XmltagAttDic1 = self.hasXml(unicode(source))
                if self.XmltagAttDic1:
                    hasState = True
                else:
                    hasState = False

                    
            elif test == 'glossarytest':
                if self.searchFileError:
                    if globalClass.tempGlsErrorDict.items():
                        self.glossaryMatch(str(source))
                    else:
                        hasState = False    
                if globalClass.tempGlsErrorDict.items():
                    tempDict = defaultdict(dict)
                    for eachitem in globalClass.tempGlsErrorDict.keys():
                        for each in globalClass.tempGlsErrorDict[eachitem]:
                            if globalClass.endPuncDict['general'] > 0 and len(each)> globalClass.endPuncDict['general']:
                                each = each[: -(globalClass.endPuncDict['general'])]
                                tempDict[eachitem][each] = each
                    
                    for eachkey in tempDict.keys():
                        globalClass.tempGlsErrorDict[eachkey].update(tempDict[eachkey])
                    hasState = True
                else:
                    hasState = False        
            if hasState:
                if test not in self.postTestList: 
                    self.postTestList.append(test)
            
            self.searchFileError=self.parent.ui.actionSearchErrors.isChecked()
            
            if self.searchFileError or globalClass.searchProjectError:
                globalClass.postTestList=self.postTestList
       
       
    def postCompute(self,  source, target, index, OverviewRow=0):
       # this function does all the operation is error checking based on the error settings obtained from precompute function
        testAndMessage={}
        if self.searchFileError or globalClass.searchProjectError:
            self.postTestList=globalClass.postTestList
        try:
            if self.emptyTarget(index):
                self.addNoteToXlf(testAndMessage)
                self.removeFromTable(index)
                
        except:
            pass
        if  target==u'' or self.postTestList==[] :
            self.postTestList =[]
            return  
        errorMessageList = []
        onlyErrors = []
        cleanNote={}
        self.index = int(index)
        for testIndex in range (0, len(self.postTestList)):
            errormessage = None
            test = self.postTestList[testIndex]
            if test == 'accelerators':
                #if error, gets the error message, otherwise gets False
                errormessage = self.standardChecker.WF_accelerators(unicode(source),unicode(target),self.AcceleratorMark)
            elif test == 'acronyms':
                errormessage = self.standardChecker.WF_acronyms(self.acronyms, unicode(target))

            elif test == 'brackets':
                errormessage = self.WF_brackets(unicode(source), unicode(target), self.bracketDict)

            elif test == 'doublequoting':
                errormessage = self.standardChecker.WF_doublequoting(unicode(source), unicode(target))
                
            elif test == 'doublewords':
                errormessage = self.standardChecker.WF_doublewords(unicode(source), unicode(target))   
                
            elif test == 'puncspacing':
                errormessage = self.standardChecker.WF_puncspacing(unicode(source), unicode(target)) 
            elif test == 'doublespacing':
                errormessage = self.standardChecker.WF_doublespacing(unicode(source), unicode(target))

            elif test == 'emails':
                errormessage = self.standardChecker.WF_emails(unicode(source), unicode(target))
# This test needs further testing.
            elif test == 'EndpunctuationEnable':
                errormessage = self.standardChecker.WF_endpunc(unicode(source), unicode(target), globalClass.endPuncDict, self.endPuncList)

            elif test == 'endwhitespace':
                errormessage = self.standardChecker.WF_endwhitespace(unicode(source), unicode(target))

            elif test == 'escapes':
                errormessage = self.WF_escapes(unicode(source), unicode(target))

            elif test == 'filepaths':
                errormessage = self.WF_filepaths(unicode(source), unicode(target), self.filepath)

            elif test == 'functions':
                errormessage = self.WF_functions(unicode(source), unicode(target), self.functions)

            elif test == 'newlines':
                errormessage = self.standardChecker.WF_newlines(unicode(source), unicode(target))

            elif test == 'options':
                errormessage = self.standardChecker.WF_options(unicode(source), unicode(target))

            elif test == 'printf':
                errormessage = self.WF_printf(unicode(source), unicode(target), self.printf)

            elif test == 'sentencecount': 
                errormessage = self.standardChecker.WF_sentencecount(unicode(source), unicode(target))
            elif test == 'singlequoting': 
                errormessage = self.standardChecker.WF_singlequoting(unicode(source), unicode(target))
            elif test == 'startwhitespace': 
                errormessage = self.standardChecker.WF_startwhitespace(unicode(source), unicode(target))
            elif test == 'tabs': 
                errormessage = self.standardChecker.WF_tabs(unicode(source), unicode(target))
            elif test == 'urls':
                errormessage = self.WF_urls(unicode(source), unicode(target), self.urlList)
            elif test == 'variables':
#                In this case a list is returned. Just take the first element of that list as an error message. Leave the other elements.
                errormessage = self.standardChecker.WF_variables(unicode(source), unicode(target),self.variableMarkList)
            elif test == 'glossarytest':
                errormessage = self.WF_glossarytest(unicode(source), unicode(target))
#                print "error", errormessage
                
            elif test == 'xmltagsAtt':
                errormessage = self.WF_xmltagsAtt(unicode(source), unicode(target), self.xmltagsAtt)  

            elif test == 'xmltags':
                errormessage = self.WF_xmltags(unicode(source), unicode(target), self.XmltagAttDic1, self.duplicate)
            if(errormessage):
                if (test == 'accelerators' or test == 'variables'):
                    for each in errormessage:
                        errorMessageList.append(index)
                        errorMessageList.append(test)
                        errorMessageList.append(each)
                        onlyErrors.append(each)
                        testAndMessage[test]=each
                        globalClass.fileErrorList.append(index)
                else:
                    errorMessageList.append(index)
                    errorMessageList.append(test)
                    errorMessageList.append(errormessage)
                    onlyErrors.append(errormessage)
                    testAndMessage[test]=errormessage
                    globalClass.fileErrorList.append(index)
        
#        print "errormessage", errorMessageList
        cleanNote['clean']=u'string is error free'
        
        storingEnable=World.settings.value("enableStore",  QtCore.QVariant(True)).toBool()
        if(errorMessageList):
            globalClass.hasErrors=0
            self.popUp(errorMessageList, onlyErrors)
            if globalClass.searchProjectError:
                if not self.parent.checkPlural:
                        self.parent.fileError= self.parent.fileError+1
                        if self.parent.hasPlural:
                            self.parent.checkPlural=True
                if storingEnable:
                    self.addNoteToProjectXlf(testAndMessage)                
            if self.searchFileError:
                index=int(index)
                self.markError=1
                self.parent.markError=1
                if self.parent.hasPlural:
                    for i in errorMessageList:
                        self.parent.pluralErrorList.append(i)
                    self.parent.errorTable.fillTable(self.parent.pluralErrorList) 
                    
                    self.parent.dockOverview.markFileError(index, self.unitNumber, self.markError)
                else:    
                    self.parent.errorTable.fillTable(errorMessageList) 
                    
                    self.parent.dockOverview.markFileError(index, self.unitNumber, self.markError)
                if storingEnable:
                    
                    if self.parent.operator.isXliff:
                        self.addNoteToXlf(testAndMessage)    
                        
            if not globalClass.searchProjectError and not self.searchFileError:
                self.parent.errorTable.fillTable(errorMessageList) 
                if storingEnable:
                    if self.parent.operator.isXliff:
                        self.addNoteToXlf(testAndMessage)
        else:
            if storingEnable and not self.parent.hasPlural:
                if self.parent.operator.isXliff:    
                    self.addNoteToXlf(cleanNote)
            if self.searchFileError:
                globalClass.hasErrors=0  
                index=int(index)
                if self.parent.hasPlural:
                    if self.parent.markError==1:
                        self.markError=1
                    else:
                        self.markError=0
                        if storingEnable:
                            if self.parent.operator.isXliff:
                                self.addNoteToXlf(cleanNote)
                    self.parent.dockOverview.markFileError(index, self.unitNumber, self.markError)
                    if not self.parent.pluralErrorList:
                        self.removeFromTable(index)
                else:
                    self.markError=0
                    self.parent.dockOverview.markFileError(index, self.unitNumber, self.markError)
                    self.removeFromTable(index)
                
            else:    
                globalClass.hasErrors=1
            if not globalClass.searchProjectError and not self.searchFileError:
                self.removeFromTable(index)
            if globalClass.searchProjectError:
                self.addNoteToProjectXlf(cleanNote)
                
        self.postTestList = []
        errorMessageList=[]   
        errormessage = None
        

    def hasAccelerator(self, str, type = 'openoffice' ):
#        This function checks whether there is any accelelator or not in the str parameter. The checking is done according to type mentioned.
        openoffice = '~'
        mozilla = '&'
        gnome = '_'
        user=''
        result = None
        if type == 'openoffice':
            result = string.find(str, openoffice, 0, len(str)-1)
        elif type == 'Mozilla' or type == 'KDE' or type == 'wx':
            result = string.find(str, mozilla, 0, len(str)-1)
        elif type == 'gnome':
            result = string.find(str, gnome, 0, len(str)-1)
        elif type == 'UserDefine':
            user=globalClass.endPuncDict['UserDefine']
            result = string.find(str, user, 0, len(str)-1)   
        if result == -1:
            return False
        else:
            return True
            
    def hasEscapes(self, str1):
#        This function finds out the existence of an escape in the source. If there is an escape then returns backslash with its next character, if not then return an empty dictionary
        escapeDict = {}
        escapelist = self.getBracket(str1, '\\')
        for each in escapelist:
            if each < len(str1)-1 and str1[each+1] == '\\':
                escapelist.remove(each+1)
            
            prunedString = str1[each:each+2]
            if prunedString not in escapeDict.keys():
                escapeDict[prunedString] = 1
            else:
                escapeDict[prunedString] += 1
                
        return escapeDict
    
    def hasFilePath(self,str1):
#        This function looks for filepath in a string. Returns a list of all the file paths.If no filepath found then returns an empty list.
        wordlist = str1.split()
        pathlist = []
        regex = r'^(.*?/|.*?\\)?([^\./|^\.\\]+)(?:\.([^\\]*)|)$'
        for each in wordlist:
            filepath = (re.findall(regex,each))
            
            if filepath:
                pathlist.append(filepath[0][0])
        return pathlist
    
    def hasFunction(self, str1, regexType):
#        This function matches regex within the source string. Returns a list containing all the function , otherwise retirns an empty list. 

        list = []
        if regexType == 'functions':
            regex = r'\w+[\ ]*\(.*?\)'
        elif regexType == 'printf':
            regex = r'\%\d?\d?\$?\d?\.?\d?\w' 
        elif regexType == 'xmltagsAtt':
            regex = r'<[^<]*?/?>' 
        elif regexType == 'xmltags':
            regex = r'<[^<]*?/?>' 
        list = re.findall(regex,str1)
        return list
     
    def isFunction(self, func):
#        This function checks for a c style function structure. If matches then returns True, otherwise returns False.
        withBracket = re.findall(r'\(.*\)', func)[0]
        withInBracket = withBracket[1:len(withBracket)-1]
        parameters = withInBracket.split(',')

        for each in parameters:
            if len(each.split()) != 1:
                return False
        return True
        
    def hasUrl(self, str1):
#        This function looks for all the urls within the string. Returns a list of Urls otherwise return an empty list
        list = []
        regex = r'^(?#Protocol)(?:(?:ht|f)tp(?:s?)\:\/\/|~/|/)?(?#Username:Password)(?:\w+:\w+@)?(?#Subdomains)(?:(?:[-\w]+\.)+(?#TopLevel Domains)(?:com|org|net|gov|mil|biz|info|mobi|name|aero|jobs|museum|travel|[a-z]{2}))(?#Port)(?::[\d]{1,5})?(?#Directories)(?:(?:(?:/(?:[-\w~!$+|.,=]|%[a-f\d]{2})+)+|/)+|\?|#)?(?#Query)(?:(?:\?(?:[-\w~!$+|.,*:]|%[a-f\d{2}])+=(?:[-\w~!$+|.,*:=]|%[a-f\d]{2})*)(?:&(?:[-\w~!$+|.,*:]|%[a-f\d{2}])+=(?:[-\w~!$+|.,*:=]|%[a-f\d]{2})*)*)*(?#Anchor)(?:#(?:[-\w~!$+|.,*:=]|%[a-f\d]{2})*)?$' 
        wordlist = str1.split()
        for each in wordlist:
            if each[-1] in globalClass.targetReplaceChars:
                each=each[:-1]
            temp = re.findall(regex, each)
            if temp:
                list += temp
        return list
        
    def hasVariables(self,source, applicationType):
#        This function looks for application specific variables, returns true if anyvariable found otherwise returns false.
        marklist = []
        
        if applicationType == 'openoffice':
            marklist = World.openOfficeVarmatches
        elif applicationType == 'Mozilla':
            marklist = World.mozillaVarmatches
        elif applicationType == 'KDE':
            marklist = World.kdeVarmatches
        elif applicationType == 'Drupal':
            marklist = World.drupalVarmatches
        
        if marklist:
            for each in marklist:
                res = None
                res = decoration.findmarkedvariables(unicode(source),each[0],each[1])
                if res:
                    return True, marklist
        return False, []
     #code added by Aditi for finding tag with attribute   
     
    def getXmlAtt(self, tag):
#        This function gets the xml tag and attribute from a particular string. 
        regex= r'\w+[\ ]*\w+[[=][\"\']?]*'
        list = re.findall(regex,tag)
        tagnameList= []
        if list:
            tagnameList=list[0].split(" ") 
            tagname=tagnameList[0]
            list[0]=tagnameList[1]
            list2=[]
            for each in list:
                each=each[:-2]
                list2.append(each)
            return tagname, list2
        else:
            return
        
    def hasXmlAttr(self, str):
#    This function checks whether there is any xml tags in the source. Reruens an empty list if none, othewrwise treturns a list of all the xml tags
        list = []
        tempList = self.hasFunction(str, 'xmltags')
        for tag in tempList:
            if self.isXmlTag(tag):
                list.append(tag)
        return list    
 
    def hasXml(self, str):
#    This function checks whether there is any xml tags with attribute in the source. Reruens an empty list if none, othewrwise treturns a list of all the xml tags
        
        self.XmltagAttDic={}
        duplicateTag1={}
        list = []
        preAttr=[]
        tempList = self.hasFunction(str, 'xmltags')
        for tag in tempList:
            if self.isXmlTag(tag):
                list.append(tag)
        lst=[]     
        duplicateTag1={}
        for each in list:
            eachlist= each.split()
            lst.append(eachlist[0])
            
        i=0
        for each in list:
            self.onlytaglist=[]
            eachlist= each.split()
            dup=lst.count(eachlist[0])
            duplicateTag1[lst[i]]=dup
            eachlen=len(eachlist)
            if eachlen>1:
                tagname, self.onlytaglist=self.getXmlAtt(each)
                if dup>1:
                    if tagname in self.XmltagAttDic: 
                        preAttr=self.XmltagAttDic[tagname]
                        for eachtag in self.onlytaglist:
                            if eachtag not in preAttr:
                                preAttr.append(eachtag)
                        self.XmltagAttDic[tagname]= preAttr   
                    else:
                        self.XmltagAttDic[tagname]=self.onlytaglist
                else:        
                    self.XmltagAttDic[tagname]=self.onlytaglist
  
            elif eachlen==1:
                self.XmltagAttDic[each]=self.onlytaglist  
            i=i+1

        return duplicateTag1, self.XmltagAttDic
     #end of Aditi's code   
     
    def isXmlTag(self, tag):
#        This function checks whether the tag is according to xml structute. If according to xml structure thene returns true, otherwise returns false.
        matchObject = self.removeXmlPrefix.match(tag,0)
        if matchObject:
            foreTruncateTuple = matchObject.span()
            foreTruncateLength = foreTruncateTuple[1] - foreTruncateTuple[0]
            tag = tag[foreTruncateLength:] 
        tag = tag.replace(u'>', u'',1)
        tagWords = tag.split()
        length = len(tagWords)
        if length == 0:
            return True
        elif length > 0:
            startIndex = 0
            while startIndex != len(tag):
                match = self.xmlAttributeMatchExpression.match(tag, startIndex)
                if match:
                    TruncateTuple = match.span()
                    TruncateLength = TruncateTuple[1] - TruncateTuple[0]
                    startIndex += TruncateLength
                else:
                    return False
        return True
        
        
    def hasBracket(self, str):
        
#        This function looks for any  bracket ((,{,[) in str. If any brachet found then return a dictionary consisting of the number of the each type of brackets in the str.   
        bracketList = []
        bracketDict = {}
        firstBracketList = self.getBracket(str, '(')
        secondBracketList = self.getBracket(str, '{')
        thirdBracketList = self.getBracket(str, '[')
        if firstBracketList or secondBracketList or thirdBracketList:
            bracketList = firstBracketList + secondBracketList + thirdBracketList
            bracketList.sort()
            bracketDict['('] = len(firstBracketList)
            bracketDict['{'] = len(secondBracketList)            
            bracketDict['['] = len(thirdBracketList)
        return bracketDict            
        
    def getBracket(self, str, char):
#        This function returns a list of indices of all occurances of the char in str.
        list = []
        start = 0
        index = 0
        while index != -1:
            index = str.find(char,start)
            if index != -1:
                list.append(index)
                start = index+1
        return list    
    
    def WF_brackets(self, str1, str2, dict):
#        This function checks whether the brackets in the source and in the target are consistent. Also checks for the target brackets sequeance using a stack like mechanism.
        tempdict = {}
        
        tempdict[u')'] = u'('
        tempdict[u'}'] = u'{'
        tempdict[u']'] = u'['
        
        targetDict = self.hasBracket(str2)
        if dict != targetDict:
            return u"Source and Target doesn't have equal number of brackets."
        else:
            expression = '\(|\)|\{|\}|\[|\]'
            lst = re.findall(expression, str2)
            templst = []
            for each in lst:
                if each == u'(' or each == u'{' or each == u'[':
                    templst.append(each)
                elif each == u')' or each == u'}' or each == u']':
                    popitem = templst.pop()
                    if popitem != tempdict[each]:
                        return u'bracket' + each + u"is not consistent." 
            if templst:
                return u'bracket is not consistent.'
        return False   
    
    
    def WF_urls(self, str1, str2, sourceUrlList):
#        This function gets the Urls's from the target and matches with the sourceUrlList. If finds an exact match then returns false otherwise returns an error message.
        targetUrlList = self.hasUrl(str2)
        if sorted(sourceUrlList) == sorted(targetUrlList):
            return False
        else:
            return u'Urls should be same in source and translation.'
    
    def WF_glossarytest(self, str1, str2):
#        This function matches whether the glossary suggestions are in the translation.
        tempsource=string.lower(str1)
        gtemplist=self.hasXmlAttr(tempsource)
        mtemp=[]
        for x in gtemplist:
            temp=[]
            x=str(x)
            x=x.lstrip()
            x=x.rstrip()
            for c in x:
                if c=='<' or c=='>' or c=='"':
                    x=x.replace(c, '')
                elif c=='=':
                    x=x.replace(c, ' ')
            temp=x.split()
            mtemp.append(temp)
                    
        for x in gtemplist:
            t=tempsource.replace(str(x), "")
            tempsource=t
        tempsource=tempsource.split()
        for i in range(len(tempsource)):
            tempsource[i]=str(tempsource[i])
#goutom error message

        list = []
        for eachitem in globalClass.glsErrorDict.keys():
            translate = str2
            keylist = globalClass.glsErrorDict[eachitem].keys()
            flag = 1
            for item in keylist:
                if flag == 0:
                    break
                if self.searchFileError:
                    if item in translate:
                        flag = 0
                else:        
                    for i in range (0, globalClass.glsMatchWordCount[eachitem]):
                        if item in translate and i == globalClass.glsMatchWordCount[eachitem]-1:
                            flag = 0
                        elif item in translate and i < globalClass.glsMatchWordCount[eachitem]-1: 
                            firstindex = string.find(translate, item)
                            firstindex += len(item) 
                            translate = translate[firstindex:]
                       
            if flag == 1:
#goutom error message
                c=1
                for tempstr in mtemp:
                    if eachitem in  tempstr and  eachitem not in tempsource:
                        c=0
                        break
                if c==1:
                    list.append(eachitem)         
#goutom error message
        if list:
            return u'One of the glossary suggestions of : ' + u', '.join(list) + u' must be in the translation.'
        else:
            return False
            
    def hasCharacter(self,str1, char):
#     Searches for any existance of char in the str1. returns an empty list if no match found.
        if char == 'doublequoting':
            expression = u'\"'
        elif char == 'doublespacing':
            expression = u'  '  
        elif char == 'newlines':
            expression = u'\n'    
        elif char == 'singlequoting':
            expression = u'\''    
        elif char == 'tabs':
            expression = u'\t'    
        lst = re.findall(expression, str1)
        
        if lst:
            return True
        else:
            return False

    def WF_xmltagsAtt(self, str1, str2, xmltagsAtt):
#        this function checks whether all the xml tags of the list xmltags is in the target. 
        lst = []
        for eachtag in xmltagsAtt:
            if eachtag not in str2:
                lst.append(eachtag)
        if lst:
            return u'Xml tag: ' +  u', '.join(lst) + 'should be exactly same as the source.'
        else:
            return False
    #start of code by Aditi
    def WF_xmltags(self, str1, str2, XmltagAttDic1, duplicateTag1):
#        this function checks whether all the xml tags of the list xmltags is in the target This test doesnot check Attribute value. 
        XmltagAttkey1=[]
        XmltagAttkey2=[]
        XmltagAttvalue1=[]
        XmltagAttvalue2=[]
        Flag=False
        duplicate, XmltagAttDic2 = self.hasXml(str2) 
        XmltagAttkey1=XmltagAttDic1.keys()
        XmltagAttkey2=XmltagAttDic2.keys()
        dup1=duplicateTag1.keys()
        dup2=duplicate.keys()
        
        if (duplicate !=duplicateTag1):
            for each in dup1:
                    sourcedup=duplicateTag1[each]
                    if each in duplicate:
                        targetdup=duplicate[each]
                        if sourcedup!=targetdup:
                            if each.find("<")!=-1:
                                each=str(each).replace("<", "")
                            if each.find(">")!=-1:
                                each=str(each).replace(">", "")    
                            return u'Xml tag or attribute: ' + each + ' should be exactly same as the source.'    
                    else:
                        if each.find("<")!=-1:
                                each=str(each).replace("<", "")
                        if each.find(">")!=-1:
                            each=str(each).replace(">", "")    
                        return u'Xml tag  or attribute: ' + each + ' should be exactly same as the source.' 
        
        if XmltagAttDic1!= XmltagAttDic2:
            
            for each in XmltagAttkey1:
               if each not in XmltagAttkey2: 
                    if each.find("<")!=-1:
                        each=str(each).replace("<", "")
                    if each.find(">")!=-1:
                        each=str(each).replace(">", "")  
                    return u'Xml tag  or attribute: ' + each + ' should be exactly same as the source.'
               else:
                    Flag=True
            if Flag: 
                    try:
                        for each in XmltagAttkey1:
                            XmltagAttvalue1=XmltagAttDic1[each]

                            XmltagAttvalue2=XmltagAttDic2[each]
                            for eachvalue in XmltagAttvalue1:
                                if eachvalue not in XmltagAttvalue2: 
                                    if each.find("<")!=-1:
                                        each=str(each).replace("<", "")
                                    if each.find(">")!=-1:
                                        each=str(each).replace(">", "")  
                                    return u'Xml tag  or attribute: ' + eachvalue + ' should be exactly same as the source.'           
                    except:
                        pass
        else:
            return False
    #end of code by Aditi        
    def WF_escapes(self, str1, str2):
#     This function checks whether the source and target escape sequences are consistent. If not then returns an error message.   
        dict = self.hasEscapes(str(str2))
        if dict != self.escapeDict:
            return u'Source and target escape sequences do not match.'
        else:
            return False
    
    def WF_filepaths(self, str1, str2, sourceFilePath):
#        This function matches whether the file paths in source and target are same or not. If same then returns False if different then returns an error message
        targetFilePath = self.hasFilePath(str2)
        for eachpath in sourceFilePath:
            if eachpath not in targetFilePath:
                return u'Target file path: "' + eachpath + '" is not same as the source file path.'
                
        return False
    
    def WF_functions(self, str1, str2, sourceFunctions):
#        This function matches whether the function names in source and target are same or not. If same then returns False if different then returns an error message.
        targetFunctions = self.hasFunction(str2, 'functions')
        for eachfunc in sourceFunctions:
            if eachfunc not in targetFunctions:
                return u'Target function name: "'+ eachfunc + u'" should be same as the source function name.'
                
        return False 
    
    def WF_printf(self, str1, str2, sourcePrintf):
#        This function finds the printf placeholders from the target. Prunes any $ and digit, then matches with the source placeholder dictionary sourcePrintf. If exactly same then retuns False, otherwise returns an error message.
        targetPrintf = self.hasFunction(str2,'printf') 
        for i in range(0, len(targetPrintf)):
            if string.find(targetPrintf[i],'$') != -1:
                targetPrintf[i] = targetPrintf[i].replace('$', '')
            for character in targetPrintf[i]:
                if character == u'0' or character == u'.':
                    break
                elif character.isdigit():
                    targetPrintf[i] = targetPrintf[i].replace(character,'', 1)
 
        from copy import copy        
        templist = copy(targetPrintf)
        for each in sourcePrintf:
            try:
                templist.remove(each)
            except:
                pass
        if templist:
            return u'Error with placeholder: '+ string.join(templist) + u' in the translation.'
            
        templist = copy(sourcePrintf)
        for each in targetPrintf:
            try:
                templist.remove(each) 
            except:
                pass
        if templist:
            return u'Error with placeholder: '+ string.join(templist) + u' in the translation.'
        return False
            
if __name__ == "__main__":
    
    app = QtGui.QApplication(sys.argv)
    errorChecking = ErrorChecking()
    errorChecking.show()
    sys.exit(app.exec_())

