#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2006 - 2007 by The WordForge Foundation
#                 www.wordforgefoundation.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details. 
#
# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)
#
# This module is working on the Dialog for Export Glossary presently into TBX formate file.

from PyQt4 import QtCore, QtGui
from WordForge.Editor.ui.Ui_Export2TBX import Ui_ExportTBX
from WordForge.Editor.modules import FileDialog
import WordForge.Editor.modules.World as World
import os, sys
from WordForge.Editor.modules import GL2tbx

class Export2TBX(QtGui.QDialog):
    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self.ui = None
    
    def showDialog(self):
        """
        Dialog a Third language dialog for merging operation.
        """
        # lazy init
        if not self.ui:
            self.ui = Ui_ExportTBX()
            self.ui.setupUi(self)
            
            self.ui.lineOutput.setFocus()
            self.connect(self.ui.btnOutput, QtCore.SIGNAL("clicked()"), self.getFile)
            self.connect(self.ui.btnOk, QtCore.SIGNAL("clicked()"), self.export2TBX)
            self.connect(self.ui.btnCancel, QtCore.SIGNAL("clicked()"), QtCore.SLOT("close()"))
            
        self.show()
    
    def getFile(self):
        """
        Open the file dialog where you can choose both file and directory.
        
        @return filename or dirname
        """
        directory = World.settings.value("WorkingDir").toString()
        filename = QtGui.QFileDialog.getSaveFileName(self, self.trUtf8("Open File"),
                                                 directory,
                                                 self.trUtf8("TBX File (*.tbx)"))
                                                 
        if filename:
            filename = str(filename)
            if not filename.endswith(".tbx"):
                filename = filename + ".tbx"
            self.ui.lineOutput.setText(str(filename))
            directory = os.path.dirname(unicode(filename))
            World.settings.setValue("WorkingDir", QtCore.QVariant(directory))
            return filename
        else:
            return False
    
    def export2TBX(self):
        outputFile = str(self.ui.lineOutput.text())
        import shutil
        inputFile = str(os.path.split(str(World.settings.fileName()))[0]) + os.path.sep +"WordForgeGL.tbx"
        if os.path.exists(inputFile) and outputFile:
            shutil.copy2(inputFile, outputFile)
            QtGui.QMessageBox.information(self, "Exporting", "The TBX File has been created.")
        
if __name__ == "__main__":
    import sys, os
    # set the path for QT in order to find the icons
    QtCore.QDir.setCurrent(os.path.join(sys.path[0], "..", "ui"))
    app = QtGui.QApplication(sys.argv)
    exportTBX = Export2TBX(None)
    exportTBX.showDialog()
    sys.exit(app.exec_())
    
