#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#Copyright (c) 2006 - 2007 by The WordForge Foundation
#                       www.wordforgefoundation.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details.
#
# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)
#
# This module is working on FileAction.

import sys, os, tempfile
from PyQt4 import QtCore, QtGui
from WordForge.Editor.modules import World
from WordForge.Editor.modules.globalClass import globalClass
from subprocess import Popen, PIPE, STDOUT


class FileAction(QtCore.QObject):
    """
    Code for the actions in File menu.
    
    @signal fileToSave(string): emitted when a file is saved. 'string' is the filename as QString.
    @signal fileToOpen(string): emitted when a file is opened. 'string' is the filename as QString.
    """

    def __init__(self, parent):
        """ 
        init the class.
        @param parent a QWidget to center the dialogs 
        """
        QtCore.QObject.__init__(self)
        self.parentWidget = parent
        self.filename = None
        self.oldFilename = None
        self.fileExtension = ""
        self.fileDescription = ""
        self.MaxRecentHistory = 10
        self.isSaved = False
        self.forceClose = False
        self.openIcon=False
        
        
        self.directory = World.settings.value(World.workingDir).toString()
        if (not self.directory) or (not os.path.exists(self.directory)):
            self.directory = QtCore.QDir.homePath()
        
    def openFile(self):
        """
        Open an OpenFile dialog.
        @return: Returns True if ok button is click.
        """
        self.openIcon=True
        #TODO: open one or more existing files selected
        newFileName = QtGui.QFileDialog.getOpenFileName(self.parentWidget, self.tr("Open File"),
                        self.directory, ";;".join(World.fileFilters))
        if (not newFileName.isEmpty()):
            # remember last open file's directory.
            self.saveoldFile=self.oldFilename
            self.setFileProperty(newFileName.replace("/", os.path.sep))
            self.emitFileOpened()
            return True
        
    def save(self, filename = None):
        """
        Emit fileToSave signal for operator to do the save file.
        Emit save signal for TUView and Overview to commit their changes.
        """
        if (not filename):  
            newfName = self.filename 
            try:    
                if self.openIcon: 
                    filename = unicode(self.saveoldFile)
                    if not filename==unicode("None"):
                        self.filename = filename
                else:
                    filename = self.filename
            except:
                    filename = self.filename
            if (filename==unicode("None")) or (not self.openIcon): 
                filename = newfName   
            self.emit(QtCore.SIGNAL("save"))
        self.emit(QtCore.SIGNAL("fileToSave"), filename)
        self.isSaved =True
        self.openIcon=False
        
    def saveAs(self):
        """
        Open an SaveAs File dialog.
        """
        # TODO: think about export in different formats
        fileDialog = QtGui.QFileDialog(self.parentWidget, self.tr("Save As"), self.directory, ";;".join(World.fileFiltersSaveAs))

        fileDialog.setHistory(World.settings.value("SaveAsHistory").toStringList())
        fileDialog.setDirectory(self.directory)
        fileDialog.setLabelText(QtGui.QFileDialog.Accept, self.tr("Save As"))
        fileDialog.setAcceptMode(QtGui.QFileDialog.AcceptSave)
        fileDialog.setConfirmOverwrite(True)
        
        # perform save as only when there is filename
        if (fileDialog.exec_()):
            files = fileDialog.selectedFiles()
            if (not files.isEmpty()):
                filename = unicode(files.first())
                sourceFileType = self.fileExtension.strip(".")
                
                # TODO: use content of file to determine the file type
                # but self.__store__ is in operator.py
#                if isinstance(self.__store__, xliff.xlifffile):
#                    print "it is xliff file."
                
                filenameNoExt, ext = os.path.splitext(filename)
                targetFileType = ext[len(os.path.extsep):].lower()
                
                # user has selected a filetype in the dialog, it is override the filename
                # TODO: use regular expression to get file extension
                filter = fileDialog.selectedFilter().__str__()
                if (filter in World.fileFiltersSaveAs):
                    index = World.fileFiltersSaveAs.index(filter)
                    # assume it is .xlf filetype
                    if (index == 1):
                        targetFileType = "xlf"
                    # assume it is .po filetype
                    elif (index == 2):
                        targetFileType = "po"
                
                converter = World.saveasdic.get(str(sourceFileType + "2" + targetFileType).lower())
                # saving this file need a conversion.
                if (converter):
                    # will save to tempfile whether the units has modified or not.
                    tempFilename = tempfile.mkstemp()[1] + "." + sourceFileType
                    self.save(tempFilename)
                    cmd = "%s -i %s -o %s --errorlevel=message" % (converter, tempFilename, filenameNoExt + "." + targetFileType)
                    s = Popen(cmd, stdout=PIPE, stderr=STDOUT, shell=isinstance(cmd, basestring)).communicate()[0]
                    try:
                        os.remove(tempFilename)
                    except:
                        pass
                    savedFile = filenameNoExt + "." + targetFileType
                
                # simple save as, just with different filename.
                else:
                    if (targetFileType in ["xlf", "xliff", "po"]):
                        savedFile = filenameNoExt + "." + targetFileType
                    elif (targetFileType):
                        savedFile = filename + "." + sourceFileType
                    else:
                        savedFile = filenameNoExt + "." + sourceFileType
                    self.save(savedFile)
                
                if os.path.isfile(savedFile):
                    self.setFileProperty(savedFile)
                    # ignore what has modified since the tempfile is has the latest changes of units.
                    self.forceClose = True
                    self.emitFileOpened()
                    self.forceClose = False
                else:
                    QtGui.QMessageBox.critical(self.parentWidget, "Error", "A problem has occured while saving file.")
    
    def clearedModified(self, parent):
        """
        Return bool indicates it has cleared the content modified.
        """
        globalClass.closeFile=True
        if (self.forceClose):
            self.forceClose = False
            return True
        
        ret = QtGui.QMessageBox.question(parent, self.tr("File Modified"),
                    self.tr("The file has been modified.\n"
                            "Do you want to save your changes?"),
                    QtGui.QMessageBox.Yes | QtGui.QMessageBox.Default,
                    QtGui.QMessageBox.No,
                    QtGui.QMessageBox.Cancel | QtGui.QMessageBox.Escape)
        
        if (ret == QtGui.QMessageBox.Yes):
            self.save()
            
            return True
        elif (ret == QtGui.QMessageBox.No):
            # reset file name
            self.filename = ""
            return True
        elif (ret == QtGui.QMessageBox.Cancel):
            return False
    
    def setFileProperty(self, filename):
        """
        Set information such as directory, extension, and description from
        filename.
        @param filename: name of file as unicode string.
        """
        self.saveoldFile=self.oldFilename
        filename = unicode(filename)
        self.filename = filename
        # remember last open file's directory.
        self.directory = os.path.dirname(unicode(filename))
        
        extension = {"po": "PO Files", "pot": "PO Template Files", 
            "xlf": "XLIFF Files", 
            "tmx": "Translation Memory eXchange (TMX) Files", "tbx": "TermBase eXchange (TBX) Files"}
        name, ext = os.path.splitext(filename)
        ext = ext[len(os.path.extsep):].lower()
        self.fileExtension = "." + ext
        self.fileDescription = extension.get(ext) 
        
        World.settings.setValue(World.workingDir, QtCore.QVariant(self.directory))
        
        self.oldFilename=self.filename
#        print "Property old...", self.oldFilename
    def emitFileOpened(self):
        """
        Send "fileToOpen" signal with a filename as string.
        """
        self.emit(QtCore.SIGNAL("fileToOpen"), str(self.filename))
    
if __name__ == "__main__":
    # set the path for QT in order to find the icons
    QtCore.QDir.setCurrent(os.path.join(sys.path[0], "..", "ui"))
    app = QtGui.QApplication(sys.argv)
    fileaction = FileAction(None)
    sys.exit(app.exec_())
