#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#Copyright (c) 2006 - 2007 by The WordForge Foundation
#                       www.wordforgefoundation.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details. 
#
# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)

from WordForge.Editor.modules import World
from PyQt4 import QtCore, QtGui
from WordForge.Editor.modules.globalClass import globalClass
import re, string

class Highlighter(QtGui.QSyntaxHighlighter):
    def __init__(self, parent):
        if World.WF_debug:
            print "Entering __init__ in class highlighter"
        QtGui.QSyntaxHighlighter.__init__(self, parent)
        self.parent = parent
        # var format
        self.varFormat = QtGui.QTextCharFormat()
        self.varFormat.setForeground(QtCore.Qt.blue)
        # variables: e.g. &python; %s %1 AppName
        if globalClass.endPuncDict:
            for i in globalClass.endPuncDict.keys():
                if (i =="Mozilla" or i =="KDE" or i =="UserDefine"):
                    if globalClass.endPuncDict[i]=="&":
                        self.varExpression = re.compile(r'%\w+%?|\(?\$[({[]?\w+[])}]?\$?|#\w+#?|\[\w+\]|[A-Z][a-z]+[A-Z][a-z](\w)+|\%[\ ]*\(\w+[\]*[\,\w+]*\)s')
                        break
                else:
                    self.varExpression = re.compile(r"&\w+;?|%\w+%?|\(?\$[({[]?\w+[])}]?\$?|#\w+#?|\[\w+\]|[A-Z][a-z]+[A-Z][a-z](\w)+|\%[\ ]*\(\w+[\]*[\,\w+]*\)s")
        else:
            self.varExpression = re.compile(r"&\w+;?|%\w+%?|\(?\$[({[]?\w+[])}]?\$?|#\w+#?|\[\w+\]|[A-Z][a-z]+[A-Z][a-z](\w)+|\%[\ ]*\(\w+[\]*[\,\w+]*\)s")       

        # tag format
        self.tagFormat = QtGui.QTextCharFormat()
        self.tagFormat.setForeground(QtCore.Qt.darkMagenta)
        # tags: e.g. <b> </ui>
        self.tagExpression = re.compile(r'<[^<]*?/?>')
        # escape format
        self.escapeExpression = re.compile(r'\\[t|r|n]')
        self.escapeFormat = QtGui.QTextCharFormat()
        self.escapeFormat.setForeground(QtCore.Qt.red)
        # glossary format
        self.glsFormat = QtGui.QTextCharFormat()
        self.glsFormat.setFontWeight(QtGui.QFont.Bold)
        self.glsFormat.setForeground(QtCore.Qt.darkGreen)
        self.glsFormat.setFontUnderline(True)
        self.glsFormat.setUnderlineColor(QtCore.Qt.darkGreen)
        self.highlightGlossary = False
        self.glsExpression = None
        self.errGlsExpression=None
        self.glossaryWords = []
        
        # search format
        self.searchFormat = QtGui.QTextCharFormat()
        self.searchFormat.setFontWeight(QtGui.QFont.Bold)
        self.searchFormat.setForeground(QtCore.Qt.white)
        self.searchFormat.setBackground(QtCore.Qt.darkMagenta)
        
        # missSpelledList format
        self.missSpelledFormat = QtGui.QTextCharFormat()
        self.missSpelledFormat.setForeground(QtCore.Qt.darkRed)
        self.missSpelledFormat.setFontUnderline(True)
        self.missSpelledFormat.setUnderlineColor(QtCore.Qt.darkRed)
        self.missSpelledExp = None
        
        # a wrong word format
        self.missedFormat = QtGui.QTextCharFormat()
        self.missedFormat.setForeground(QtCore.Qt.white)
        self.missedFormat.setBackground(QtCore.Qt.red)
        
        self.searchString = None
        self.missed = None
        
    def highlightBlock(self, text):
        """
        highlight the text according to the self.expression
        @ param text: a document text.
       
        if World.WF_debug:
            print "Entering highlightBlock in class highlighter"
         """

        # clear glossaryWords not by rehighlight block.
#           start of modification for  issue number :2818343
#           if the target text contains any of the characters of World.regularExpressionCharacters then we have to represent them with a backslash before that. If these characters come at the begining or the end, then we need to add an extra character (here n) before nd after that for matching with glossary.
# nAppended is a flag that holds True if we append an n in the end. Otherwise False.
        nAppended = False
        try:
            if (str(text))[0]  in World.regularExpressionCharacters:
                text.prepend('n')
            if (str(text))[-1]  in World.regularExpressionCharacters:
                text.append('n')
                nAppended = True
        except:
            pass
        text1 = text[:]
#     end of modification for  issue number :2818343
        if (self.previousBlockState() == -1):
            self.glossaryWords = []
            self.blockWordCount = 0
        self.setCurrentBlockState(0)
        if (self.highlightGlossary) and (self.glsExpression):
            text1.replace("\\t", "\\ ")
            text1.replace("\\r", "\\ ")
            text1.replace("\\n", "\\ ")
            glsIndex = text1.indexOf(self.glsExpression)

        else:
            glsIndex = -1
        self.glsHtIndex= glsIndex
        #the line below had to be added for Qt 4.5 to solve issue 2815051.
        length1=0
        
        text1.replace("\\t", "\\ ")
        text1.replace("\\r", "\\ ")
        text1.replace("\\n", "\\ ")
        while (glsIndex >= 0):
            # highlight glossary
            if (self.highlightGlossary) and (self.glsExpression):
               #the lines below had to be added for Qt 4.5 to solve issue 2815051.
                length1 = self.glsExpression.indexIn(text1, length1)
                length = self.glsExpression.matchedLength()
                length1 += length
                if str(text1)[0]=="n" and str(text1)[1] in World.regularExpressionCharacters:
                    self.setFormat(glsIndex-1, length, self.glsFormat)
                else:
                    self.setFormat(glsIndex, length, self.glsFormat)
#code added for showing glossary suggestions (when right clicked) for more than two consecutive highlighted words.  
#Issue number: 2817377  	  
                matchedWord = self.glsExpression.capturedTexts()[0]
                if matchedWord:
                    if matchedWord in  globalClass.tempGlsMatchWordCount.keys():
                        globalClass.tempGlsMatchWordCount[string.lower(str(matchedWord))] += 1
                    elif matchedWord != None:
                        globalClass.tempGlsMatchWordCount[string.lower(str(matchedWord))] = 1
                    try:
                        if  globalClass.glsDict[str(text)][glsIndex]:
                            pass
                    except:
                            globalClass.glsDict[str(text)][glsIndex] = matchedWord
                            globalClass.glsLastIndexDict[str(text)][glsIndex] = glsIndex + length
#end of my code                           
                self.glossaryWords.append(unicode(self.glsExpression.capturedTexts()[0]))
                glsIndex = text1.indexOf(self.glsExpression, glsIndex + length)

        #the following segment of code had to be added for neglecting the extra chracter added previously (here n) for those text containing any of the characters of World.regularExpressionCharacters. this will help to highlight a variables correctly. 
        
        try:
            if str(text)[0]=="n" and str(text)[1] in World.regularExpressionCharacters:
                str1= str(text)[1:]
                text= QtCore.QString.fromUtf8(str1)
#            Check if n is appended, otherwise it will remove tailing \n from the string
            if nAppended and str(text)[-1]=="n" and str(text)[-2] in World.regularExpressionCharacters :
                str1= str(text)[:-1]
                text= QtCore.QString.fromUtf8(str1)
        except:
            pass
# highlight tag
        tagIndex = -1
        matchObj = self.tagExpression.search(unicode(text),0)
        if matchObj:
            tagIndex = matchObj.start()

        while (tagIndex >= 0):
            if matchObj:
                matchRange = matchObj.span()
                length = matchRange[1] - matchRange[0]
                self.setFormat(tagIndex, length, self.tagFormat)
        
                matchObj = self.tagExpression.search(unicode(text),tagIndex + length)
                if matchObj:
                    tagIndex = matchObj.start()
                else:
                    tagIndex = -1

    #start of code by Aditi
        
# highlight variable
        varIndex = -1
        matchObj = self.varExpression.search(unicode(text),0)
        if matchObj:
            varIndex = matchObj.start()
        while (varIndex >= 0):
            if matchObj:
                matchRange = matchObj.span()
                length = matchRange[1] - matchRange[0]
                self.setFormat(varIndex, length, self.varFormat)
        
                matchObj = self.varExpression.search(unicode(text),varIndex + length)
                if matchObj:
                    varIndex = matchObj.start()
                else:
                    varIndex = -1
             #end of code modification for highlighting variable correctly.
#        Highlight escapes
        escapeIndex = -1
        matchObj = self.escapeExpression.search(unicode(text),0)
        if matchObj:
            escapeIndex = matchObj.start()

        while (escapeIndex >= 0):
        # highlight tag and variable
            if matchObj:
                matchRange = matchObj.span()
                length = matchRange[1] - matchRange[0]
                self.setFormat(escapeIndex, length, self.escapeFormat)        
                matchObj = self.escapeExpression.search(unicode(text),escapeIndex + length)
                if matchObj:
                    escapeIndex = matchObj.start()
                else:
                    escapeIndex = -1
        # highlight search
        if (self.searchString):            
            index = self.foundPosition - self.blockWordCount
            self.setFormat(index, len(self.searchString), self.searchFormat)
            cursor = self.parent.textCursor()
            cursor.setPosition(self.blockWordCount + index)
            self.parent.setTextCursor(cursor)
            captured = unicode(text[index:index + len(self.searchString)])
            if (captured.lower() == self.searchString.lower()):
                self.searchString = None
        # highlight missSpelled:
        missSpelledIndex = -1
        if self.missSpelledExp:
            matchSpellObj = self.missSpelledExp.search(unicode(str(text)),0)
            if matchSpellObj:
                missSpelledIndex = matchSpellObj.start()
        while (missSpelledIndex >= 0):

            self.targetReplaceChars = globalClass.targetReplaceChars
            if matchSpellObj:
                matchRange = matchSpellObj.span()
                length = matchRange[1] - matchRange[0]
                try:
                    if ((missSpelledIndex>0 and (text[(missSpelledIndex-1)] in self.targetReplaceChars) or text[missSpelledIndex-2:missSpelledIndex] in ['\\n', '\\t', '\\r']) or missSpelledIndex == 0 ) and (((missSpelledIndex + length)== len(text)) or (text[missSpelledIndex + length] in self.targetReplaceChars)):
                        
                        self.setFormat(missSpelledIndex, length, self.missSpelledFormat)
                except:
                    pass
                matchSpellObj = self.missSpelledExp.search(unicode(str(text)),missSpelledIndex + length)
                if matchSpellObj:
                    missSpelledIndex = matchSpellObj.start()
                else:
                    missSpelledIndex = -1                

        # highlight search
        if (self.missed):
            index = self.missedPosition - self.blockWordCount
            self.setFormat(index, len(self.missed), self.missedFormat)
        
        self.blockWordCount += len(text) + 1
    
    def chckaccelerator(self, globalDict):
        for i in globalDict.keys():
            if (i =="Mozilla" or i =="KDE" or i =="UserDefine"):
                if globalClass.endPuncDict[i]=="&":
                    self.varExpression = re.compile(r'%\w+%?|\(?\$[({[]?\w+[])}]?\$?|#\w+#?|\[\w+\]|[A-Z][a-z]+[A-Z][a-z](\w)+|\%[\ ]*\(\w+[\]*[\,\w+]*\)s')
                    break
            else:
                self.varExpression = re.compile(r"&\w+;?|%\w+%?|\(?\$[({[]?\w+[])}]?\$?|#\w+#?|\[\w+\]|[A-Z][a-z]+[A-Z][a-z](\w)+|\%[\ ]*\(\w+[\]*[\,\w+]*\)s")
        self.refresh()
        
    def refresh(self):
        """
        mark contents dirty, to make it rehighlight.
   
        if World.WF_debug:
            print "Entering refresh in class highlighter"
       """          
        self.parent.document().markContentsDirty(0, len(self.parent.document().toPlainText()))
    
    def setPattern(self, patternList):
        """
        build self.glsExpression base on given pattern.
        @param patternList: list of string.
        """      
#code added for fixing issue number 2817377,2818343     
#first creates a regular expression,for taking out the strings with closing bracket,then removes those strings from the patternList.
        modifiedPatternList = []      
        for element in patternList:
            modifiedPatternList.append(element)
       
        tempList = QtCore.QStringList()
        tempList << modifiedPatternList
      
        pattern = "\\b(" + "|".join(p for p in modifiedPatternList) + ")\\b"
        self.glsExpression = QtCore.QRegExp(pattern)
        self.glsExpression.setCaseSensitivity(QtCore.Qt.CaseInsensitive)
        
#end of code modification for fixing issue number 2817377  
    
    def setSearchString(self, searchString, foundPosition):
        """
        set searchString and make document() dirty then it will
        re highlightBlock().
        @param searchString: string to be searched for
        @param foundPosition: Position of found string in the document
       
        if World.WF_debug:
            print "Entering setSearchString in class highlighter"
         """
        self.searchString = searchString
        self.foundPosition = foundPosition
        self.refresh()
        
    def setHighlightGlossary(self, bool):
        """ 
        if World.WF_debug:
            print "Entering setHighlightGlossary in class highlighter"
             """
        self.highlightGlossary = bool
    
    def setMissSpelledList(self, missSpelledList):
#        for each in missSpelledList:
#            print 'list', QtGui.QApplication.translate("MainWindow",each, None, QtGui.QApplication.UnicodeUTF8)
        if missSpelledList:
            pattern = "|".join(unicode(each) for each in missSpelledList)
            self.missSpelledExp = re.compile(pattern)
            self.refresh()
        else:
            self.missSpelledExp = None
            self.refresh()
            
    def highlightAnWrongWord(self, missed, missedPosition):
        """
        set searchString and make document() dirty then it will
        re highlightBlock().
        @param missed: string to be searched for
        @param foundPosition: Position of found string in the document
        if World.WF_debug:
            print "Entering setHighlightGlossary in class highlighter"
             """
        self.missSpelledExp = None
        self.missed = missed
        self.missedPosition = missedPosition
        self.refresh()
        

