#!/usr/bin/python
# -*- coding: utf-8 -*-
#Copyright (c) 2006 - 2007 by The WordForge Foundation
#                       www.wordforgefoundation.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
# 
# See the LICENSE file for more details. 
#
# Developed by:
#       Keo Sophon (keosophon@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)
# 
# This module is working on the main windows of Editor

import os,  sys
from PyQt4 import QtCore
from WordForge.translate.storage import factory
from WordForge.translate.search import match
from WordForge.translate.storage import xliff
from WordForge.translate.storage import tbx
import WordForge.Editor.modules.World as World
from lxml import etree
import WordForge. __version__
import optparse
import string, re
    
strREQUIRED = 'required'

def embedGlossary(glossaryFile, sourceFile, targetFile = "",  Mycounts = []):
        """
        embed glossary into a file or directory.
        @param glossaryFile: glossary file.
        @param sourceFile: source file or directory.
        @param targetFile: target or output file or directory. If not specified, it will generate
        a name automaticaly.
        """

#        if World.WF_debug:
#            print "Enter embedGlossary in InternalGlossary"

        if (os.path.isfile(sourceFile) and (sourceFile.endswith(".xlf") or sourceFile.endswith(".xliff"))):
            if (not targetFile):
#                targetFile = getFileName(sourceFile)
                targetFile = sourceFile
            embedGlossaryIntoFile(glossaryFile, sourceFile, targetFile,   Mycounts)

        elif (os.path.isdir(sourceFile)):
            if (not targetFile):                
#                targetFile = getDirectoryName(sourceFile)                          
                targetFile = sourceFile
            else:
                if not os.path.exists(targetFile):
                    os.makedirs(targetFile)
            
            for root, dirs, files in os.walk(sourceFile):                
                targetRoot = targetFile
                # file
                for file in files:
                    iFileName = os.path.join(root + os.path.sep + file)                    
                    tFileName = os.path.join(targetRoot + os.path.sep + file)
                    embedGlossary(glossaryFile, iFileName, tFileName, Mycounts)
                # folder
                for folder in dirs:                    
                    iFileName = os.path.join(root + os.path.sep + folder)
                    tFileName = os.path.join(targetRoot + os.path.sep + folder)                    
                    embedGlossary(glossaryFile, iFileName, tFileName,  Mycounts)
                break

def blankTargetFilter(candidate):
    if candidate.target:
        return True
    else:
        return False
        
def embedGlossaryIntoFile(glossaryFile, sourceFile, targetFile,   Mycounts = []):

#    if World.WF_debug:
#        print "Enter embedGlossary in InternalGlossary"
    glossaryList=[]
    matchedlist=[]
    try:
        glossaryStore = factory.getobject(glossaryFile)
        sourceStore = factory.getobject(sourceFile)
    except Exception, e:
        raise str(e)
    
    matcher = match.terminologymatcher(glossaryStore)
    
    for unit in matcher.candidates.units:
        element =unit.source
#        if str(element[0]) in World.regularExpressionCharacters:
#            element = 'n' + element
#        if str(element[-1]) in World.regularExpressionCharacters:
#            element = element + 'n'
        glossaryList.append(element)    
    pattern = "\\b(" + "|".join(p for p in glossaryList) + ")\\b"
    InternalglsExpression = QtCore.QRegExp(pattern)
    InternalglsExpression.setCaseSensitivity(QtCore.Qt.CaseInsensitive)
    # glossary object of founded words
    glossaryTbx = tbx.tbxfile()
    
    # Initialize what will be the list of already included words
    addedWords = list()
    
    # Create the internal glossary, going through all the words of the source file, looking 
    # them up in the glossary, and including them in a new XLIFF file that will become the
    # internal glossary.
    for unit in sourceStore.units:
        if (not unit.source):
                continue
        else:
            str1=unit.source
#            if (str(str1))[0] in World.regularExpressionCharacters:
#                str1='n' +str1
#            if (str(str1))[-1] in World.regularExpressionCharacters:
#                str1=str1+'n'
            str1 = QtCore.QString(str1)
            glsIndex=1
            length1=0
            glsIndex = str1.indexOf(InternalglsExpression)
            while(glsIndex>=0):
                length1 = InternalglsExpression.indexIn(str1, length1)
                length = InternalglsExpression.matchedLength()
                length1 += length
                matchedWord = InternalglsExpression.capturedTexts()[0]
                matchedlist.append(unicode(matchedWord))
                
                glsIndex = str1.indexOf(InternalglsExpression, glsIndex + length)
                
#        words = unit.source.split()
        for word in matchedlist:
            #strip punctuation and digits from word
#            word = str(word).translate(string.maketrans("",""), string.punctuation)            
            # if word is included in embedded file already
            if word.lower() in addedWords:
                continue
            
            candidates = matcher.matches(word)
            
            if candidates:
                addedWords.append(word.lower())
                glossaryTbx.addsourceunitWF(word)
                currentUnit = glossaryTbx.units[len(glossaryTbx.units) - 1]
                currentUnit.target = ','.join(candidate.target for candidate in filter(blankTargetFilter,candidates))
#              If this candidate has other info  
                if hasattr(candidates[0],'entryDefinition' ):
                    currentUnit.po2tbxmap(currentUnit, candidates[0])
                    currentUnit.setInfoInTbx(currentUnit)
                
    # Read the source file and find its root.
    sourceTree= etree.parse(sourceFile)
    root = sourceTree.getroot()
    fileNode = root[0]
    
    Mycounts[0] = matcher.countUnits
    Mycounts[1] = matcher.countFuzzy
    Mycounts[2] = len(addedWords)
    if len(addedWords) > 0: #if the glossary is not empty
        # If there is a header in the file, do not create one (we use rfind because the name of the tag
        # might include also the namespace). If there is no glossary tag in the header, create one
        # if there is no internal-file tag in the glossary, create one. 
        # If there is an XLIFF glossary already in the file (XLIFF tag inside internal-file, clear it (delete it)
        # because it will be later replace by the new one.
        if (fileNode[0].tag.rfind("header") == -1): # if the first entry does not have the tag header (no header)
            fileNode.insert(0, etree.Element("header"))
        header = fileNode[0]
        glossaryNode = None
        for aHeaderEntry in header:
            if (aHeaderEntry.tag.rfind("glossary") != -1):
                glossaryNode = aHeaderEntry
        if (glossaryNode == None):
            glossaryNode = etree.SubElement(header, "glossary")
        if len (glossaryNode) == 0: 
            internalFileNode = etree.SubElement(glossaryNode, "internal-file")
        else: 
            internalFileNode = glossaryNode[0]
        if (len(internalFileNode) > 0):
            internalFileNode.clear()
    
        # Open the target file that will contain the embedded glossary (could be same as source)
        fout = open(targetFile, 'wb')
        fout.writelines(str(glossaryTbx))
        fout.close()   
    
        # Parse the output file, and append to it the specific glossary that we have created
        glossaryTree= etree.parse(targetFile)
        internalFileNode.append(glossaryTree.getroot())
    
    # Write the target file.
    fout = open(targetFile, 'wb')
    fout.writelines(etree.tostring(sourceTree, pretty_print=True, xml_declaration=True, encoding='utf-8'))
    fout.close()

class OptionWithDefault(optparse.Option):
    ATTRS = optparse.Option.ATTRS + [strREQUIRED]
    
    def __init__(self, *opts, **attrs):
#        if World.WF_debug:
#            print "Enter __init__ in Class OptionWithDefault (Internal Glossary)"

        if attrs.get(strREQUIRED, False):
            attrs['help'] = '(Required) ' + attrs.get('help', "")
        optparse.Option.__init__(self, *opts, **attrs)

class OptionParser(optparse.OptionParser):
    def __init__(self, **kwargs):
        if World.WF_debug:
            print "Enter __init__ in Class OptionParser (Internal Glossary)"

        kwargs['option_class'] = OptionWithDefault
        optparse.OptionParser.__init__(self, **kwargs)
    
    def check_values(self, values, args):
        if World.WF_debug:
            print "Enter check values in Class OptionParser (Internal Glossary)"
        for option in self.option_list:
            if hasattr(option, strREQUIRED) and option.required:
                if not getattr(values, option.dest):
                    self.error("option %s is required" % (str(option)))
        return optparse.OptionParser.check_values(self, values, args)
            
def main(argv=None):
    if World.WF_debug:
        print "Enter main in Internal Glossary"

    if (sys.argv[0].endswith("py")):
        py = "python "
    else:
        py = ""
    
    version = WordForge. __version__
    usage = py + """%prog [-g|--glossary ] glossaryFile [-i|--input] inputFile [-o|--output] outputFile"""
    strPro = "%prog Version" + ' '  + version.ver
    strVersion = strPro + ' \n\
    Copyright (C) 2006-2007 The WordForge Foundation. www.wordforgefoundation.org\
    This is free software. You may redistribute copies of it under the terms of\
    the GNU General Public License <http://www.gnu.org/licenses/gpl.html>.\
    There is NO WARRANTY, to the extent permitted by law.\
    Written by Keo Sophon, Seth Chanratha.'
    
    parser = OptionParser(usage = usage, version = strVersion)
    parser.add_option("-g", "--glossary", required=True,  dest="glossaryFile",  metavar = "FILE", type="string",
            help="a glossary file to be searched through")
    parser.add_option("-i", "--input", required=True, dest="inputFile",  metavar = "FILE",  type="string",
            help="an xlf file name or a directory")
    parser.add_option("-o", "--output", dest="outputFile",   metavar = "FILE", action="store", type="string",
            help="an xlf file name or directory to store output after embedding process", default=None)
    
    options, args = parser.parse_args()
    
    if (options.glossaryFile and options.inputFile):
        embedGlossary(options.glossaryFile, options.inputFile, options.outputFile,  ycounts)

if __name__ == "__main__":
    if World.WF_debug:
        print "Enter __name == __main__ in Internal Glossary"
    main()


