#!/usr/bin/python
# -*- coding: utf-8 -*-
#Copyright (c) 2006 - 2007 by The WordForge Foundation
#                       www.wordforgefoundation.org
#  
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
# 
# See the LICENSE file for more details. 
#
# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)
#       Aditi Barua(aditi@ankur.org.bd)
#       Asheq Hamid(asheq@ankur.org.bd)
#       Md. Rezwan Shahid(rezwan@ankur.org.bd)
#       Goutom Roy(goutom@ankur.org.bd)
# This module is working on the main windows of Editor
import os.path
import sys
import os
from PyQt4 import QtCore, QtGui
from WordForge.Editor.ui.Ui_MainEditor import Ui_MainWindow
from WordForge.Editor.modules.TUview import TUview
from WordForge.Editor.modules.Overview import OverviewDock
from WordForge.Editor.modules.Comment import CommentDock
from WordForge.Editor.modules.Operator import Operator
from WordForge.Editor.modules.Catalog import Catalog
from WordForge.Editor.modules.FileAction import FileAction
from WordForge.Editor.modules.currentLanguage import curLanguage
from WordForge.Editor.modules.Find import Find
from WordForge.Editor.modules.BatchProcess import BatchProcess
import WordForge.Editor.modules.World as World
from WordForge.Editor.modules import TableTM
from WordForge.Editor.modules import TableGlossary
from WordForge.Editor.modules import SpellCheck
from WordForge import __version__
from WordForge.Editor.modules import ThirdLanguages
from WordForge.Editor.modules import ErrorChecking
from WordForge.Editor.modules.globalClass import globalClass
from collections import defaultdict
import WordForge.Editor.ui.images_rc
import string
#from threading import Thread

class MainWindow(QtGui.QMainWindow):
    """
    The main window which holds the toolviews.
    """
    def __init__(self, parent = None):
        if World.WF_debug:
            print "Enter __init__ in Class MainWindow"
               
        QtGui.QMainWindow.__init__(self, parent)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.createTestFiles()
        self.maximumSize()
        self.recentaction = []
        self.bookmarkaction = []
        self.setWindowTitle(World.settingApp + ' ' + __version__.ver)
        self.maxBookmark = 10
        self.delimiter = " : "
        self.currentFile=""
        self.hasThird = False
        self.createBookmarkAction()
        imagesdir = "../images"
        self.iconBookmark = QtGui.QIcon(":/bookmark.png")
        app = QtGui.QApplication.instance()
        self.connect(app, QtCore.SIGNAL("focusChanged(QWidget *,QWidget *)"), self.focusChanged)
        self.fileError=0
        self.hasPlural=False
        if World.WF_debug:
            print "  Last Geometry in Class MainWindow"
        
        # get the last geometry
        geometry = World.settings.value("lastGeometry")
        if (geometry.isValid()):
            self.setGeometry(geometry.toRect())

        sepAction = self.ui.menuWindow.actions()[0]

        if World.WF_debug:
            print "  Overview widget in Class MainWindow"
        
        #plug in overview widget.
        self.dockOverview = OverviewDock(self)
        self.addDockWidget(QtCore.Qt.TopDockWidgetArea, self.dockOverview)
        action = self.dockOverview.toggleViewAction()
        action.setText("&Overview")
        action.setStatusTip("Toggle the Overview window")
        action.setWhatsThis("<h3>Toggle the Overview window</h3>If the Overview window is hidden then display it. If it is displayed then close it.")
        self.ui.menuWindow.insertAction(sepAction, action)
        
#        self.connect(self.ui.actionUndoall , QtCore.SIGNAL("triggered()"), self.undoing)
        
        if World.WF_debug:
            print "  TUview widget in Class MainWindow"
            
        #plug in TUview widget
        self.getPreferenceValue()
        self.dockTUview = TUview(self)
        self.setCentralWidget(self.dockTUview)
        action = self.dockTUview.toggleViewAction()
        action.setText("&Detail")
        action.setStatusTip("Toggle the Detail window")
        action.setWhatsThis("<h3>Toggle the Detail window</h3>If the Detail window is hidden then display it. If it is displayed then close it.")
        self.ui.menuWindow.insertAction(sepAction, action)

        if World.WF_debug:
            print "  Glossary widget in Class MainWindow"

        #Plug in glossary widget
        self.tableGlossary = TableGlossary.TableGlossary(self)
        self.addDockWidget(QtCore.Qt.RightDockWidgetArea, self.tableGlossary)
        action = self.tableGlossary.toggleViewAction()
        action.setText("&Glossary's Suggestions")
        action.setStatusTip("Toggle the Glossary window")
        action.setWhatsThis("<h3>Toggle the Glossary window</h3>If the Glossary window is hidden then display it. If it is displayed then close it.")
        self.ui.menuWindow.insertAction(sepAction, action)

        if World.WF_debug:
            print "  Comment widget in Class MainWindow"
        
        #plug in comment widget
        self.dockComment = CommentDock(self)
        self.addDockWidget(QtCore.Qt.TopDockWidgetArea, self.dockComment)
        action = self.dockComment.toggleViewAction()
        action.setText("&Comment")
        action.setStatusTip("Toggle the Comment window")
        action.setWhatsThis("<h3>Toggle the Comment window</h3>If the Comment window is hidden then display it. If it is displayed then close it.")
        self.ui.menuWindow.insertAction(sepAction, action)

                #plug in error checking widget
        self.errorTable = ErrorChecking.ErrorChecking(self)
        self.addDockWidget(QtCore.Qt.BottomDockWidgetArea, self.errorTable)
        action = self.errorTable.toggleViewAction()
        action.setText("&Error Window")
        action.setStatusTip("Toggle the Error window")
        action.setWhatsThis("<h3>Toggle the Error window</h3>If the Error window is hidden then display it. If it is displayed then close it.")
        self.ui.menuWindow.insertAction(sepAction, action)
        
        if World.WF_debug:
            print "  Statusbar widgets in Class MainWindow"

        self.statuslabel = QtGui.QLabel()
        self.statuslabel.setFrameStyle(QtGui.QFrame.NoFrame)
        self.statuslabel.setMargin(1)
        self.ui.statusbar.addWidget(self.statuslabel)
        self.ui.statusbar.setWhatsThis("<h3>Status Bar</h3>Shows the progress of the translation in the file and messages about the current state of the application.")

        if World.WF_debug:
            print "  Actions for toolbars in Class MainWindow"

        #add action from each toolbar toggleviewaction to toolbars submenu of view menu
        self.ui.menuToolbars.addAction(self.ui.toolFile.toggleViewAction())
        self.ui.menuToolbars.addAction(self.ui.toolEdit.toggleViewAction())
        self.ui.menuToolbars.addAction(self.ui.toolNavigation.toggleViewAction())
        self.ui.menuToolbars.addAction(self.ui.toolFilter.toggleViewAction())
        self.ui.menuToolbars.addAction(self.ui.toolBookmark.toggleViewAction())
        self.ui.menuToolbars.addAction(self.ui.toolTools.toggleViewAction())
        #create operator
        self.operator = Operator(self)  

        self.connect(self.ui.actionCatalogManager, QtCore.SIGNAL("triggered()"), self.showCatalogManager)

        if World.WF_debug:
            print "  TM Table in Class MainWindow"
        
        # TM table
        self.table = TableTM.TableTM(self)
        self.addDockWidget(QtCore.Qt.RightDockWidgetArea, self.table)
        action = self.table.toggleViewAction()
        action.setText("&Translation Memory's Suggestions")
        action.setStatusTip("Toggle the Translation Memory window")
        action.setWhatsThis("<h3>Toggle the Translation Memory window</h3>If the Translation Memory window is hidden then display it. If it is displayed then close it.")
        self.ui.menuWindow.insertAction(sepAction, action)
        
        self.connect(self.operator, QtCore.SIGNAL("requestTargetChanged"), self.dockTUview.emitTargetChanged)

        if World.WF_debug:
            print "  TM in Class MainWindow"
        
        # translation memory
        self.connect(self.dockOverview, QtCore.SIGNAL("requestUnit"), self.operator.emitRequestUnit)
        self.connect(self.operator, QtCore.SIGNAL("tmRequest"), self.dockOverview.popupTranslation)
#        self.connect(self.operator, QtCore.SIGNAL("tmCandidates"), self.table.fillTable)
#        self.connect(self.table, QtCore.SIGNAL("translation2target"), self.dockTUview.translation2target)
        self.connect(self.table, QtCore.SIGNAL("visible"), self.operator.setIsTMTableVisible)
#        self.connect(self.table, QtCore.SIGNAL("addAsAltTrans"), self.operator.addAsAltTrans)
#        self.connect(self.table, QtCore.SIGNAL("RemoveAltTran"), self.operator.removeAltTran)
        
        if World.WF_debug:
            print "  Glossary in Class MainWindow"

        # glossary
        self.connect(self.dockTUview, QtCore.SIGNAL("lookupTerm"), self.operator.emitTermRequest)
        self.connect(self.operator, QtCore.SIGNAL("termRequest"), self.dockTUview.popupTerm)
        self.connect(self.operator, QtCore.SIGNAL("termCandidates"), self.tableGlossary.fillTable)
        self.connect(self.tableGlossary, QtCore.SIGNAL("closed"), self.toggleTUviewTermSig)
        self.connect(self.tableGlossary, QtCore.SIGNAL("shown"), self.toggleTUviewTermSig)
        self.connect(self.dockTUview, QtCore.SIGNAL("clearTableGlossary"), self.tableGlossary.viewSetting)

        if World.WF_debug:
            print "  Converter in Class MainWindow"
        #Converter
        self.connect(self.ui.actionConverter, QtCore.SIGNAL("triggered()"), self.showConverterDialog)
        self.connect(self.ui.actionXLIFF_Upgrade, QtCore.SIGNAL("triggered()"), self.showXliffUpgradeDialog)

        if World.WF_debug:
            print "  Third Language in Class MainWindow"
            
        #Third Language
        self.connect(self.ui.actionThird_Language, QtCore.SIGNAL("triggered()"), self.showThirdLanguageDialog)  
        #Error Checking
        
        self.curval = ''
        self.lastval = ''
        self.connect(self.ui.actionError_Checking, QtCore.SIGNAL("triggered()"), self.showHideErrorDoc)
        if World.WF_debug:
            print "  Help menu of AboutQt in Class MainWindow"
        
        #preference
        from WordForge.Editor.modules.Preference import Preference
        self.preference = Preference(self)
            
        #Help menu of aboutQt
        action = QtGui.QWhatsThis.createAction(self)
        action.setText(self.tr("&What's This?"))
        action.setStatusTip("Context sensitive help")
        action.setWhatsThis("<h3>Display context sensitive help</h3>In what's This? mode, the mouse cursor shows an arrow with a question mark, and you can click on the interface elements to get a short description of what they do and how to use them in a dialog, the feature can be accessed using the context help button in the titlebar.")
        self.ui.menuHelp.insertAction(self.ui.actionAbout , action)
        self.ui.menuHelp.insertSeparator(self.ui.actionAbout)
        self.connect(self.ui.actionAbout, QtCore.SIGNAL("triggered()"), self.showAboutDialog)
        self.connect(self.ui.actionAboutQT, QtCore.SIGNAL("triggered()"), QtGui.qApp, QtCore.SLOT("aboutQt()"))

        if World.WF_debug:
            print "  File actions objects in Class MainWindow"
        
        # create file action object and file action menu related signals
        self.fileaction = FileAction(self)
        self.connect(self.ui.actionOpen, QtCore.SIGNAL("triggered()"), self.fileaction.openFile)
        self.connect(self.ui.action_Close, QtCore.SIGNAL("triggered()"), self.closeFile)
        self.connect(self.ui.actionSave, QtCore.SIGNAL("triggered()"), self.fileaction.save)
        self.connect(self.ui.actionSave, QtCore.SIGNAL("triggered(bool)"), self.ui.actionSave.setEnabled)
        self.connect(self.ui.actionSaveas, QtCore.SIGNAL("triggered()"), self.fileaction.saveAs)
        
        self.connect(self.operator, QtCore.SIGNAL("saveAs"), self.fileaction.saveAs)
        self.connect(self.ui.actionExit, QtCore.SIGNAL("triggered()"), QtCore.SLOT("close()"))
        self.createRecentAction()

        if World.WF_debug:
            print "  Create Find Widget in Class MainWindow"
            

        # create Find widget and connect signals related to it
        self.findBar = Find(self)
        self.addDockWidget(QtCore.Qt.BottomDockWidgetArea, self.findBar)
        self.connect(self.ui.actionFind, QtCore.SIGNAL("triggered()"), self.findBar.showFind)
        
#         goutom 
        self.connect(self.ui.actionFindNext, QtCore.SIGNAL("triggered()"), self.findBar.findNext)
        self.connect(self.ui.actionFindPrevious, QtCore.SIGNAL("triggered()"), self.findBar.findPrevious)
#        goutom 
        
        self.connect(self.ui.actionFindNext, QtCore.SIGNAL("triggered()"), self.searchNext)
        self.connect(self.ui.actionFindPrevious, QtCore.SIGNAL("triggered()"), self.searchPrevious)
        self.connect(self.ui.actionReplace, QtCore.SIGNAL("triggered()"), self.findBar.showReplace)
        self.connect(self.findBar, QtCore.SIGNAL("initSearch"), self.operator.initSearch)
        self.connect(self.findBar, QtCore.SIGNAL("searchNext"), self.searchNext)
        self.connect(self.findBar, QtCore.SIGNAL("searchPrevious"), self.searchPrevious)
        self.connect(self.table, QtCore.SIGNAL("findUnit"), self.operator.slotFindUnit)
        self.connect(self.operator, QtCore.SIGNAL("searchStatus"), self.findBar.setColorStatus)
        self.connect(self.operator, QtCore.SIGNAL("searchStatus"), self.displayTempMessage)
        
        self.connect(self.findBar, QtCore.SIGNAL("replace"), self.operator.replace)
        self.connect(self.findBar, QtCore.SIGNAL("replaceAll"), self.operator.replaceAll)
        
        self.connect(self.operator, QtCore.SIGNAL("glossaryPattern"), self.dockTUview.setPattern)
        self.connect(self.operator, QtCore.SIGNAL("highlightglossary"), self.dockTUview.setHighlightGlossary)
        self.connect(self.operator, QtCore.SIGNAL("searchResult"), self.dockTUview.setSearchString)
        self.connect(self.operator, QtCore.SIGNAL("searchResult"), self.dockComment.setSearchString)
        # "replaceText" sends text field, start, length, and text to replace.
        self.connect(self.operator, QtCore.SIGNAL("replaceText"), self.dockTUview.replaceText)
        self.connect(self.operator, QtCore.SIGNAL("replaceText"), self.dockComment.replaceText)

        if World.WF_debug:
            print "  Goto menu action in Class MainWindow"
       
        # Goto menu action
        self.iconGoto = QtGui.QIcon(":/goto.png")
        self.ui.actionGoTo.setIcon(self.iconGoto)
        self.connect(self.ui.actionGoTo, QtCore.SIGNAL("triggered()"), self.showGoto)

        if World.WF_debug:
            print "  Bookmarks menu action in Class MainWindow"
        
        # Bookmarks menu action
        self.connect(self.ui.actionAddBookmarks, QtCore.SIGNAL("triggered()"), self.toggleBookmark)
        self.connect(self.ui.actionNextBookmark, QtCore.SIGNAL("triggered()"), self.nextBookmark)
        self.connect(self.ui.actionPreviousBookmark, QtCore.SIGNAL("triggered()"), self.previousBookmark)
        self.connect(self.ui.actionClearBookmarks, QtCore.SIGNAL("triggered()"), self.clearBookmarks)
      
        self.connect(self.ui.actionPreferences, QtCore.SIGNAL("triggered()"), self.showPreferenceDialog)

        if World.WF_debug:
            print "  Get signal from TM in Class MainWindow"

        # get signal from TM
        self.connect(self.operator, QtCore.SIGNAL("ShowPreference"), self.showPreferenceDialog)
        

        if World.WF_debug:
            print "  Lookuptext and Instant TM in Class MainWindow"
        
        # action lookup text and auto translation from TM
        self.connect(self.ui.actionAuto_translate, QtCore.SIGNAL("triggered()"), self.operator.autoTranslate)
        
        if World.WF_debug:
            print "  Show TM dialog in Class MainWindow"
        
        # action show TM dialog
        self.connect(self.ui.actionManageTM, QtCore.SIGNAL("triggered()"), self.showTmsettingDialog)

        if World.WF_debug:
            print "  Show glossary dialog in Class MainWindow"
        
        # action show glossary dialog
        self.connect(self.ui.actionManageGlossary, QtCore.SIGNAL("triggered()"), self.showGlossaryDialog)
        self.connect(self.ui.actionEdit_Header, QtCore.SIGNAL("triggered()"), self.showHeaderDialog)
        
        #Batch Process
        self.connect(self.ui.actionBatch_Process, QtCore.SIGNAL("triggered()"), self.showBatchProcessDialog)
        if World.WF_debug:
            print "Batch Processing Class MainWindow"

        # Other actions
        self.connect(self.ui.actionFirst, QtCore.SIGNAL("triggered()"), self.dockOverview.scrollFirst)
        self.connect(self.ui.actionPrevious, QtCore.SIGNAL("triggered()"), self.dockOverview.scrollPrevious)
        self.connect(self.ui.actionNext, QtCore.SIGNAL("triggered()"), self.dockOverview.scrollNext)
        self.connect(self.ui.actionLast, QtCore.SIGNAL("triggered()"), self.dockOverview.scrollLast)
        self.connect(self.ui.actionCopySource2Target, QtCore.SIGNAL("triggered()"), self.dockTUview.source2target)

#        self.connect(self.ui.actionEraseTarget, QtCore.SIGNAL("triggered()"), self.dockTUview.eraseTarget)

#        self.connect(self.ui.actionCopySearchResult2Target, QtCore.SIGNAL("triggered()"), self.table.emitTarget)

        # action filter menu
        # there is connecting signal to dock.overview.updateText, cause incorrectly filter fuzzy
        self.connect(self.ui.actionFilterFuzzy, QtCore.SIGNAL("toggled(bool)"), self.filterFuzzy)
        self.connect(self.ui.actionFilterTranslated, QtCore.SIGNAL("toggled(bool)"), self.operator.filterTranslated)
        self.connect(self.ui.actionFilterUntranslated, QtCore.SIGNAL("toggled(bool)"), self.operator.filterUntranslated)
        self.connect(self.ui.actionToggleFuzzy, QtCore.SIGNAL("triggered()"), self.operator.toggleFuzzy)
        self.connect(self.ui.actionClearFuzzy, QtCore.SIGNAL("triggered()"), self.operator.clearFuzzies)
        self.connect(self.ui.actionErasetarget, QtCore.SIGNAL("triggered()"), self.dockOverview.eraseTarget)
    
        # "currentUnit" sends currentUnit, currentIndex
        self.connect(self.operator, QtCore.SIGNAL("currentUnit"), self.dockOverview.updateView)
        self.connect(self.operator, QtCore.SIGNAL("currentUnit"), self.dockTUview.updateView)
        self.connect(self.operator, QtCore.SIGNAL("currentUnit"), self.dockComment.updateView)
        self.connect(self.operator, QtCore.SIGNAL("currentUnit"), self.table.viewSetting)
        
#        self.connect(self.operator, QtCore.SIGNAL("currentUnit"), self.addFuzzyIcon)
        self.connect(self.dockOverview, QtCore.SIGNAL("filteredIndex"), self.operator.setUnitFromPosition)
        self.connect(self.dockTUview, QtCore.SIGNAL("scrollToRow"), self.dockOverview.scrollToRow)
        
        self.connect(self.dockOverview, QtCore.SIGNAL("targetChanged"), self.operator.setTarget)
        self.connect(self.dockTUview, QtCore.SIGNAL("targetChanged"), self.operator.setTarget)
        self.connect(self.dockTUview, QtCore.SIGNAL("textChanged"), self.dockOverview.updateText)
#        self.connect(self.dockTUview, QtCore.SIGNAL("textChanged"), self.dockOverview.rowtext)
        
        self.connect(self.dockComment, QtCore.SIGNAL("textChanged"), self.dockOverview.markComment)
        self.connect(self.dockComment, QtCore.SIGNAL("commentChanged"), self.operator.setComment)
        self.connect(self.fileaction, QtCore.SIGNAL("save"), self.dockTUview.emitTargetChanged)
        self.connect(self.fileaction, QtCore.SIGNAL("save"), self.dockComment.emitCommentChanged)
        self.connect(self.fileaction, QtCore.SIGNAL("fileToSave"), self.operator.saveStoreToFile)
        self.connect(self.dockTUview, QtCore.SIGNAL("contentChanged"), self.operator.contentChanged)
        self.connect(self.dockComment, QtCore.SIGNAL("contentChanged"), self.operator.contentChanged)
        self.connect(self.operator, QtCore.SIGNAL("contentChanged"), self.ui.actionSave.setEnabled)
        
        # get "returnFilename" signal from operator of saveStoreToFile function
#        self.connect(self.operator, QtCore.SIGNAL("returnFilename"), self.updateStatistic)

        self.connect(self.operator, QtCore.SIGNAL("returnFilename"), self.setTitle)
        self.connect(self.dockOverview, QtCore.SIGNAL("toggleFirstLastUnit"), self.toggleFirstLastUnit)

        self.connect(self.operator, QtCore.SIGNAL("newUnits"), self.dockOverview.slotNewUnits)
        self.connect(self.operator, QtCore.SIGNAL("newUnits"), self.dockTUview.viewSetting)
        self.connect(self.operator, QtCore.SIGNAL("newUnits"), self.dockComment.viewSetting)
        # "filterChanged" sends filter and number of filtered items.
        self.connect(self.operator, QtCore.SIGNAL("filterChanged"), self.dockOverview.filterChanged)
        self.connect(self.operator, QtCore.SIGNAL("filterChanged"), self.dockTUview.filterChanged)

        # set file status information to text label of status bar.
        self.connect(self.operator, QtCore.SIGNAL("unitsStatus"), self.setStatus)
        self.connect(self.fileaction, QtCore.SIGNAL("fileToOpen"), self.openFile)

        if World.WF_debug:
            print "  Progress bar in Class MainWindow"

        # progress bar
        self.progressBar = QtGui.QProgressBar()
        self.progressBar.setEnabled(True)
        self.progressBar.setProperty("value",QtCore.QVariant(0))
        self.progressBar.setOrientation(QtCore.Qt.Horizontal)
        self.progressBar.setObjectName("progressBar")
        self.progressBar.setVisible(False)
        self.ui.statusbar.addPermanentWidget(self.progressBar)
        self.connect(self.dockOverview, QtCore.SIGNAL("progressBarValue"), self.updateProgress)
        self.connect(self.operator, QtCore.SIGNAL("progressBarValue"), self.updateProgress)

        if World.WF_debug:
            print "  Get status in Class MainWindow"

        # get the last state of mainwindows's toolbars and dockwidgets
        state = World.settings.value("MainWindowState")
        # check if the last state is valid it will restore
        if (state.isValid()):
            self.restoreState(state.toByteArray(), 1)
        
        tuViewHidden = World.settings.value("TuViewHidden", QtCore.QVariant(False))
        self.dockTUview.setHidden(tuViewHidden.toBool())
        self.findBar.setHidden(True)
      
        self.connect(self.table, QtCore.SIGNAL("openFile"), self.openFile)
        self.connect(self.table, QtCore.SIGNAL("goto"), self.dockOverview.gotoRow)
        
        self.operator.applySettings()
        catalogProject = World.settings.value("CurrentProject").toString()

        if World.WF_debug:
            print "  Action spellchecker in Class MainWindow"
        
        # action spellchecker
        self.ui.actionAutoSpellCheck.setChecked(World.settings.value("AutoSpellCheck",  QtCore.QVariant(False)).toBool())
        if self.ui.actionAutoSpellCheck.isChecked():
            self.operator.autoSpellCheck(True)
        self.spellcheck = SpellCheck.SpellCheck(self)
        self.connect(self.ui.action_SpellTrans, QtCore.SIGNAL("triggered()"), self.spellcheck.showDialog)
        # emit target to update unit before starting spellcheck
        self.connect(self.spellcheck, QtCore.SIGNAL("spellChecked"), self.dockTUview.emitTargetChanged)
        self.connect(self.spellcheck,  QtCore.SIGNAL("spellChecked"),  self.operator.spellTrans)
        self.connect(self.ui.actionAutoSpellCheck, QtCore.SIGNAL("toggled(bool)"), self.dockTUview.emitTargetChanged)
        self.connect(self.ui.actionAutoSpellCheck, QtCore.SIGNAL("toggled(bool)"), self.dockTUview.stopHighlighting)
        self.connect(self.ui.actionAutoSpellCheck, QtCore.SIGNAL("toggled(bool)"), self.operator.autoSpellCheck)
    
        self.connect(self.operator, QtCore.SIGNAL("spellCheckMaximum"), self.spellcheck.setMaximum)
        
        self.connect(self.operator, QtCore.SIGNAL("missNsuggestedList"), self.dockTUview.setMissSpelledList)
        self.connect(self.operator, QtCore.SIGNAL("missNsuggestedList"), self.spellcheck.initializeFillingResult)
        self.connect(self.operator, QtCore.SIGNAL("spellCheckCompleted"), self.spellcheck.spellCheckCompleted)
        self.connect(self.spellcheck,  QtCore.SIGNAL("changeOnce"),  self.operator.replaceSubString)
        # when cancelling spellcheck, set contentdirty in TUview to false and then undo replacing wrong words
        self.connect(self.spellcheck,  QtCore.SIGNAL("cancelSpellCheck"),  self.dockTUview.setContentDirty)
        self.connect(self.spellcheck,  QtCore.SIGNAL("cancelSpellCheck"),  self.operator.replaceListOfSubStrings)
        
        self.connect(self.dockTUview,  QtCore.SIGNAL("breakerEncountered"),  self.operator.checkOnTheFly)
        self.connect(self.spellcheck,  QtCore.SIGNAL("highlightAnWrongWord"),  self.dockTUview.highlightAnWrongWord)
        self.connect(self.operator, QtCore.SIGNAL("spellCheckCompleted"), self.dockTUview.clearHighlightWrongWord)
        self.connect(self.spellcheck, QtCore.SIGNAL("spellCheckCompleted"), self.dockTUview.clearHighlightWrongWord)
        
        self.connect(self.ui.actionOpen_in_New_Window, QtCore.SIGNAL("triggered()"), self.openNewWindow)
        
        
        self.connect(self.ui.actionClean_an_XLIFF_file_2, QtCore.SIGNAL("triggered()"), self.showCleanXLIFFDialog)
        #action errorChecking
        self.connect(self.ui.actionError_Checking, QtCore.SIGNAL("toggled(bool)"), self.operator.errorChecking)
        self.ui.actionError_Checking.setChecked(World.settings.value("errorChecking",  QtCore.QVariant(False)).toBool())
        self.connect(self.ui.actionSearchErrors, QtCore.SIGNAL("toggled(bool)"), self.checkingErrorFile)
        
        self.connect(self.ui.actionChangeFileLanguage, QtCore.SIGNAL("triggered()"), self.changeFileLanguage)
       
        #action showing invisible character
        self.connect(self.ui.actionShow_Nonprinting_Char, QtCore.SIGNAL("toggled(bool)"), self.dockOverview.shownonprintChar)
        
        self.applySettings()

        if World.WF_debug:
            print "  Initialize Ref language windows in Class MainWindow"
        
        # Initialise the Reference Language window when we start the editor
        self.dockThirdLanguage = ThirdLanguages.ThirdLanguages(self)
        self.dockThirdLanguage.setWindowTitle("Reference Languages")
        self.dockThirdLanguage.setVisible(False)
        action = self.dockThirdLanguage.toggleViewAction()
        self.addDockWidget(QtCore.Qt.RightDockWidgetArea, self.dockThirdLanguage)
        action.setText("&Reference Languages")
        action.setStatusTip("Toggle the Reference Languages window")
        action.setWhatsThis("<h3>Toggle the Third Languages window</h3>If the Third Languages window is hidden then display it. If it is displayed then close it.")
        self.ui.menuWindow.insertAction(sepAction, action)
        self.connect(self.operator, QtCore.SIGNAL("currentUnit"), self.dockThirdLanguage.updateView)

        if World.WF_debug:
            print "  Initialize TM info when start editor in Class MainWindow"
        
        # Initialise Translation Memory information when we start the editor
#        self.connect(self.ui.actionSearch_Suggestions,  QtCore.SIGNAL("triggered()"),  self.operator.emitTMCandidates)
        self.connect(self.ui.actionSearch_Suggestions,  QtCore.SIGNAL("triggered()"),  self.operator.ApplyTMToCurrentMsg)

        if World.WF_debug:
            print "  Initialize Glossary options when start editor in Class MainWindow"
                
        # Initialise Glossary options when we start the editor
#        self.connect(self.operator,  QtCore.SIGNAL("EmbedInternalGlossary"), self.ui.actionEmbedGlossary.setEnabled)
#        self.connect(self.ui.actionEmbedGlossary,  QtCore.SIGNAL("triggered()"),  self.operator.embedGlossary)
        self.connect(self.ui.actionEmbedGlossary,  QtCore.SIGNAL("triggered()"),  self.showEmbedGlossaryDialog)
        
#        self.connect(self.operator, QtCore.SIGNAL("setExportTM"), self.ui.actionExport_Translation_Memory.setEnabled)
        
        self.seEnabledExportTMGlossssary()
        
        self.connect(self.ui.actionExport_Translation_Memory, QtCore.SIGNAL("triggered()"), self.showExportTM2TMX)
        self.connect(self.ui.actionExport_Glossary, QtCore.SIGNAL("triggered()"), self.showExport2TBX)

        if World.WF_debug:
            print "Leave __init__ in Class MainWindow"
        self.Catalog = Catalog(self)    
        self.connect(self.Catalog, QtCore.SIGNAL("showPreferenceError"), self.showPreferenceDialog)    
        
        self.getPreferenceValue()

#        this list contains all the mainEditor type objects that has been created using new window option.
        self.newWindowList = []

#    current language     
       # self.curLanguage =curLanguage(self)

    def createTestFiles(self):
        # This function tests whether the test_file directory exists in the wordforge config folder. If it is not in there then creates the folder and copy the test_files from installation directory into this folder.
        if (os.name == 'posix') :
            localpath = os.path.split(os.path.realpath( __file__ ))[0] + os.path.sep
        else:
            localpath = ""
#        targetDir = os.path.join(os.path.expanduser("~"), World.logDir, 'test_files')        
        targetDir = os.path.join(World.configDir, 'test_files')        
        if os.path.isdir(targetDir):
            return
        else:
            if (os.name == 'posix') :
                sourceDir = os.path.abspath(localpath + '..' + os.path.sep + 'test_files')
            else:
                sourceDir = os.path.abspath(os.path.dirname(sys.argv[0]) +os.path.sep   + '..' + os.path.sep + 'test_files')
            #create the wordforge directory.
            import distutils.dir_util  
#            sourceDir = os.path.abspath(os.path.dirname(sys.argv[0]) +os.path.sep   + '..' + os.path.sep + 'test_files')
#            sourceDir = os.path.abspath(localpath + '..' + os.path.sep + 'test_files')
            distutils.dir_util.copy_tree(sourceDir, targetDir)
     

    def checkingErrorFile(self, bool, store=None):
        #search for errors in currently opened file or opened project
        if not bool:
            return 
        else:
            units=None
            self.getPreferenceValue()
            if self.ui.actionSearchErrors.isChecked():
                globalClass.searchProjectError=False
                units=self.dockOverview.units
                
            elif store:
                globalClass.fileErrorList=[]
                globalClass.searchProjectError=True
                units=store.units 
            if units: 
                self.fileError =0
                index=-1
                OverviewRow=-1
                prevSource=''
                prevTarget=''
                globalClass.postTestList=[]
                self.checkPlural=False
                for unit in units:
                    if unit.source=='':
                        continue
                    index=(index+1) 
                    if unit.hasplural():
                        self.hasPlural=True
                        self.pluralErrorList=[]
                        self.markError=0
                        index=(index+1) 
                        self.sourceStrings = unit.source.strings
                        self.targetStrings = unit.target.strings
                        for i in range(len(unit.source.strings)):
                            if prevSource or prevTarget:
                                
                                self.errorTable.postCompute(prevSource,prevTarget, str(index), OverviewRow)
                            self.errorTable.preCompute(self.sourceStrings[i])
                            prevSource=self.sourceStrings[i]
                            prevTarget=self.targetStrings[i]
                            OverviewRow=OverviewRow+1
                            globalClass.unitid = unit.getid()
                          
                        self.errorTable.postCompute(prevSource,prevTarget, str(index), OverviewRow)
                        prevSource=''
                        prevTarget=''
                        index=(index-1)  
                        self.hasPlural=False
                        self.pluralErrorList=[]
                        self.markError=0
                        self.checkPlural=False
                    else: 
                        globalClass.glsErrorDict = globalClass.tempGlsErrorDict
                        globalClass.tempGlsErrorDict = defaultdict(dict)
                        if prevSource or prevTarget:
                            self.errorTable.postCompute(prevSource,prevTarget, str(index), OverviewRow)
                        self.errorTable.preCompute(unit.source)
                        OverviewRow=OverviewRow+1
                        globalClass.unitid = unit.getid()
                        prevSource=unit.source
                        prevTarget=unit.target        
                globalClass.glsErrorDict = globalClass.tempGlsErrorDict
                globalClass.tempGlsErrorDict = defaultdict(dict)        
                self.errorTable.postCompute(prevSource,prevTarget, str(index+1), OverviewRow)    
            self.ui.actionSearchErrors.setChecked(False)
            globalClass.preComputeFlag = 1    
            globalClass.postTestList=[]
            storingEnable=World.settings.value("enableStore",  QtCore.QVariant(True)).toBool()
            globalClass.hasErrors=2
            if storingEnable:
                self.ui.actionSave.setEnabled(True)
                self.fileaction.isSaved=False
                self.operator.modified=True
# goutom code start
        
    def undoing(self):
        lstlen = int(len(self.dockOverview.indexlist))
        if lstlen:
            lainlst= self.dockOverview.indexlist[lstlen-1]
            if not self.dockOverview.undo_dic[lainlst]:
                self.dockOverview.indexlist.pop()
                lstlen = int(len(self.dockOverview.indexlist))
                if lstlen:
                    lainlst= self.dockOverview.indexlist[lstlen-1]
                    self.dockOverview.scrollToRow(lainlst)
                    self.undoing()
            else:
                self.undo_processing(lainlst)
    
    def undo_processing(self,  lainlst):
        txt=self.dockOverview.undo_dic[lainlst].pop()
        self.dockOverview.prevtext=str(txt)
        self.dockTUview.setTargetText(txt)
        
    #goutom code end
    
    def showHideErrorDoc(self):
        if self.ui.actionError_Checking.isChecked():
#            self.errorTable.show()
            globalClass.errorCheckEnableFlag = 1
            self.getPreferenceValue()

        else:
#            self.errorTable.hide()
            globalClass.errorCheckEnableFlag = 0

 
    def updateProgress(self, value):
        if (not self.progressBar.isVisible()):
            self.progressBar.setVisible(True)
        self.progressBar.setValue(value)
        if (value == 100):
            self.progressBar.setVisible(False)

    def getPreferenceValue(self):
#it gets the errorchecking flags and options information from  preferences-> errorchecking for the language of the currently opened file.
        if (not hasattr(self, "Preference") ):
            from WordForge.Editor.modules.Preference import Preference
            self.preference = Preference(self)
        homeDir = os.path.expanduser('~')
#        defaultErrorCheckingFilePath = homeDir + os.path.sep + '.wordforge' + os.path.sep + 'test_files' + os.path.sep + 'bn_BD_tests.xml'    
        defaultErrorCheckingFilePath = World.configDir + os.path.sep + 'test_files' + os.path.sep + 'bn_BD_tests.xml'    

        if (os.name == 'posix') :
            localpath = os.path.split(os.path.realpath( __file__ ))[0] +os.path.sep
        else:
            localpath = "" 
#        activeErrorCheckingFilePath=World.settings.value("activeErrorCheckingFilePath", QtCore.QVariant(os.path.abspath(localpath + '..' + os.path.sep + 'test_files' + os.path.sep + "bn_BD_tests.xml"))).toString() 
        activeErrorCheckingFilePath=World.settings.value("activeErrorCheckingFilePath", QtCore.QVariant(os.path.abspath(World.configDir + os.path.sep + 'test_files' + os.path.sep + "bn_BD_tests.xml"))).toString() 
        tagValueList=[]
        tagValueList=self.preference.load_file(activeErrorCheckingFilePath)
        try:
            self.preference.loadGlobalPreferenceValue(tagValueList)
        except:
            pass

    def enableCopyCut(self, bool):
        self.ui.actionCopy.setEnabled(bool)
        self.ui.actionCut.setEnabled(bool)

    def enableUndo(self, bool):
        self.ui.actionUndo.setEnabled(bool)

    def enableRedo(self, bool):
        self.ui.actionRedo.setEnabled(bool)

    def showGoto(self):
        status = self.operator.getStatusObject()
        value, ok = QtGui.QInputDialog.getInteger(self, self.tr("Go"), self.tr("Go to string number:"), 1, 1, status.getTotalOfStrings())
        if ok:
            self.dockOverview.gotoRow(value)

    def autoSave(self):
        # call save function in fileaction file
        if (self.ui.actionSave.isEnabled()):
            if (self.autoSaveInterval <= 1):
                minute=" minutes)."
            else:
                minute=" minute)."
            self.displayTempMessage(self.tr("Autosave... (your file is saved every " + str(((self.autoSaveInterval / 1000))/60) + minute))
            self.fileaction.save()
            self.ui.actionSave.setEnabled(True) 
            self.fileaction.isSaved=False

    def bookmarkValue(self, unit):
        # return unit source with current index of overview.
#        unit = self.operator.getCurrentUnit()
        id = self.dockOverview.getCurrentIndex()
        self.bookmarkString = str(id) + World.delimiter + unit.source[:15] + "..."
        return self.bookmarkString

    def toggleBookmark(self):
        unit = self.operator.getCurrentUnit()
        bookmark = World.settings.value("bookmarkList").toStringList()
        bookmarkString = self.bookmarkValue(unit)
        #get current item
        currentRow = self.dockOverview.ui.tableOverview.currentRow()
        curritem = self.dockOverview.ui.tableOverview.item(currentRow, 0)
        #set bookmark
        commenttext = unicode(self.dockComment.ui.txtTranslatorComment.toPlainText())
        errornote= unit.getnotes()
        if not(bookmark.__contains__(bookmarkString)):
            if self.operator.isXliff:
                unit.markbookmarked(True) 
            self.setBookmarks(unit)
            self.ui.actionNextBookmark.setEnabled(True)
            self.ui.actionPreviousBookmark.setEnabled(True)
            self.ui.actionClearBookmarks.setEnabled(True)
            # set icon for current row of bookmark
            if commenttext or errornote:
                curritem.setIcon(self.dockOverview.iconnoteBookmark)
            else:
                curritem.setIcon(self.iconBookmark)
        # clear bookmark
        else:
            if self.operator.isXliff:
                unit.markbookmarked(False)
            blankIcon = QtGui.QIcon()
            if commenttext or errornote:
                curritem.setIcon(self.dockOverview.noteIcon)
            else: 
                curritem.setIcon(blankIcon)  
            index = bookmark.indexOf(bookmarkString)
            bookmark.removeAt(index)
            self.bookmarkaction[index].setVisible(False)
            World.settings.setValue("bookmarkList", QtCore.QVariant(bookmark))
            bookmarks = World.settings.value("bookmarkList").toStringList()

            if (len(bookmarks)== 0):
                self.ui.actionPreviousBookmark.setEnabled(False)
                self.ui.actionNextBookmark.setEnabled(False)
        self.ui.actionSave.setEnabled(True)
        self.fileaction.isSaved=False
        self.operator.modified=True        

    def setBookmarks(self, unit):
        #set bookmark in the row of the overview 
        bookmarkedUnit=unit
        bookmarkString = self.bookmarkValue(bookmarkedUnit)
        bookmarkDic = {}
        bookmarkList = []
        bookmarks = World.settings.value("bookmarkList").toStringList()
        bookmarks.removeAll(bookmarkString)
        bookmarks.prepend(bookmarkString)
        for bookmark in bookmarks:
            bookmarkStNum = int(str(bookmark).split(World.delimiter)[0])
            bookmarkSt = World.delimiter + str(bookmark).split(World.delimiter)[1]
            bookmarkList.append(bookmarkStNum)
            bookmarkDic[bookmarkStNum] = bookmarkSt
        bookmarks = QtCore.QStringList()
        bookmarkList.sort()
        for bookmarkNum in bookmarkList:
            bookmarkString = str(bookmarkNum) + bookmarkDic[bookmarkNum]
            if bookmarkString not in bookmarks: 
                bookmarks.append(bookmarkString)
#        while bookmarks.count() > self.maxBookmark:
#            bookmarks.removeAt(bookmarks.count() - 1) 
        World.settings.setValue("bookmarkList", QtCore.QVariant(bookmarks))
        self.updateBookmarkAction()
        
    def createBookmarkAction(self):
        #create click event to bookmark current row in overview when bookmark menu item is clicked.
        for i in range(self.maxBookmark):
            self.bookmarkaction.append(QtGui.QAction(self))
            self.bookmarkaction[i].setVisible(False)
            self.connect(self.bookmarkaction[i], QtCore.SIGNAL("triggered()"), self.startBookmarkAction)
            self.ui.menuBookmark.addAction(self.bookmarkaction[i])
        self.updateBookmarkAction()
    
    def startBookmarkAction(self):
        #start bookmark operation when click event has been thrown from menu item for bookmark
        action = self.sender()
        if action:
            index = str(action.data().toString()).split(World.delimiter)
            self.dockOverview.gotoRow(int(index[0]))
    
    def updateBookmarkAction(self):
        """
        Update bookmark action
        """
        #update bookmark in the overview .
        bookmark = World.settings.value("bookmarkList").toStringList()
        numBookmark = min(bookmark.count(), self.maxBookmark)
        for i in range(numBookmark):
            self.bookmarkaction[i].setText(bookmark[i])
            self.bookmarkaction[i].setData(QtCore.QVariant(bookmark[i]))
            self.bookmarkaction[i].setVisible(True)
        for j in range(numBookmark, self.maxBookmark):
            self.bookmarkaction[j].setVisible(False)
            
    def getBookmarkedStNums(self):
        """
        Get a list of bookmarked string number: eg: [2,5,7]
        """
        bookmarkStNums = []
        bookmarks = World.settings.value("bookmarkList").toStringList()
        for bookmark in bookmarks:
            bookmarkStNum = int(str(bookmark).split(World.delimiter)[0])
            bookmarkStNums.append(bookmarkStNum)
        return bookmarkStNums

    def nextBookmark(self):
        """
        go to next bookmark, loop to first bookmark if end of bookmark.
        """
        bookmarkStNums = self.getBookmarkedStNums()
        currentIdex = self.dockOverview.getCurrentIndex()
        for i in range(len(bookmarkStNums)):
            if (currentIdex >= bookmarkStNums[len(bookmarkStNums)-1]):
                self.dockOverview.gotoRow(bookmarkStNums[0])
                break
            if (currentIdex < bookmarkStNums[i]):
                self.dockOverview.gotoRow(bookmarkStNums[i])
                break

    def previousBookmark(self):
        """
        go to previous bookmark, loop to last bookmark if reached the begining of bookmark.
        """
        bookmarkStNums = self.getBookmarkedStNums()
        currentIdex = self.dockOverview.getCurrentIndex()
        for i in range(len(bookmarkStNums)-1, -1, -1):
            if (currentIdex > bookmarkStNums[i]):
                self.dockOverview.gotoRow(bookmarkStNums[i])
                break
            if (currentIdex <= bookmarkStNums[0]):
                self.dockOverview.gotoRow(bookmarkStNums[len(bookmarkStNums)-1])
                break
            if (currentIdex == bookmarkStNums[i]):
                self.dockOverview.gotoRow(bookmarkStNums[i -1])
                break

    def clearBookmarks(self, saveFlag=True):
        """
        Clear all bookmarks should clears all icons in overview.
        """
        if saveFlag:
            self.ui.actionSave.setEnabled(True)
            self.fileaction.isSaved=False
            self.operator.modified=True
            units=self.dockOverview.units
            for unit in units:
                if unit.isbookmarked():
                    unit.clearbookmarked()
        # clear icons from overview.
        bookmarkStNums = self.getBookmarkedStNums()
        blankIcon = QtGui.QIcon()
        for bookmark in bookmarkStNums:
            curritem = self.dockOverview.ui.tableOverview.item(bookmark-1, 0)
            curritemsource = self.dockOverview.ui.tableOverview.item(bookmark-1, 1)
            if curritem:
                curritem.setIcon(blankIcon)
        #clear from bookmark menu
        
        bookmarks = World.settings.value("bookmarkList").toStringList() 
        numClear = min(bookmarks.count(), self.maxBookmark)
        for i in range(numClear):
            self.bookmarkaction[i].setVisible(False)
        World.settings.remove("bookmarkList")
        #Disable Clear All Bookmarks, Bookmark Next, Bookmark Previous if there is  no bookmark
        self.ui.actionClearBookmarks.setEnabled(False)
        self.ui.actionPreviousBookmark.setEnabled(False)
        self.ui.actionNextBookmark.setEnabled(False)
        
            
    def prependOpenRecent(self, fileName):
        """
        Append the recentely opened file to the open recent list.
        @param fileName string, the filename to append to Open Recent.
        """               
        files = World.settings.value("recentFileList").toStringList()
        files.removeAll(fileName)
        files.prepend(fileName)
        while files.count() > World.MaxRecentFiles:
            files.removeAt(files.count() - 1)
        if (files.count() > 0):
            self.ui.menuOpen_Recent.setEnabled(True)
        World.settings.setValue("recentFileList", QtCore.QVariant(files))
        self.updateRecentAction()
        return
        
    def startRecentAction(self):
        action = self.sender()
        if action:
            # TODO: remove filename from recent file if it doesn't exist.
            filename = unicode(action.data().toString())
            # A dialog ask to remove death file name.
            if (not(os.path.isfile(filename))):
                titled = unicode(self.tr("%s \nwas not found.")) % (filename)
                ret = QtGui.QMessageBox.critical(self, self.tr("Error"), 
                titled,
                QtGui.QMessageBox.Ok | QtGui.QMessageBox.Default)
                if (ret == QtGui.QMessageBox.Ok):
                    files = World.settings.value("recentFileList").toStringList()
                    filenameIndex = files.indexOf(filename)
                    if (filenameIndex >= 0):
                        files.removeAt(filenameIndex)
                    World.settings.setValue("recentFileList", QtCore.QVariant(files))
                    self.updateRecentAction()
                    self.closeFile()
                return
            
            self.openFile(filename)
    
    def clearRecentAction(self):
        self.ui.menuOpen_Recent.clear()
        self.ui.menuOpen_Recent.setEnabled(False)
        World.settings.remove("recentFileList")
        # remove menu Open_Recent action, and add Open action to toolbar
        self.addOpenToBar()
    
    def createRecentAction(self):
        for i in range(World.MaxRecentFiles):
            self.recentaction.append(QtGui.QAction(self))
            self.recentaction[i].setVisible(False)
            self.connect(self.recentaction[i], QtCore.SIGNAL("triggered()"), self.startRecentAction)
            self.ui.menuOpen_Recent.addAction(self.recentaction[i])
        self.ui.menuOpen_Recent.addSeparator()
        self.clearAction = QtGui.QAction("&Clear", self)
        self.connect(self.clearAction, QtCore.SIGNAL("triggered()"), self.clearRecentAction)
        self.ui.menuOpen_Recent.addAction(self.clearAction)
        self.ui.menuOpen_Recent.setEnabled(True)
        self.updateRecentAction()

    def updateRecentAction(self):
        """
        Update recent actions of Open Recent Files with names of recentely opened files
        """        
        if (not len(self.ui.menuOpen_Recent.actions())):
            self.createRecentAction()
        files = World.settings.value("recentFileList").toStringList()
        
        if (files.count() > 0):
            self.ui.menuOpen_Recent.setEnabled(True)
        numRecentFiles = min(files.count(), World.MaxRecentFiles)
        for i in range(numRecentFiles):
            self.recentaction[i].setText(("&" + str(i+1) + ": ") + files[i])
            self.recentaction[i].setData(QtCore.QVariant(files[i]))
            self.recentaction[i].setVisible(True)

        for j in range(numRecentFiles, World.MaxRecentFiles):
            self.recentaction[j].setVisible(False)
        # remove Open action from toolbar, and add menu Open_Recent action to toolbar
        self.addOpenToBar()

    def closeEvent(self, event):
        """
        @param QCloseEvent Object: received close event when closing mainwindows
        """
        if World.WF_debug:
            print "Enter closeEvent in Class MainWindow"
        
        if (self.operator.modified):
            if (self.fileaction.clearedModified(self)):
                event.accept()
            else:
                event.ignore()
        # remember last geometry
        World.settings.setValue("lastGeometry", QtCore.QVariant(self.geometry()))
        
        # remember last state
        state = self.saveState(1)
        World.settings.setValue("MainWindowState", QtCore.QVariant(state))
        World.settings.setValue("TuViewHidden", QtCore.QVariant(self.dockTUview.isHidden()))
        self.operator.langcur=""
        World.settings.setValue("CurrentLanguage", QtCore.QVariant(self.operator.langcur)) 
        
    def setTitle(self, filename):
        """
        set the title of program.
        @param filename: a filename with full path, type as string.
        """
        if (filename):
            filename=unicode(filename)
            self.setWindowTitle("%s - %s" % (filename, World.settingApp))
            self.prependOpenRecent(filename)
        else:
            self.setWindowTitle(World.settingApp)
    
    def toggleFirstLastUnit(self, atFirst, atLast):
        """set enable/disable first, previous, next, and last unit buttons
        @param atFirst: bool indicates that the unit is at first place
        @param atLast: bool indicates that the unit is at last place
        """
        self.ui.actionFirst.setDisabled(atFirst)
        self.ui.actionPrevious.setDisabled(atFirst)
        self.ui.actionNext.setDisabled(atLast)
        self.ui.actionLast.setDisabled(atLast)
    
    def searchNext(self):
        """Find Next from submenu """
        self.operator.searchNext()
        self.findBar.ui.lineEdit.setFocus()
    
    def searchPrevious(self):
        """Find Previous from submenu."""
        self.operator.searchPrevious()
        self.findBar.ui.lineEdit.setFocus()
        
    def focusChanged(self, oldWidget, newWidget):

        if World.WF_debug:
            print "Enter focusChanged in Class MainWindow"
        if (oldWidget):            
            self.disconnect(oldWidget, QtCore.SIGNAL("copyAvailable(bool)"), self.enableCopyCut)
            self.disconnect(oldWidget, QtCore.SIGNAL("undoAvailable(bool)"), self.enableUndo)
            self.disconnect(oldWidget, QtCore.SIGNAL("redoAvailable(bool)"), self.enableRedo)
            # cut, copy and paste in oldWidget
            if (callable(getattr(oldWidget, "cut", None))):
                self.disconnect(self.ui.actionCut, QtCore.SIGNAL("triggered()"), oldWidget, QtCore.SLOT("cut()"))

            if (callable(getattr(oldWidget, "copy", None))):
                self.disconnect(self.ui.actionCopy, QtCore.SIGNAL("triggered()"), oldWidget, QtCore.SLOT("copy()"))

            if (callable(getattr(oldWidget, "paste", None))):
                self.disconnect(self.ui.actionPaste, QtCore.SIGNAL("triggered()"), oldWidget, QtCore.SLOT("paste()"))
            # undo, redo and selectAll in oldWidget
            if (callable(getattr(oldWidget, "document", None))):
                self.disconnect(self.ui.actionUndo, QtCore.SIGNAL("triggered()"), oldWidget.document(), QtCore.SLOT("undo()"))
                self.disconnect(self.ui.actionRedo, QtCore.SIGNAL("triggered()"), oldWidget.document(), QtCore.SLOT("redo()"))

        if (newWidget):
            self.connect(newWidget, QtCore.SIGNAL("copyAvailable(bool)"), self.enableCopyCut)
            self.connect(newWidget, QtCore.SIGNAL("undoAvailable(bool)"), self.enableUndo)
            self.connect(newWidget, QtCore.SIGNAL("redoAvailable(bool)"), self.enableRedo)
            # cut, copy and paste in newWidget
            if (callable(getattr(newWidget, "cut", None))):
                self.connect(self.ui.actionCut, QtCore.SIGNAL("triggered()"), newWidget, QtCore.SLOT("cut()"))

            if (callable(getattr(newWidget, "copy", None))):
                self.connect(self.ui.actionCopy, QtCore.SIGNAL("triggered()"), newWidget, QtCore.SLOT("copy()"))

            if (callable(getattr(newWidget, "paste", None))):
                self.connect(self.ui.actionPaste, QtCore.SIGNAL("triggered()"), newWidget, QtCore.SLOT("paste()"))
                if (callable(getattr(newWidget, "isReadOnly", None))):
                    self.ui.actionPaste.setEnabled(not newWidget.isReadOnly())
                else:
                    self.ui.actionPaste.setEnabled(True)
            else:
                self.ui.actionPaste.setEnabled(False)

            if (callable(getattr(newWidget, "textCursor", None))):
                hasSelection = newWidget.textCursor().hasSelection()
                self.enableCopyCut(hasSelection)
            else:
                self.enableCopyCut(False)

            #it will not work for QLineEdits
            if (callable(getattr(newWidget, "document", None))):
                undoAvailable = newWidget.document().isUndoAvailable()
                redoAvailable = newWidget.document().isRedoAvailable()
                self.enableUndo(undoAvailable)
                self.enableRedo(redoAvailable)
                self.connect(self.ui.actionUndo, QtCore.SIGNAL("triggered()"), newWidget.document(), QtCore.SLOT("undo()"))
                self.connect(self.ui.actionRedo, QtCore.SIGNAL("triggered()"), newWidget.document(), QtCore.SLOT("redo()"))
            else:
                self.enableUndo(False)
                self.enableRedo(False)
    
    def openFile(self, filename):
        """
        Open filename and prepend it in recent list.
        @param filename: file to open.
        """
        if World.WF_debug:
            print "Enter openFile in Class MainWindow"
            
        self.dockOverview.prevtext=""
        self.dockOverview.undo_dic={}
        self.dockOverview.indexlist=[]
        
        globalClass.postTestList=[]
        self.errorTable.postTestList=[]
        globalClass.closeFile=False
        closed = self.closeFile()
        globalClass.closeFile=True
        self.unitnum=0
        self.clearBookmarks(False)
        if (closed):
            if globalClass.errorCheckEnableFlag == 1:
                self.getPreferenceValue()
            if (self.operator.getUnits(filename)):
                self.fileaction.setFileProperty(filename)
                self.setStatusForFile(filename)
                self.prependOpenRecent(filename)
                
        self.show()
        units=self.dockOverview.units
        self.unitnum=len(units)
        self.errorTable.CountUnitNumber(self.unitnum)
        if self.errorTable.isVisible():
            self.connect(self.errorTable.ui.errorTable, QtCore.SIGNAL("cellDoubleClicked(int, int)"), self.errorTable.jumpToMessage)
        globalClass.closeFile=True
        if self.ui.actionError_Checking.isChecked():
            self.operator.fileerrorlist()
        self.currentFile=filename 
        
    def removeTableContent(self, table):
        
        row = table.rowCount()
        for r in range(0, row):
#            this has been done to make sure that the table is searched from the bottom so that after deleting a row,  it does not get out of the range
            reverse = (row-r)-1 
            table.removeRow(reverse)

   
    def closeFile(self, force = False):
        """
        Return True when successfully close file, else return False.
        @param force: bool whether to check if content is modified.
        """
#        clearing the existing error table when a file is closed
        if World.WF_debug:
            print "Enter closeFile in Class MainWindow"
        self.operator.TMDict = {}
        self.operator.__tmCache__ = {}
        if (force) or (not self.operator.modified):
            self.setStatusForFile(None)
            self.operator.closeFile()
            self.statuslabel.setText("")
            self.ui.actionSave.setEnabled(False)
            self.ui.actionAuto_translate.setEnabled(False)
            self.ui.actionSearch_Suggestions.setEnabled(False)
            table = self.errorTable.ui.errorTable
            self.removeTableContent(table)
#            self.ui.actionEmbedGlossary.setEnabled(False)
        elif (self.operator.modified):
            if self.fileaction.clearedModified(self):
                table = self.errorTable.ui.errorTable
                self.removeTableContent(table)
                self.closeFile(True)  
            else:
                self.clearBookmarks(False)
                return False
        self.clearBookmarks(False)  
        self.operator.langcur=""
        World.settings.setValue("CurrentLanguage", QtCore.QVariant(self.operator.langcur))  
        return True
        
    def setStatusForFile(self, filename):
        """
        Enable/disable buttons according to availability of filename.
        @param filename: filename to set title and determine enable/disable.
        """
        if World.WF_debug:
            print "Enter statusForFile in Class MainWindow"

        self.ui.actionSave.setEnabled(False)
        
        value = bool(filename)
        self.setTitle(filename)
        self.ui.action_Close.setEnabled(value)
        self.ui.actionSaveas.setEnabled(value)
        self.ui.actionPaste.setEnabled(value)
        self.ui.actionFindNext.setEnabled(value)
        self.ui.actionFindPrevious.setEnabled(value)
        self.ui.actionFind.setEnabled(value)
        self.ui.actionReplace.setEnabled(value)
        self.ui.actionCopySource2Target.setEnabled(value)
        self.ui.actionErasetarget.setEnabled(value)
        self.ui.actionChangeFileLanguage.setEnabled(value)
        self.ui.actionBatch_Process.setEnabled(value)
        uniFilename=unicode(filename)
        if (uniFilename.endswith(".xlf") or uniFilename.endswith(".xliff")):
            self.ui.actionEdit_Header.setEnabled(False)
        else:    
            self.ui.actionEdit_Header.setEnabled(value)
        self.ui.actionGoTo.setEnabled(value)
        self.ui.actionAddBookmarks.setEnabled(value)
        self.ui.actionToggleFuzzy.setEnabled(value)
        self.ui.actionClearFuzzy.setEnabled(value)
        self.ui.actionFilterFuzzy.setEnabled(value)
        self.ui.actionFilterTranslated.setEnabled(value)
        self.ui.actionFilterUntranslated.setEnabled(value)
        self.ui.action_SpellTrans.setEnabled(value)
        self.ui.actionAutoSpellCheck.setEnabled(value)
        self.ui.actionAuto_translate.setEnabled(value)
        self.ui.actionSearch_Suggestions.setEnabled(value)  
        self.ui.actionError_Checking.setEnabled(value)   
        self.ui.actionSearchErrors.setEnabled(value)
        self.ui.actionShow_Nonprinting_Char.setEnabled(value)  
        if self.ui.actionError_Checking.isChecked():
            globalClass.errorCheckEnableFlag = 1
        else:
            globalClass.errorCheckEnableFlag = 0
            
#        if self.errorTable.isHidden():
##            self.ui.actionError_Checking.setChecked(False)
#            globalClass.errorCheckEnableFlag = 0
#        else :
##            self.ui.actionError_Checking.setChecked(True)
#            globalClass.errorCheckEnableFlag = 1
##            self.getPreferenceValue()
            
        # disconnect and reconnect fuzzy, translated, and untranslated button actions.
        self.disconnect(self.ui.actionFilterFuzzy, QtCore.SIGNAL("toggled(bool)"), self.operator.filterFuzzy)
        self.disconnect(self.ui.actionFilterTranslated, QtCore.SIGNAL("toggled(bool)"), self.operator.filterTranslated)
        self.disconnect(self.ui.actionFilterUntranslated, QtCore.SIGNAL("toggled(bool)"), self.operator.filterUntranslated)
        self.ui.actionFilterFuzzy.setChecked(False)
        self.ui.actionFilterTranslated.setChecked(False)
        self.ui.actionFilterUntranslated.setChecked(False)
        self.connect(self.ui.actionFilterFuzzy, QtCore.SIGNAL("toggled(bool)"), self.operator.filterFuzzy)
        self.connect(self.ui.actionFilterTranslated, QtCore.SIGNAL("toggled(bool)"), self.operator.filterTranslated)
        self.connect(self.ui.actionFilterUntranslated, QtCore.SIGNAL("toggled(bool)"), self.operator.filterUntranslated)
        
        #TODO: it is enable unless Automatically lookup translation in TM is checked.
        self.ui.actionCopySearchResult2Target.setEnabled(value)
        self.findBar.toggleViewAction().setVisible(value)
        
        # reset TM table and Glossary Table
        if (not value):
            self.table.viewSetting()
            self.tableGlossary.viewSetting()
            self.dockThirdLanguage.viewSetting()
            
    def addOpenToBar(self):
        '''
        add Open action or menu Open_Recent action to toolbar
        '''
        if World.WF_debug:
            print "Enter addOpenToBar in Class MainWindow"
                  
        if (hasattr(self, "actionOpen")):
            self.ui.toolFile.removeAction(self.actionOpen)
        if not self.ui.menuOpen_Recent.isEnabled():
            self.actionOpen = self.ui.actionOpen
        else:
            self.actionOpen = self.ui.menuOpen_Recent.menuAction()
            self.disconnect(self.actionOpen, QtCore.SIGNAL("triggered()"), self.fileaction.openFile)
            self.connect(self.actionOpen, QtCore.SIGNAL("triggered()"), self.fileaction.openFile)
        self.actionOpen.setToolTip(self.tr("Open"))
        self.actionOpen.setWhatsThis("<h3>Open a file</h3>You will be asked for the name of a file to be opened and open recent file in an editor window.")
        self.ui.toolFile.insertAction(self.ui.actionSave, self.actionOpen)
        
    def toggleTUviewTermSig(self):
        """Toggle TUview term signal."""

        if World.WF_debug:
            print "Enter __init__ in Class MainWindow"
             
        if self.tableGlossary.toggleViewAction().isChecked():
            self.connect(self.dockTUview, QtCore.SIGNAL("termList"), self.operator.emitGlossaryCandidates)
            self.dockTUview.emitGlossaryWords()
        else:
            self.disconnect(self.dockTUview, QtCore.SIGNAL("termList"), self.operator.emitGlossaryCandidates)
    
#    def updateStatistic(self, filename):
#        """
#        Call Operator.saveStoreToFile() to return signal of filename
#        """
#        if not hasattr(self, "Catalog"):
#            self.initCatalogManager()
#        self.Catalog.updateFileStatus(filename)

    def setStatus(self, statusObject):
        """
        display units staus
        """
        
        if World.WF_debug:
            print "Enter setStatus in Class MainWindow"
            
        numTotal = statusObject.getTotalOfStrings()
        numUntran = statusObject.getTotalOfUntranslatedStrings()
        numTran = statusObject.getTotalOfTranslatedStrings()
        numFuzzy = statusObject.getTotalOfFuzzyStrings()
        
        filename = self.operator.getFileName()
        if not hasattr(self, "Catalog"):
            self.initCatalogManager()
            
        self.Catalog.updateFileStatus(filename, numUntran, numFuzzy, numTran)
        
        statusString = "Total number of messages: " + str(numTotal) + "  |  " + \
                "Untranslated: " + str(numUntran) + "  |  " + \
                "Fuzzy: " +  str(numFuzzy) + "  |  " + \
                "Translated: " +  str(numTran)
        
        self.statuslabel.setText(statusString)
        
    def displayTempMessage(self, message):
        """
        display a quick message for 5 seconds in status bar.
        """
        if (message == "reachedEnd"):
            self.ui.statusbar.showMessage(self.tr("Search has reached the end."), 5000)
        elif (message == "notFound"):
            self.ui.statusbar.showMessage(self.tr("Search was not found."), 5000)
        elif (message == "found"):
            pass
        else:
            self.ui.statusbar.showMessage(message, 5000)
    
    def filterFuzzy(self,  bool):
        #
        self.disconnect(self.dockTUview, QtCore.SIGNAL("textChanged"), self.dockOverview.updateText)
        self.operator.filterFuzzy(bool)
        self.connect(self.dockTUview, QtCore.SIGNAL("textChanged"), self.dockOverview.updateText)
        
    def openNewWindow(self):
        
        self.newWindowList.append(MainWindow())
        self.newWindowList[len(self.newWindowList)-1].show()

#        import platform
#        from subprocess import Popen, PIPE, STDOUT
#        
#        if World.WF_debug:
#            print "Enter openNewWindow in Class MainWindow"
#            
#        ostype = platform.system()
#        
#        if (ostype == "Linux"):
#            cmd = 'wordforge.py'
#            Popen(cmd, stdout=None, stderr=STDOUT, shell=isinstance(cmd, basestring))
#        elif (ostype == "Windows"):
#            import win32process
#            if hasattr(sys.stderr, 'fileno'):
#                childStdErr = sys.stderr
#            elif hasattr(sys.stderr, '_file') and hasattr(sys.stderr._file, 'fileno'):
#                childStdErr = childstderr = sys.stderr._file
#            else:
#                childStdErrPath = 'nul'
#                childStdErr = file(childStdErrPath, 'a')
#                
#            Popen(cmd, stdin = None, stdout=None, stderr=childStdErr, shell=False,creationflags = win32process.CREATE_NO_WINDOW)
        
    def showAboutDialog(self):
        """
        Display the about dialog.
        """
        from WordForge.Editor.modules.AboutEditor import AboutEditor
        if (not hasattr(self, "aboutDialog") ):
            self.aboutDialog = AboutEditor(self)
        self.aboutDialog.showDialog()
    
    def showBatchProcessDialog(self):
        """
        Display the batch Processing dialog.
        """
        if (not hasattr(self, "BatchProcessDialog") ):
            from WordForge.Editor.modules.BatchProcess import BatchProcess
            self.BatchProcessDialog = BatchProcess(self)
        self.BatchProcessDialog.showDialog()    

    def showHeaderDialog(self):
        """
        Display the header dialog.
        """
        if (not hasattr(self, "headerDialog") ):
            from WordForge.Editor.modules.Header import Header
            self.headerDialog = Header(self, self.operator)
            self.connect(self.headerDialog, QtCore.SIGNAL("showPreference()"), self.showPreferenceDialog)

        self.headerDialog.showDialog()

    def initCatalogManager(self):
        """
        init the catalog manager.
        sync(): Writes any unsaved changes to permanent storage, and reloads any settings that have
        been changed in the meantime by another application.
        """
        self.connect(self.Catalog, QtCore.SIGNAL("initSearch"), self.operator.initSearch)
        self.connect(self.Catalog, QtCore.SIGNAL("searchNext"), self.operator.searchNext)
        self.connect(self.operator, QtCore.SIGNAL("searchStatus"), self.Catalog.setSearchStatus)
        self.connect(self.Catalog, QtCore.SIGNAL("openFile"), self.openFile)
        self.initTMsetting()
        self.connectCatalogTmsetting()


    def connectCatalogTmsetting(self):
        """
        connect tmsetting dialog with catalog manager.
        """
        self.connect(self.Catalog, QtCore.SIGNAL("buildTM"), self.tmsetting.buildMatcher)
        self.connect(self.tmsetting, QtCore.SIGNAL("buildPercentage"), self.Catalog.updateProgress)
                
    def initTMsetting(self):
        """
        init TMsettings
        """
        if (not hasattr(self, "tmsetting") ):
            from WordForge.Editor.modules import TMSetting
            self.tmsetting = TMSetting.TMSetting(self)
            self.connect(self.tmsetting, QtCore.SIGNAL("matcher"), self.operator.setMatcher)
            self.connect(self.tmsetting, QtCore.SIGNAL("matcher"), self.seEnabledExportTMGlossssary)
                
            if (hasattr(self, "Catalog") ):
                self.connectCatalogTmsetting()
                
    def showTmsettingDialog(self):
        """
        Display the tmsetting dialog.
        """
        self.initTMsetting()
        self.tmsetting.showDialog()

    def showGlossaryDialog(self):
        """
        Display the glossary dialog.
        """
        if World.WF_debug:
            print "Enter showGlossaryDialog in Class MainWindow"
            
        if (not hasattr(self, "glossary") ):
            from WordForge.Editor.modules import TMSetting
            self.glossary = TMSetting.glossarySetting(self)
            self.connect(self.glossary, QtCore.SIGNAL("matcher"), self.operator.setMatcher)
            self.connect(self.glossary, QtCore.SIGNAL("matcher"), self.seEnabledExportTMGlossssary)
        self.glossary.showDialog()
        
    def showCatalogManager(self):
        """
        Display the catalog manager.
        """
        self.initCatalogManager()
        self.Catalog.showDialog()
        
    def showPreferenceDialog(self, tabNumber = 0):
        """
        Display the preference dialog.
        """
        if World.WF_debug:
            print "Enter showPreferenceDialog in Class MainWindow"
        
        if (not hasattr(self, "preference") ):
            from WordForge.Editor.modules.Preference import Preference
            self.preference = Preference(self)
#        self.connect(self.ui.actionPreferences, QtCore.SIGNAL("triggered()"), self.preference.showDialog)
        self.connect(self.preference, QtCore.SIGNAL("settingsChanged"), self.dockComment.applySettings)
#        self.connect(self.preference, QtCore.SIGNAL("settingsChanged"), self.dockOverview.applySettings)
        self.connect(self.preference, QtCore.SIGNAL("settingsChanged"), self.dockTUview.applySettings)
        self.connect(self.preference, QtCore.SIGNAL("settingsChanged"), self.operator.applySettings)
        self.connect(self.preference, QtCore.SIGNAL("settingsChanged"), self.table.applySettings)
        self.connect(self.preference, QtCore.SIGNAL("settingsChanged"), self.tableGlossary.applySettings)
#        self.connect(self.preference,  QtCore.SIGNAL("setDictionary"),  self.operator.setDictionary)
        self.connect(self.preference,  QtCore.SIGNAL("settingsChanged"),  self.applySettings)
            
        self.preference.showDialog(tabNumber)
    
    def applySettings(self):
        # set timer for auto save function
        autoSaveEnabled = World.settings.value("EnableAutoSave", QtCore.QVariant(True)).toBool()
        if autoSaveEnabled:
            self.timer = QtCore.QTimer()
            self.disconnect(self.timer,  QtCore.SIGNAL("timeout()"), self.autoSave)
            self.connect(self.timer,  QtCore.SIGNAL("timeout()"), self.autoSave)
            self.autoSaveInterval = World.settings.value("autoSaveInterval", QtCore.QVariant("5")).toInt()[0] * 60000
            self.timer.start(self.autoSaveInterval)
#            print "will save in", autoSaveInterval / 1000, "seconds."
        
    def showThirdLanguageDialog(self):
        """
        Display the third language dialog.
        """
        if World.WF_debug:
            print "Enter showThirdLanguageDialog in Class MainWindow"
        
        if (not hasattr(self, "third") ):
#            from WordForge.Editor.modules.ThirdLanguages import ThirdLanguages
#            self.third = ThirdLanguages(self)
#            self.connect(self.third, QtCore.SIGNAL("addThirdlanguage"), self.operator.addThirdLanguage)
#            self.connect(self.third, QtCore.SIGNAL("removeThirdLanguage"), self.operator.removeThirdLanguage)
#            self.connect(self.operator, QtCore.SIGNAL("currentUnit"), self.third.updateView)
#        self.third.showDialog()
#        self.operator.emitUnit(self.operator.getCurrentUnit())
            from WordForge.Editor.modules.DialogThird import DialogThird
            self.DialogThird = DialogThird(self)
        self.DialogThird.showDialog()
            
        
    def showConverterDialog(self):
        """
        Display the converter dialog.
        """
        if (not hasattr(self, "converter")):
            from WordForge.Editor.modules import Converter
            self.converter = Converter.Converter(self)
#            self.connect(self.operator, QtCore.SIGNAL("export"), self.converter.convertStore)
        self.converter.showDialog()

    def showXliffUpgradeDialog(self):
        """
        Display the xliff update dialog.
        """
        from WordForge.Editor.modules import XliffMerger
        self.mergerXliff = XliffMerger.XLIFFMerger(self)
        self.mergerXliff.show()


    def showEmbedGlossaryDialog(self):
        """
        Display the third language dialog.
        """
        if World.WF_debug:
            print "Enter showEmbedGlossaryDialog in Class MainWindow"
        
        if (not hasattr(self, "embedGlossary") ):
            from WordForge.Editor.modules.EmbedGlossary import EmbedGlossaryDialog
            self.embedGlossary = EmbedGlossaryDialog(self)
        self.embedGlossary.showDialog()    
    
    def showCleanXLIFFDialog(self):
        """
        Display the third language dialog.
        """
        if World.WF_debug:
            print "Enter showGCleanXLIFFDialog in Class MainWindow"
        
        if (not hasattr(self, "CleanXLIFFDialog") ):
            from WordForge.Editor.modules import CleanXLIFFDialog
            self.CleanXLIFFDialog = CleanXLIFFDialog.cleanXLIFF(self)
        self.CleanXLIFFDialog.showDialog()
        
        
    def seEnabledExportTMGlossssary(self, matcher=None,  section=None):
        if (self.operator.__termMatcher__ == None):
            self.ui.actionExport_Glossary.setEnabled(False)
        else:
            self.ui.actionExport_Glossary.setEnabled(True)
            
        if (self.operator.__tmMatcher__ == None):
            self.ui.actionExport_Translation_Memory.setEnabled(False)
        else:
            self.ui.actionExport_Translation_Memory.setEnabled(True)
    
    def showExportTM2TMX(self):
        """
        Display the third language dialog.
        """
        if (not hasattr(self, "exportTM2TMX") ):
            from WordForge.Editor.modules import ExportTM2TMX
            self.exportTM2TMX = ExportTM2TMX.ExportTM2TMX(self)
        self.exportTM2TMX.showDialog()
    
    def showExport2TBX(self):
        """
        Display the third language dialog.
        """
        if (not hasattr(self, "export2TBX") ):
            from WordForge.Editor.modules import Export2TBX
            self.export2TBX = Export2TBX.Export2TBX(self)
        self.export2TBX.showDialog()
        
    def callPreCompute(self):
#this function calls the precompute function of errorchecking.py  
            self.errorTable.preCompute(globalClass.currentSource)
    
    
    def callPostCompute(self, index, isPlural):
#this function calls the postCompute function of errorchecking.py   
            globalClass.glsErrorDict = globalClass.tempGlsErrorDict
            globalClass.glsMatchWordCount = globalClass.tempGlsMatchWordCount
            globalClass.tempGlsErrorDict = defaultdict(dict)
            globalClass.tempGlsMatchWordCount = {}
            if not isPlural:
                globalClass.lastTarget = self.dockTUview.ui.txtTarget.toPlainText()
            else:    
                globalClass.lastTarget=self.dockTUview.curtarget
#                if globalClass.lastTarget:
#                    globalClass.lastTarget=self.dockTUview.ui.txtTarget.toPlainText()  
            globalClass.lastIndex = str(index)
            self.errorTable.postCompute(globalClass.lastSource,globalClass.lastTarget, globalClass.lastIndex)

    def uncheckErrorAction(self):
# unchecks the errorchecking menu. This function is called from errorchecking.py closeevent event.
            self.ui.actionError_Checking.setChecked(False)
            
    def changeFileLanguage(self):
        #this function insert or update the language of the present file. 
        if self.operator.isXliff:
            self.operator.loadopenwindow("xliff")
        else:    
            self.operator.loadopenwindow("po")
            
def main(inputFile = None):    
    # set the path for QT in order to find the icons
    if World.WF_debug:
        print "Enter main in Class MainWindow"
            
    
    if (__name__ == "__main__"):
        QtCore.QDir.setCurrent(os.path.join(sys.path[0], "../ui"))
    
    else:        
        import distutils.sysconfig
        packagesdir = distutils.sysconfig.get_python_lib()
        packagesdir = os.path.join(packagesdir, "WordForge")
        QtCore.QDir.setCurrent(os.path.join(packagesdir, "Editor", "ui"))
            
    app = QtGui.QApplication(sys.argv)
    editor = MainWindow()
    editor.show()
    
    if (inputFile):
        if os.path.exists(inputFile):
            editor.openFile(inputFile)
        else:
            msg = editor.tr("%1 file name doesn't exist").arg(inputFile)
            QtGui.QMessageBox.warning(editor, editor.tr("File not found") , msg)
    sys.exit(app.exec_())
    
if __name__ == "__main__":
    main()
    
