#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Editor
# Copyright 2006 WordForge Foundation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details. 
#
# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)
#       Aditi Barua (aditi@ankur.org.bd)
#       Rezwan Shahid (rezwan@ankur.org.bd)
#       Asheq Hamid (asheq@ankur.org.bd)
# This module is providing an setting path of catalog dialog 

from PyQt4 import QtCore, QtGui
from WordForge.Editor.ui.Ui_NewGlossaryTerm import Ui_NewGlossaryTerm
from WordForge.translate.storage import lisa
from WordForge.translate.storage import xliff
from WordForge.translate.storage import tbx
from WordForge.translate.storage import base
from WordForge.translate.search import match
from lxml import etree

class NewGlossaryTerm(QtGui.QDialog):
    """
    Code for setting path of catalog dialog
    """
    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent, QtCore.Qt.Window)
        self.ui =  Ui_NewGlossaryTerm()
        self.ui.setupUi(self)
        self.parent = parent
        self.setWindowModality(QtCore.Qt.WindowModal)
        self.applySettings()
        self.fillTable()

        self.connect(self.ui.tableWidget, QtCore.SIGNAL("currentCellChanged(int,int,int,int)"), self.setFocusOnCell)
        self.connect(self.ui.Addsyn, QtCore.SIGNAL("clicked()"), self.addSynonym)
        self.connect(self.ui.delsyn, QtCore.SIGNAL("clicked()"), self.deleteSynonym)
        
        self.connect(self.ui.buttonBox, QtCore.SIGNAL("rejected()"), self.close)
        self.connect(self.ui.buttonBox, QtCore.SIGNAL("accepted()"), self.addTerm2Glossary)
     
  
    def addTerm2Glossary(self):
#      This function adds new term to internal glossary
        dict = {}
        for rowindex in range(self.ui.tableWidget.rowCount()):
            translation = unicode(self.ui.tableWidget.item(rowindex, 1).text())
            if translation:
                term =  unicode(self.ui.tableWidget.item(rowindex, 0).text())
                if not dict.get(term):
                    dict[term] = []
                dict[term].append(translation)
#       Update the internal glossary tree
        self.setTarget(self.parent.operator.internalGlossary, dict)
#      Update the matcher units
        matcherCandidates = self.parent.operator.__termMatcher__.candidates
        for term in dict.keys():
            for unit in matcherCandidates.units:
                if term == unit.source and not unit.target:
#  Add simple units to the candidate list.                  
                    for index in range(len(dict[term])):
                        simpleunit = base.TranslationUnit("")
                        simpleunit.source = term
                        simpleunit.target = dict[term][index]
    #                  Add tbx specific info.
                        simpleunit.entryDefinition  = []
                        simpleunit.subjectField = [] 
                        simpleunit.context = []
                        simpleunit.targetPOSDict = {}
                        simpleunit.languageDependentDefinition = {}
                        simpleunit.addnote(unit.getnotes(origin="translator"))
                        simpleunit.fuzzy = unit.isfuzzy()
                        matcherCandidates.units.append(simpleunit)
                
#  Remove the item with no translation.
                    matcherCandidates.units.remove(unit)
#  Remove items from the glossary cache.                   
                    for key in self.parent.operator.__termCache__.keys():
                        if key.lower() == term.lower():
                            del self.parent.operator.__termCache__[key]
                
        self.parent.dockTUview.emitGlossaryWords() 
        if not self.parent.ui.actionSave.isEnabled():
            self.parent.operator.contentChanged(True)
        self.setInternalGlossaryInFile(self.parent.operator.__store__.document, self.parent.operator.internalGlossary.document)
        self.close()
     
    def setInternalGlossaryInFile(self, document, internalGlossary):
#  This function sets the internalGlossary in the correct place of document.
        docRoot = document.getroot()
        glossRoot = internalGlossary.getroot()
        fileNode = docRoot.find(self.namespaced('file'))
        if fileNode is not None:
            headerNode = fileNode.find(self.namespaced('header'))
            if headerNode is not None:
                glossaryNode = headerNode.find(self.namespaced('glossary'))
                if glossaryNode is not None:
                    internalFileNode = glossaryNode.find(self.namespaced('internal-file'))
                    if internalFileNode is not None:
                        martifNode = internalFileNode.find(self.namespaced('martif'))
                        if martifNode is not None:
                            internalFileNode.replace(martifNode, glossRoot)
                
    def namespaced(self, name):

        namespace = 'urn:oasis:names:tc:xliff:document:1.1'
        return "{%s}%s" % (namespace, name)
            
    def setTarget(self, glossary, dict):
#        This function sets the targets in the proper unit
        for key in dict.keys():
            unit = glossary.findunit(key)
            if unit:
                if not unit.target:
                    unit.po2tbxmap(unit)
                    unit.target = ','.join(dict[key])
                    unit.setInfoInTbx(unit)

    def deleteSynonym(self):
#        This function deletes a row containing synonym.
            currow = self.ui.tableWidget.currentRow() 
            term =  self.ui.tableWidget.item(currow, 0)
            totalRows = self.ui.tableWidget.rowCount()
            isSynonym = False
            for row in range (totalRows):
                if self.ui.tableWidget.item(row, 0).text() == term.text() and row != currow:
                    isSynonym = True
            if isSynonym:
                self.ui.tableWidget.removeRow(currow)
                if currow == totalRows-1:
                    self.ui.tableWidget.selectRow(currow-1)
                else:
                    self.ui.tableWidget.selectRow(currow)
                
    def addSynonym(self):
        
#        This function adds synonym after the selected row.
            currow = self.ui.tableWidget.currentRow() 
            self.ui.tableWidget.insertRow(currow+1)
            termItem = self.ui.tableWidget.item(currow, 0).clone()
            self.ui.tableWidget.setItem(currow+1, 0,termItem)
            item = QtGui.QTableWidgetItem()
            self.ui.tableWidget.setItem(currow+1, 1, item)
            self.ui.tableWidget.selectRow(currow+1)
           
    def setFocusOnCell(self, currow, curcol, prevrow, prevcol):
        if curcol != 0:
            curItem = self.ui.tableWidget.item(currow, curcol)
            self.ui.tableWidget.editItem(curItem)

    def fillTable(self):
#        This function fills the table with the untranslated terms.
        row = 0 
        for unit in self.parent.tableGlossary.glossaryCandidates:
            if not unit.target:
                self.ui.tableWidget.insertRow(row)
                itemTerm = QtGui.QTableWidgetItem(unit.source)
                itemTerm.setFlags(QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable)
                self.ui.tableWidget.setItem(row, 0, itemTerm)
                itemTrans = QtGui.QTableWidgetItem()
                self.ui.tableWidget.setItem(row, 1, itemTrans)
                row += 1
         
        if row != 0:
            self.ui.tableWidget.selectRow(0)
            
    def applySettings(self):
#        This function sets the window, loads data in widgets.
#       Settings for QTableWidget.
        self.headerLabels = [self.tr("Term"),self.tr("Translation")]
        
        self.ui.tableWidget.clear()
        self.ui.tableWidget.setColumnCount(len(self.headerLabels))
        self.ui.tableWidget.setHorizontalHeaderLabels(self.headerLabels)   
        self.ui.tableWidget.resizeColumnToContents(1)
        self.ui.tableWidget.horizontalHeader().setResizeMode(1, QtGui.QHeaderView.Stretch)        
        self.ui.tableWidget.verticalHeader().hide()
        
        
if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    GlossaryTerm =  NewGlossaryTerm(None)
    GlossaryTerm.show()
    sys.exit(tm.exec_())
