#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2006 - 2007 by The WordForge Foundation
#                     www.wordforgefoundation.org
#
# Version 0.1 (29 December 2006)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details. 
#
# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)
#       Aditi Barua(aditi@ankur.org.bd)
#       Asheq Hamid(asheq@ankur.org.bd)
#       Md. Rezwan Shahid(rezwan@ankur.org.bd)
#
# This module is working on Project of Catatog File.


from PyQt4 import QtCore, QtGui
from WordForge.Editor.ui.Ui_NewProject import Ui_NewProject
#from WordForge.Editor.modules import FileDialog
from lxml import etree
import WordForge.translate.lang.data as data
import WordForge.Editor.modules.World as World
from WordForge.Editor.modules.Preference import Preference
from WordForge.Editor.modules.globalClass import globalClass
from WordForge.translate.storage import factory
from WordForge.translate.misc import quote
import os, time

class newProject(QtGui.QDialog):
    """
    This module implementation with newProject, openProject and openrecentProject
    """
    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self.parent=parent
        self.ui = Ui_NewProject()
        self.ui.setupUi(self)
        self.ui.btnOK.setEnabled(True)
        self.getPrefernce=self.parent.parent.preference
        self.connect(self.ui.btnOK, QtCore.SIGNAL("clicked()"), self.accept)
        self.connect(self.ui.btnCancel, QtCore.SIGNAL("clicked()"), QtCore.SLOT("close()"))

        # call dialog box of FileDialog
#        self.connect(self.ui.btnAdd, QtCore.SIGNAL("clicked()"), self.getFileOrDiretory)
        self.connect(self.ui.btnAdd, QtCore.SIGNAL("clicked()"), self.addFiles)
        self.connect(self.ui.btnAddDir, QtCore.SIGNAL("clicked()"), self.addFolder)
        self.connect(self.ui.btnRemove, QtCore.SIGNAL("clicked()"), self.removeLocation)
        self.connect(self.ui.btnClear, QtCore.SIGNAL("clicked()"), self.clearLocation)
        self.connect(self.ui.btnMoveUp, QtCore.SIGNAL("clicked(bool)"), self.moveUp)
        self.connect(self.ui.btnMoveDown, QtCore.SIGNAL("clicked(bool)"), self.moveDown)

        self.connect(self.ui.projectName, QtCore.SIGNAL("textChanged(QString)"), self.enableAccept)
        self.connect(self.ui.listLocation, QtCore.SIGNAL("currentItemChanged(QListWidgetItem *, QListWidgetItem *)"), self.enableDeleteButton)
#        self.connect(self.ui.listLocation, QtCore.SIGNAL("itemSelectionChanged()"), self.enableDeleteButton)
#        self.connect(self.ui.listLocation, QtCore.SIGNAL("itemChanged(QListWidgetItem)"), self.enableDeleteButton)

        # language code of the country
        self.languageList=[]
        self.countryList=[]
        self.languageList=self.getPrefernce.load_language_ISO2(self.ui.cattargetlang)
        self.countryList=self.getPrefernce.load_countries(self.ui.cattargetcontry)
        self.connect(self.ui.cattargetlang, QtCore.SIGNAL("currentIndexChanged(const QString &)"), self.moreLang)
        
        self.projectFilename = ""
        self.projectName = ""
        self.targetlang = ""
        globalClass.projectlanguage=self.targetlang
        self.path = []
        self.includeSub = False
        self.ui.btnMoveUp.setVisible(False)
        self.ui.btnMoveDown.setVisible(False)
        #current language settings s
        # determine a directory for project files
        homedir = os.path.expanduser('~')
        sep = os.path.sep
#        self.defaulthProjectPath = homedir + sep + World.logDir + sep + "projects" + sep
        self.defaulthProjectPath = World.configDir + sep + "projects" + sep
        if (not os.path.exists(self.defaulthProjectPath)):
            os.makedirs(self.defaulthProjectPath)
    
    def moreLang(self, languageCode):  
        #shows the languages list  with 3 characters language code
        if languageCode=="More Languages":
            self.getPrefernce.load_language_ISO3(self.ui.cattargetlang)
        else:
            return
            
    def addFiles(self):
        """
        Open the file dialog where you can choose files to add to project.
        @msg: waiting an dialog box will appear up when user click to add item into listLocation.
        """
        self.ui.listLocation.insertItem(0, "Please wait...")
        
        directory = World.settings.value("workingDir").toString()
        filenames = QtGui.QFileDialog.getOpenFileNames(self, self.tr("Add files to project"),
                directory, ";;".join(World.fileFilters))
        
        if (filenames):
            for filename in filenames:
                self.addLocation(filename)
            directory = os.path.dirname(unicode(filenames[0]))
            World.settings.setValue("workingDir", QtCore.QVariant(directory))
        self.ui.listLocation.takeItem(0)
        self.enableSave()
    
    def addFolder(self):
        """
        Open the file dialog where you can choose a folder to add to project.
        @msg: waiting an dialog box will appear up when user click to add item into listLocation.
        """
        self.ui.listLocation.insertItem(0, "Please wait...")
        
        directory = World.settings.value("workingDir").toString()
        folderPath = QtGui.QFileDialog.getExistingDirectory(self, self.tr("Add folder to project"), directory)
        
        if (folderPath):
            # remove separator at the end of string
            folderPath = str(folderPath).rstrip(os.path.sep)
            self.addLocation(folderPath)
            directory = os.path.dirname(folderPath)
            World.settings.setValue("workingDir", QtCore.QVariant(directory))
        
        self.ui.listLocation.takeItem(0)
        self.enableSave()
#    def getFileOrDiretory(self):
#        """
#        Open the file dialog where you can choose both file and directory.
#        Add path to Catalog list.
#        @msg: waiting an dialog box will appear up when user click to add item into listLocation.
#        """
#        msg = "Please wait..."
#        item = QtGui.QListWidgetItem(msg)
#        self.ui.listLocation.addItem(item)
#
#        directory = World.settings.value("workingDir").toString()
#        filenames = FileDialog.fileDialog().getExistingPath(
#                self,
#                directory,
#                World.fileFilters)
#        if (filenames):
#            self.ui.listLocation.takeItem(self.ui.listLocation.count() -1)
#            for filename in filenames:
#                self.addLocation(filename)
#            directory = os.path.dirname(unicode(filenames[0]))
#            World.settings.setValue("workingDir", QtCore.QVariant(directory))
#        else:
#            self.ui.listLocation.takeItem(self.ui.listLocation.count() -1)

#    def getFilename(self):
#        directory = World.settings.value("workingDir").toString()
#        dirname = QtGui.QFileDialog.getExistingDirectory(self,
#                    self.tr("Select directory"),
#                    directory,
#                    QtGui.QFileDialog.ShowDirsOnly
#                    | QtGui.QFileDialog.DontResolveSymlinks)
#
#        lblEdit = self.ui.projectName.text()
#        if (lblEdit):
#            if (not lblEdit.endsWith(".ini", QtCore.Qt.CaseInsensitive)):
#                lblEdit = lblEdit + ".ini"
#        else:
#            self.ui.projectName.setFocus()
#        # Concatenate string between directory path and filename
#        filename = os.path.join(dirname + lblEdit)
#        filename = unicode(filename)
#        self.ui.entryPath.setText(filename)
#        directory = os.path.dirname(filename)
#        World.settings.setValue("workingDir", QtCore.QVariant(directory)) 

    def addLocation(self, path):
        """
        Add path to location list.
        """
        items = self.ui.listLocation.findItems(path, QtCore.Qt.MatchCaseSensitive)
        if (not items):
            item = QtGui.QListWidgetItem(path)
            self.ui.listLocation.addItem(item)

    def clearLocation(self):
        """
        Clear all paths from location list, uncheck include sub combobox.
        """
        self.ui.listLocation.clear()
        self.ui.checkIncludeSub.setChecked(False)
        self.ui.checkIncludeSub.setChecked(False)
        self.ui.btnOK.setEnabled(False)
        
    def removeLocation(self):
        """
        Remove the selected path from the location list.
        """
        self.ui.listLocation.takeItem(self.ui.listLocation.currentRow())
        self.enableSave()
    def moveItem(self, distance):
        """
        Move an item up or down depending on distance        
        @param distance: int
        """
        currentrow = self.ui.listLocation.currentRow()
        currentItem = self.ui.listLocation.item(currentrow)
        distanceItem = self.ui.listLocation.item(currentrow + distance)
        if (distanceItem and currentItem):
            temp = distanceItem.text()
            distanceItem.setText(currentItem.text())
            currentItem.setText(temp)
            self.ui.listLocation.setCurrentRow(currentrow + distance)

    def moveUp(self):
        """
        Move item up by 1 distance.
        """
        self.moveItem(-1)

    def moveDown(self):
        """
        Move item down by 1 distance
        """
        self.moveItem(1)

    def accept(self):
        """
        Save and open new project, or emit project's properties depend on
        show mode.
        """
        if (self.mode == World.projectModeNew):
            self.projectName = self.ui.projectName.text()
            self.targetlang=self.getPrefernce.striplangcontryname(self.ui.cattargetlang.currentText(), self.ui.cattargetcontry.currentText())
            globalClass.projectlanguage=self.targetlang
            self.path = QtCore.QStringList()
            for i in range(self.ui.listLocation.count()):
                self.path.append(self.ui.listLocation.item(i).text())
            self.includeSub = self.ui.checkIncludeSub.isChecked()
            self.getPrefernce.fileswithnolanguage=[]
            self.getPrefernce.fileswithdifflanguage=[]   
            self.purpose="newproject"       
            for path in self.path:
                filelanguage=self.getPrefernce.checkfileLanguage(path, "newproject")
            if self.getPrefernce.fileswithnolanguage or self.getPrefernce.fileswithdifflanguage:          
                flag=self.Newprojectwanring(self.path)  
            
            if (self.projectName):
                self.prjRoot = None
                self.projectFilename = self.getNewProjectFilename()
                self.saveProject(self.projectFilename)
                self.openProject(self.projectFilename)

        # save project and re-open
        elif (self.mode == World.projectModeProperty):
            modified = self.detectChanges()
            if (modified):
                self.projectName = self.ui.projectName.text()
                self.targetlang=self.getPrefernce.striplangcontryname(self.ui.cattargetlang.currentText(), self.ui.cattargetcontry.currentText())
                globalClass.projectlanguage=self.targetlang
                self.path = QtCore.QStringList()
                for i in range(self.ui.listLocation.count()):
                    self.path.append(self.ui.listLocation.item(i).text())
                self.includeSub = self.ui.checkIncludeSub.isChecked()
                self.saveProject(self.projectFilename)
                self.openProject(self.projectFilename)

        elif (self.mode == World.projectModeOpen):
            item = self.ui.listLocation.item(self.ui.listLocation.currentRow())
            project = item.data(QtCore.Qt.UserRole).toString()
            self.openProject(project)

        elif (self.mode == World.projectModeDelete):
            item = self.ui.listLocation.item(self.ui.listLocation.currentRow())
            if (item and hasattr(item, "data")):
                project = item.data(QtCore.Qt.UserRole).toString()
                currentProject = World.settings.value("CurrentProject").toString()

                # close the project
                if (project == currentProject):
                    ret = QtGui.QMessageBox.question(self, self.tr("Warning"),
                        self.tr("You have to close the current project before you can delete it.\n"
                                "Do you want to continue?"),
                        QtGui.QMessageBox.Yes | QtGui.QMessageBox.Default,
                        QtGui.QMessageBox.No)

                    if (ret == QtGui.QMessageBox.Yes):
                        self.closeProject()
                        os.remove(project)
                        QtGui.QMessageBox.information(self, self.tr("Information"), self.tr("You have deleted a project."))
                    else:
                        return False
                else:
                    os.remove(project)
                    QtGui.QMessageBox.information(self, self.tr("Information"), self.tr("You have deleted a project."))

        self.close()

    def openProject(self, filename):
        """
        Open a .prj project file, store its properties, save current project entry to World.

        @param filename: the xml project file (.prj) to read.
        @return bool: indicates the function has accept or reject.
        """
        if (not filename):
            # show a open dialog for choosing a project
            filename = self.showDialogOpen()
            return False

        # Set current project
        World.settings.setValue("CurrentProject", QtCore.QVariant(filename))

        # open .prj xml project file
        if os.path.isfile(filename):
            prjFile = open(filename, "r")
            prjXmlData = prjFile.read()
            prjFile.close()

            prjRoot = etree.fromstring(prjXmlData)

            # store project's properties
            self.projectFilename = filename
            self.projectName = prjRoot.get("name")
            self.targetlang = prjRoot.get("xmllang")
            globalClass.projectlanguage=self.targetlang
            self.includeSub = bool(prjRoot.get("includeSub") == "True") or False 
            self.path = []

            for child in prjRoot:
                if (child.tag == "catalog"):
                    catalog = child.get("path")
                    if catalog:
                        self.path.append(catalog)

            self.emit(QtCore.SIGNAL("updateCatalog"), prjRoot)
            return True
        return False

    def closeProject(self):
        """
        Clear properties. Remove CurrentProject entry from World.
        """
        World.settings.setValue("CurrentProject", QtCore.QVariant(""))

        # Clear project properties
        self.projectFilename = ""
        self.projectName = ""
        self.targetlang = ""
        globalClass.projectlanguage=self.targetlang
        self.path = []
        self.includeSub = False

        # send a signal to Catalog, so it will clear its content.
        self.emit(QtCore.SIGNAL("updateCatalog"), None)
        return True

    def saveAsProject(self):
        directory = World.settings.value("workingDir").toString()
        filename = QtGui.QFileDialog.getSaveFileName(self,
                self.tr("Save File As"),
                directory, self.tr("Project File (*.prj)"))
        if (filename):
            if (not filename.endsWith(".prj", QtCore.Qt.CaseInsensitive)):
                filename = filename + ".prj"
            self.projectFilename = filename
        else:
            return

        World.settings.setValue("CurrentProject", QtCore.QVariant(filename))
        proSettings = QtCore.QSettings(filename, QtCore.QSettings.IniFormat)
        proSettings.setValue("language", QtCore.QVariant(self.targetlang))
        proSettings.setValue("path", QtCore.QVariant(self.path))
        proSettings.setValue("includeSub", QtCore.QVariant(self.includeSub))
        self.openProject(filename)

    def saveProject(self,  filename = None):
        """
        Save a .prj xml project file to filename. Project will have: name, language, path,
        and includeSub.

        @param filename: file name to save to.
        """
        
        if (not filename):
            filename = World.settings.value("CurrentProject").toString()
        World.settings.setValue("CurrentProject", QtCore.QVariant(filename))

        if (hasattr(self, "prjRoot")) and (self.prjRoot) :
            self.prjRoot.set("name", str(self.projectName))
            self.prjRoot.set("xmllang", str(self.targetlang))
            self.prjRoot.set("includeSub", str(self.includeSub))
            # remove all catalog paths in self.prjRoot
            for catalog in self.prjRoot:
                if catalog.tag == "catalog":
                    self.prjRoot.remove(catalog)
        else:
            # create root for .prj xml file
            self.prjRoot = etree.Element("project", name=str(self.projectName), xmllang=str(self.targetlang), includeSub=str(self.includeSub))
        # add path to prjRoot
        
        for path in self.path:
            filelanguage=self.getPrefernce.checkfileLanguage(path, "newproject")
            catPath = etree.SubElement(self.prjRoot, "catalog", path=unicode(path)) 
        xmldata = etree.tostring(self.prjRoot, pretty_print=True)
        prjFile = open(filename,  "w")
        prjFile.write(xmldata)
        prjFile.close()
    
    def changelanguage(self, pathlist):
         # changes the translation language code of the files of Tm or project to the langueg specified in the corresponding window. 
        for file in pathlist:
            file=str(file)
            if (file.endswith(".xlf") or file.endswith(".xliff")):
                isXliff=True
            elif (file.endswith(".po")): 
                isXliff=False
            else:
                continue
            if (file in self.getPrefernce.fileswithnolanguage) or (file in self.getPrefernce.fileswithdifflanguage):     
                if isXliff:
                    store = factory.getobject(file)
                    if self.purpose=="newproject":
                        store.settargetlang(self.targetlang)
                    elif self.purpose=="TM":
                        store.settargetlang(globalClass.TMGLStargetLanguageCode)
                    store.savefile(file) 
                else:
                    if self.purpose=="newproject":
                        self.getPrefernce.currentpoheaderDic["X-Language"]=self.targetlang
                    elif self.purpose=="TM":
                        self.getPrefernce.currentpoheaderDic["X-Language"]=globalClass.TMGLStargetLanguageCode
                    self.getPrefernce.headerpo.msgstr = ['""']
                    for (key, value) in self.getPrefernce.currentpoheaderDic.items():
                        self.getPrefernce.headerpo.msgstr.append(quote.quotestr("%s: %s\\n" % (key, value)))    
                    self.getPrefernce.postore.savefile(file)     
        self.getPrefernce.fileswithnolanguage=[]
        self.getPrefernce.fileswithdifflanguage=[]
        self.postore=None
        self.getPrefernce.headerpo=None
        self.getPrefernce.currentpoheaderDic={}
    
    def Newprojectwanring(self, pathlist):
        #shows warning for missing or mismatched langauge code.  
        messageBox = QtGui.QMessageBox
        fileswithnolanguage= self.trUtf8("Among the files you specified some files do not have any translation language. Do you wish to save <b>"+ self.targetlang + "</b> in these files(Recommended)?")
        
        fileswithdifflanguage = self.trUtf8("Among the files you specified some files whose translation language do not match with the language specified while creating this project (<b>"+ self.targetlang + "</b>). Do you wish to change those language to <b>"+ self.targetlang + "</b> in these files(Recommended)?")
        
        fileslanguage = self.trUtf8("Among the files you specified some file do not have any translation language. Some files' translation language do not match  with the language specified while creating this project (<b>"+ self.targetlang + "</b>). the Do you wish to save <b>"+ self.targetlang + "</b> in these files(Recommended)?")
        ret = messageBox()
        ret.setTextFormat(QtCore.Qt.RichText)
        if self.getPrefernce.fileswithnolanguage and self. getPrefernce.fileswithdifflanguage:
            ret.setText(fileslanguage)
        elif self.getPrefernce.fileswithnolanguage and not self.getPrefernce.fileswithdifflanguage:
            ret.setText(fileswithnolanguage)
        elif self.getPrefernce.fileswithdifflanguage and not self.getPrefernce.fileswithnolanguage:
            ret.setText(fileswithdifflanguage)    
        YesButton = ret.addButton(self.trUtf8("Yes"),messageBox.YesRole)
        NoButton = ret.addButton(self.trUtf8("No"),messageBox.NoRole)
        ret.exec_()

        if (ret.clickedButton() == YesButton):
            self.changelanguage(pathlist)
            return True
        elif (ret.clickedButton() == NoButton):
            return False
       
    def setPrjRoot(self, prjRoot):
        """
        set project xml data to self.prjRoot
        """
        self.prjRoot = prjRoot
    
    def showDialog(self, mode = "mode"):
        """
        Show a specific dialog mode.
        """
        if (mode == World.projectModeNew):
            # show new project dialog.
            self.mode = mode
            self.configureDialog(mode)
            # set default values
            self.languageList=[]
            self.countryList=[]
            self.languageList=self.getPrefernce.load_language_ISO2(self.ui.cattargetlang)
            self.countryList=self.getPrefernce.load_countries(self.ui.cattargetcontry)
            currentlang=World.settings.value("FullLanguage",  QtCore.QVariant("Bengali [bn]")).toString()
            currentlang= currentlang +"\n"
            j=-1
            for i in self.languageList:
                j=j+1
                if(i==currentlang):
                    break
            self.ui.cattargetlang.setCurrentIndex(j)
            currentcountry=World.settings.value("perCountryName",  QtCore.QVariant("BANGLADESH [BD]")).toString()
            j=-1
            for i in self.countryList:
                j=j+1
                if(i==currentcountry):
                    break
            self.ui.cattargetcontry.setCurrentIndex(j)
            
            self.clearLocation()
            self.ui.checkIncludeSub.setChecked(True)

        elif (mode == World.projectModeOpen):
            # pop up a dialog for choosing project.
            self.mode = mode
            self.configureDialog(mode)

            # read project files and add to a pop up dialog
            self.ui.listLocation.clear()
            filenameList = self.getProjectFilenameList()
            for location in filenameList:
                prjFile = open(location, "r")
                prjXmlData = prjFile.read()
                prjFile.close()
                prjRoot = etree.fromstring(prjXmlData)
                projectName = prjRoot.get("name")
                item = QtGui.QListWidgetItem(projectName)
                self.ui.listLocation.addItem(item)
                item.setData(QtCore.Qt.UserRole, QtCore.QVariant(location))

        elif (mode == World.projectModeProperty):
            # show current project properties.
            self.mode = mode
            self.configureDialog(mode)

#            if (not self.projectFilename):
#                filename = World.settings.value("CurrentProject").toString()
#                self.openProject(filename)

            # fill in properties
            
            self.ui.projectName.setText(self.projectName)
            
            list=self.targetlang.split("_")
            language="["+list[0] + "]"
            country="["+list[1] + "]"
            self.getPrefernce.setcurrentItem(self.ui.cattargetlang, language, "target")
            self.getPrefernce.setcurrentItem(self.ui.cattargetcontry, country, "target")
            
#            self.clearLocation()
            for location in self.path:
                self.addLocation(location)
            self.ui.checkIncludeSub.setChecked(self.includeSub)

        elif (mode == World.projectModeDelete):
            # pop up a dialog for deleting project.
            self.mode = mode
            self.configureDialog(mode)

            # read project files and add to a pop up dialog
            self.ui.listLocation.clear()
            filenameList = self.getProjectFilenameList()
            for location in filenameList:
                prjFile = open(location, "r")
                prjXmlData = prjFile.read()
                prjFile.close()
                prjRoot = etree.fromstring(prjXmlData)
                projectName = prjRoot.get("name")
                item = QtGui.QListWidgetItem(projectName)
                self.ui.listLocation.addItem(item)
                item.setData(QtCore.Qt.UserRole, QtCore.QVariant(location))

        self.show()

    def configureDialog(self, mode = "mode"):
        """
        Show and hide ui elements for specific dialog mode.
        """
        if (mode == World.projectModeNew) or (mode == World.projectModeProperty):
            listTitle = "Project files<br>and folders:"
            self.ui.lblName.setVisible(True)
            self.ui.projectName.setVisible(True)
            self.ui.lbltargetlanguage.setVisible(True)
            self.ui.cattargetlang.setVisible(True)
            self.ui.cattargetcontry.setVisible(True)
            self.ui.btnAdd.setVisible(True)
            self.ui.btnAddDir.setVisible(True)
            self.ui.btnRemove.setVisible(True)
            self.ui.btnClear.setVisible(True)
#            self.ui.btnMoveUp.setVisible(True)
#            self.ui.btnMoveDown.setVisible(True)
            self.ui.checkIncludeSub.setVisible(True)
            self.resize(578,401)

        elif (mode == World.projectModeOpen) or (mode == World.projectModeDelete):
            listTitle = "Project name:"
            self.ui.lblName.setVisible(False)
            self.ui.projectName.setVisible(False)
            self.ui.lbltargetlanguage.setVisible(False)
            self.ui.cattargetlang.setVisible(False)
            self.ui.cattargetcontry.setVisible(False)
            self.ui.btnAdd.setVisible(False)
            self.ui.btnAddDir.setVisible(False)
            self.ui.btnRemove.setVisible(False)
            self.ui.btnClear.setVisible(False)
#            self.ui.btnMoveUp.setVisible(False)
#            self.ui.btnMoveDown.setVisible(False)
            self.ui.checkIncludeSub.setVisible(False)
            self.ui.line.setVisible(False)
            self.show()
            self.resize(400,300)

        if (mode == World.projectModeNew):
            windowTitle = "New Project"
            headerTitle = "Creation of a new project"
            self.ui.btnOK.setText(self.tr("&Save"))
            self.ui.projectName.setText("")
            self.ui.projectName.setEnabled(True)
            # disable the save button
            self.ui.btnOK.setEnabled(False)

        elif (mode == World.projectModeOpen):
            windowTitle = "Open Project"
            headerTitle = "Choose a project to open:"
            self.ui.btnOK.setText(self.tr("&Open"))
            # enable the open button
            self.ui.btnOK.setEnabled(True)
            self.ui.line.setVisible(False)
            
        elif (mode == World.projectModeProperty):
            windowTitle = "Project Properties"
            headerTitle = "Property"
            self.ui.btnOK.setText(self.tr("&Save and Reload"))
            self.ui.projectName.setEnabled(False)
            # enable the open button
            self.ui.line.setVisible(False)

        elif (mode == World.projectModeDelete):
            windowTitle = "Delete Project"
            headerTitle = "Choose a project to delete:"
            self.ui.btnOK.setText(self.tr("&Delete"))
            # disable the delete button
            self.ui.btnOK.setEnabled(False)
            self.ui.line.setVisible(False)

        self.setWindowTitle(self.tr(windowTitle))
        self.ui.projectTitle.setText(self.tr(headerTitle))
        self.ui.listTitle.setText(self.tr(listTitle))

    def getNewProjectFilename(self):
        """
        Return a generated filename of project###.prj in a default project path.
        """
        for num in range(100):
            filename = self.defaulthProjectPath + "project" + ("%03d" % num) + ".prj"
            if (not os.path.exists(filename)):
                break
        return filename

    def getProjectFilenameList(self):
        """
        Return a list of project###.prj in a default project path.
        """
        filenameList = []
        for num in range(100):
            filename = self.defaulthProjectPath + "project" + ("%03d" % num) + ".prj"
            if (os.path.exists(filename)):
                filenameList.append(filename)
        return filenameList

    def detectChanges(self):
        """
        Check if changes have been made to project, return True or False.
        """
        nameModified = False
        langModified = False
        
        # 
        if not World.settings.value("CurrentProject").toString():
            return False
        
        if (self.projectName != self.ui.projectName.text()):
            nameModified = True
        list=self.targetlang.split("_")
        language="["+list[0] + "]"
        country="["+list[1] + "]"
        if ((language not in self.ui.cattargetlang.currentText()) and (country not in self.ui.cattargetcontry.currentText())):
            langModified = True
        
        pathModified = False
        stringlist = QtCore.QStringList()
        for i in range(self.ui.listLocation.count()):
            stringlist.append(self.ui.listLocation.item(i).text())
        if (self.path != stringlist):
            pathModified = True
        
        if (self.includeSub != self.ui.checkIncludeSub.isChecked()):
            pathModified = True
        
        return nameModified or langModified or pathModified

    def enableAccept(self):
        #enable ok button when there is a project name and some files included in the project
        if (self.mode == World.projectModeNew):
            if ((self.ui.projectName.text() != "") and (self.ui.listLocation.count() > 0)):
                self.ui.btnOK.setEnabled(True)
            else:
                self.ui.btnOK.setEnabled(False)

    def enableDeleteButton(self):
        #enable delete button if there is a project selected for deletion
        if (self.mode == World.projectModeDelete):
            if (self.ui.listLocation.count() > 0):
                self.ui.btnOK.setEnabled(True)

    def enableSave(self):  
        if (self.mode == World.projectModeNew):
            if ((self.ui.projectName.text() != "") and (self.ui.listLocation.count() > 0)):    
                self.ui.btnOK.setEnabled(True)


if __name__ == "__main__":
    import os, sys
    app = QtGui.QApplication(sys.argv)
    projectDialog = newProject(None)
    projectDialog.showDialog("mode")
    sys.exit(projectDialog.exec_())

