#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2006 - 2007 by The WordForge Foundation
#                       www.wordforgefoundation.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
# 
# See the LICENSE file for more details. 

# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)
#       Aditi Barua(aditi@ankur.org.bd)
#       Asheq Hamid(asheq@ankur.org.bd)
#       Md. Rezwan Shahid(rezwan@ankur.org.bd)
#
# This module is working on Operator.

from PyQt4 import QtCore, QtGui
from WordForge.translate.storage import po
from WordForge.translate.misc import quote
from WordForge.translate.storage import poheader
from WordForge.translate.storage import xliff
from WordForge.translate.storage import tbx
from WordForge.translate.storage import pypo
import WordForge.Editor.modules.World as World
from WordForge.Editor.modules.PickleTM import PickleTM
from WordForge import __version__
import os, sys
import re, string, sets
from WordForge.translate.search import match
from WordForge.Editor.modules import InternalGlossary
from WordForge.Editor.modules.Overview import OverviewDock
from WordForge.Editor.modules.NewGlossaryTerm import NewGlossaryTerm
from WordForge.Editor.modules.globalClass import globalClass
from WordForge.Editor.modules.Preference import Preference
from lxml import etree
import platform
from subprocess import Popen, PIPE, STDOUT, mswindows


import enchant
from threading import Thread

reload(sys)
sys.setdefaultencoding("utf-8")


if (platform.system() == "Linux"):
    localpath = os.path.split(os.path.realpath( __file__ ))[0] +os.path.sep
else:
    localpath = ""
    
debug = World.WF_debug

class Operator(QtCore.QObject):
    """
    Operates on the internal datastructure.
    The class loads and saves files and navigates in the data.
    Provides means for searching and filtering.
    
    @signal unitsStatus(string): emitted with the new status message
    @signal newUnits(store.units): emitted with the new units
    @signal currentUnit(unit): emitted with the current unit
    @signal updateUnit(): emitted when the views should update the units data
    @signal toggleFirstLastUnit(atFirst, atLast): emitted to allow dis/enable of actions
    @signal filterChanged(filter, lenFilter): emitted when the filter was changed
    @signal readyForSave(False): emitted when a file was saved
    """
    
    def __init__(self, mainEditor):
        if World.WF_debug:
            print "Entering __init__ in class Operator"
        QtCore.QObject.__init__(self)
        self.__mainEditor__ = mainEditor
        self.mainEditor = mainEditor
        self.__store__ = None
        self.__currentUnitIndex__ = 0
        self.__filteredList__ = []
        self.__filter__ = None
        self.glossaryChanged = True
        self.__autoLookupUnit__ = False
        self.__isTMTableVisible__ = False
        self.__termMatcher__ = None
        self.__tmMatcher__ = None
        self.__termCache__ = {} # for improve glossary search speed
        self.__tmCache__ = {} # for improve TM search speed
        self.foundString = None
        self.lastFoundUnit = None
        self.searchStartIndex = 0
        self.searchFields = None
        self.__status__ = None
#        self.autoSpell = None
        self.dictlanguage = None
        self.missSpelledList = None
        self.suggestedList = None
        self.TMDict = {}
        self.dictIsExist = False
        self.isXliff=True
        self.langcur=""
        self.currentfileName=""
        self.filepath4error=""
        # indicator if the current unit was modified
        self.modified = False
        self.changedcontent=False
        self.usingGLb4TM = False
        self.sourcelang=''

        # initialize Translation Memory/TM Matcher
        if (not self.__tmMatcher__):
            World.settings.beginGroup("TM")
            pickleFile = World.settings.value("pickleFile").toString()
            World.settings.endGroup()
            # if application remember TM file name and file really exist
            if (pickleFile and os.path.exists(pickleFile)):
                p = PickleTM()
                self.__tmMatcher__ = p.getMatcher(pickleFile)
                
        # initialize glossary/term Matcher
        if (not self.__termMatcher__):
            World.settings.beginGroup("Glossary")
            pickleFile = World.settings.value("pickleFile").toString()
            World.settings.endGroup()
            if (pickleFile and os.path.exists(pickleFile)): # file really exists
                p = PickleTM()
                self.__termMatcher__ = p.getMatcher(pickleFile)
                
        self.__standardGlossary__ = None
        
        # FileTM preference read from the init file
        self.__AutoTargetInclusion__ = World.settings.value("TM_TargetInclusion", QtCore.QVariant(False)).toBool()
        self.__minAddAltTrans__ = World.settings.value("TM_min", QtCore.QVariant("70")).toInt()[0]
        self.__maxAddAltTrans__ = World.settings.value("TM_max", QtCore.QVariant("100")).toInt()[0]
        self.max_candidates = World.settings.value("TM_max_candidates", QtCore.QVariant("10")).toInt()[0]
        self.max_string_len = World.settings.value("TM_max_string_len", QtCore.QVariant("200")).toInt()[0]
        self.__AutoAddAltsTran__ = World.settings.value("AutoAddAltsTran", QtCore.QVariant(False)).toBool()
        self.__eliminateIdenticalSug__ = World.settings.value("TM_EliminateIdentical",  QtCore.QVariant(False)).toBool()
        self.__autoLookupUnit__ = World.settings.value("TM_Instant").toBool()
        
        # Target Language for TM and Glossary
        self.TMTargetLanguage = World.settings.value("TMTargetLanguage").toString()
        self.GlossaryTargetLanguage = World.settings.value("GlossaryTargetLanguage").toString()

#       for GLb4TM
        self.__useGLb4TM__ = World.settings.value("TM_GLb4TM", QtCore.QVariant(True)).toBool()
        self.__FuzzyGLb4TM__ = World.settings.value("FuzzyGLb4TM", QtCore.QVariant(False)).toBool()
        self.AcceleratorMark = World.settings.value("CurrentUserDefineAccelerator").toString()
        
        self.linuxPlatform = platform.system() == "Linux"
        self.windowsPlatform = mswindows
        self.hunspellDir = "..\\..\\hunspell-1.2.8\\"
        self.autoSpell = World.settings.value("AutoSpellCheck",  QtCore.QVariant(False)).toBool()
        self.spellCheckIgnoreSource = World.settings.value("notHighlightInSource", QtCore.QVariant(True)).toBool()
        self.dictObject = None
        if (self.autoSpell):
            self.autoSpellCheck(True)
#      This flag is true if an internal glossary is being used.      
        self.internalGlossaryUsed = False
        self.newGlossaryTerm = QtGui.QDialog()

    def getUnits(self, fileName):
        """
        parseing the file
        @param fileName the file to be parse, either a string or a file object.
        @return True of False indicates success open of file.
        """
        self.currentfileName=fileName
        if (fileName.endswith(".xlf") or fileName.endswith(".xliff")):
            self.isXliff=True
        else:
            self.isXliff=False
        from WordForge.translate.storage import factory
        try:
            store = factory.getobject(fileName)
        except Exception, e:
            QtGui.QMessageBox.critical(None, self.trUtf8('Error'), self.trUtf8('Error while trying to open file ') + fileName + '\n' + str(e))
            return False
        self.setFileName(fileName)
        self.__setNewStore__(store)
        self.store=store
        self.getsourcelanguage(fileName) 
        if (fileName.endswith(".xlf") or fileName.endswith(".xliff") or fileName.endswith(".po")):
            self.setfiletranslationlanguage(fileName) 
        
#        for unit in store.units:
#            print "getUnits here", unit.source        
        if not isinstance(self.__store__, xliff.xlifffile):    
            self.isXliff=False
        if isinstance(self.__store__, xliff.xlifffile):
            self.isXliff=True
            self.internalGlossary = self.getInternalGlossary(fileName)
                  
#        else:    
#            self.isXliff=False
            if self.internalGlossary:    # if there is an internal glossary in the xliff file
                if (self.__termMatcher__):  # if there is a standard glossary loaded in the WordForge
                    ret = QtGui.QMessageBox.question(None, self.trUtf8("Glossary"), self.trUtf8("This file has an internal glossary embedded. Do you wish to use the internal glossary of this file (recommended)? Otherwise the glossary that is already loaded in the WordForge editor will be used."),  QtGui.QMessageBox.Yes | QtGui.QMessageBox.Default,  QtGui.QMessageBox.No | QtGui.QMessageBox.Escape)
                else :
                    ret = QtGui.QMessageBox.Yes     #if there is no standard glossary, use the internal glossary
                    
                if (ret == QtGui.QMessageBox.Yes):  #use internal glossary and keep the nornmal glossary in the variable self.__standardGlossarY__
                    self.__standardGlossary__ = self.__termMatcher__
                    self.__termMatcher__ = match.terminologymatcher(self.internalGlossary)
                    self.__termCache__ = {}         #emtpy the cache for highlighting word
                    self.glossaryChanged= True      # this ensure highlight is correct.
                    self.applySettings()   
                    self.internalGlossaryUsed = True   
                else:
                    self.internalGlossaryUsed = False
        return True
    
    def __setNewStore__(self, store):
        """
        Setup the internal structure of parsed file
        
        @param store the new storage class of parsed file
        """
        if World.WF_debug:
            print "Entering __setNewStore__ in class Operator"
            
        self.setFileOpenedObject(store)
        # filter flags
        self.__filter__ = World.filterAll
        units = self.__store__.units
        from WordForge.Editor.modules.Status import Status
        # get status for units
        self.setStatusObject(Status(self.__store__))
        
        for unit in units:
            # set x_editor_state for all units.
            if (unit.source and not (isinstance(self.__store__, xliff.xlifffile) and ( "gettext-domain-header" in (unit.getrestype() or "")))):   #if the unit is not the header
                unit.x_editor_state = self.__status__.unitState(unit)
                unit.x_editor_filterIndex = len(self.__filteredList__)
                self.__filteredList__.append(unit)   
        self.__emitNewUnits__()
        self.setUnitFromPosition(0, 0)

        self.emit(QtCore.SIGNAL("spellCheckMaximum"), len(self.__filteredList__))
        
    def __emitNewUnits__(self):
        """
        Emit "newUnits" signal with a list of units.
        """
        if World.WF_debug:
            print "Entering __emitNewUnits__ in class Operator"
        self.emit(QtCore.SIGNAL("newUnits"), self.__filteredList__)
        
    def __emitUnitsStatus__(self):
        """
        Emit "unitsStatus" signal with a string contains current, total, fuzzy,
        translated, and untranslated messages of current file.
        """
        if World.WF_debug:
            print "Entering __emitUnitsStatus__ in class Operator"
            
            
        if (self.__status__):
            self.emit(QtCore.SIGNAL("unitsStatus"),  self.__status__)
        
    def emitUnit(self, unit):
        """
        Emit "currentUnit" signal with a unit.
        @param unit: class unit.
        """
        if World.WF_debug:
            print "Entering emitUnit in class Operator"
        if (hasattr(unit, "x_editor_filterIndex")):
            self.__currentUnitIndex__ = unit.x_editor_filterIndex
#            print "emitunit", unit.source,  self.__currentUnitIndex__
            self.emit(QtCore.SIGNAL("currentUnit"), unit)
            self.__emitUnitsStatus__()
        
    def getCurrentUnit(self):
        """
        @return the current unit.
        """
        if World.WF_debug:
            print "Entering getCurrentUnit in class Operator"
            
        if (self.__currentUnitIndex__ < len(self.__filteredList__)):
            return self.__filteredList__[self.__currentUnitIndex__]
        else:
            return None

      
            
    def filterFuzzy(self, bool):
        """
        Add/remove fuzzy to filter, and send filter signal.
        @param bool True or False when Fuzzy checkbox is checked or unchecked.
        """
        
        if World.WF_debug:
            print "Entering filterFuzzy in class Operator"
            
        filter = self.__filter__
        if (not bool):
            filter |= World.fuzzy
        elif (filter):
            filter &= ~World.fuzzy
        self.emitFiltered(filter)
    
    def filterTranslated(self, bool):
        """
        Add/remove translated to filter, and send filter signal.
        @param bool True or False when Translated checkbox is checked or unchecked.
        """
        
        if World.WF_debug:
            print "Entering filterTranslated in class Operator"
            

        filter = self.__filter__
        if (not bool):
            filter |= World.translated
        elif (filter):
            filter &= ~World.translated
        self.emitFiltered(filter)
    
    def filterUntranslated(self, bool):
        """
        Add/remove untranslated to filter, and send filter signal.
        @param bool True or False when Untranslated checkbox is checked or unchecked.
        """
        
        if World.WF_debug:
            print "Entering filterUntranslated in class Operator"
            
        filter = self.__filter__
        if (not bool):
            filter |= World.untranslated
        elif (filter):
            filter &= ~World.untranslated
        self.emitFiltered(filter)
    
    def emitFiltered(self, filter):
        """
        Emit "filterChanged" signal with a filter and length of filtered list.
        """
        
        if World.WF_debug:
            print "Entering emitFiltered in class Operator"
            

        self.emitRequestTargetChanged()
        
        # remember unit before filtering
        if len(self.__filteredList__):
            unitBeforeFiltered = self.__filteredList__[self.__currentUnitIndex__]
        else:
            unitBeforeFiltered = None
        
        # build a new filteredList when only filter has changed.
        if (filter != self.__filter__):
            self.__filter__ = filter
            self.__filteredList__ = []
            for unit in self.__store__.units:
                try:
                    if (unit.source and (self.__filter__ & unit.x_editor_state)):
                        unit.x_editor_filterIndex = len(self.__filteredList__)
                        self.__filteredList__.append(unit)
                except:
                    pass
        self.emit(QtCore.SIGNAL("filterChanged"), filter, len(self.__filteredList__))
        
        # emit new current unit
        if (unitBeforeFiltered) and (unitBeforeFiltered in self.__filteredList__):
            unit = unitBeforeFiltered
        elif len(self.__filteredList__):
            unit = self.__filteredList__[0]
        else:
            unit = None
        self.emitUnit(unit)

    def headerData(self):
        """
        Get header info from the file.
        @return Header comment and Header dictonary.
        """
        
        if World.WF_debug:
            print "Entering headerData in class Operator"
            

        if (not isinstance(self.__store__, poheader.poheader)):
            return (None, None)

        header = self.__store__.header() 
        if header:
            headerDic = self.__store__.parseheader()
            translator = header.getnotes("translator")
            try:
                translator = translator.decode('utf-8')
            except Exception:
                pass
            return (translator, headerDic)
        else:
            return ("", {})

    def makeNewHeader(self, headerDic):
        """
        Create a header with the information from headerDic.
        @param headerDic: a dictionary of arguments that are neccessary to form
        a header.
        @return: a dictionary with the header items.
        """
        if World.WF_debug:
            print "Entering __init__ in class TUview"

        if (hasattr(self.__store__, "x_generator")):
            self.__store__.x_generator = World.settingApp + ' ' + __version__.ver
        if isinstance(self.__store__, poheader.poheader):
            self.__store__.updateheader(add=True, **headerDic)
            self.contentChanged(True)
            return self.__store__.makeheaderdict(**headerDic)
          
        else: return {}
        
    def saveNewHeader(self, othercomments, headerDic):
        #        save a header with the information from headerDic.
        if (not isinstance(self.__store__, poheader.poheader)):
            return {}
        header = self.__store__.header()
        if (header):
            header.removenotes()
            header.addnote(unicode(othercomments))
            #TODO this code is also in the library po.py, so we should combine it.
            header.msgid = ['""']
            headeritems = self.__store__.makeheaderdict(**headerDic)          
            header.msgstr = ['""']
            for (key, value) in self.newHeaderItems.items():
                header.msgstr.append(quote.quotestr("%s: %s\\n" % (key, value)))
        self.contentChanged(True)    
    
    def updateNewHeader(self, othercomments, headerDic):
        """
        Will update the existing header.
        @param othercomments: The comment of header file.
        @param headerDic: A header dictionary that has information about header.
        """
        #TODO: need to make it work with xliff file
        
        if World.WF_debug:
            print "Entering updateNewHeader in class TUview"

        if (not isinstance(self.__store__, poheader.poheader)):
            return {}
        headeritems={}       
        header = self.__store__.header()    
        if (header):
            header.removenotes()
            header.addnote(unicode(othercomments))
            #TODO this code is also in the library po.py, so we should combine it.
            header.msgid = ['""']
            headeritems = self.__store__.makeheaderdict(**headerDic)          
            header.msgstr = ['""']
            for (key, value) in headeritems.items():
                header.msgstr.append(quote.quotestr("%s: %s\\n" % (key, value)))
        self.newHeaderItems=headeritems
        self.contentChanged(True)
        
    def saveStoreToFile(self, fileName):
        """
        Save the temporary store into a file.
        
        @param fileName: String type.
        """
        
        if World.WF_debug:
            print "Entering __init__ in class Operator"
            

        if (World.settings.value("headerAuto", QtCore.QVariant(True)).toBool()):
            self.emit(QtCore.SIGNAL("headerAuto"))
        try:
            if (fileName):
                self.__store__.savefile(fileName)
                self.emit(QtCore.SIGNAL("returnFilename"), unicode(fileName))
            else:
                self.__store__.save()
            self.contentChanged(False)
        except Exception, e:
            QtGui.QMessageBox.critical(None, 
                                    self.trUtf8('Error'), 
                                    self.trUtf8('Error while trying to write file ' )
                                    + fileName  + 
                                    '\n' + str(e))
        
        
    def setComment(self, comment, unit = None):
        """
        Set the comment to the unit or current unit.
        Call emitUnit() if unit is not current unit.
        @param comment: QString type.
        """
        
        if World.WF_debug:
            print "Entering setComment in class Operator"

        if (self.__currentUnitIndex__ < 0 or not self.__filteredList__):
            return
        newUnit = False
        if (not unit):
            unit = self.getCurrentUnit()
            newUnit = True
        if self.isXliff:
            unit.removenotes(origin="translator")
            unit.addnote(unicode(comment),'translator')
        else:
            unit.removenotes()
            unit.addnote(unicode(comment))
        if (newUnit):
            self.emitUnit(unit)
        self.contentChanged(True)
    
    def stripspecial(self,target):
#take out the special characters when showing non-printing characters is not enabled
        origtarget=target
        unicodechar=""
        name=""
        orig=0
        targetlan= len(target)    
        for Char in range(0,targetlan):
            if origtarget[Char]=="\\":
                for i in range(6):
                    unicodechar=unicodechar+origtarget[Char+i]
                for k, v in World.dict_invisibleSpecial.iteritems():
                    origfromlist=v[1]
                    if unicodechar==origfromlist:
                        name=k	
                        for k, v in World.dict_nonprintable.iteritems():
                            if v==name:
                                orig=k
                                break
                        original=unichr(orig)
                        target=target.replace(unicodechar,original)
                unicodechar=""
                Char=Char+6
        return target        

    def setTarget(self, target, unit = None):
        """
        Set the target to the unit or current unit.
        Call emitUnit() if unit is not current unit.
        @param target: Unicode sting type for single unit and list type for
        plural unit.
        """
        
        # if there is no translation unit in the view.
        activestyle=World.settings.value("invisiblecharacater", QtCore.QVariant(True)).toBool()
        if World.WF_debug:
            print "Entering setTarget in class Operator"
            
        if (self.__currentUnitIndex__ < 0 or not self.__filteredList__):
            return
        newUnit = False
        if (not unit):
            unit = self.getCurrentUnit()
            newUnit = True
        # update target for current unit        
        if self.mainEditor.ui.actionShow_Nonprinting_Char.isChecked():
            if not activestyle:
                target=self.stripspecial(target)
            else:
                target=self.mainEditor.dockTUview.origtarget
        unit.settarget(target)
        #FIXME: this mark works single not plural unit.
        self.__status__.markTranslated(unit, (unit.target and True or False))
            
        if (newUnit):
            if self.mainEditor.dockTUview.contentDirty and self.mainEditor.dockComment.comment4bookmark:
                self.changedcontent=True
            else:
                self.changedcontent=False    
            self.emitUnit(unit)
        else:
            self.__emitUnitsStatus__()
        self.contentChanged(True)

    def showNewGlossaryTerm(self):
#        This function checks whether an untranslated glossary term is present and shows a term addition window if applicable.
        for unit in self.__mainEditor__.tableGlossary.glossaryCandidates:
            if not unit.target:
                if not self.newGlossaryTerm.isVisible():
                    self.newGlossaryTerm = NewGlossaryTerm(self.__mainEditor__)  
                    parentgeometry = self.__mainEditor__.geometry()
                    x= parentgeometry.x() + (parentgeometry.height()/6)
                    y = parentgeometry.y() + (parentgeometry.width()/6)
                    self.newGlossaryTerm.move(x, y)
                    self.newGlossaryTerm.show()
                    break

    def setUnitFromPosition(self, position, prePosition):
        """
        Build a unit from position and call emitUnit().
        @param position: position inside the filtered list.
        """
        if World.WF_debug:
            print "Entering setUnitFromPosition in class Operator"
        
        if (position in range(len(self.__filteredList__))):
            unit = self.__filteredList__[position]
#the following code is for enabling error checking everytime focus is changed in in overview
            globalClass.lastSource=globalClass.currentSource
            globalClass.lastTarget=globalClass.currentTarget
            globalClass.lastUnitId=globalClass.currentUnitId
            isPlural = unit.hasplural()

            globalClass.currentSource=unit.source
            globalClass.currentTarget=unit.target
            globalClass.currentUnitId=unit.getid()
            globalClass.sourceWordFlag = 1
            globalClass.preComputeFlag = 1   
            if globalClass.errorCheckEnableFlag:
                currentItem=self.mainEditor.dockOverview.ui.tableOverview.item(prePosition, 0)
                
                currentItem=int(currentItem.text())
                self.mainEditor.callPostCompute(currentItem, isPlural)

            #end of code modification for error checking
            self.emitUnit(unit)
            self.mainEditor.dockComment.comment4bookmark=""
            self.checkOnTheFly()
            self.emitTMCandidates()
            # re-init search by reset found position and current filed 
            # when changed unit. and remember the last unit index.
            self.foundPosition = -1
            self.lastFoundUnit = None
            self.searchStartIndex = self.__currentUnitIndex__
            if (self.searchFields):
                self.currentField = self.searchFields[0]
                
    def clearFuzzies(self, flag =None):
        """
        Change all fuzzy message to translated message 
        """
        if World.WF_debug:
            print "Entering clearFuzzies in class Operator"
        messageBox = QtGui.QMessageBox
        title = self.trUtf8("Warning")
        msg = self.trUtf8("You might lose some translation process information, and this action cannot be undone.\nAre you sure you want to change all fuzzy messages to translated?")
        result = messageBox.warning(None, title, msg, 
                    messageBox.Yes | messageBox.Default,
                    messageBox.No | messageBox.Escape)
        
        if (result == messageBox.Yes):
           #start of Aditi's code for clear all fuzzy messages
            for unit in self.__store__.units:
               if (not hasattr(unit, "x_editor_state")):
                    unit.x_editor_state = self.__status__.unitState(unit)
            OverIn=1
            for unit in self.__store__.units:
                if (unit.source and not (isinstance(self.__store__, xliff.xlifffile) and ( "gettext-domain-header" in (unit.getrestype() or "")))):
                    if (unit.x_editor_state & World.fuzzy):
                        self.mainEditor.dockOverview.ui.tableOverview.setSortingEnabled(False)
                        self.__status__.markFuzzy(unit, False)                    
                        for row in range(self.mainEditor.unitnum):
                            rowindex=self.mainEditor.dockOverview.ui.tableOverview.item(row, 0)
                            rowindex=int(rowindex.text())
                            if OverIn==rowindex:
                                self.mainEditor.dockOverview.markState(row, unit.x_editor_state)
                                self.mainEditor.dockOverview.markError(row, unit.x_editor_state)
                                self.mainEditor.dockOverview.ui.tableOverview.setSortingEnabled(True)
                                break
                    OverIn=OverIn+1
            self.contentChanged(True)        
            self.filterFuzzy(False)
                    

    def toggleFuzzy(self):
        """
        Toggle fuzzy state for current unit.
        """
        
        if World.WF_debug:
            print "Entering toggleFuzzy in class Operator"
            

        self.emitRequestTargetChanged()
        if (self.__currentUnitIndex__ < 0):
            return
        unit = self.getCurrentUnit()
        if (unit.x_editor_state & World.fuzzy):
            self.__status__.markFuzzy(unit, False)
        elif (unit.x_editor_state & World.translated):
            self.__status__.markFuzzy(unit, True)
        else:
            return
        self.emitUnit(unit)
        self.__emitUnitsStatus__()
        self.contentChanged(True)
    
    def initSearch(self, searchString, searchFields, matchCase):
        """
        Initilize the needed variables for searching.
        @param searchString: string to search for.
        @param searchFields: text fields to search through.
        @param matchCase: bool indicates case sensitive condition.
        """
        
        if World.WF_debug:
            print "Entering initSearch in class Operator"
            

        if (searchFields):
            self.searchStartIndex = self.__currentUnitIndex__
            self.foundPosition = -1
            self.searchString = unicode(searchString)
            self.searchFields = searchFields
            self.currentField = self.searchFields[0]
            self.matchCase = matchCase
        
    def searchNext(self):
        """
        Search forward through units/text fields.
        """
        
        if World.WF_debug:
            print "Entering searchNext in class Operator"
            

        # if it starts with last-not-found string, do not search.

        if (not self.__filteredList__) or (not hasattr(self, "searchFields")):
            return False
        
        currentUnit = self.getCurrentUnit()
        resumePoint = currentUnit.x_editor_filterIndex
        
        for unit in (self.__filteredList__[resumePoint:] + self.__filteredList__[:resumePoint]):
            resumeField = self.searchFields.index(self.currentField)
            # slice the first path out, since we came thru it already.
            for field in self.searchFields[resumeField:]:
                unitString = self._getStringFromUnit(unit, field)
                if (not self.matchCase):
                    self.foundPosition = unitString.lower().find(self.searchString.lower(), self.foundPosition + 1)
                else:
                    self.foundPosition = unitString.find(self.searchString, self.foundPosition + 1)
                if (self.foundPosition > -1):
                    self.currentField = field
                    break
            
            if (self.foundPosition > -1):
            
                self.emitFoundUnit(unit)
                self.foundString = self.searchString
                self.lastFoundUnit = unit
                self.emit(QtCore.SIGNAL("searchStatus"), "found")
                return True
            self.currentField = self.searchFields[0]
            self.foundPosition = -1
        else:
            # Searched through file, but not found.
            if (self.lastFoundUnit and (self.foundString == self.searchString)):
                self.emit(QtCore.SIGNAL("searchStatus"), "reachedEnd")
            else:
                self.emit(QtCore.SIGNAL("searchStatus"), "notFound")
                self.emitNotFound()
            return False
    
    def searchPrevious(self):
        """
        Search backward through units/text fields.
        """
        
        if World.WF_debug:
            print "Entering searchPrevious in class Operator"
            

        if (not self.__filteredList__) or (not hasattr(self, "searchFields")):
            return False
        
        currentUnit = self.getCurrentUnit()
        resumePoint = currentUnit.x_editor_filterIndex
        
        for i in range(len(self.__filteredList__)):
            if (i <= resumePoint):
                unit = self.__filteredList__[resumePoint - i]
            else:
                unit = self.__filteredList__[len(self.__filteredList__) + resumePoint - i]
                
            resumeField = self.searchFields.index(self.currentField) + 1
            # get back from where we came thru to the first.
            
            for field in reversed(self.searchFields[:resumeField]):
                unitString = self._getStringFromUnit(unit, field)
                if self.foundPosition == -1:
                    self.foundPosition = len(unitString)-1
                if (not self.matchCase):
                    self.foundPosition = unitString.lower().rfind(self.searchString.lower(), 0, self.foundPosition)
                else:
                    self.foundPosition = unitString.lower().rfind(self.searchString.lower(), 0, self.foundPosition)
                if (self.foundPosition > -1):
                    self.currentField = field
                    break
            if (self.foundPosition > -1):
                self.emitFoundUnit(unit)
                self.foundString = self.searchString
                self.lastFoundUnit = unit
                self.emit(QtCore.SIGNAL("searchStatus"), "found")
                return True
            self.currentField = self.searchFields[-1]
            self.foundPosition = -1            
        else:
            # Searched through file, but not found.
            if (self.lastFoundUnit and (self.foundString == self.searchString)):
                self.emit(QtCore.SIGNAL("searchStatus"), "reachedEnd")
            else:
                self.emit(QtCore.SIGNAL("searchStatus"), "notFound")
                self.emitNotFound()
            return False
    
    def _getStringFromUnit(self, unit, field):
        """
        Return QString of unit's field; source, target, or comment.
        """
        
        if World.WF_debug:
            print "Entering _getStringFromUnit in class Operator"
            

        if (field == World.source):
            unitString = unit.source
        elif (field == World.target):
            unitString = unit.target
        elif (field == World.comment):
            unitString = unit.getnotes()
        else:
            unitString = ""
        return unitString
    
    def replace(self, text):
        """
        Replace the found text in the text fields.
        @param text: text to replace.
        """
        
        if World.WF_debug:
            print "Entering replace in class Operator"
            

        if (self.foundPosition > -1):
            self.emit(QtCore.SIGNAL("replaceText"), \
                self.currentField, \
                self.foundPosition, \
                len(unicode(self.searchString)), \
                text)
            self.searchNext()
            return True
        return False

    def replaceAll(self, text):
        """
        Replace the found text in the text fields through out the units.
        @param text: text to replace.
        """
        
        if World.WF_debug:
            print "Entering replaceAll in class Operator"
            

        if (unicode(text) == unicode(self.searchString)):
            return
        
        while self.replace(text):
            pass
        
    def emitFoundUnit(self, unit):
        """
        Emit "searchResult" signal with searchString, textField, and
        foundPosition.
        """
        
        if World.WF_debug:
            print "Entering emitFoundUnit in class Operator" 
        self.emitUnit(unit)
        self.emit(QtCore.SIGNAL("searchResult"), self.searchString, self.currentField, self.foundPosition)
    
    def emitNotFound(self):
        """
        Emit "searchResult" signal with searchString="", textField, and
        foundPosition... indicates result not found.
        """
        
        if World.WF_debug:
            print "Entering emitNotFound in class Operator"
            

        self.emit(QtCore.SIGNAL("searchResult"), "", self.currentField, -1)
    
    def closeFile(self):
        
        if World.WF_debug:
            print "Entering closeFile in class Operator"
            
#        self.__store__ = None
#        self.__status__ = None
#        self.__filter__ = None
        self.__filteredList__ = []
        self.__emitNewUnits__()
        self.emitUnit(None)
        self.contentChanged(False)
        if (self.__standardGlossary__):
            self.__termMatcher__ = self.__standardGlossary__
            self.__termCache__ = {}         #emtpy the cache for highlighting word
            self.glossaryChanged= True      # this ensure highlight is correct.
            self.__standardGlossary__ = None
            
        
    def setMatcher(self, matcher, section):
        """
        Set matcher or termMatcher to new matcher.
        @param matcher: class matcher.
        @param section: string indicates TM or glossary.
        """
        if World.WF_debug:
            print "Entering setMatcher in class Operator"
        
        if (section == "TM"):
            self.__tmMatcher__ = matcher
            self.__tmCache__ = {}
        else:
            self.__termMatcher__ = matcher
            self.__termCache__ = {}
            self.glossaryChanged = True
                
        self.applySettings()
        
    def lookupTerm(self, term):
        """
        Lookup a term in glossary.
        @param term: a word to lookup in termMatcher.
        @return candidates as list of units
        """
        
        if World.WF_debug:
            print "Entering lookupTerm in class Operator"
#        print "Entering lookupTerm...", term
#        start of modification needed for issue number :2818343
#     Adding an extra n at the begining and end for words containing characters of World.regularExpressionCharacters.
        term =  str(term)    
        if len(term) > 1 and term[0] == str('n') and term[1] in World.regularExpressionCharacters:
                        term = term[1:]
        if len(term) > 1 and term[-1] == str('n') and term[-2] in World.regularExpressionCharacters:
                        term = term[:-1]
#end of modification needed for issue number :2818343
        if (self.__autoLookupTerm__ and self.__termMatcher__):
            # emit "glossaryPattern" when glnossary changed
            if (self.glossaryChanged):
                self.glossaryChanged = False
                pattern = []
                for unit in self.__termMatcher__.candidates.units:
                    
#        start of modification needed for issue number :2818343
#     Represnting regular expression characters with backslash before them to match them with glossary and  adding an extra n at the begining and end for words containing characters of World.regularExpressionCharacters at the begining or at the end.
                    element = unit.source
                    lenElement = len(element) - 1
                    for each in World.regularExpressionCharacters:
                # To find all of the the starting positions of each in element
                        starts = [match.start() for match in re.finditer(re.escape(each), element)]
                        if starts:                    
                            element = element.replace(each, '\\' + each)
                        else:
                            continue
            
                        if str(element[0]) in World.regularExpressionCharacters:
                            element = 'n' + element
                        if str(element[-1]) in World.regularExpressionCharacters:
                            element = element + 'n'
                    pattern.append(element)
                    
#        end of modification needed for issue number :2818343
                    
                self.emit(QtCore.SIGNAL("glossaryPattern"), pattern)
            if (not term):
                return None
            # use cache to improve glossary search speed within 20 terms.
            if self.__termCache__.has_key(term):    
                candidates = self.__termCache__[term]
            else:               
                candidates = self.__termMatcher__.matches(term)
                # remove not exact term from candidates, if we remove directly from candidate, then the list is shortened hence not iterated for desired number of times.
                tempCandidates = candidates[:]
                for candidate in candidates:
                    if (candidate.source.lower() != term.lower()):
                        tempCandidates.remove(candidate)
                candidates = tempCandidates
                self.__termCache__[term] = candidates
                if (len(self.__termCache__) > 20): 
                    self.__termCache__.popitem()
             #      The following lines are taking the values in a dictionary, for using in glossary testing. 

            for each in candidates:
                globalClass.tempGlsErrorDict[each.source][unicode(each.target)] = unicode(each.target)        
            return candidates
    
    def emitGlossaryCandidates(self, termList):
        """
        @emit "glossaryCandidates" signal as list of units.
        @param termList list of words to find in glossary        
        """
        if World.WF_debug:
            print "Entering emitGlossaryCandidates in class Operator"
        candidates = []
        for term in termList:
            alistOfCandidates = []
            alistOfCandidates = self.lookupTerm(term)
            if (alistOfCandidates):
                candidates += alistOfCandidates
        self.__mainEditor__.tableGlossary.glossaryCandidates = candidates
        self.emit(QtCore.SIGNAL("termCandidates"), candidates)
            
    def emitTermRequest(self, term):
        #    "goutom  glossary right click"
        tempstr=string.lower(globalClass.currentSource)
        myglist=self.mainEditor.errorTable.hasXmlAttr(tempstr)
        for x in myglist:
            t=tempstr.replace(str(x), "")
            tempstr=t
        c=0
        for x in myglist:
            if str(string.lower(term)) in x and  str(string.lower(term)) not in tempstr:
                c=1
                break
        if c==1:
            return
        """
        @emit "termRequest" signal as list of units.
        """
        if World.WF_debug:
            print "Entering emitTermRequest in class Operator"
        candidates = self.lookupTerm(term)
        self.emit(QtCore.SIGNAL("termRequest"), candidates)
    
    def lookupUnitTranslation(self, unit):
        """
        lookup suggested translations in translation memory (TM)"
        @param unit: is an xliff object, or pounit object
        @return candidates: list of found units in TM 
        """
        if World.WF_debug:
            print "Entering lookupUnitTranslation in class Operator"
        tempoCandidates = []
        if self.__tmMatcher__:
            self.__tmMatcher__.setparameters(self.max_candidates, self.__minAddAltTrans__, self.max_string_len)
        if (unit and self.__tmMatcher__ and (unit.target == "" or (unit.isfuzzy() and self.__SuggestOnFuzzy__))):
            candidates = []
#            Handle the xliffunits differently as there is no strings attribute in xliffunit.
            if isinstance(unit,xliff.xliffunit ):
                candidates = candidates + self.scanTM(unit.source)                
            else:
                for string in unit.source.strings:
                    candidates = candidates + self.scanTM(string)
    #            tempoCandidates = []
            if candidates:
                for candidate in candidates:           
                    candidate.matchquality = self.getMatchQuality(candidate)      
                    if (candidate.matchquality >= self.__minAddAltTrans__):
                        tempoCandidates.append(candidate)
        return tempoCandidates
        
    def scanTM(self, string):
        """
        Lookup suggested translations from Translation Memory for a unit.
        @param unit: class unit.
        @return candidates as list of units, or None on error.
        """
        
        if World.WF_debug:
            print "Entering scanTM in class Operator"

        # use cache to improve search speed within 20 units.
        if self.__tmCache__.has_key(string):
            candidates = self.__tmCache__[string]
        else:
            candidates = self.__tmMatcher__.matches(string)
            self.__tmCache__[string] = candidates
            if (len(self.__tmCache__) > 20 ): 
                self.__tmCache__.popitem()
        
        return candidates
    
    def ApplyTMToCurrentMsg(self):
        """
        Apply TM to current Massage
        @emit "tmCandidates" signal as list of units.
        """
        if World.WF_debug:
            print "Entering ApplyTMToCurrentMsg in class Operator"
        if (self.__isTMTableVisible__):
            unit = self.getCurrentUnit()
            candidates = self.lookupUnitTranslation(unit)

#            self.emit(QtCore.SIGNAL("tmCandidates"), candidates)  
            self.mainEditor.table.InstantTMFillTable(candidates)
    
    def emitTMCandidates(self):
        """
        If instant TM is activated, TM suggestion candidates from the standard TM of WF are
        included in a list of candidates.If there are TM suggestion candidates embeded as
        Alternative translations in the XLIFF file, then they are also appended to the list
        of candidates.
        @emit "tmCandidates" signal as list of units.
        if World.WF_debug:
            print "Entering emitTMCandidates in class Operator"
        """
        unit = self.getCurrentUnit()
        candidates = []
        if unit and not unit.istranslated():
            if (self.__autoLookupUnit__ and self.__isTMTableVisible__): # if instant TM it returs a list of units.
                candidates = self.lookupUnitTranslation(unit)

            if (isinstance(self.__store__, xliff.xlifffile)):
                candidates += unit.getalttransWF("proposal") # concatenate instant TM list with internal TM of the file

#            elif(isinstance(self.__store__, pypo.pofile)):
            if self.TMDict:
#                    when a message is clicked,  then it sends the TM suggestions of only the first string (in case of plurals) to TM window
                if self.TMDict.has_key(unit.source):
                    self.mainEditor.table.fillTable(unit.source, self.TMDict[unit.source])

            if candidates:
                self.mainEditor.table.InstantTMFillTable(candidates)
#                self.emit(QtCore.SIGNAL("tmCandidates"), candidates)

    def emitRequestUnit(self):
        """
        Lookup current unit's translation.
        @emit "tmRequest" signal as list of units.
        """
        if World.WF_debug:
            print "Entering emitRequestUnit in class Operator"
            
        unit = self.getCurrentUnit()
        candidates = self.lookupUnitTranslation(unit)
        self.emit(QtCore.SIGNAL("tmRequest"), candidates)
    
    def setIsTMTableVisible(self, bool):
        """
        Set whether to lookup suggested translations of current unit from TM e.g. Translation Memorny Table is not
        visible, so don't bother to lookup unit.
        """
        
        if World.WF_debug:
            print "Entering __init__ in class Operator"
            

        self.__isTMTableVisible__ = bool
        # when tm table shown, it need to be filled.
        if (bool):
            self.emitTMCandidates()
            
    def autoTranslate(self):
        """
        Auto translate units.
        """
        
        if World.WF_debug:
            print "Entering autoTranslate in class Operator"
            

        self.__minAddAltTrans__ = World.settings.value("TM_min", QtCore.QVariant("70")).toInt()[0]
        self.__maxAddAltTrans__ = World.settings.value("TM_max", QtCore.QVariant("100")).toInt()[0]
        self.max_candidates = World.settings.value("TM_max_candidates", QtCore.QVariant("10")).toInt()[0]
        self.max_string_len = World.settings.value("TM_max_string_len", QtCore.QVariant("200")).toInt()[0]
        
        self.__useGLb4TM__ = World.settings.value("TM_GLb4TM", QtCore.QVariant(True)).toBool()
        self.__FuzzyGLb4TM__ = World.settings.value("FuzzyGLb4TM", QtCore.QVariant(False)).toBool()
        self.__AutoTargetInclusion__ = World.settings.value("TM_TargetInclusion", QtCore.QVariant(False)).toBool()
        self.__Mark100PerSuggestionsAsFuzzy__ = World.settings.value("TM_MarkStringAsFuzzy",  QtCore.QVariant(True)).toBool()
        self.__AutoAddAltsTran__ = World.settings.value("TM_AddSuggestions",  QtCore.QVariant(True)).toBool()
        self.__SuggestOnFuzzy__ = World.settings.value("TM_IncludeFuzzy",  QtCore.QVariant(True)).toBool()
        self.__eliminateIdenticalSug__ = World.settings.value("TM_EliminateIdentical",  QtCore.QVariant(False)).toBool()
        self.__autoLookupUnit__ = World.settings.value("TM_Instant").toBool()
        
        if (self.__Mark100PerSuggestionsAsFuzzy__): 
            MarkStringAsFuzzy = ""
        else: 
            MarkStringAsFuzzy = "not "
        
        if (self.__eliminateIdenticalSug__): 
            EliminateIdentical = "not "
        else: 
            EliminateIdentical = ""
        
        if (self.__AutoAddAltsTran__): 
            AddSuggestions = " For translations that are "+ str(self.__minAddAltTrans__) +"% to "+ str(self.__maxAddAltTrans__) +"% identical to the source, it will create suggestions that will be displayed in the Translation Memory window .</li> <li>It will not propose more than "+str(self.max_candidates)+" suggestion for each message</li><li>It will "+ EliminateIdentical+"display suggestions that are identical to each other.</li>"
        else:
            AddSuggestions = ""

        if ( self.__SuggestOnFuzzy__): 
            IncludeFuzzy = ""
        else: 
            IncludeFuzzy = "not "
        
        messageBox = QtGui.QMessageBox
        msgxliff = self.trUtf8("<html><body><p align ='justify'>WordForge will look in its Translation Memory for suggestions for the translation of as many messages as possible in the current file.\n<ul><li>If it finds a suggestion that is at least "+ str(self.__maxAddAltTrans__)  +"% identical to the source of the message, it will place it in the translation window, and will " + MarkStringAsFuzzy + "mark the message as fuzzy.</li><li>"+ AddSuggestions + "<li>When calculating the percentage match of a message, only the first  " + str(self.max_string_len) + " characters will be considered. </li><li>It will "+ IncludeFuzzy +" suggest translations for messages that are already fuzzy. </li><li>All suggestions will be stored in the XLIFF file, and will be usable the next time that you work on it, without needing to look up Translation Memory again.</li></ul>\n\nIf you want to change the above settings, please click on the <b>Preferences</b> button.</p></body></<html>")

        msgpo = self.trUtf8("<html><p align ='justify'>WordForge will look in its Translation Memory for suggestions for the translation of as many messages as possible in the current file.\n<ul><li>If it finds suggestions that are "+ str(self.__maxAddAltTrans__) +"% identical to the source of the message, it will place the translation in the translation window, and will " + MarkStringAsFuzzy + "mark the message as fuzzy. </li><li>It will not take into account source messages that are longer than "+str(self.max_string_len)+" characters.</li></ul>If you want to change the above settings, please click on the <b>Preferences</b> button</p><html>.")

        ret = messageBox()
        ret.setTextFormat(QtCore.Qt.RichText)
        if (isinstance(self.__store__, xliff.xlifffile)):
            ret.setText(msgxliff)
           
        else:
            ret.setText(msgpo)

        PreferenceButton = ret.addButton(self.trUtf8("Preferences"), messageBox.ActionRole)
        YesButton = ret.addButton(self.trUtf8("Yes"),messageBox.YesRole)
        NoButton = ret.addButton(self.trUtf8("Cancel"), messageBox.NoRole)
        ret.exec_()
        
        if (ret.clickedButton() == PreferenceButton):
            self.emit(QtCore.SIGNAL("ShowPreference"), 2)
        elif (ret.clickedButton() == NoButton):
            return
        elif (ret.clickedButton() == YesButton):
            #            clear the tmcache
            self.__tmCache__={}
            # get tmMatcher from pickle
            if (not self.__tmMatcher__):
                messageBox.critical(None,
                    self.trUtf8("Error"),
                    self.trUtf8("No Translation Memory. You must build a Translation Memory before you use this feature. "), 
                    self.trUtf8("&OK"))
                return
            
            self.__tmMatcher__.setparameters(self.max_candidates, self.__minAddAltTrans__, self.max_string_len)
            
            self.__emitNewUnits__()
            # for autoTranslate all units
            self.__mainEditor__.setCursor(QtCore.Qt.WaitCursor)
            self.centPercent = 0
            self.idMatch = 0
            self.overNinety = 0
            self.overEighty = 0
            self.overSeventy = 0
            self.totalMessages = str(self.__status__.getTotalOfStrings())
            for unit in self.__store__.units:
                # if a unit is translated or if it is not a header (empty source string) it is  fuzzy (and we are not looking for suggestions for fuzzy strings) ignore this unit
                if (unit.istranslated() or \
                    (not unit.source)  or \
                    (isinstance(self.__store__, xliff.xlifffile) and ("gettext-domain-header" in (unit.getrestype() or ""))) or \
                    (unit.isfuzzy() and not self.__SuggestOnFuzzy__)):
                    continue   #skip because unit doesn't need translation memory
                
                candidates = []
#                if the unit is xliffunit then do not look for strings attribute in it. 
                if isinstance(unit,xliff.xliffunit ):
                    if not(unit.isfuzzy() and not self.__SuggestOnFuzzy__):
#                        checking if glossary should be used before TM
                        if (self.__useGLb4TM__ and self.__AutoTargetInclusion__ and self.__autoLookupTerm__ and self.__termMatcher__):
#                            using glossary before TM
                            GLb4TM = self.findGLb4TM(unit.source)
                            if GLb4TM:
                                candidates.append(GLb4TM)
                                self.usingGLb4TM = True
                            else:
                                candidates = self.scanTM(unit.source)
                        else:
                            candidates = self.scanTM(unit.source)
                        if candidates:
                            self.setAutoTransToTarget(unit, unit.source, candidates)
                else:
                    for string in unit.source.strings:
                        if not(unit.isfuzzy() and not self.__SuggestOnFuzzy__):
    #                        checking if glossary should be used before TM
                            if (self.__useGLb4TM__ and self.__AutoTargetInclusion__ and self.__autoLookupTerm__ and self.__termMatcher__):
    #                            using glossary before TM
                                GLb4TM = self.findGLb4TM(unit.source)
                                if GLb4TM:
                                    candidates.append(GLb4TM)
                                    self.usingGLb4TM = True
                                else:
                                    candidates = self.scanTM(unit.source)                                
                            else:
                                candidates = self.scanTM(unit.source)                                    
                            if candidates:
                                self.setAutoTransToTarget(unit, unit.source, candidates)
                                
#                        if(unit.isfuzzy() and not self.__SuggestOnFuzzy__):
#                            continue
#                        candidates = self.scanTM(string)
#                        if candidates:
#                            self.setAutoTransToTarget(unit, string, candidates)

            self.__emitNewUnits__()
            self.__emitUnitsStatus__()
            if (len(self.__filteredList__) > 0):
                self.emitUnit(self.__filteredList__[0])
            self.contentChanged(True)
            self.emitTMCandidates()
            self.__mainEditor__.setCursor(QtCore.Qt.ArrowCursor)
            
            self.showTMStatistics()


    def findGLb4TM(self, source):
        self.usingGLb4TM = False
        self.accFlag = False
        self.startPunc = False
        self.endPunc = False
        
        terms = len(source.split())
        if(terms <> 1):
            return False
#        searching for accelerators in the source
        if self.AcceleratorMark in source:
            source = source.split(self.AcceleratorMark)
            source = ''.join(source)
            self.accFlag = True
#        print 'term af: ', source
        
#        making sure that there are no punctuations in the middle of the source
        for each in globalClass.preferencePuncList:
            if each in source[1:-1]:
                return False
        
#      searching punctuations at the start and end of the source
        for each in globalClass.preferencePuncList:
            if source[0] == each:
                source = source[1:]
                self.startPunc = each

            if source[-1] == each:
                source = source[:-1]
                self.endPunc = each
        
#        sending the source for matching and returning the first glossary match
        glossaryMatch = self.lookupTerm(source)
        if glossaryMatch:
            glossaryMatch = glossaryMatch[0]
        
        return glossaryMatch

    def showTMStatistics(self):
        from WordForge.Editor.modules.TMStatistics import TMStatistics
        TMStat = TMStatistics(self.mainEditor)
        
#        totalMessages = self.__status__.getTotalOfStrings()
        TMApplied = str(len(self.TMDict))
        
        percentApplied = str(100*float(TMApplied)/float(self.totalMessages))
        percentApplied = percentApplied[0:percentApplied.find(".")+3] + "%"
        
        percent100 = str(100*float(self.centPercent)/float(self.totalMessages))
        percent100 = percent100[0:percent100.find(".")+3] + "%"
        
        percentIDMatch = str(100*float(self.idMatch)/float(self.totalMessages))
        percentIDMatch = percentIDMatch[0:percentIDMatch.find(".")+3] + "%"
        
        percent90 = str(100*float(self.overNinety)/float(self.totalMessages))
        percent90 = percent90[0:percent90.find(".")+3] + "%"
        
        percent80 = str(100*float(self.overEighty)/float(self.totalMessages))
        percent80 = percent80[0:percent80.find(".")+3] + "%"
        
        percent70 = str(100*float(self.overSeventy)/float(self.totalMessages))
        percent70 = percent70[0:percent70.find(".")+3] + "%"

        noMatch = str(int(self.totalMessages) - int(TMApplied))
        percentNoMatch = str(100*float(noMatch)/float(self.totalMessages))
        percentNoMatch = percentNoMatch[0:percentNoMatch.find(".")+3] + "%"

        fileName = unicode(self.__fileName__.split(os.path.sep)[-1])
        
#TM writing code starts here , done by mushfiq               
        #TM copy paste feature
        a = "Statistics Of:"+"\t\t"+fileName +"\n"+"\n"+"Number Of Messages:"+"\t\t"+self.totalMessages+ "\n"+"TM applied on"+"\t\t"+TMApplied+"\t"+percentApplied+"\n"+"100% match:"+"\t\t"+str(self.centPercent)+"\t"+percent100+"\n"+"ID match"+"\t\t\t"+str(self.idMatch)+"\t"+percentIDMatch+"\n"+"Between 90-99%"+"\t\t"+str(self.overNinety)+"\t"+percent90+"\n"+"Between 80-89%"+"\t\t"+str(self.overEighty)+"\t"+percent80+"\n"+"Between 70-79%"+"\t\t"+str(self.overSeventy)+"\t"+percent70+"\n"+"No TM match"+"\t\t"+str(noMatch)+"\t"+str(percentNoMatch)

        TMStat.ui.TMtxtEdit.setPlainText(a)          
#end of Mushfiq's code
        
        TMStat.show()
        

    def setAutoTransToTarget(self, unit, string, candidates):
        """
        The current unit
        A list of sorted xliff object unit
        """
        if World.WF_debug:
            print "Entering setAutoTransToTarget in class Operator"

#            processing for the glossary before TM
        if self.usingGLb4TM:
            target = candidates[0].target
            makeFuzzy = False
            
            if self.__FuzzyGLb4TM__:
                makeFuzzy = True
                
            if self.startPunc:
               target = unicode(self.startPunc) + unicode(target)
               self.__status__.markFuzzy(unit, True)
               makeFuzzy = True
               
            if self.endPunc:
                target = unicode(target) + unicode(globalClass.endPuncDict[self.endPunc])
                self.__status__.markFuzzy(unit, True)
                makeFuzzy = True

            if self.accFlag:
                target = unicode(self.AcceleratorMark) + unicode(target)
                self.__status__.markFuzzy(unit, True)
                makeFuzzy = True

            target = unicode(target)
            unit.settarget(target)
            
            if makeFuzzy:
                self.__status__.markFuzzy(unit, True)
            else:
                self.__status__.markTranslated(unit, True)
                
            unit.x_editor_state = self.__status__.unitState(unit)
            return


        if (self.__store__):
            # Translation Memory's Suggestions, Add alternative translations if in range
            # if we want to add translations as alt-trans, and the file we have is XLIFF
            # if the file is PO, then we store alt-trans in a dictionary
            if (self.__AutoAddAltsTran__ and (isinstance(self.__store__, xliff.xlifffile) or isinstance(self.__store__, pypo.pofile))):
#                if isinstance(self.__store__, pypo.pofile):
                self.TMDict[str(string)] = []
                for i in range(len(candidates)): # For each candidate
                    match = self.getMatchQuality(candidates[i])  # get its match quality
                    # if this unit will be written in target, ignore it.
                    if (i==0 and match >= self.__maxAddAltTrans__ and not unit.isfuzzy() and self.__AutoTargetInclusion__):
                        continue
#                    if ((match >= self.__minAddAltTrans__) and (match < self.__maxAddAltTrans__)):
#                    if (match >= self.__minAddAltTrans__):
                        # if eliminate identical suggestions
                    if isinstance(self.__store__, xliff.xlifffile) and (self.__eliminateIdenticalSug__) and self.detectAlt(candidates[i], unit):
                        continue
#                        altTran = self.detectAlt(candidates[i], unit)
#                        if (altTran):                        
#                            unit.delalttrans(altTran)
                    if isinstance(self.__store__, xliff.xlifffile):
                        unit.addalttransWF(candidates[i].target, "TM", str(self.TMTargetLanguage), candidates[i].source, str(match), "proposal"  )
                    
#                    put the TM suggestions in a dictionary
#                    elif isinstance(self.__store__, pypo.pofile):
                    self.TMDict[str(string)].append(str(match))
                    self.TMDict[str(string)].append(str(candidates[i].source))
                    self.TMDict[str(string)].append(str(candidates[i].target))
#            # If the candidate is good enough, according to the parameter for placing in the target
            # then the translation is placed directly in the target. If required by parameter it is turned 
            # to fuzzy or to translated.
            match = self.getMatchQuality(candidates[0])      #This is the best candidate
            if match == 100:
                self.centPercent +=1
            elif match >=90:
                self.overNinety +=1
            elif match >=80:
                self.overEighty +=1
            elif match >=70:
                self.overSeventy +=1
                
            if (match >= self.__maxAddAltTrans__ and self.__AutoTargetInclusion__ and not unit.isfuzzy()):
                unit.settarget(candidates[0].target)

#                if ((match == 100) and not self.__Mark100PerSuggestionsAsFuzzy__):
#                    self.__status__.markTranslated(unit, True)
#                else:
#                    self.__status__.markFuzzy(unit, True)
                if (match == 100):
                    if unit.getid() == candidates[0].id:
                        self.idMatch +=1
                    if not (self.__Mark100PerSuggestionsAsFuzzy__) or (unit.getid() == candidates[0].id):
                        self.__status__.markTranslated(unit, True)
                    else:
                        self.__status__.markFuzzy(unit, True)
                else:
                    self.__status__.markFuzzy(unit, True)

#               this line is for showing the fuzzy state correctly
                unit.x_editor_state = self.__status__.unitState(unit)

    def detectAlt(self, candidate, unit):
        """
        detecting whether the candidate is already existing or not
        
        @param: candidate as xliff object
        @param: xliff object
        @return: detected alternative translatin or None
        """
        
        if World.WF_debug:
            print "Entering detectAlt in class Operator"
            

        if (unit):
            translist = unit.getalttrans()
            if translist:
                for altTran in translist:
                    # can't detect only source and target
                    if (candidate.target == altTran.target):
                        return altTran
                else:
                    return None
    
#    def addAsAltTrans(self, candidate):
#        """
#        add candidate from translation memory as alternative translation to the xliff file
#        @param candidate: as an xliff object unit
#        """
#        unit = self.getCurrentUnit()        #current translation unit
#        matchquality = int(candidate.matchquality)
#        messageBox = QtGui.QMessageBox
#        
#        if (matchquality  == 100):
#            messageBox.critical(None,
#            self.trUtf8("Alternative Translation"),
#            self.trUtf8("""It has a 100% match quality. It must not be added as alternative translation."""),
#            messageBox.Ok)
#            return            
#        else:        
#            # detect that altTrans exists already. If existing, ask whether to add or not.
#            if (self.detectAlt(candidate, unit)):
#                ret = messageBox.information(None,
#                self.trUtf8("Alternative Translation"),
#                self.trUtf8("""It is already existing as an alternative translation.\n Do you still want to add?"""),
#                messageBox.Yes | messageBox.Default, messageBox.No | messageBox.Escape)
#                if (ret == messageBox.No):
#                    return
#        
#        unit.addalttrans(candidate.target, "TM Search", None)
#        self.emitUnit(unit)
#        self.contentChanged(True)
#        messageBox.information(None,
#            self.trUtf8("Add alternative Translation"),
#            self.trUtf8("Success"),
#            messageBox.Ok)
        
#    def removeAltTran(self, candidate):
#        """
#        remove an alternative translation form alternative translations
#        @param candidate: as an xliff object unit
#        """
#        unit = self.getCurrentUnit()
#        matchquality = int(candidate.matchquality)
#        messageBox = QtGui.QMessageBox
#                
#        if (matchquality  == 100):
#            messageBox.critical(None,
#            self.trUtf8("Alternative Translation"),
#            self.trUtf8("It's the unit translation. It can not be removed"),
#            messageBox.Ok)
#            return            
#        else:        
#            # detect that altTrans exists already and remove it or not
#            altTran = self.detectAlt(candidate, unit)
#            if (altTran):
#                ret = messageBox.information(None,
#                self.trUtf8("Alternative Translation"),
#                self.trUtf8("Are you sure you want to remove it from alternative translations?"),
#                messageBox.Yes | messageBox.Default, messageBox.No | messageBox.Escape)                
#                if (ret == messageBox.No):
#                    return            
#            
#        unit.delalttrans(altTran)
#        
#        self.emitUnit(unit)
#        self.contentChanged(True) 
#        messageBox.information(None,
#            self.trUtf8("Remove alternative Translation"),
#            self.trUtf8("Success"),
#            messageBox.Ok)
        
        
    def getMatchQuality(self, candidate):
        """Get the match quality and return as string (73)"""

        if World.WF_debug:
            print "Entering getMatchQuality in class Operator"


        matchQuality = (candidate.getnotes("translator"))[:-1]
        try:
            matchQuality = int(matchQuality)
        except ValueError:
            matchQuality = 0
        return matchQuality
        
    def slotFindUnit(self, source):
        """
        Find a unit that contain source then emit currentUnit.
        @param source: source string used to search for unit.
        """
        
        if World.WF_debug:
            print "Entering slotFindUnit in class Operator"
            

        unit = self.__store__.findunit(source)
        if unit:
            self.emitUnit(unit)
    
    def applySettings(self):
        if World.WF_debug:
            print "Entering applySettings in class Operator"
            
        self.__AutoTargetInclusion__ = World.settings.value("TM_TargetInclusion").toBool()
        self.__useGLb4TM__ = World.settings.value("TM_GLb4TM", QtCore.QVariant(True)).toBool()
        self.__FuzzyGLb4TM__ = World.settings.value("FuzzyGLb4TM", QtCore.QVariant(False)).toBool()
        self.__Mark100PerSuggestionsAsFuzzy__ = World.settings.value("TM_MarkStringAsFuzzy").toBool()
        self.__AutoAddAltsTran__ = World.settings.value("TM_AddSuggestions").toBool()
        self.__SuggestOnFuzzy__ = World.settings.value("TM_IncludeFuzzy").toBool()
        self.__eliminateIdenticalSug__ = World.settings.value("TM_EliminateIdentical").toBool()
        self.__autoLookupUnit__ = World.settings.value("TM_Instant").toBool()
        
        
        #self.__autoLookupTerm__ indicates if glossaryTerm should be highlighted.
        #it's read from the init file  
        self.__autoLookupTerm__ = World.settings.value("GlossaryPreference",  QtCore.QVariant(True)).toBool()
        self.lookupTerm(None)
        if  (self.__autoLookupTerm__ and self.__termMatcher__):
            self.emit(QtCore.SIGNAL("highlightglossary"), True)
        else:
            self.emit(QtCore.SIGNAL("highlightglossary"), False)
        # FileTM settings read from the init file
        self.__minAddAltTrans__ = World.settings.value("TM_min", QtCore.QVariant("70")).toInt()[0]
        self.__maxAddAltTrans__ = World.settings.value("TM_max", QtCore.QVariant("100")).toInt()[0]
        self.max_candidates = World.settings.value("TM_max_candidates", QtCore.QVariant("10")).toInt()[0]
        self.max_string_len = World.settings.value("TM_max_string_len", QtCore.QVariant("200")).toInt()[0]
        
        # Target Language for TM and Glossary
        self.TMTargetLanguage = World.settings.value("TMTargetLanguage").toString()
        self.GlossaryTargetLanguage = World.settings.value("GlossaryTargetLanguage").toString()
        
        
        
        # check if the active dictionary has been changed.
        dictionaryName = World.settings.value("activeDictionary").toString().__str__()
        dicPath = os.path.abspath(".." + os.path.sep + "dict") + os.path.sep
        #set dictionary language for spellchecker
        if (self.dictlanguage != dicPath + dictionaryName):
            self.dictlanguage = dicPath + dictionaryName
            globalClass.createDictFlag = 1
            self.checkOnTheFly()

        
    def contentChanged(self, bool = False):
        """
        The signal is emitted when content is changed
        """
        if World.WF_debug:
            print "Entering contentChanged in class Operator"
        self.modified = bool
        self.mainEditor.fileaction.isSaved=not bool
        self.emit(QtCore.SIGNAL("contentChanged"), bool or False)

    def fileerrorlist(self):
# load the settings of error options from xml file into the globalpreferences value
        homePath = os.path.join(World.configDir, 'test_files' + os.path.sep)
        if not self.langcur:
            return
        fileName= self.langcur + "_tests.xml"
        self.filepath4error = os.path.abspath(homePath + fileName)
        if (not os.path.exists(self.filepath4error)):
            self.mainEditor.preference.make_File(fileName)
            inifilePath=homePath+"initial.xml"
            iniFile = open(inifilePath, 'r')
            contentprev = iniFile.readlines()
            content=[]
            lineno=0 
            for line in contentprev:
                lineno=lineno+1
                if lineno==3:
                    line="<Language code=\"" + self.langcur +"\">\n"
                content.append(line)
            file = open(self.filepath4error, "w")
            file.writelines(content)
            file.close()    
            iniFile.close()    
        tagValueList=self.mainEditor.preference.load_file(self.filepath4error)
        self.mainEditor.preference.loadGlobalPreferenceValue(tagValueList)    
#        print "fileerrorlist glolist", globalClass.globalerrorlist
#        print "fileerrorlist glodict", globalClass.endPuncDict

    def errorChecking(self, bool):
        """
        set errorchecking flag
        """
        if World.WF_debug:
            print "Entering errorChecking in class Operator"
        if self.langcur:
            self.fileerrorlist()
        else:    
            globalClass.globalerrorlist=[]
            globalClass.endPuncDict={}
            
        globalClass.errorCheckEnableFlag = 1
        self.errorChecking = bool
        World.settings.setValue("errorChecking", QtCore.QVariant(self.errorChecking))
            
    def autoSpellCheck(self, bool):
        """
        set autoSpell flag
        """
        if World.WF_debug:
            print "Entering autoSpellCheck in class Operator"   
        globalClass.spellCheckEnableFlag = bool
        self.autoSpell = bool
        World.settings.setValue("AutoSpellCheck", QtCore.QVariant(self.autoSpell)) 
        self.dictIsExist = self.checkAvailableDict()
        if (bool and self.dictIsExist):
            self.spellUnit(self.getCurrentUnit())
        elif (bool and not self.dictIsExist and self.langcur):
            messageBox = QtGui.QMessageBox
            message = self.trUtf8("No dictionary avaiable for current language "+self.langcur+".\nSet it under Tools -> Preferences -> Spellcheck")
            title = self.trUtf8("Spellchecker Dictionary")
            messageBox.critical(None, title, message, messageBox.Ok)
            self.__mainEditor__.ui.actionAutoSpellCheck.setChecked(False)
        if not bool:
            self.emitUnit(self.getCurrentUnit())
        if (bool and not self.dictIsExist and not self.langcur):    
            self.mainEditor.ui.actionAutoSpellCheck.setChecked(False)    
        self.emitTMCandidates()
        return
    
    def checkOnTheFly(self):
        '''
        spellcheking when typing
        '''
        if World.WF_debug:
            print "Entering checkOnTheFly in class Operator"
            
        if (self.autoSpell and self.dictIsExist):
            self.spellUnit(self.getCurrentUnit())
            
    def spellUnit(self, unit):
        """
        spellcheck a unit
        @param unit: type a unit
        """
        if World.WF_debug:
            print "Entering spellUnit in class Operator"
            
        if (not unit):
            return
        message = unit.target.encode('utf-8')
        if (self.spellCheck(message,unit.source)):
            if (self.missSpelledList):
                self.emitUnit(unit)
                if not sets.Set(self.missSpelledList).issubset(sets.Set(self.__mainEditor__.spellcheck.ignoreAll)):
                    self.emitMissNsuggestedList(self.__store__.units.index(unit))
                return True
        # the list is blank but ...
        self.emitMissNsuggestedList(self.__store__.units.index(unit))
        return False
    
    def spellTrans(self, newCheckRange = None, includeFuzzy = None):
        """
        spellcheck a range of translations
        
        @param newCheckRange: type as int or [int, int]
        @param includeFuzzy: type as bool
        """
        if World.WF_debug:
            print "Entering spellTrans in class Operator"
        loopInUnits = self.spellingLoop 
        if (newCheckRange):     # start new check
            #check available dictionary for spellchecker
            if (not self.checkAvailableDict() and self.langcur):
                messageBox = QtGui.QMessageBox
                message = self.trUtf8("No active dictionary for spellchecker.\nSet it under Tools -> Preferences -> Spellcheck")
                title = self.trUtf8("Spellchecker Dictionary")
                messageBox.critical(None, title, message, messageBox.Ok)
                self.emit(QtCore.SIGNAL("spellCheckCompleted"))
                return
                
            # reset resume point
            self.spellResumePoint = None
            
            # spellcheck fuzzy or not
            if (includeFuzzy):
                self.includeFuzzy = World.fuzzy
            else:
                self.includeFuzzy = 0
            
            # determine searching from where to where
            units = self.__store__.units
            if (units[0].isheader()):
                starting = 1
            else:
                starting = 0
                
            if (newCheckRange == World.wholeFile):
                self.offset = 0 + starting
                self.end = len(units)
            elif (newCheckRange == World.currentToFirst):
                currentUnit = self.getCurrentUnit()
#Special consideration for poxliffunits as index function fails to find index of poxliffunit
                if 'PoXliffUnit' in str(type(currentUnit)):
                    for i in range(0, len(units)):
                        if str(units[i]) == str(currentUnit):
                            self.offset = i
                            break
                else:
                    self.offset = units.index(currentUnit)
                self.end = starting - 1
            elif (newCheckRange == World.currentToLast):
                currentUnit = self.getCurrentUnit()
#Special consideration for poxliffunits as index function fails to find index of poxliffunit
                if 'PoXliffUnit' in str(type(currentUnit)):
                    for i in range(0, len(units)):
                        if str(units[i]) == str(currentUnit):
                            self.offset = i
                            break
                else:
                    self.offset = units.index(currentUnit)

                self.end = len(units)
            else:
                startingIndex = newCheckRange[0]
                lastIndex = newCheckRange[1]
                self.offset = startingIndex - 1 + starting
                if (startingIndex <= lastIndex):
                    self.end = lastIndex + starting
                else:
                    self.end = lastIndex - starting
            offset = self.offset
        else:       # continue from last check
            offset = self.spellResumePoint
        # start spellchecking each units
        if (not loopInUnits(offset, self.end)):
            self.emit(QtCore.SIGNAL("spellCheckCompleted"))
        
    def spellingLoop(self, offset, end):
        
        if World.WF_debug:
            print "Entering __init__ in class Operator"
        spellUnit = self.spellUnit
        i = offset
        if (offset < end):
            # spell forward
            while (i <end):
                unit = self.__store__.units[i]
                if hasattr(unit,'x_editor_state') and ((unit.x_editor_state == World.translated) or (unit.x_editor_state and self.includeFuzzy)):
                    globalClass.sourceWordFlag = 1
                    if (spellUnit(unit)):
                        self.spellResumePoint = i + 1
                        if sets.Set(self.missSpelledList).issubset(sets.Set(self.__mainEditor__.spellcheck.ignoreAll)):
                            self.emitMissNsuggestedList(self.__store__.units.index(unit))
                        return True
                i += 1
            else:   
                return False
        else:
            # spell backward
            while (i >self.end):
                unit = self.__store__.units[i]
                if ((unit.x_editor_state == World.translated) or (unit.x_editor_state and self.includeFuzzy)):
                    globalClass.sourceWordFlag = 1
                    if (spellUnit(unit)):
                        self.spellResumePoint = i -1
                        if sets.Set(self.missSpelledList).issubset(sets.Set(self.__mainEditor__.spellcheck.ignoreAll)):
                            self.emitMissNsuggestedList(self.__store__.units.index(unit))
                        return True
                i -= 1
            else:
                return False
    
    def spellCheck(self, message, source = ''):
        """
        spellchecking process.
        @param filename: type as string
        """
        self.linuxPlatform = platform.system() == "Linux"
        spellCheckCandidate = []
        # Takes the source only for the first time when focus appears on the source.
        if globalClass.sourceWordFlag == 1:
            globalClass.sourceWordFlag = 0
            self.missSpelledList = []
            self.suggestedList = []
            if self.spellCheckIgnoreSource:
                globalClass.SpellCheckedWordList = self.tokenize(source,'source')
            else:
                globalClass.SpellCheckedWordList = []
        if self.dictObject:
            targetWordList = self.tokenize(message, 'target')
            for each in targetWordList:
                if len(each)>1 and each not in globalClass.SpellCheckedWordList:
                    spellCheckCandidate.append(each)
            
            self.dummyList = []
            try:
                for each in spellCheckCandidate:
                    spellCheckerThread = SpellCheckerThread(self.dictObject,each,self.missSpelledList,self.suggestedList,self.dummyList)
#                    Creating and Calling the thread.
                    spellCheckerThread.start()
                    globalClass.SpellCheckedWordList.append(each)
                while len(self.dummyList) < len(spellCheckCandidate):
                    continue
            except:
                pass
        
        if self.missSpelledList:  
            return True
        else:
            return False
        
    def tokenize(self, text, textType):       
#        This function tokenizes the text according to the textType (source/target). Returns a list of tokens
        self.targetReplaceChars=globalClass.targetReplaceChars 
        if textType == 'source':
            return re.findall(r'\w+', unicode(text))
        elif textType == 'target':
            if globalClass.endPuncDict['accelerators'] != 'False':
                self.targetReplaceChars = self.targetReplaceChars.replace(globalClass.endPuncDict[globalClass.endPuncDict['accelerators']],'')
            for each in ['\\n', '\\t', '\\r']:
                try:
                    text = text.replace(each,' ')
                except:
                    pass
            for each in self.targetReplaceChars:
                try:
                    text = text.replace(each,' ')
                except:
                    pass
        return text.split()
        
    def emitMissNsuggestedList(self, unitIndex):
        """
        emit missspelled list and suggested list
        @param unitIndex: type as int
        """
        
        if World.WF_debug:
            print "Entering emitMissNsuggestedList in class Operator"
            
        currentUnit = self.getCurrentUnit()
#Special consideration for poxliffunits as index function fails to find index of poxliffunit
        if 'PoXliffUnit' in str(type(currentUnit)):
            units = self.__store__.units
            for i in range(0, len(units)):
                if str(units[i]) == str(currentUnit):
                    unitIndex = i
                    break
            
        self.emit(QtCore.SIGNAL("missNsuggestedList"), unitIndex, self.missSpelledList, self.suggestedList)

        
    def replaceSubString(self, old, new, unitIndex = None):
        """"
        replace an old substring with new substring in target
        @param unitIndex: type as int
        @param old: type as string
        @param new: type as string
        """
        
        if World.WF_debug:
            print "Entering replaceSubString in class Operator"
            

        if (unitIndex):
            unit = self.__store__.units[unitIndex]
        else:
            unit = self.getCurrentUnit()
        unit.target = unit.target.replace(old, new)
        self.emitUnit(unit)
        
    def replaceListOfSubStrings(self, para):
        """
        replace a lists of substring with new substring in targets
        @param para: type as a list of tuples(string, string, int) 
        ex: [(index, oldString, newString), (index1, oldString1, newString1),...]
        """
        
        if World.WF_debug:
            print "Entering replaceListOfSubStrings in class Operator"
            

        i = len(para) -1
        while (i>=0):
            item = para.pop(i)
            unit = self.__store__.units[item[0]]
            unit.target = unit.target.replace(item[2], item[1])
            i -=1   
        self.emitUnit(unit)
            
    def checkAvailableDict(self):
        """
        check whether dictionary is set for spellchecker
        """
        if World.WF_debug:
            print "Entering checkAvailableDict in class Operator"
            # get remembered dictionary
        globalClass.createDictFlag = 0
        dicPath = os.path.abspath(localpath + ".." + os.path.sep + "dict") + os.path.sep
        dictionaryName = World.settings.value("CurrentLanguage").toString().__str__()
        hasdict=self.mainEditor.preference.searchActiveDictionary(dictionaryName)
        if (hasdict):
            self.dictlanguage = dicPath + dictionaryName
#                my code for adding dictionary obj starts
            try:
                self.dictObject = enchant.Dict(dictionaryName)
                globalClass.sourceWordFlag = 1
                self.checkOnTheFly()
            except:
                self.dictObject = None
#                my code for adding dictionary obj ends
            return True
        else:
            return False
    
    def emitRequestTargetChanged(self):
        
        if World.WF_debug:
            print "Entering emitRequestTargetChanged in class Operator"
            

        self.emit(QtCore.SIGNAL("requestTargetChanged"))
    
#    def addThirdLanguage(self, text, lang, origin= None):
#        """"Adds an alt-trans tag and alt-trans components to the unit.
#        
#        @type txt: String
#        @param txt: Alternative translation of the source text.
#        """
#        unit = self.getCurrentUnit()
#        if (hasattr(unit, "addalttrans")):
#            unit.addalttrans(text, origin, lang)
#            self.contentChanged(True)
#        
#    def removeThirdLanguage(self, alternative):
#        """Removes the supplied alternative from the list of alt-trans tags"""
#        unit = self.getCurrentUnit()
#        if (hasattr(unit, "delalttrans")):
#            unit.delalttrans(alternative)
#            self.contentChanged(True)
    
    def getInternalGlossary(self, fileName):
        """
        get store of internal glossary
        @type fileName String
        @param fileName name of available file
        """
        
        if World.WF_debug:
            print "Entering __init__ in class Operator"
            

        from xml.dom import minidom
        xmldoc = minidom.parse(fileName)
        try:
            internalGlossary = xmldoc.getElementsByTagName("glossary")[0]
            internalFile = internalGlossary.getElementsByTagName("internal-file")[0]
            tbxElement = internalFile.getElementsByTagName("martif")[0]
            glossaryStore = tbx.tbxfile().parsestring(tbxElement.toxml())
            return glossaryStore
        except Exception,  e:
            return None
    
    def getFileName(self):
        """
        get opened FileName
        """
        
        if World.WF_debug:
            print "Entering __init__ in class Operator"
            

        return self.__fileName__
    
    def setFileName(self, fileName):
        """
        set filename opened
        @param fileName as QString
        """        
        
        if World.WF_debug:
            print "Entering setFileName in class Operator"
            

        self.__fileName__ = fileName
        
    def getStatusObject(self):
        """
        @return file's statistics object
        """
        
        if World.WF_debug:
            print "Entering getStatusObject in class Operator"
            

        return self.__status__
        
    def setStatusObject(self, status):
        """
        set file's statistics of object
        @param file's statistic object
        """
        
        if World.WF_debug:
            print "Entering setStatusObject in class Operator"
            

        self.__status__ = status
        
    def getFileOpenedObject(self):
        """
        @return file opened object as xliff, po, tmx object
        """
        
        if World.WF_debug:
            print "Entering getFileOpenedObject in class Operator"
            

        return self.__store__
    
    def setFileOpenedObject(self,  store):
        """
        set file opened object
        @param file opened object as xliff, po, tmx object
        """
        
        if World.WF_debug:
            print "Entering setFileOpenedObject in class Operator"
            

        self.__store__ = store

    def getCurrentUnitIndex(self):
        """
        @return the index of current unit
        """
        
        if World.WF_debug:
            print "Entering getCurrentUnitIndex in class Operator"
            

        return self.__currentUnitIndex__
    
    def embedGlossary(self):
        InternalGlossary.embedGlossary()
        
    def setfiletranslationlanguage(self, fileName): 
  # sets the translation language into the po and xliff file 
        if self.isXliff:
            self.getcurlanguage(fileName)
        else:
            self.getpocurlanguage(fileName)
            
       # self.parent.timersetting4Thesaurus()
        
    def moreLang(self, languageCode):  
        #loads the ISO3 langauge list in language combo box 
        if languageCode=="More Languages":
            self.mainEditor.preference.load_language_ISO3(self.curLanguageUi.ui.cbxcurLanguage)
        else:
            return
            
    def setcurlanguage(self):
        self.curlang=self.curLanguageUi.ui.cbxcurLanguage.currentText()
        self.curlangindex=self.curLanguageUi.ui.cbxcurLanguage.currentIndex()
        self.curcontry=self.curLanguageUi.ui.cbxcurCountry.currentText()
        self.curcontryindex=self.curLanguageUi.ui.cbxcurCountry.currentIndex()
        self.langcur=self.mainEditor.preference.make_level("currentLang")
        if self.filetype=="xliff":
            self.store.settargetlang(self.langcur)
        elif self.filetype=="po":
            self.currentheaderDic["X-Language"]=self.langcur
            otherComments, self.headerDic = self.headerData()
            self.updateNewHeader(otherComments, self.currentheaderDic)
        World.settings.setValue("CurrentLanguage", QtCore.QVariant(self.langcur))  
        World.settings.setValue("FullLanguage", QtCore.QVariant(self.curlang))
        self.curLanguageUi.close()
        self.mainEditor.ui.actionSave.setEnabled(True)
        self.mainEditor.fileaction.isSaved=False
        self.mainEditor.operator.modified=True
        self.mainEditor.preference.writeMoreLanguage(self.curLanguageUi.ui.cbxcurLanguage)
        
    def loadopenwindow(self, filetype):
        #show the current language window
        from WordForge.Editor.modules.currentLanguage import curLanguage
        self.curLanguageUi = curLanguage(self)
        self.curLanguageUi.setWindowTitle(self.tr("Current Translation Language"))
        ISO2List=self.mainEditor.preference.load_language_ISO2(self.curLanguageUi.ui.cbxcurLanguage)
        self.mainEditor.preference.load_countries(self.curLanguageUi.ui.cbxcurCountry)
        currentlang=str(World.settings.value("FullLanguage",  QtCore.QVariant("Bengali [bn]")).toString())
        currentlang=currentlang+"\n"
        j=-1
        for i in ISO2List:
            j=j+1
            if(i==currentlang):
                break    
        self.curLanguageUi.ui.cbxcurLanguage.setCurrentIndex(j)
        countryCode=World.settings.value("CountryCode",  QtCore.QVariant("18")).toInt()[0]
        self.curLanguageUi.ui.cbxcurCountry.setCurrentIndex(countryCode)
        self.connect(self.curLanguageUi.ui.cbxcurLanguage, QtCore.SIGNAL("currentIndexChanged(const QString &)"), self.moreLang)
        self.filetype=filetype
        self.connect(self.curLanguageUi.ui.OkButton, QtCore.SIGNAL("clicked()"), self.setcurlanguage)

    def openwanringmsg(self, codefromPersonalize):
        #shows wanring for the mismatch between current language window setting and preferences personalize settings
        messageBox = QtGui.QMessageBox
        msgx = self.trUtf8("The language of this file <b>" + self.langcur + "</b> is different from the language in Preferences <b>" + codefromPersonalize + "</b>. If you want to change the language in Preferences click on the <b>Preferences</b> button. If you want to change the language to which this file will be translated, change it from Edit->Change Language of Present File")
        ret = messageBox()
        ret.setTextFormat(QtCore.Qt.RichText)
        ret.setText(msgx)
        PreferenceButton = ret.addButton(self.trUtf8("Preferences"), messageBox.ActionRole)
        OkButton = ret.addButton(self.trUtf8("Ok"),messageBox.YesRole)
        ret.exec_()

        if (ret.clickedButton() == PreferenceButton):
            self.emit(QtCore.SIGNAL("ShowPreference"), 0)
        elif (ret.clickedButton() == OkButton):
                    return
                    
    def getcurlanguage(self, fileName, purpose=""):
        #saves the translation language code in xliff file
        self.langcur=""
        sourceTree= etree.parse(fileName)
        root = sourceTree.getroot()
        self.fileNode = root[0]
        self.langcur=self.fileNode.get('target-language')
        if not self.langcur:
            self.loadopenwindow("xliff")
        else:
            World.settings.setValue("CurrentLanguage", QtCore.QVariant(self.langcur))        
            codefromPersonalize=World.settings.value("personalcountryLanguageCode", QtCore.QVariant("bn_BD")).toString()
            wanring=World.settings.value("warningLangMismatch", QtCore.QVariant(True)).toBool()
            if codefromPersonalize!=self.langcur and wanring:
                self.openwanringmsg(codefromPersonalize)
        
        
    def getpocurlanguage(self, fileName, purpose=""):
        #saves the translation language code in po file
        header = self.__store__.header() 
        self.langcur=""
        self.currentheaderDic=[]
        if header:
            self.currentheaderDic = self.__store__.parseheader()        
            if "X-Language" not in self.currentheaderDic.keys():
                    self.loadopenwindow("po")    
            elif not self.currentheaderDic["X-Language"]:
                    self.loadopenwindow("po")
            else:
                self.langcur = self.currentheaderDic["X-Language"]
                codefromPersonalize=World.settings.value("personalcountryLanguageCode", QtCore.QVariant("bn_BD")).toString()
                wanring=World.settings.value("warningLangMismatch", QtCore.QVariant(True)).toBool()
                if codefromPersonalize!=self.langcur and wanring:
                    self.openwanringmsg(codefromPersonalize)
        else:
                userProfileDic = userProfileDic = {'charset':"CHARSET", 'encoding':None, 'project_id_version': None, 'pot_creation_date':None, 'po_revision_date': False, 'last_translator': None, 'language_team':None, 'mime_version':None, 'plural_forms':None, 'report_msgid_bugs_to':None}
                self.headerDic = self.makeNewHeader(userProfileDic) 
                self.getpocurlanguage(fileName)
            
        World.settings.setValue("CurrentLanguage", QtCore.QVariant(self.langcur))      
        
    def getsourcelanguage(self, fileName):
        if self.isXliff:
            self.langcur=""
            sourceTree= etree.parse(fileName)
            root = sourceTree.getroot()
            self.fileNode = root[0]
            self.sourcelang=self.fileNode.get('source-language')  
    
class SpellCheckerThread(Thread):
#    This thread class checks spelling of a single word
    def __init__ (self,checkerObj,word,missSpelledList,suggestedList, dummyList):
#        checkerObj is the hunspell object,dummyList is used for halting the execution until the globallist is filled up.  
        Thread.__init__(self)
        self.checkerObj = checkerObj
        self.word = word
        self.missSpelledList = missSpelledList
        self.suggestedList = suggestedList
        self.dummyList = dummyList
        #      0 means send for spellcheck, 1 means don't send for spellcheck
        self.flag = 0 
        # 0 means don't need to change the word, 1 means variable word has been changed due to accelerator. 
        self.flag1 = 0
         
    def run(self):
#        This function is called when the thread method start() is complete.
        if globalClass.endPuncDict['accelerators'] != 'False' and globalClass.endPuncDict[globalClass.endPuncDict['accelerators']] in self.word:
            if len(self.word)>2:
                self.tempword = self.word
                self.word = self.word.replace(globalClass.endPuncDict[globalClass.endPuncDict['accelerators']], '') 
                self.flag1 = 1
            elif len(self.word)<=2:
                self.flag = 1
        if self.flag == 0:
            if not self.checkerObj.check(self.word):
                self.suggestedList.append(self.checkerObj.suggest(self.word))
                if self.flag1 == 1:
                    self.word = self.tempword
                    self.flag1 = 0
                self.missSpelledList.append(self.word)
        self.dummyList.append(1)
