#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#Copyright (c) 2006 - 2007 by The WordForge Foundation
#                    www.wordforgefoundation.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details. 
# 
# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)

# This module is working on overview of source and target

from PyQt4 import QtCore, QtGui
from WordForge.Editor.ui.Ui_Overview import Ui_Form
import WordForge.Editor.modules.World as World
from WordForge.translate.storage import xliff
from WordForge.Editor.modules.globalClass import globalClass
from WordForge.Editor.modules.EraseTarget import eraseTarget
from WordForge.Editor.modules.Status import Status
from WordForge.Editor.modules.Preference import Preference
import WordForge.Editor.ui.images_rc
import unicodedata

class OverviewDock(QtGui.QDockWidget):
    def __init__(self, parent):
        self.parent = parent
        QtGui.QDockWidget.__init__(self, parent)
        self.setObjectName("overviewDock")
        self.setWindowTitle(self.tr("Overview"))
        self.form = QtGui.QWidget(self)
        self.ui = Ui_Form()
        self.ui.setupUi(self.form)
        self.setWidget(self.form)
        
        # set up table appearance and behavior
        self.viewSetting()
        self.ui.tableOverview.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        self.ui.tableOverview.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        self.ui.tableOverview.horizontalHeader().setSortIndicatorShown(True)
        self.ui.tableOverview.resizeColumnToContents(0)
        self.ui.tableOverview.resizeColumnToContents(3)
        self.ui.tableOverview.horizontalHeader().setResizeMode(1, QtGui.QHeaderView.Stretch)
        self.ui.tableOverview.horizontalHeader().setResizeMode(2, QtGui.QHeaderView.Stretch)
        self.ui.tableOverview.horizontalHeader().setHighlightSections(False)
        self.ui.tableOverview.verticalHeader().hide()
        self.applySettings()
        
        self.fuzzyColor = QtGui.QColor(246, 238, 156, 140)
        self.blankColor = QtGui.QColor(255, 255, 255, 0)
        
        self.iconBookmark = QtGui.QIcon(":/bookmark.png")
        self.iconnoteBookmark = QtGui.QIcon(":/noteBookmark.png")
        self.fuzzyIcon = QtGui.QIcon(":/fuzzy.png")
        self.noteIcon = QtGui.QIcon(":/note.png")
        self.approvedIcon = QtGui.QIcon(":/approved.png")
        self.pluralIcon = QtGui.QIcon(":/plural.png")
        self.pluralFuzzyIcon = QtGui.QIcon(":/pluralfuzzy.png")
        self.error = QtGui.QIcon(":/Error_checking.png")
        self.blankIcon = QtGui.QIcon()
        self.normalState = QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable
        self.indexMaxLen = 0
        self.units = []
        self.visibleRow = []
        self.filter = None
        self.unitList = []
        self.prevItem=0
        self.bookmark=False
        self.sourcewithnonprint=""
        self.targetwithnonprint=""
        self.hasError=None
        self.changedSignal = QtCore.SIGNAL("currentCellChanged(int, int, int, int)")
        self.connect(self.ui.tableOverview, self.changedSignal, self.emitCurrentIndex)
        self.connect(self.ui.tableOverview.model(), QtCore.SIGNAL("layoutChanged()"), self.showFilteredItems)
        self.connect(self.ui.tableOverview, QtCore.SIGNAL("cellDoubleClicked(int, int)"), self.fillTarget)
        self.prevRow =0
        self.prevIndexItem=0
        self.ui.tableOverview.installEventFilter(self)
        itemError = QtGui.QTableWidgetItem("Unchecked")
        self.prevtext=""
        self.undo_dic={}
        self.indexlist=[]
    
    def eventFilter(self, obj, event):
        if (obj == self.ui.tableOverview):  
            if (event.type() == QtCore.QEvent.ContextMenu):
                """
                Request a translation for unit's source.
                """
                self.tableContextMenu(event)
                return True
            else:
                return False
        else:
            return self.eventFilter(obj, event)
        
        
    def tableContextMenu(self, event):
        """
        Save the position of mouse, and emit "requestUnit" signal.
        """
        self.globalPos = event.globalPos()
        self.emit(QtCore.SIGNAL("requestUnit"))
    
    def popupTranslation(self, candidates):
        """
        Popup menu of translation.
        """
        menu = QtGui.QMenu()
        if (not candidates):
            menuAction = menu.addAction(self.tr("(no translation)"))
            menu.exec_(self.globalPos)
            return
        
        for candidate in candidates:
            strCopy = unicode(self.tr("Copy \"%s\" to target.")) % unicode(candidate.target)
            menuAction = menu.addAction(strCopy)
            menuAction.setData(QtCore.QVariant(candidate.target))
            self.connect(menuAction, QtCore.SIGNAL("triggered()"), self.copyToTarget)
        menu.exec_(self.globalPos)
        self.disconnect(menuAction, QtCore.SIGNAL("triggered()"), self.copyToTarget)
    
    def copyToTarget(self):
        """
        emit "targetChanged" signal with self.sender().text().
        """
        text = self.sender().data().toString()
        self.emit(QtCore.SIGNAL("targetChanged"), unicode(text))
    
    def closeEvent(self, event):
        """
        set text of action object to 'show Overview' before closing Overview
        @param QCloseEvent Object: received close event when closing widget
        """
        QtGui.QDockWidget.closeEvent(self, event)
        self.toggleViewAction().setChecked(False)
        
    def slotNewUnits(self, units):
        """
        set the filter to filterAll, fill the table with units.
        @param units: list of unit class.
        """
        self.targetBeforeEdit = None
        self.viewSetting(units)
        if (not units):
            self.emitFirstLastUnit()
            return
        self.indexMaxLen = len(str(len(units)))
        self.filter = World.filterAll
        self.units = units
        self.ui.tableOverview.setSortingEnabled(False)
        
        self.setUpdatesEnabled(False)
        oldValue = None
        lenUnit = len(units)
        i = 0.0
        for unit in units:
            if (self.filter & unit.x_editor_state):
                self.addUnit(unit)
                unit.x_editor_index = int(i)
            i += 1
            value = int((i / lenUnit) * 100)
            # emit signal when only percentage changed
            if (oldValue != value):
                self.emit(QtCore.SIGNAL("progressBarValue"), value)
                oldValue = value    
             
        self.ui.tableOverview.setSortingEnabled(True)
        self.ui.tableOverview.sortItems(0)
        self.setUpdatesEnabled(True)

    def filterChanged(self, filter, lenFilter):
        """
        show the items which are in filter.
        @param filter: helper constants for filtering.
        @param lenFilter: len of filtered items.
        """
        if (filter == self.filter):
            return
        self.filter = filter
        self.showFilteredItems()
        
    def addUnit(self, unit):
        """
        add the unit to table.
        @param unit: unit class.
        """
        self.storingEnable=World.settings.value("enableStore",  QtCore.QVariant(True)).toBool()
        self.unitList.append(unit)
        row = self.ui.tableOverview.rowCount()
        self.ui.tableOverview.setRowCount(row + 1)
        self.ui.tableOverview.setIconSize(QtCore.QSize(40, 18))
        self.ui.tableOverview.setColumnWidth (0, 60 )
        # index field
        itemIndex = QtGui.QTableWidgetItem(self.indexString(row + 1))
        itemIndex.setTextAlignment(QtCore.Qt.AlignRight + QtCore.Qt.AlignVCenter)
        itemIndex.setFlags(self.normalState)
        itemIndex.setData(QtCore.Qt.UserRole, QtCore.QVariant(row))
        unit.x_editor_tableItem = itemIndex
        self.ui.tableOverview.setItem(row, 0, itemIndex)
        self.markComment(row, unit.getnotes(), unit)
        # source field
        source = unit.source
        line = source.find("\n")
        if (line >= 0):
            source = source[:line] + "..."
        itemSource = QtGui.QTableWidgetItem(source)
        itemSource.setFlags(self.normalState)
        self.ui.tableOverview.setItem(row, 1, itemSource)
        # target field
        target = unit.target or ""
        line = target.find("\n")
        if (line >= 0):
            target = target[:line] + "..."
        itemTarget = QtGui.QTableWidgetItem(target)
        self.ui.tableOverview.setItem(row, 2, itemTarget)
        # error field
        attrlist=[]
        note=unit.getnotes()
        if note:
            try:
                attrlist=unit.getErrorTypeAttribute("error")
            except: 
                pass
            if attrlist:
                count=len(attrlist)
                for i in range(count):
                        if attrlist[i]=="clean":
                            itemError = QtGui.QTableWidgetItem("Clean")
                            break
                        else:
                            itemError = QtGui.QTableWidgetItem("Error")
            else: 
                itemError = QtGui.QTableWidgetItem("Unchecked")
        else:   
            itemError = QtGui.QTableWidgetItem("Unchecked")
        itemError.setFlags(self.normalState)
        itemError.setTextAlignment(QtCore.Qt.AlignHCenter + QtCore.Qt.AlignVCenter)
        self.ui.tableOverview.setItem(row, 4, itemError)
        # status field
        itemStatus = QtGui.QTableWidgetItem()
        itemStatus.setFlags(self.normalState)
        itemStatus.setTextAlignment(QtCore.Qt.AlignHCenter + QtCore.Qt.AlignVCenter)
        self.ui.tableOverview.setItem(row, 3, itemStatus)
        self.markState(row, unit.x_editor_state)
        #get bookmarked value
        if self.parent.operator.isXliff:
            if unit.isbookmarked():
                curritem = self.ui.tableOverview.item(row, 0)
                self.setBookmarks(unit, curritem)
                note=unit.getnotes()
                if note:
                    curritem.setIcon(self.iconnoteBookmark)
                else:
                    curritem.setIcon(self.iconBookmark)    
                self.parent.ui.actionNextBookmark.setEnabled(True)
                self.parent.ui.actionPreviousBookmark.setEnabled(True)
                self.parent.ui.actionClearBookmarks.setEnabled(True)
    
    def setBookmarks(self, unit, id):
# sets bokkmark for current row
        bookmarkedUnit=unit
        bookmarkString =  str(id.text()) + World.delimiter + unit.source[:15] + "..."
        bookmarkDic = {}
        bookmarkList = []
        bookmarks = World.settings.value("bookmarkList").toStringList()
#        for i in bookmarks:
#            print "o", i    
        bookmarks.removeAll(bookmarkString)
        bookmarks.prepend(bookmarkString)
        for bookmark in bookmarks:
            bookmarkStNum = int(str(bookmark).split(World.delimiter)[0])
            bookmarkSt = World.delimiter + str(bookmark).split(World.delimiter)[1]
            bookmarkList.append(bookmarkStNum)
            bookmarkDic[bookmarkStNum] = bookmarkSt
        bookmarks = QtCore.QStringList()
        bookmarkList.sort()
        for bookmarkNum in bookmarkList:
            bookmarkString = str(bookmarkNum) + bookmarkDic[bookmarkNum]
            if bookmarkString not in bookmarks:
                bookmarks.append(bookmarkString)
#        for i in bookmarks:
#            print "setBookmarks over", i    
        World.settings.setValue("bookmarkList", QtCore.QVariant(bookmarks))
        self.parent.updateBookmarkAction()
    def rowtext(self,  text):
#        text=str(text)
        if self.prevtext == text:
            return 
        r=int(self.ui.tableOverview.currentRow())
        if r not in self.indexlist:
            self.indexlist.append(r)
            undo_lst=[]
            self.undo_dic[r]=undo_lst
            
#        if len(text)==0 and len(self.undo_dic[r])==0 and text[len(text)-1]==' ':
#            s=self.partitioning(text)
#            self.undo_dic[r].append(s)
#            self.prevtext=text
            
        if len(text):
            if text[len(text)-1]==' ':
                self.undo_dic[r].append(text)
                self.prevtext=text
        
        
    def partitioning(self,  text):
#        text=str(text.rstrip())
        text=text.rstrip()
        lngth=len(text)
        s=""
        if lngth >= 3:
            for  i in range(lngth-1, -1,  -1):
                if text[i]==' ':
                    s=text[0:i+1]+""
                    break
        return s
        
    def emitCurrentIndex(self, row, col, preRow, preCol):
        """
        emit filteredIndex signal with selected unit's index.
        """
        # do not emit signal when select the same row.
        if (row == preRow):
            return

        if row != -1 or col != -1:
            if self.parent.operator.internalGlossaryUsed:
                self.parent.operator.showNewGlossaryTerm()
                
        # emit the index of current unit.
        self.emitTargetChanged(preRow)
        item = self.ui.tableOverview.item(row, 0)
        if (hasattr(item, "data")):
            index = item.data(QtCore.Qt.UserRole).toInt()[0]
            self.targetItem = self.ui.tableOverview.item(self.ui.tableOverview.currentRow(), 2).text()
            self.emit(QtCore.SIGNAL("filteredIndex"), index, preRow)
        
    def emitTargetChanged(self, preRow = None):
        """
        emit targetChanged signal if target column has changed.
        """
        targetItem = self.ui.tableOverview.item(preRow, 2)
        if (not hasattr(targetItem, "text")):
            return
        targetAfterEdit = targetItem.text()
        self.targetBeforeEdit = self.unit.target
        if (targetAfterEdit != self.targetBeforeEdit) and (self.targetBeforeEdit != None):
            # target has changed
            target = unicode(targetAfterEdit)
            if (self.unit and self.unit.hasplural()):
                # change only the first string of plural
                string0 = target
                target = []
                for string in self.unit.target.strings:
                    target.append(string)
                target[0] = string0
            self.emit(QtCore.SIGNAL("targetChanged"), target)
            if (self.unit):
                if not self.parent.ui.actionShow_Nonprinting_Char.isChecked():
                    self.updateText(self.unit.target or "")
                else:    
                    self.updateText(unicode(targetItem.text()) or "")
      
            
    def fillTarget(self, row, column):
        """
        Fill current item row #2 with self.unit.target.
        This call when user double click on target field to edit.
        """
        targetItem = self.ui.tableOverview.item(self.ui.tableOverview.currentRow(), 2)
        # unmark item fuzzy when text changed
        row = self.ui.tableOverview.row(targetItem)
        if (targetItem and self.unit):
            self.targetBeforeEdit = self.unit.target
            targetItem.setText(self.targetBeforeEdit)
            
    def keyReleaseEvent(self, event):
#        This event is called automatically when a key release occurs.
        target = self.parent.dockTUview.ui.txtTarget.toPlainText()
        source = self.parent.dockTUview.ui.txtSource.toPlainText()
        if source == globalClass.currentSource:
                if globalClass.preComputeFlag == 1:
                    self.parent.callPreCompute()
                    globalClass.preComputeFlag = 0        
    
    def updateText(self, text, plural = 0):
        """
        Set text to current item.
        @param text: text to set in target column.
        @param plural: bool to indicate a current text need to update.
        """
        if (plural) or (not self.unit):
            # do not update plural strings.
            return
        targetItem = self.ui.tableOverview.item(self.ui.tableOverview.currentRow(), 2)
        row = self.ui.tableOverview.row(targetItem)
        
        if (targetItem) and (not self.ui.tableOverview.isRowHidden(row)):
#            # TODO: use a function here
#            # unmark item fuzzy when text changed
#            self.unit.x_editor_state &= ~World.fuzzy
#            self.markState(row, self.unit.x_editor_state)
            
            # shorten text for display only.
            line = text.find("\n")
            if (line >= 0):
                text = text[:line] + "..."  
            targetItem.setText(text)
        
        # unmark item fuzzy when text changed
        if (self.updateTextRow != row):
            self.item0 = self.ui.tableOverview.item(row, 0)
            self.item1 = self.ui.tableOverview.item(row, 1)
            self.item2 = self.ui.tableOverview.item(row, 2)
            self.item3 = self.ui.tableOverview.item(row, 3)
            self.item4 = self.ui.tableOverview.item(row, 4)
        self.updateTextRow = row
        
        if (hasattr(self, "item3")):
            try:
                if (text):
                    self.item3.setText("Ok")
                else:
                    self.item3.setText("")
                    self.item4.setText("Unchecked")
#                    self.unit.x_editor_state=4
                    self.item0.setBackgroundColor(self.blankColor)
                    self.item1.setBackgroundColor(self.blankColor)
                    self.item2.setBackgroundColor(self.blankColor)
                    self.item3.setBackgroundColor(self.blankColor)
                    self.item4.setBackgroundColor(self.blankColor)
                
                if (self.item3.text() == "Ok"):
                    self.item0.setBackgroundColor(self.blankColor)
                    self.item1.setBackgroundColor(self.blankColor)
                    self.item2.setBackgroundColor(self.blankColor)
                    self.item3.setBackgroundColor(self.blankColor)
                    self.item4.setBackgroundColor(self.blankColor)
                if (text):
                    self.item3.setText("Ok")
                else:
                    self.item3.setText("")
            except:
                pass

    def eraseTarget(self):
                # shows the erase target window
        from WordForge.Editor.modules.EraseTarget import eraseTarget
        self.eraseTargetUi = eraseTarget(self)
        self.eraseTargetUi.setWindowTitle(self.tr("Erase Target"))
        self.connect(self.eraseTargetUi.ui.OkButton, QtCore.SIGNAL("clicked()"), self.eraseAllTarget)
    
    def shownonprintChar(self, bool):
        #shows the non printing characters
        activestyle=World.settings.value("invisiblecharacater", QtCore.QVariant(True)).toBool()
        row=0
        replacechar=[]
        for unit in self.units:
            textsource=unicode(unit.source)
            texttarget=unicode(unit.target)
#            if row==self.ui.tableOverview.currentRow():
#                texttarget=currtarget
            self.sourcewithnonprint=""
            self.targetwithnonprint=""
            orgchar=unicode(" ")
            decimal=0
            for char in textsource:
                unicodeno=ord(char)
                if bool:
                    #showing invisible chars in source
                    if unicodedata.category(unichr(unicodeno)) in World.catsCrl:
                        for decimal in World.dict_nonprintable.keys():
                            if unicodeno==decimal:
                                name=World.dict_nonprintable[decimal]
                                replacechar=World.dict_invisibleSpecial[name]
                                if activestyle:
                                    self.sourcewithnonprint=self.sourcewithnonprint+replacechar[0]
                                else:
                                    self.sourcewithnonprint=self.sourcewithnonprint+replacechar[1]    
                    else: 
                        self.sourcewithnonprint=self.sourcewithnonprint+char 
                else:
                        #bool else
                        if char in World.lst_invisibleSpecial:
                            for k, v in World.dict_invisibleSpecial.iteritems():
                                if char==v:
                                    name=k
                                    for k, v in World.dict_nonprintable.iteritems():
                                        if name==v:
                                            decimal=k
                                            orgchar=unichr(decimal)
                                            break
#                            orgchar=unichr(32)
                            self.sourcewithnonprint=self.sourcewithnonprint+orgchar
                        else:
                            self.sourcewithnonprint=self.sourcewithnonprint+char
            for char in texttarget:
                unicodeno=ord(char)
                if bool:
                    #showing invisible chars in target
                    if unicodedata.category(unichr(unicodeno)) in World.catsCrl:
                        for decimal in World.dict_nonprintable.keys():
                            if unicodeno==decimal:
                                name=World.dict_nonprintable[decimal]
                                replacechar=World.dict_invisibleSpecial[name]
                                if activestyle:
                                    self.targetwithnonprint=self.targetwithnonprint+replacechar[0]
                                else:
                                    self.targetwithnonprint=self.targetwithnonprint+replacechar[1]    
                    else:    
                        self.targetwithnonprint=self.targetwithnonprint+char 
                else: 
                        #bool else
                        if unicodeno in World.dict_nonprintable.keys():
                            orgchar=unichr(unicodeno)
                            self.targetwithnonprint=self.targetwithnonprint+orgchar
                        else:
                            self.targetwithnonprint=self.targetwithnonprint+char
            self.ui.tableOverview.item(row, 1).setText(self.sourcewithnonprint)
            self.ui.tableOverview.item(row, 2).setText(self.targetwithnonprint)
            row=row+1
        currtarget=self.parent.dockTUview.origtarget
        self.parent.dockTUview.origtarget=unicode(self.parent.dockTUview.ui.txtTarget.toPlainText())
        source2=(self.ui.tableOverview.item(self.ui.tableOverview.currentRow(), 1)).text()
        self.parent.dockTUview.ui.txtSource.setPlainText(source2)
        target2=unicode((self.ui.tableOverview.item(self.ui.tableOverview.currentRow(), 2)).text())
        self.parent.dockTUview.ui.txtTarget.setPlainText(target2)
        if not bool:
            if activestyle:
                self.parent.dockTUview.ui.txtTarget.setPlainText(currtarget)
            else:
                currtarget=self.parent.operator.stripspecial(self.parent.dockTUview.ui.txtTarget.toPlainText())
                self.parent.dockTUview.ui.txtTarget.setPlainText(currtarget)
                
        
    def eraseAllTarget(self):
        # erases all translations of the file
        row=0    
        removedRows=[]
        numOfRow=self.parent.errorTable.ui.errorTable.rowCount()
        for unit in self.units:
            if self.eraseTargetUi.ui.chkEraseTarget.isChecked():
                if unit.source==unit.target:
                    self.parent.operator.__status__.markTranslated(unit, False)
                    self.parent.operator.__emitUnitsStatus__()
                    unit.x_editor_state=4
                    unit.target=""
                    try:
                        notlist=unit.removeallnotes("error")
                    except:
                        pass
#                    notlist=unit.removeallnotes("po-translator")
                    item1 = int(self.ui.tableOverview.item(row, 0).text())
                    item2 = self.ui.tableOverview.item(row, 2)
                    item3 = self.ui.tableOverview.item(row, 3)
                    item4 = self.ui.tableOverview.item(row, 4)
                    item2.setText("")
                    item3.setText("")
                    item4.setText("Unchecked")
                    self.parent.dockTUview.ui.txtTarget.setText("")
                    self.markState(row, unit.x_editor_state)
                    removedRows.append(item1)
                row=row+1
            else:    
                self.parent.operator.__status__.markTranslated(unit, False)
                self.parent.operator.__emitUnitsStatus__()
                unit.x_editor_state=4
                target=u''
                unit.settarget(target)
                try:
                        notlist=unit.removeallnotes("error")
                except:
                    pass
#                for row in range(self.parent.unitnum):
                item2 = self.ui.tableOverview.item(row, 2)
                item3 = self.ui.tableOverview.item(row, 3)
                item4 = self.ui.tableOverview.item(row, 4)
                item2.setText("")
                item3.setText("")
                item4.setText("Unchecked")
                self.parent.dockTUview.ui.txtTarget.setText("")
                self.markState(row, unit.x_editor_state)
                row=row+1
                for i in range(numOfRow):
                    self.parent.errorTable.ui.errorTable.removeRow(i)    
            self.eraseTargetUi.close()
            self.parent.ui.actionSave.setEnabled(True)
            self.parent.fileaction.isSaved=False
            self.parent.operator.modified=True
            if self.eraseTargetUi.ui.chkEraseTarget.isChecked():
                for i in range(numOfRow):
                    if self.parent.errorTable.ui.errorTable.item(i, 0)!=None:
                        errorRow=int(self.parent.errorTable.ui.errorTable.item(i, 0).text())
                        if errorRow in removedRows:
                            self.parent.errorTable.ui.errorTable.removeRow(i)     
        
    def markFileError(self, index, unitNumber, markError):   
       # marks error, clean  and unchecked in the overview. 
        #Aditi's code for error column in overview
#        if globalClass.hasErrors:
        for row in range(unitNumber):
            itemInOverView = int(self.parent.dockOverview.ui.tableOverview.item(row, 0).text())
            
            if index==itemInOverView:
                self.ui.tableOverview.setSortingEnabled(False)
                if markError==1:  
                    hasError=QtGui.QTableWidgetItem("Error")
                if markError==0:   
                    hasError=QtGui.QTableWidgetItem("Clean")    
                hasError.setFlags(self.normalState)
                hasError.setTextAlignment(QtCore.Qt.AlignHCenter + QtCore.Qt.AlignVCenter)
                self.ui.tableOverview.setItem(row,  4, hasError)
                self.ui.tableOverview.setSortingEnabled(True)
                break
            
#        #end of Aditi's code           
    def setError(self, errorEntry, row, unit):
        # inesrts the error , clean and unchecked in the 5th column of overview
        self.ui.tableOverview.item(row, 4).setText(errorEntry)
        self.parent.errorTable.preCom=False
        
    def updateView(self, unit):
        """
        highlight the table's row, mark comment icon, mark state icon,
        and set the target text according to unit.
        @param unit: unit class
        """
#        print "current row", self.ui.tableOverview.item(self.ui.tableOverview.currentRow(), 1).text()
#        
        currentError=''

        if (not unit) or (not hasattr(unit, "x_editor_tableItem")):
            self.unit = None
            return
        self.targetBeforeEdit = None
        self.unit = unit
        
        # TODO: old item that cached in unit could be delete everytime the table add its new item
        # get rid of using it later.
        try:
            row = self.ui.tableOverview.row(unit.x_editor_tableItem)
            unit.x_editor_row = self.visibleRow.index(row)
        except:
            return
        targetItem = self.ui.tableOverview.item(row, 2)
        # update target column only if text has changed.
        
        if not self.parent.ui.actionShow_Nonprinting_Char.isChecked():
#            if (targetItem.text() != unit.target):
            target = self.shorten(unit.target or "")
            targetItem.setText(target)
        else:
            itemtarget=unicode(targetItem.text())
            target = self.shorten(itemtarget or "")
            targetItem.setText(target)
        self.markComment(row, unit.getnotes(), unit)
        self.setIconBookmark()

        self.disconnect(self.ui.tableOverview, self.changedSignal, self.emitCurrentIndex)
        self.ui.tableOverview.selectRow(row)
        self.connect(self.ui.tableOverview, self.changedSignal, self.emitCurrentIndex)
        # mark state only after the current row is selected.
        self.markState(row, unit.x_editor_state)
        
        #Aditi's code for error column in overview
        self.storingEnable=World.settings.value("enableStore",  QtCore.QVariant(True)).toBool()
        unitNote=''
        unitNote=unit.getnotes()  
        
        currentRow=self.ui.tableOverview.currentRow()
        
        currentError=self.ui.tableOverview.item(self.ui.tableOverview.currentRow(), 4)
        currentter=self.ui.tableOverview.item(self.ui.tableOverview.currentRow(), 2)
        rownum=self.parent.unitnum            
        row=0
        for row in range(rownum):
            index=self.ui.tableOverview.item(row, 0)
            index=int(index.text())  
            if index==self.prevIndexItem:
                self.prevRow=row
                break               
        if currentError:
            currentError=currentError.text() 
        self.ui.tableOverview.setSortingEnabled(False) 
        if globalClass.hasErrors ==2:                 
            if currentError== "Error": 
                self.setError("Error", currentRow, unit)
            elif currentError== "Clean":    
                self.setError("Clean", currentRow, unit)
            elif currentError== "Unchecked":    
                self.setError("Unchecked", currentRow, unit)
        elif globalClass.hasErrors ==0: 
            self.setError("Error", self.prevRow, unit)
            globalClass.hasErrors=2
        elif globalClass.hasErrors ==1:
            globalClass.hasErrors=2
            self.setError("Clean", self.prevRow, unit)
        self.prevUnit= self.unit
        self.prevIndexItem=int((self.ui.tableOverview.item(self.ui.tableOverview.currentRow(), 0)).text())
        #end of Aditi's code
        if ((not self.parent.ui.actionFilterTranslated.isChecked()) and (not self.parent.ui.actionFilterFuzzy.isChecked()) and (not self.parent.ui.actionFilterUntranslated.isChecked())):
            self.ui.tableOverview.setSortingEnabled(True) 
        else: 
            self.ui.tableOverview.setSortingEnabled(False)    
        self.emitFirstLastUnit()
        # for updateText function
        self.updateTextRow = -1
        
        #show error message into the error table if "show error" checkbox is checked in preference,done by Aditi
        showErrorEnable=World.settings.value("enableErrorShow",  QtCore.QVariant(True)).toBool() 
        visible=self.parent.errorTable.isVisible()
        note=''
        attrlist=[]
        notelist=[]
        showerrorMessageList=[]
        if showErrorEnable and visible and self.parent.operator.isXliff:
            note=unit.getnotes("error")
            if note:
                item = self.ui.tableOverview.item(self.ui.tableOverview.currentRow(), 0)
                item=str(item.text())
                
                attrlist=unit.getErrorTypeAttribute("error")
                notelist=note.split("\n")         
                count=len(attrlist)
            
            if(attrlist):
                for i in range(count):
                    if attrlist[i]=="clean":
                        break  
                    showerrorMessageList.append(item)     
                    showerrorMessageList.append(attrlist[i])
                    showerrorMessageList.append(notelist[i])
                    self.parent.errorTable.fillTable(showerrorMessageList) 
        if showErrorEnable and visible and not attrlist:
            currentRowIndex=self.ui.tableOverview.item(self.ui.tableOverview.currentRow(), 0)  
            currentRowIndex=int(currentRowIndex.text())
            rownum = self.parent.errorTable.ui.errorTable.rowCount()
            row=0
            for row in range(rownum):
                index=self.parent.errorTable.ui.errorTable.item(row, 0)
                if index!=None:
                    index=index.text()
                    index=str(index)
                    index=int(index)
                    if index==currentRowIndex:
                        self.parent.errorTable.ui.errorTable.selectRow(row)    
                        break 
        #            self.ui.tableOverview.setSortingEnabled(True) 
            #end of Aditi's code
    def markError(self, index, state):
        """
        mark icon indicate state of unit on note column.
        @param index: row in table.
        @param state: unit's state.
        """
        errState=9
        item3 = self.ui.tableOverview.item(index, 3)
        item4 = self.ui.tableOverview.item(index, 4)
        if (not item3):
            return
        if (state & World.fuzzy):
            item4.setBackgroundColor(self.fuzzyColor)
        elif (state & World.translated) or (state == errState):
            item4.setBackgroundColor(self.blankColor)

    def markState(self, index, state):
        """
        mark icon indicate state of unit on note column.
        @param index: row in table.
        @param state: unit's state.
        """
        item0 = self.ui.tableOverview.item(index, 0)
        item1 = self.ui.tableOverview.item(index, 1)
        item2 = self.ui.tableOverview.item(index, 2)
        item3 = self.ui.tableOverview.item(index, 3)
        item4 = self.ui.tableOverview.item(index, 4)
        if (not item3):
            return
        if (state & World.fuzzy):
            item3.setText("F")
#            if (state & World.plural):
#                item3.setIcon(self.pluralFuzzyIcon)
#                item3.setToolTip("Plural string is fuzzy")
#            else:
#                item3.setIcon(self.fuzzyIcon)
#                item3.setToolTip("String is fuzzy")
            item0.setBackgroundColor(self.fuzzyColor)
            item1.setBackgroundColor(self.fuzzyColor)
            item2.setBackgroundColor(self.fuzzyColor)
            item3.setBackgroundColor(self.fuzzyColor)
            item4.setBackgroundColor(self.fuzzyColor)
            
        elif (state & World.translated):
            item3.setText("Ok")
            item0.setBackgroundColor(self.blankColor)
            item1.setBackgroundColor(self.blankColor)
            item2.setBackgroundColor(self.blankColor)
            item3.setBackgroundColor(self.blankColor)
            item4.setBackgroundColor(self.blankColor)
        else:
            item3.setText("")
#            if (state & World.plural):
#                item3.setIcon(self.pluralIcon)
#                item3.setToolTip("Plural strings")
#            else:
#                item3.setIcon(self.blankIcon)
#                item3.setToolTip("")
            item0.setBackgroundColor(self.blankColor)
            item1.setBackgroundColor(self.blankColor)
            item2.setBackgroundColor(self.blankColor)
            item3.setBackgroundColor(self.blankColor)
            item4.setBackgroundColor(self.blankColor)
#        if (not self.ui.tableOverview.item(index, 3)):
#            return
#        
#        if (state & World.fuzzy):
#            self.ui.tableOverview.item(index, 3).setText("f")
##            if (state & World.plural):
##                self.ui.tableOverview.item(index, 3).setIcon(self.pluralFuzzyIcon)
##                self.ui.tableOverview.item(index, 3).setToolTip("Plural string is fuzzy")
##            else:
##                self.ui.tableOverview.item(index, 3).setIcon(self.fuzzyIcon)
##                self.ui.tableOverview.item(index, 3).setToolTip("String is fuzzy")
#            self.ui.tableOverview.item(index, 0).setBackgroundColor(self.fuzzyColor)
#            self.ui.tableOverview.item(index, 1).setBackgroundColor(self.fuzzyColor)
#            self.ui.tableOverview.item(index, 2).setBackgroundColor(self.fuzzyColor)
#            self.ui.tableOverview.item(index, 3).setBackgroundColor(self.fuzzyColor)
#        
#        else:
#            # TODO: do not setBackgroundColor when item is not dirty yet.
#            self.ui.tableOverview.item(index, 3).setText("")
##            if (state & World.plural):
##                self.ui.tableOverview.item(index, 3).setIcon(self.pluralIcon)
##                self.ui.tableOverview.item(index, 3).setToolTip("Plural strings")
##            else:
##                self.ui.tableOverview.item(index, 3).setIcon(self.blankIcon)
##                self.ui.tableOverview.item(index, 3).setToolTip("")
#            self.ui.tableOverview.item(index, 0).setBackgroundColor(self.blankColor)
#            self.ui.tableOverview.item(index, 1).setBackgroundColor(self.blankColor)
#            self.ui.tableOverview.item(index, 2).setBackgroundColor(self.blankColor)
#            self.ui.tableOverview.item(index, 3).setBackgroundColor(self.blankColor)

        
    def applySettings(self):
        """
        set color and font to the table.
        """
        overviewColor = World.settings.value("overviewColor")
        if (overviewColor.isValid()):
            colorObj = QtGui.QColor(overviewColor.toString())
            palette = self.ui.tableOverview.palette()
            palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(QtGui.QPalette.Text), colorObj)
            palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(QtGui.QPalette.Text), colorObj)
            self.ui.tableOverview.setPalette(palette)

        font = World.settings.value("overviewFont")
        if (font.isValid()):
            fontObj = QtGui.QFont()
            if (fontObj.fromString(font.toString())):
              self.ui.tableOverview.setFont(fontObj)
              
        font = World.settings.value("overviewHeaderFont")
        if (font.isValid()):
            fontObj = QtGui.QFont()
            if (fontObj.fromString(font.toString())):
              self.ui.tableOverview.horizontalHeader().setFont(fontObj)
              
        self.ui.tableOverview.resizeRowsToContents()
        
    def showFilteredItems(self):
        """
        hide and show the item in the table according to filter.
        calculating the visibleRow for navigation.
        """
        self.setUpdatesEnabled(False)
        i = 0
        self.visibleRow = []
        for unit in self.units:
            if hasattr(unit, "x_editor_tableItem"):
                item = unit.x_editor_tableItem
                row = self.ui.tableOverview.row(item)
                if (unit.x_editor_state & self.filter):
                    self.ui.tableOverview.showRow(row)
                    self.visibleRow.append(row)
                    item.setData(QtCore.Qt.UserRole, QtCore.QVariant(i))
                    i += 1
                else:
                    self.ui.tableOverview.hideRow(row)
        self.ui.tableOverview.repaint()
        self.visibleRow.sort()
        # reselect the current item in the table to correct the highlight the whole row
        self.ui.tableOverview.selectRow(self.ui.tableOverview.currentRow())
        self.emitFirstLastUnit()
        
        # for updateText function
        self.updateTextRow = -1
        self.setUpdatesEnabled(True)
    
    def indexString(self, index):
        """converting index which is integer string."""
        return str(index).rjust(self.indexMaxLen) + "  "
            
    
    def markComment(self, index = None, note = "", unit=None):
        """
        mark icon indicate unit has comment on index column, and add tooltips.
        @param index: row in table.
        @param note: unit's comment as tooltips in index column.
        """
        bookmark=False
        bookindexList=[]
        booklist=[]
        if unit:
             if self.parent.operator.isXliff:
                bookmark= unit.isbookmarked()
                self.bookmark=bookmark
        else:
            bookmark = World.settings.value("bookmarkList").toStringList()
            id=self.ui.tableOverview.item(self.ui.tableOverview.currentRow(), 0)
            if id!=None:
                bookmarkString =  str(id.text())
                bookmarkString=bookmarkString.replace(" ", "")
                for i in bookmark:
                    booklist=i.split(" ")
                    bookindexList.append(str(booklist[0]))
                if  bookmarkString in bookindexList:  
                    bookmark=True
                else:
                    bookmark=False       
        if (type(index) != int):
            note = index
            index = None
        # get the current item
        if (not index):
            item = self.ui.tableOverview.item(self.ui.tableOverview.currentRow(), 0)
        else:
            item = self.ui.tableOverview.item(index, 0)
        if (not item):
            return
        if self.parent.dockTUview.contentDirty:
            if (self.parent.dockComment.commentText):
                if bookmark:
                    item.setIcon(self.iconnoteBookmark)
                else:
                    item.setIcon(self.noteIcon)
                item.setToolTip(unicode(note))
            else:
                if bookmark:
                    item.setIcon(self.iconBookmark)
                else:   
                    item.setIcon(self.blankIcon)
                item.setToolTip("")
        else:
            if (note):
                if bookmark:
                    item.setIcon(self.iconnoteBookmark)
                else:
                    item.setIcon(self.noteIcon)
                item.setToolTip(unicode(note))
            else:
                if bookmark:
                    item.setIcon(self.iconBookmark)
                else:   
                    item.setIcon(self.blankIcon)
                item.setToolTip("") 
        note = ""
        self.parent.dockComment.commentText=""        

    def setIconBookmark(self):
        # sets the bookmark icon in the 1st column of overview
        unit = self.parent.operator.getCurrentUnit()
        bookmarkIndexs = []
        bookmarks = World.settings.value("bookmarkList").toStringList()
        for bookmark in bookmarks:
            bookmarkIndex = int(str(bookmark).split(World.delimiter)[0])
            bookmarkIndexs .append(bookmarkIndex)
        currentRow = self.ui.tableOverview.currentRow()
        item = self.ui.tableOverview.item(currentRow , 0)
        if (not item):
            return
        if bookmarkIndexs.__contains__(currentRow + 1):
            if self.parent.dockComment.commentText or unit.getnotes() :
                item.setIcon(self.iconnoteBookmark)
            else:
                item.setIcon(self.iconBookmark)
        else:
            if self.parent.dockComment.commentText or unit.getnotes():
                item.setIcon(self.noteIcon)      
             
        
    def scrollPrevious(self):
        """move to previous row inside the table."""
        currentRow = self.ui.tableOverview.currentRow()
        currentIndex = self.visibleRow.index(currentRow)
        preRow = self.visibleRow[currentIndex - 1]
        #              Set the current cell before moving to the next.
        self.ui.tableOverview.setCurrentCell(currentRow, 2)
        self.ui.tableOverview.selectRow(preRow)
    
    def scrollNext(self):
        """move to next row inside the table."""
        currentRow = self.ui.tableOverview.currentRow()
        currentIndex = self.visibleRow.index(currentRow)
        nextRow = self.visibleRow[currentIndex + 1]
        #              Set the current cell before moving to the next.
        self.ui.tableOverview.setCurrentCell(currentRow, 2)
        self.ui.tableOverview.selectRow(nextRow)
    
    def scrollFirst(self):
        """move to first row of the table."""
        self.ui.tableOverview.selectRow(0)
    
    def scrollLast(self):
        """move to last row of the table."""
        self.ui.tableOverview.selectRow(self.visibleRow[-1])
    
    def scrollToRow(self, value):
        """move to row number specified by value.
        @param value: row number."""
        if (len(self.visibleRow) > 0):
            nextRow = self.visibleRow[value]
            self.ui.tableOverview.selectRow(nextRow)
    
    def emitFirstLastUnit(self):
        currentRow = self.ui.tableOverview.currentRow()   
        lenSelItem = len(self.ui.tableOverview.selectedItems())
        firstUnit = (lenSelItem == 0) or (currentRow == self.visibleRow[0])
        lastUnit = (lenSelItem == 0) or (currentRow == self.visibleRow[-1])
        self.emit(QtCore.SIGNAL("toggleFirstLastUnit"), firstUnit, lastUnit)
        
        if (hasattr(self.parent.dockTUview, "unit")):
            if self.parent.dockTUview.unit:
                if self.parent.dockTUview.unit.hasplural() and globalClass.closeFile:
                    self.parent.dockTUview.ui.tabWidgetSource.clear()
                    self.parent.dockTUview.ui.tabWidgetTarget.clear()

    def gotoRow(self, value):
        item = self.ui.tableOverview.findItems(self.indexString(value), QtCore.Qt.MatchExactly)
        if (len(item) > 0):
            row = self.ui.tableOverview.row(item[0])
            if (not self.ui.tableOverview.isRowHidden(row)):
                self.ui.tableOverview.selectRow(row)
    
    def getCurrentIndex(self):
        """
        return the current (selected) unit index.
        """
        row = self.ui.tableOverview.currentRow()
        item = self.ui.tableOverview.item(row, 0)
        return int(item.text())
    
    def viewSetting(self, argc = None):
        # sets the property of table overview
        bool = (argc and True or False)
        self.ui.tableOverview.clear()
        self.headerLabels = [self.tr("Index"), self.tr("Source"), self.tr("Translation"), (u"    \u221a    "), self.tr("Error")]
        self.ui.tableOverview.setColumnCount(len(self.headerLabels))
        self.ui.tableOverview.setHorizontalHeaderLabels(self.headerLabels)
        self.ui.tableOverview.setRowCount(0)
        self.ui.tableOverview.setEnabled(bool)
    def shorten(self, text):
        """
        Cut the text which has more than one line and append with three dots.
        """
        line = text.find("\n") 
        if (line >= 0):
            text = text[:line] + "..."
        return text
    
    
if __name__ == "__main__":
    import sys, os
    # set the path for QT in order to find the icons
    QtCore.QDir.setCurrent(os.path.join(sys.path[0], "..", "ui"))
    app = QtGui.QApplication(sys.argv)
    overview = OverviewDock(None)
    overview.show()
    sys.exit(app.exec_())

