#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (c) 2006 - 2007 by The WordForge Foundation
#                       www.wordforge.org
#
# This program is free sofware; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your op tion) any later version.
#
# See the LICENSE file for more details. 
#
# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)
#
# This module is working on source and target of current TU.

from PyQt4 import QtCore, QtGui
from WordForge.Editor.ui.Ui_SpellCheck import Ui_frmSpellCheck
import WordForge.Editor.modules.World as World
from WordForge.Editor.modules.globalClass import globalClass
class SpellCheck(QtGui.QDialog):
    '''
    Code for suggestions
    '''
    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self.ui = None
        self.max = None
        self.index = None
        self.unitIndex = None
        self.missSpelledList = None
        self.suggestedList = None
        self.parent = parent
        self.ignoreAll = []
        self.changedAll = {}
        self.rememberChange = []
        
    def showDialog(self):
        #lazy init
        if (not self.ui):
            self.ui = Ui_frmSpellCheck()
            self.setModal(True)
            self.ui.setupUi(self)
        if (self.max):
            self.ui.fromUnit.setMaximum(self.max)
            self.ui.toUnit.setMaximum(self.max)
            self.ui.toUnit.setValue(self.max)
        QtCore.QObject.disconnect(self.ui.Check,QtCore.SIGNAL("clicked()"), self.check)
        QtCore.QObject.disconnect(self.ui.Cancel,QtCore.SIGNAL("clicked()"), self.closed)
        QtCore.QObject.disconnect(self.ui.Finish,QtCore.SIGNAL("clicked()"), self.closed)
        QtCore.QObject.disconnect(self.ui.fromUnit,QtCore.SIGNAL("valueChanged(int)"), self.setFromToUnitStatus)
        QtCore.QObject.disconnect(self.ui.toUnit,QtCore.SIGNAL("valueChanged(int)"), self.setFromToUnitStatus)
        QtCore.QObject.connect(self.ui.Check,QtCore.SIGNAL("clicked()"), self.check)
        QtCore.QObject.connect(self.ui.Cancel,QtCore.SIGNAL("clicked()"), self.closed)
        QtCore.QObject.connect(self.ui.Finish,QtCore.SIGNAL("clicked()"), self.closed)
        QtCore.QObject.connect(self.ui.fromUnit,QtCore.SIGNAL("valueChanged(int)"), self.setFromToUnitStatus)
        QtCore.QObject.connect(self.ui.toUnit,QtCore.SIGNAL("valueChanged(int)"), self.setFromToUnitStatus)
        
        self.disconnect(self.ui.ignoreOnce, QtCore.SIGNAL("clicked(bool)"),  self.moveToNext)
        self.disconnect(self.ui.ignoreAll, QtCore.SIGNAL("clicked(bool)"),  self.setIgnoreAll)
        self.connect(self.ui.ignoreOnce, QtCore.SIGNAL("clicked(bool)"),  self.moveToNext)
        self.connect(self.ui.ignoreAll, QtCore.SIGNAL("clicked(bool)"),  self.setIgnoreAll)
        self.disconnect(self.ui.changeOnce, QtCore.SIGNAL("clicked(bool)"),  self.changeOnce)
        self.disconnect(self.ui.changeAll, QtCore.SIGNAL("clicked(bool)"),  self.changeAll)
        self.connect(self.ui.changeOnce, QtCore.SIGNAL("clicked(bool)"),  self.changeOnce)
        self.connect(self.ui.changeAll, QtCore.SIGNAL("clicked(bool)"),  self.changeAll)
        
        self.ui.ignoreOnce.setEnabled(False)
        self.ui.ignoreAll.setEnabled(False)
        self.ui.changeOnce.setEnabled(False)
        self.ui.changeAll.setEnabled(False)
        self.show()
        
    def setMaximum(self,  max):
        self.max = max
        
    def setFromToUnitStatus(self):
        self.ui.fromToUnit.setChecked(True)
        
    def closeEvent(self, event):
            self.closed()
        
    def closed(self):
        if (self.sender() == self.ui.Cancel):
            if (not self.rememberChange):
                self.reject()
            else:
                result = QtGui.QMessageBox.warning(self, self.tr("Cancel Spellchecker"),
                    self.tr("Change(s) of wrong word will be cancelled, are you sure?"),
                    QtGui.QMessageBox.Yes | QtGui.QMessageBox.Default,
                    QtGui.QMessageBox.No | QtGui.QMessageBox.Escape)
                if (result == QtGui.QMessageBox.Yes):
                    self.emit(QtCore.SIGNAL("cancelSpellCheck"), self.rememberChange)
                    self.reject()
                else:
                    return
        else:
            self.accept()
        self.ui.txtNotIndict.clear()
        self.ui.suggestions.clear()
        self.index = None
        self.unitIndex = None
        self.missSpelledList = None
        self.suggestedList = None
        self.ignoreAll = []
        self.changedAll = {}
        self.rememberChange = []
        self.emit(QtCore.SIGNAL("spellCheckCompleted"))
        if self.parent.ui.actionAutoSpellCheck.isChecked():
            globalClass.sourceWordFlag = 1
            self.parent.operator.autoSpellCheck(True)
            
    def check(self):
        """
        start a new spellcheck process
        """
        self.index = None
        self.unitIndex = None
        self.missSpelledList = None
        self.suggestedList = None
        self.ignoreAll = []
        self.changedAll = {}
        self.rememberChange = []
        self.setCursor(QtCore.Qt.WaitCursor)
        whatToCheck = None
        includeFuzzy = False
        if(self.ui.wholeFile.isChecked()):
            whatToCheck = World.wholeFile
        elif (self.ui.currentToFirst.isChecked()):
            whatToCheck = World.currentToFirst
        elif (self.ui.currentToLast.isChecked()):
            whatToCheck = World.currentToLast
        else:
            whatToCheck = [self.ui.fromUnit.value(), self.ui.toUnit.value()]
        if (self.ui.checkFuzzy.isChecked()):
            includeFuzzy = True
        self.emit(QtCore.SIGNAL("spellChecked"), whatToCheck, includeFuzzy)
        
    def fillResult(self):
        """
        show missSpelled words one by one and its suggestions in a dialog
        """
        missSpelled = self.missSpelledList
        if (not self.isHidden()):
            if (missSpelled):
                if (self.index < len(missSpelled)):
                    missed = missSpelled[self.index]
                    if (missed in self.ignoreAll):
                        self.moveToNext()
                    elif (self.changedAll.has_key(missed)):
                        self.changeOnce(missed, self.changedAll[missed])
                    else:
                        self.setCursor(QtCore.Qt.ArrowCursor)
                        self.ui.txtNotIndict.setPlainText(missed)
                        self.ui.suggestions.clear()
                        suggestedList = self.suggestedList[self.index]
                        for i in range(len(suggestedList)):
                            self.ui.suggestions.addItem(suggestedList[i])
                        self.ui.suggestions.setCurrentRow(0)
                        self.emit(QtCore.SIGNAL("highlightAnWrongWord"), missed)
                elif (self.index >= len(missSpelled)):
                    self.emit(QtCore.SIGNAL("spellChecked"),  None)
            else:
                self.ui.txtNotIndict.clear()
                self.ui.suggestions.clear()
                self.setCursor(QtCore.Qt.ArrowCursor)

    def moveToNext(self):
        '''
        move to next missSpelled word
        '''
        self.setCursor(QtCore.Qt.WaitCursor)
        self.index +=1
        self.fillResult()
        
    def setIgnoreAll(self):
        '''
        set the wrong word to be ignored for the next occurence
        '''
        ignoreText = self.missSpelledList[self.index]
        if ignoreText not in self.ignoreAll:
            self.ignoreAll.append(ignoreText)
        self.moveToNext()
        
    def initializeFillingResult(self, unitIndex, missSpelledList, suggestedList):
        """
        initialize before dialog show
        @param unitIndex: type as int, missSpelledList as a list of string, suggestedList as a list of string
        @param missSpelledList: type as a list of strings
        @param suggestedList: type as a list of strings
        """
        if (not self.isHidden()):
            self.index = 0
            self.unitIndex = unitIndex
            translatedMessage = unicode(self.parent.dockTUview.ui.txtTarget.toPlainText())
            tokenizedtranslatedMessage = self.parent.operator.tokenize(translatedMessage, 'target')
            self.missSpelledList = []
            self.suggestedList = []
            for eachword in missSpelledList:
                if eachword in tokenizedtranslatedMessage:
                    index = missSpelledList.index(eachword)
                    self.missSpelledList.append(eachword)
                    self.suggestedList.append(suggestedList[index])
            self.fillResult()
            self.ui.ignoreOnce.setEnabled(True)
            self.ui.ignoreAll.setEnabled(True)
            self.ui.changeOnce.setEnabled(True)
            self.ui.changeAll.setEnabled(True)
    
    def changeOnce(self, missed = None, suggested = None):
        """
        change wrong word with suggested word
        """
        row = self.ui.suggestions.currentRow()
        if (not missed):
            missed = self.missSpelledList[self.index]
        if (not suggested):
            suggested = self.suggestedList[self.index][row]
        self.emit(QtCore.SIGNAL("changeOnce"), missed, suggested, self.unitIndex)
        self.rememberChange.append((self.unitIndex, missed, suggested))
        if not self.parent.ui.actionSave.isEnabled():
            self.parent.operator.contentChanged(True)
        self.moveToNext()
        
    def changeAll(self):
        """
        change and set the wrong word to be replaced for the next occurence
        """
        missed = self.missSpelledList[self.index]
        if (not self.changedAll.has_key(missed)):
            row = self.ui.suggestions.currentRow()
            self.changedAll[missed] = self.suggestedList[self.index][row]
        self.changeOnce()
        
    def spellCheckCompleted(self):
        self.setCursor(QtCore.Qt.ArrowCursor)
        self.ui.ignoreOnce.setEnabled(False)
        self.ui.ignoreAll.setEnabled(False)
        self.ui.changeOnce.setEnabled(False)
        self.ui.changeAll.setEnabled(False)
        self.ui.txtNotIndict.clear()
        self.ui.suggestions.clear()
##        self.index = None
##        self.unitIndex = None
##        self.missSpelledList = None
##        self.suggestedList = None
##        self.ignoreAll = []
##        self.changedAll = {}
##        self.rememberChange = []
        QtGui.QMessageBox.information(self, self.tr("Spellchecker"), self.tr("Spellcheck completed"))
    
if __name__ == "__main__":
    import os, sys
    app = QtGui.QApplication(sys.argv)
    SpellCheck = SpellCheck(None)
    SpellCheck.showDialog()
    sys.exit(app.exec_())
    
