#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#Copyright (c) 2006 - 2007 by The WordForge Foundation
#                       www.wordforgefoundation.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details.
#
# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)
#       Aditi Barua(aditi@ankur.org.bd)
#       Asheq Hamid(asheq@ankur.org.bd)
#       Md. Rezwan Shahid(rezwan@ankur.org.bd)
#
# This module is providing an setting path of translation memory dialog

import sys, os
import tempfile
from PyQt4 import QtCore, QtGui
from WordForge.Editor.ui.Ui_tmSetting import Ui_tmsetting
from WordForge.Editor.modules import World
from WordForge.Editor.modules.PickleTM import PickleTM
from WordForge.translate.storage import factory
from WordForge.Editor.modules.NewProject import newProject
from WordForge.Editor.modules.Preference import Preference
from WordForge.Editor.modules.globalClass import globalClass
from WordForge.translate.storage import poheader
from WordForge.translate.search import match
from WordForge.Editor.modules import TM2tmx
from WordForge.Editor.modules import GL2tbx

class globalSetting(QtGui.QDialog):
    """Code for setting path of translation memory and Glossary dialog.

    @signal matcher: emitted with self.matcher, self.section when the timer finishes the last filename in self.filenames
    @signal buildPercentage: emitted with percentage to update progress bar
    """

    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self.parent=parent
        self.ui = None
        self.title = None
        self.section = None

    def lazyInit(self):
        if (self.ui):
            # already has ui
            return
        self.tmPrefernce=self.parent.preference
        self.ui = Ui_tmsetting()
        self.ui.setupUi(self)
        self.setModal(True)
        self.setWindowTitle(self.title)
        self.connect(self.ui.btnOk, QtCore.SIGNAL("clicked(bool)"), self.startBuild)
        self.connect(self.ui.btnCancel, QtCore.SIGNAL("clicked(bool)"), QtCore.SLOT("close()"))
        self.connect(self.ui.btnAdd, QtCore.SIGNAL("clicked(bool)"), self.addFiles)
        self.connect(self.ui.btnAddDir, QtCore.SIGNAL("clicked(bool)"), self.addFolder)
        self.connect(self.ui.btnRemove, QtCore.SIGNAL("clicked(bool)"), self.removeLocation)
        self.connect(self.ui.btnRemoveAll, QtCore.SIGNAL("clicked(bool)"), self.ui.listWidget.clear)
        self.connect(self.ui.btnExport, QtCore.SIGNAL("clicked(bool)"), self.exportTM)
        self.ui.listWidget.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)

        if (self.section == World.settingModeTM):
            self.setToolWhatsThis("Translation Memory")
        elif (self.section == World.settingModeGlossary):
            self.setToolWhatsThis("Glossary")
            self.ui.lblTargetLanguage.setText("Target Language for this glossary")

        # get pickleFile
        World.settings.beginGroup(self.section)
        self.pickleFile = World.settings.value("pickleFile").toString()
        World.settings.endGroup()
        appBaseName, tail = os.path.split(str(World.settings.fileName()))
        wordforgedir=appBaseName + os.path.sep
        # first time setting pickle file name
        if (not self.pickleFile):
            if (self.section == World.settingModeTM):
                self.pickleFile = appBaseName + os.path.sep + "PKL_WordForge_TM"
            else:
                self.pickleFile = appBaseName + os.path.sep  + "PKL_WordForge_GLOSSARY"

        # create if pickle file doesn't really exists
        if (not os.path.exists(wordforgedir)):
            os.makedirs(wordforgedir)
        if (not os.path.exists(self.pickleFile)):
            temp = open(self.pickleFile, 'w')
            temp.close()
        #language
        self.tmPrefernce.load_language_ISO2(self.ui.tmsettingtargetlang)
        self.tmPrefernce.load_countries(self.ui.tmsettingtargetcontry)
        self.connect(self.ui.tmsettingtargetlang, QtCore.SIGNAL("currentIndexChanged(const QString &)"), self.moreLangForTMGls)

    def moreLangForTMGls(self, languageCode):
        if languageCode=="More Languages":
            self.tmPrefernce.load_language_ISO3(self.ui.tmsettingtargetlang)
        else:
            return

    def setToolWhatsThis(self, tool):
        """
        Set what's this for TM or glossary dialog.

        @param tool: whether it is a TM or Glossary, type as string.
        """
        list = "<h3>Path for " + tool + "</h3>List of path to scan for " + tool + ". Paths which are checked will be used. "
        self.ui.listWidget.setWhatsThis(self.tr(list))
        dive = "<h3>Dive into subfolders</h3>Check this option, " + tool + " will include subfolders of the above path(s)."
        self.ui.checkBox.setWhatsThis(self.tr(dive))
        progress = "<h3>Build " + tool + "Process</h3>This bar shows the progression of building a " + tool + " from the above select path(s)."
        self.ui.progressBar.setWhatsThis(self.tr(progress))

    def addFiles(self):
        """
        Open the file dialog where you can choose files to add to project.
        @msg: waiting an dialog box will appear up when user click to add item into listLocation.
        """
        if (self.section == World.settingModeTM):
            fileFilters = World.fileFiltersTM
            title = self.tr("Translation Memory")
        elif (self.section == World.settingModeGlossary):
            fileFilters = World.fileFiltersGlossary
            title = self.tr("Glossary")
        else:
            fileFilters = World.fileFilters
        directory = World.settings.value("workingDir").toString()
        filenames = QtGui.QFileDialog.getOpenFileNames(self, self.tr("Add files to ") + title,
                directory, ";;".join(fileFilters))

        if (filenames):
            for filename in filenames:
                self.addLocation(filename)
            directory = os.path.dirname(unicode(filenames[0]))
            World.settings.setValue("workingDir", QtCore.QVariant(directory))

    def addFolder(self):
        """
        Open the file dialog where you can choose a folder to add to project.
        @msg: waiting an dialog box will appear up when user click to add item into listLocation.
        """
        if (self.section == World.settingModeTM):
            title = self.tr("Translation Memory")
        else:
            title = self.tr("Glossary")

        directory = World.settings.value("workingDir").toString()
        folderPath = QtGui.QFileDialog.getExistingDirectory(self, self.tr("Add folder to ") + title, directory)

        if (folderPath):
            # remove separator at the end of string
            folderPath = str(folderPath).rstrip(os.path.sep)
            self.addLocation(folderPath)
            directory = os.path.dirname(folderPath)
            World.settings.setValue("workingDir", QtCore.QVariant(directory))

    def showDialog(self):
        """
        Make the Translation Memory or Glossary Setting dialog visible.
        """
        self.lazyInit()
        self.ui.progressBar.setValue(0)

        # get application setting file, and parse it.
        self.loadSettings()
        self.ui.btnOk.setEnabled(True)
        self.show()

    def loadSettings(self):
        """
        Load settings of TM/Glossary
        """

        self.ui.listWidget.clear()
        World.settings.beginGroup(self.section)
        enabledPath = World.settings.value("enabledpath").toStringList()
        disabledPath = World.settings.value("disabledpath").toStringList()
        includeSub = World.settings.value("diveintosub").toBool()
        World.settings.endGroup()
        if (self.section == World.settingModeTM):
            TMlangaugevalue=World.settings.value("TMTargetLanguage",QtCore.QVariant("bn_BD")).toString()
            list=[]
            list=TMlangaugevalue.split("_")
            language="["+list[0] + "]"
            country="["+list[1] + "]"
            self.tmPrefernce.setcurrentItem(self.ui.tmsettingtargetlang, language, "target")
            self.tmPrefernce.setcurrentItem(self.ui.tmsettingtargetcontry, country, "target")
        elif (self.section == World.settingModeGlossary):
            list=[]
            GLSlangaugevalue=World.settings.value("GlossaryTargetLanguage",QtCore.QVariant("bn_BD")).toString()
            list=GLSlangaugevalue.split("_")
            language="["+list[0] + "]"
            country="["+list[1] + "]"
            self.tmPrefernce.setcurrentItem(self.ui.tmsettingtargetlang, language, "target")
            self.tmPrefernce.setcurrentItem(self.ui.tmsettingtargetcontry, country, "target")


        for path in enabledPath:
            self.addLocation(path)
        for path in disabledPath:
            self.addLocation(path, QtCore.Qt.Unchecked)

        if (len(enabledPath) > 0) or (len(disabledPath) > 0):
            self.ui.listWidget.setCurrentRow(0)

        self.ui.checkBox.setChecked(includeSub)
#        self.ui.spinMaxLen.setValue(maxLen)

    def addLocation(self, TMpath, checked = QtCore.Qt.Checked):
        """
        Add TMpath to TM list.

        @param TMpath: Filename as string
        """
        items = self.ui.listWidget.findItems(TMpath, QtCore.Qt.MatchCaseSensitive)
        if (not items):
            item = QtGui.QListWidgetItem(TMpath)
            item.setCheckState(checked and QtCore.Qt.Checked or QtCore.Qt.Unchecked)
            self.ui.listWidget.addItem(item)

    def removeLocation(self):
        """
        Remove selected path TM list.
        """
        items = self.ui.listWidget.selectedItems()
        for item in items:
            self.ui.listWidget.setCurrentItem(item)
            self.ui.listWidget.takeItem(self.ui.listWidget.currentRow())

    def getPreferenceLangcode(self):
        currentList=[]
        currentLanguage=str(World.settings.value("FullLanguage").toString())
        currentList=self.tmPrefernce.setCountryLanguage(currentLanguage)
        languagecodeLevel=currentList[0].replace("]", "")
        languagecode=languagecodeLevel.replace("[", "")
        currentList=[]
        currentCountry=str(World.settings.value("perCountryName").toString())
        currentList=self.tmPrefernce.setCountryLanguage(currentCountry)
        countrycodeLevel=currentList[0].replace("[", "")
        countrycode=countrycodeLevel.replace("]", "")
        if currentCountry=="No country specified":
            code=languagecode
        else:
            code=languagecode + "_" + countrycode
        return code

    def mismatchwarning(self):
        code=self.getPreferenceLangcode()
        if code!=self.targetLanguageCode:
            msgText= self.trUtf8("Some files do not have the same Language you specified in this window. And also the language you specified in this window does not match with the language setting in <b>Personalize</b> page of <b>Preferences</b>. Do you want to continue?")
        else:
            msgText= self.trUtf8("Some files do not have the same Language you specified in this window. Do you want to continue?")

        messageBox = QtGui.QMessageBox
        ret = messageBox()
        ret.setTextFormat(QtCore.Qt.RichText)
        ret.setText(msgText)
        YesButton = ret.addButton(self.trUtf8("Yes"),messageBox.YesRole)
        NoButton = ret.addButton(self.trUtf8("No"),messageBox.NoRole)
        ret.exec_()
        if (ret.clickedButton() == YesButton):
            return True
        elif (ret.clickedButton() == NoButton):
            return False

    def startBuild(self):
        """
        Collect filename into self.filenames, call buildMatcher(),
        dump matcher, and save settings.
        """
        self.targetLanguageCode = self.tmPrefernce.striplangcontryname(self.ui.tmsettingtargetlang.currentText(), self.ui.tmsettingtargetcontry.currentText())
        globalClass.TMGLStargetLanguageCode=self.targetLanguageCode

        self.ui.btnOk.setEnabled(False)

        disabledPath = self.getPathList(QtCore.Qt.Unchecked)
        # save some settings
        World.settings.beginGroup(self.section)
        World.settings.setValue("disabledpath", QtCore.QVariant(disabledPath))
        World.settings.endGroup()

        # get filenames from checked list.
        enabledPath = self.getPathList(QtCore.Qt.Checked)
        includeSub = self.ui.checkBox.isChecked()
        self.buildMatcher(enabledPath, includeSub)

        if not enabledPath:
            from WordForge.Editor.modules import Operator
            operator=Operator.Operator(self)
            operator.applySettings()

    def buildMatcher(self, paths, includeSub=True):
        """
        create matcher, start a timer for extend tm.
        """
        self.lazyInit()
        # save some settings
        if (self.section == World.settingModeTM):
            World.settings.setValue("TMTargetLanguage", QtCore.QVariant(self.targetLanguageCode))
        elif (self.section == World.settingModeGlossary):
            World.settings.setValue("GlossaryTargetLanguage", QtCore.QVariant(self.targetLanguageCode))
        flag=True
        self.matcher = None
        self.filenames = []
        self.tmPrefernce.fileswithnolanguage=[]
        self.tmPrefernce.fileswithdifflanguage=[]
        for path in paths:
            self.getFiles(path, includeSub)
        Tmfilelanguage=[]
        for file in self.filenames:
            filelanguage=self.tmPrefernce.checkfileLanguage(file, "TM")
            if self.tmPrefernce.filelangcur not in Tmfilelanguage:
                Tmfilelanguage.append(self.tmPrefernce.filelangcur)
        World.settings.setValue("TM_File_Language", QtCore.QVariant(Tmfilelanguage))
        if self.tmPrefernce.fileswithnolanguage or self.tmPrefernce.fileswithdifflanguage:
                flag=self.mismatchwarning()
        test=[]
        tmlist=World.settings.value("TM_File_Language",  QtCore.QVariant(test)).toStringList()
        if flag:
            World.settings.beginGroup(self.section)
            World.settings.setValue("enabledpath", QtCore.QVariant(paths))
            World.settings.setValue("diveintosub", QtCore.QVariant(includeSub))
            World.settings.setValue("pickleFile", QtCore.QVariant(self.pickleFile))
            World.settings.endGroup()
        # close dialog if no filename.
            if (len(self.filenames) <= 0):
    #            self.timer.stop()
    #            self.iterNumber = 1
                self.dumpMatcher()
                self.emitMatcher()
                self.close()
                return

            # start build matcher with self.filenames[0]
            self.matcher = None
            store = None
            iterNumber = 1
            for afile in self.filenames:
                store = self.createStore(afile)
                if (store):
                    if (not self.matcher):
                        if (self.section == World.settingModeTM):
                            self.matcher = match.matcher(store)
                        else:
                            self.matcher = match.terminologymatcher(store)
                    else:
                            if (self.section == World.settingModeTM):
                                self.matcher.extendtm(store.units, store)
                            else:
                                i=0
                                for i in range(len(store.units)):
                                    store.units[i].source = store.units[i].source.lower().strip()
                                self.matcher.extendtm(store.units, store)
                else:
                    if (self.section == World.settingModeTM):
                        QtGui.QMessageBox.information(self, "TM Settings",  "File "+afile+" could not be opened")
                    else:
                        QtGui.QMessageBox.information(self, "Glossary Settings",  "File "+afile+" could not be opened")

                perc = int((float(iterNumber) / len(self.filenames)) * 100)
                self.ui.progressBar.setValue(perc)
                self.emit(QtCore.SIGNAL("buildPercentage"), perc)
                iterNumber += 1

            self.dumpMatcher()
            self.emitMatcher()
    #        targetLanguageCode = World.settings.value("CurrentLanguage").toString()
            targetLanguageCode = str(self.targetLanguageCode)

            if (self.section == World.settingModeTM):
                TM2tmx.convertTM2tmx(self.matcher, os.path.split(str(World.settings.fileName()))[0] + os.path.sep +"WordForgeTM.tmx", 'en', targetLanguageCode)
                QtGui.QMessageBox.information(self, "Information",  "Your Translation Memory has been successfully built.")
            else:
                GL2tbx.convertGL2tbx(self.matcher, os.path.split(str(World.settings.fileName()))[0] + os.path.sep +"WordForgeGL.tbx", "en", targetLanguageCode)
                if (self.matcher.countFuzzy > 0):
                    QtGui.QMessageBox.information(self, "Information", "Your Glossary of "+str(self.matcher.countUnits)+" terms has been successfully imported into WordForge. "+str (self.matcher.countFuzzy)+" fuzzy units have ben ignored.")
                else:
                    QtGui.QMessageBox.information(self, "Information", "Your Glossary of "+str(self.matcher.countUnits)+" terms has been successfully imported into WordForge.")
            self.close()
        if not flag:
           self.close()
     # extend matcher, start with self.filenames[1]
#        self.iterNumber = 1
#        self.timer.stop()
#        self.timer.start(10)

    def getFiles(self, path, includeSub):
        """
        Get the filenames. Only supported files are taken into account;
        and if it's directory and includeSub is True, dive into it and add files.

        @param paths: file or directory path for adding to Catalog.
        @param includeSub: a bool value; dive into sub, if it is true.
        """
        path = unicode(path)
        if (os.path.isfile(path)):
            # if file is already existed in the item's child... skip.
            if (path.endswith(".po") or path.endswith(".pot") or path.endswith(".xlf") or path.endswith(".xliff") or path.endswith(".tbx") or path.endswith(".tmx")):
                self.filenames.append(path)

        if (os.path.isdir(path)) and (not path.endswith(".svn")):
            for root, dirs, files in os.walk(path):
                for file in files:
                    path = os.path.join(root + os.path.sep + file)
                    self.getFiles(path, includeSub)

                # whether dive into subfolder
                if (includeSub):
                    for folder in dirs:
                        path = os.path.join(root + os.path.sep + folder)
                        self.getFiles(path, includeSub)
                break

    def emitMatcher(self):
        """
        emit "matcher" with self.matcher, self.section
        """
        self.emit(QtCore.SIGNAL("matcher"), self.matcher, self.section)

    def dumpMatcher(self):
        """
        call pickleTM.dumpMatcher()
        """
        p = PickleTM()
        p.dumpMatcher(self.matcher, self.pickleFile)

    def createStore(self, file):
        """
        Create a store object from file.
        add translator, date, and filepath properties to store object.

        @param file: as a file path or object
        @return: store as a base object
        """
        try:
            store = factory.getobject(file)
        except:
            store = None
            return None
        store.filepath = file
        if (isinstance(store, poheader.poheader)):
            headerDic = store.parseheader()
            store.translator = headerDic.get('Last-Translator')
            store.date = headerDic.get('PO-Revision-Date')
            if (store.translator == None):
                store.translator = ""
            if (store.date == None):
                store.date = ""
        else:
            store.translator = ""
            store.date = ""
        return store

    def getPathList(self, isChecked):
        """
        Return list of path according to the parameter isChecked or unChecked.

        @param isChecked: as bool type
        @return: itemList as list of unchecked or checked path
        """
        itemList = QtCore.QStringList()
        count = self.ui.listWidget.count()
        for i in range(count):
            item = self.ui.listWidget.item(i)
            if (not (item.checkState() ^ isChecked)):
                itemList.append(str(item.text()))
        return itemList

    def getMaximumCandidates(self):
        """
        Return the maximum candidates number as integer.
        """
        World.settings.beginGroup(self.section)
        result = World.settings.value("max_candidates", QtCore.QVariant(10)).toInt()[0]
        World.settings.endGroup()
        return result

    def getMaximumLenght(self):
        """
        returen the maximum string lenth
        """
        if (self.section == World.settingModeTM):
            defValue = 70
        else:
            defValue = 500
        World.settings.beginGroup(self.section)
        result = World.settings.value("max_string_len", QtCore.QVariant(defValue)).toInt()[0]
        World.settings.endGroup()
        return result

    def exportTM(self):
        # initialize Translation Memory/TM Matcher
        World.settings.beginGroup(World.settingModeTM)
        pickleFile = World.settings.value("pickleFile").toString()
        World.settings.endGroup()

        # if application remember TM file name and file really exist
        if (pickleFile and os.path.exists(pickleFile)):
            p = PickleTM()
            matcher = p.getMatcher(pickleFile)

        if (matcher):
            directory = World.settings.value("workingDir").toString()
            #FIXME: Do not hard cord
            filename = QtGui.QFileDialog.getSaveFileName(self,  self.trUtf8("Export to File"), directory,   self.trUtf8("XLIFF Files (*.xlf)"))
            if (filename):
                if ((not str(filename).endswith(".xlf"))):
                    filename = filename + ".xlf"

                directory = os.path.dirname(unicode(filename))
                World.settings.setValue("workingDir", QtCore.QVariant(directory))

                from WordForge.translate.storage import xliff
                xliffObject = xliff.xlifffile()

                for unit in matcher.candidates.units:
                    xliffObject.addsourceunit(unit.source)
                    xliffObject.units[len(xliffObject.units) - 1].target = unit.target

                fileObj = open(filename, "w")
                fileObj.writelines(str(xliffObject))
                fileObj.close()

class TMSetting(globalSetting):
    """
        Translation Memory Section
    """
    def __init__(self, parent):
        globalSetting.__init__(self, parent)
        self.title = "Configure translation memory"
        self.section = World.settingModeTM

class glossarySetting(globalSetting):
    def __init__(self, parent):
        """
        Glossary Section
        """
        globalSetting.__init__(self, parent)
        self.title = "Configure glossary"
        self.section = World.settingModeGlossary

if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    tm = TMSetting(None)
    tm.showDialog()
    sys.exit(tm.exec_())


