#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (c) 2006 - 2007 by The WordForge Foundation
#                       www.wordforgefoundation.org
#
# This program is free sofware; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details. 
#
# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)
#       Aditi Barua(aditi@ankur.org.bd)
#       Asheq Hamid(asheq@ankur.org.bd)
#       Md. Rezwan Shahid(rezwan@ankur.org.bd)
#       Goutom Roy(goutom@ankur.org.bd)
# This module is working on source and target of current TU.
import sys                     # to make module find World.py calling it directly...
sys.path.append("../../..")
from PyQt4 import QtCore, QtGui
from WordForge.Editor.ui.Ui_TUview import Ui_TUview
from WordForge.Editor.modules import World
from WordForge.Editor.modules.Highlighter import Highlighter
from WordForge.Editor.modules.globalClass import globalClass
from WordForge.Editor.modules.Preference import Preference
from WordForge.myThes import example as myThesaurus
#from WordForge.Editor.modules.thesaurusThread import thesaurusThread
import string, unicodedata, os, platform
SHOWTIP = 1
CONTEXTMENU = 2

if platform.system() == "Linux":
    thesDirPath = os.path.expanduser('~') + '/.config/WordForge/myThesaurus'
elif platform.system() == "Windows":
    thesDirPath = os.path.expanduser('~') + '\Application Data\WordForge/myThesaurus'

class TUview(QtGui.QDockWidget):
    
    """
    Code for TUview.
    
    @signal scrollToRow: emited with value as row start from 0.
    @signal term: emited with word found in self.sourceHighlighter.glossaryWords
    @signal targetChanged: emitted content in textTarget is dirty.
    """
   
    def __init__(self, parent):
        """"
        if World.WF_debug:
            print "Entering __init__ in class TUview"
            """
        self.parent = parent
        QtGui.QDockWidget.__init__(self, parent)
        self.setObjectName("detailDock")
        self.setWindowTitle(self.tr("Source"))
        self.form = QtGui.QWidget(self)
        self.ui = Ui_TUview()
        self.ui.setupUi(self.form)
        self.setWidget(self.form)
        self.setFeatures(QtGui.QDockWidget.DockWidgetClosable)
        self.ui.lblComment.hide()
        self.applySettings()
#      Create a timer that changes the tool tip into the default tooltip after 10 seconds 
        self.definitionTimer = QtCore.QTimer()
        self.timeoutInterval = 20000
        
        self.connect(self.definitionTimer, QtCore.SIGNAL("timeout()"), self.changeToolTip)

        self.connect(self.ui.fileScrollBar, QtCore.SIGNAL("valueChanged(int)"), self.emitCurrentIndex)
        
        self.connect(self.ui.tabWidgetSource, QtCore.SIGNAL("currentChanged(int)"), self.sourceIndexChanged)
        self.connect(self.ui.tabWidgetTarget, QtCore.SIGNAL("currentChanged(int)"), self.targetIndexChanged)
        
        
        # create highlighter
        self.sourceLength = 0
        self.sourceHighlighter = Highlighter(self.ui.txtSource)
        self.targetHighlighter = Highlighter(self.ui.txtTarget)
        # TODO: with plural target
        # TODO: spellcheck with plural
        
        # subclass of event
        self.ui.txtSource.contextMenuEvent = self.customContextMenuEvent
        self.ui.txtTarget.contextMenuEvent = self.customContextMenuEvent
        
        self.ui.txtSource.installEventFilter(self)
        self.ui.txtTarget.installEventFilter(self)
        
        self.eventObj = None
        self.termWithoutSpaceLog = None
        self.missSpelledList = []
        self.suggestedList = []
        self.modifiedWords = []
        self.contentDirty = False
#        If true then keyreleaseevent allows up,down,left,right button, otherwise ignores. 
        self.allowNavigationKeys = False        
        QtCore.QObject.connect(self, QtCore.SIGNAL("suggestedWords"), self.popupTerm)        
        self.cursorPosition = None
        self.targetReplaceChars = globalClass.targetReplaceChars
        self.pluralTarget=False
        self.tabIndex=0
        self.pluralTab=0
        self.curtarget=''
        self.sourceStrings=[]
        self.wasInPlural = False
        self.isSingle=True
        self.pluralTextBoxList = []
        self.origtarget=""
        self.prevtarget=""
        self.cursorTerm=""
        
        #Thesaurus 
        self.synonymsList=[]
        self.Thesaurus_done=False
        
    def sourceIndexChanged(self, int):
        """
        Slot to receive currentChanged signal when switching source tab.
        @param int: the index of the tab, type as int.       
        if World.WF_debug:
            print "Entering sourceIndexChanged in class TUview"
        """
        TMDict = self.parent.operator.TMDict
        if TMDict:
#          when a tab is clicked,  then it sends the TM suggestions of only that string (in case of plurals) to TM window
            if TMDict.has_key(self.unit.source.strings[int]):
                    self.parent.table.fillTable(self.unit.source.strings[int], TMDict[self.unit.source.strings[int]])
            else:
                self.parent.table.ui.listWidget.clear()

        self.disconnect(self.ui.tabWidgetTarget, QtCore.SIGNAL("currentChanged(int)"), self.targetIndexChanged)
        try:
            self.ui.tabWidgetTarget.setCurrentIndex(int)
            currentsource=self.sourceStrings[int]
            globalClass.currentSource=currentsource
            self.tabIndex=int

        except Exception:
            pass
        self.connect(self.ui.tabWidgetTarget, QtCore.SIGNAL("currentChanged(int)"), self.targetIndexChanged)
    
    def targetIndexChanged(self, int):
        """
        Slot to receive currentChanged signal when switching target tab.
        @param int: the index of the tab, type as int.
        if World.WF_debug:
            print "Entering targetIndexChanged in class TUview"
        """
        TMDict = self.parent.operator.TMDict
        if TMDict:
#          when a tab is clicked,  then it sends the TM suggestions of only that string (in case of plurals) to TM window
            if TMDict.has_key(self.unit.source.strings[int]):
                    self.parent.table.fillTable(self.unit.source.strings[int], TMDict[self.unit.source.strings[int]])

            else:
                self.parent.table.ui.listWidget.clear()
        self.disconnect(self.ui.tabWidgetSource, QtCore.SIGNAL("currentChanged(int)"), self.sourceIndexChanged)
        try:
            self.ui.tabWidgetSource.setCurrentIndex(int)
#            currentsource=self.sourceStrings[int]
#            globalClass.currentSource=currentsource
#            self.tabIndex=int
        except IndexError:
            target = ""
        self.connect(self.ui.tabWidgetSource, QtCore.SIGNAL("currentChanged(int)"), self.sourceIndexChanged)
        
    def eventFilter(self, obj, event):
        """
        Subclass of eventFilter.
        
        @param obj: an obj whose event will be filtered.
        @param event: type as QEvent.
        
         
        if World.WF_debug:
            print "Entering eventFilter in class TUview"
        """
        if (obj == self.ui.txtSource or obj == self.ui.txtTarget or obj in self.pluralTextBoxList):
            if ((obj == self.ui.txtTarget) and (event.type() == QtCore.QEvent.KeyRelease)):
                if (event.key() in World.SpellCheckIgniteKey):
                    self.emitTargetChanged()
                    self.emit(QtCore.SIGNAL("breakerEncountered"))
            
            self.eventObj = obj
            
# code added to avoid calling this function each time the cursor moves. Omited tooltip event and used MouseButtonPress event
#Issue number: 2817377  	  
#            if (event.type() == QtCore.QEvent.ToolTip):

            if (event.type() == QtCore.QEvent.MouseButtonPress):
#end of my code
                """
                Request a tooltip for word in position pos of txtSource.
                """
                self.requestAction = SHOWTIP
                self.globalPos = event.globalPos()
                self.emitTermRequest(event.pos())
                return True
            else:
                return False
        else:
            return self.eventFilter(obj, event)
    
#    def shownonprintChar(self):
#        print "source", self.currentUnit.source 
#        print "target", self.currentUnit.target
#        text = self.ui.txtTarget.toPlainText()
#        text=str(text)
#        textwithnonprint=""
##        print "text", text
#        for i in text:
#            unicodeno=ord(i)
#            if unicodedata.category(unichr(unicodeno)) in World.catsCrl:
#                for decimal in World.dict_nonprintable.keys():
#                    if unicodeno==decimal:
#                        name=World.dict_nonprintable[decimal]
#                        repacechar=World.dict_invisibleSpecial[name]
#                        textwithnonprint=textwithnonprint+repacechar
#            else: 
#                textwithnonprint=textwithnonprint+i
#        self.ui.txtTarget.setPlainText(unicode(textwithnonprint))        

    def customContextMenuEvent(self, e):
        """
        Request a context menu for word in position pos of txtSource.
        
        @param e: type as QContextMenuEvent
        """
        self.cursorPos=e.pos()
        if World.WF_debug:
            print "Entering customContextMenuEvent in class TUview"    
        self.requestAction = CONTEXTMENU
        if (self.eventObj == self.ui.txtSource or self.eventObj in self.pluralTextBoxList):
            self.copyMenu()
        self.globalPos = e.globalPos()
        self.emitTermRequest(e.pos())
        
    def copyMenu(self):
        """
        Create a copy context menu.
        if World.WF_debug:
            print "Entering copyMenu in class TUview"
           """ 
        if (self.requestAction == CONTEXTMENU):
            textCursor = self.ui.txtSource.textCursor()
            selected = textCursor.selectedText()
            if selected:
                menu = QtGui.QMenu()
                menuAction = menu.addAction(unicode(self.tr("copy")))
                menuAction.setData(QtCore.QVariant(selected))
                self.connect(menuAction, QtCore.SIGNAL("triggered()"), self.copyTranslation)
                menu.exec_(QtGui.QCursor.pos())

    def showDefinition(self):
#        This function shows the term definition in a tooltip.
        EntryList = self.sender().data().toPyObject()
        termEntryList = EntryList[0]
        langDepDefList = EntryList[1]
        subjectField = EntryList[2]
        context = EntryList[3]
        definition =  ''
        if termEntryList:
            definition += 'Definition' + ' : '
            for eachdefinition in termEntryList:
                definition += eachdefinition + '\n'
        if subjectField:
            definition += 'Subject Field' + ' : '
            for each in subjectField:
                definition += each + '\n'
        if context:
            definition += 'Context' + ' : '
            for each in context:
                definition += each + '\n'        
        for eachKey in langDepDefList.keys():
            if langDepDefList[eachKey]:
                definition += eachKey + ' definition : '
                for eachdefinition in langDepDefList[eachKey]:
                    definition += eachdefinition + '\n'

        definition = definition[:-1]
        self.ui.txtSource.setToolTip(definition)
        
#      strt the timer.
        if self.definitionTimer.isActive():
            self.definitionTimer.stop()
        
        self.definitionTimer.start(self.timeoutInterval)
        
    def changeToolTip(self):
#      This function is called when the definitiontimer is timed out. It changes the txtSource's tooltip to default value.
        self.ui.txtSource.setToolTip(QtGui.QApplication.translate("TUview", "Original Source String", None, QtGui.QApplication.UnicodeUTF8))
            
    def popupTerm(self, candidates):
        """
        Popup menu or show tooltip of glossary word's translation.
        
        @param candidates: as list of units.
      
        if World.WF_debug:
            print "Entering popupterm in class TUview"
            """
        if (not candidates):
            return
                
        if (self.requestAction == CONTEXTMENU and self.eventObj == self.ui.txtSource or self.eventObj in self.pluralTextBoxList):
            menu = QtGui.QMenu()
            text = self.ui.txtTarget.toPlainText()
            textCursor = self.ui.txtSource.textCursor()
            selected = textCursor.selectedText()
            if selected:
                self.copyMenu()
                return
#          Add definition only once.      
            definitionAdded = False
#            This flag is true if the menu trigger signal is connected
            connected = False
            for candidate in candidates:  
                if not definitionAdded:
                    if hasattr(candidate, 'entryDefinition') and candidate.entryDefinition:
                        menuAction = menu.addAction("Show Definition and Context")
                        menuAction.setData(QtCore.QVariant([candidate.entryDefinition, candidate.languageDependentDefinition, candidate.subjectField, candidate.context]))
                        self.connect(menuAction, QtCore.SIGNAL("triggered()"), self.showDefinition)
                        definitionAdded = True
                if  candidate.target:
                    strReplaceCursor = unicode(self.tr("Copy \"%s\" to cursor position in translation.")) % (unicode(candidate.target))
                    
                    expression = QtCore.QRegExp(candidate.source, QtCore.Qt.CaseInsensitive)
                    index = text.indexOf(expression)
                    menuAction = menu.addAction(strReplaceCursor)
                    menuAction.setData(QtCore.QVariant(unicode(candidate.target)))
                    self.connect(menuAction, QtCore.SIGNAL("triggered()"), self.replaceTranslationInCursor)
                    length4=0
                    if (index >= 0):
                        length4 = expression.indexIn(text, 0)
                        length = expression.matchedLength()
                        cText = expression.capturedTexts()[0]
                        strReplace = unicode(self.tr("Replace \"%s\" with \"%s\" in translation.")) % (unicode(cText), unicode(candidate.target))
                        menuAction = menu.addAction(strReplace)
                        stringList = QtCore.QStringList([unicode(cText), unicode(candidate.target)])
                        menuAction.setData(QtCore.QVariant(stringList))
                        self.connect(menuAction, QtCore.SIGNAL("triggered()"), self.replaceTranslation)                    
                    strCopy = unicode(self.tr("Copy \"%s\" to clipboard.")) % unicode(candidate.target)
                    menuAction = menu.addAction(strCopy)
                    menuAction.setData(QtCore.QVariant(candidate.target))
                    self.connect(menuAction, QtCore.SIGNAL("triggered()"), self.copyTranslation)
                    connected = True
                    
           #self.addThesaurusToPopup(menu)
            if self.synonymsList:
                for i in range(len(self.synonymsList)):
                    synonyms = unicode(self.tr("%s")) % (unicode(self.synonymsList[i]))
                    menuAction = menu.addAction(synonyms)                        
                    menuAction.setData(QtCore.QVariant(self.synonymsList[i]))
            self.synonymsList=[]
            if not connected and definitionAdded:
                menu.exec_(self.globalPos)
                
            if connected :
                menu.exec_(self.globalPos)
                self.disconnect(menuAction, QtCore.SIGNAL("triggered()"), self.copyTranslation)
                connected = False
            
        elif (self.requestAction == CONTEXTMENU and self.eventObj == self.ui.txtTarget):
            menu = QtGui.QMenu()
            for candidate in candidates:
                strReplace = unicode(self.tr("%s")) % (unicode(candidate))
                menuAction = menu.addAction(strReplace)
                stringList =QtCore.QStringList([unicode(self.termWithoutSpaceLog[2]), unicode(candidate)])
                menuAction.setData(QtCore.QVariant(stringList))
                self.connect(menuAction, QtCore.SIGNAL("triggered()"), self.replaceTranslation)
            menu.exec_(self.globalPos)
            
        elif (self.requestAction == SHOWTIP):
            tips = ""
            if (self.eventObj == self.ui.txtSource): 
                for candidate in candidates:
                    tips += candidate.target + "\n"
            if (self.eventObj == self.ui.txtTarget):
                for candidate in candidates:
                    tips += candidate + "\n"
            tips = tips[:-1]
            QtGui.QToolTip.showText(self.globalPos, tips)
        
    def copyTranslation(self):
        """
        Copy self.sender().data() to clipboard.
        if World.WF_debug:
            print "Entering copyTranslation in class TUview"
          """ 
        text = self.sender().data().toString()
        clipboard = QtGui.QApplication.clipboard()
        clipboard.setText(text)
    
    def replaceTranslation(self):
        """
        Replace self.sender().data()[0] with self.sender().data()[1]
        in txtTarget.

         
        if World.WF_debug:
            print "Entering replaceTranslation in class TUview"
              """
        source, target = self.sender().data().toStringList()
        if (source.toLower() == target.toLower()):
            return
        text = self.ui.txtTarget.toPlainText()
        expression = QtCore.QRegExp(source, QtCore.Qt.CaseInsensitive)
        length5=0
        if (self.eventObj == self.ui.txtSource):
            index = text.indexOf(expression)    # replace from beginning
            #replace all found
            while (index >= 0):
                length5 = expression.indexIn(text, 0)
                length = expression.matchedLength()
                text.replace(index, length, target)
                index = text.indexOf(expression)
        elif (self.eventObj == self.ui.txtTarget):
            index = text.indexOf(expression, self.termWithoutSpaceLog[0]) #replace from right-clicked position
            #replace only one
            if (index >=0):
                length5 = expression.indexIn(text, 0)
                length = expression.matchedLength()
                text.replace(index, length, target)
                self.eventObj = None
        
        self.setTargetText(text)
        self.ui.txtTarget.setFocus()
        return True

    def replaceTranslationInCursor(self, replaceText=""):
        """
        Copy self.sender().data() to cusor position​ in textTarget.

         
        if World.WF_debug:
            print "Entering replaceTranslationInCursor in class TUview"
        """
        
        if self.isSingle:
#            if there is no plural
            text = self.ui.txtTarget.toPlainText()
            
            if (replaceText == ""):
                cText =  self.sender().data().toString()
            else:
                cText = replaceText
              
            cursor = self.ui.txtTarget.textCursor()
            cursor = cursor.position()

            if (len(text) <= 0):
                text = cText
                self.setTargetText(cText)
            elif (cursor == len(text)):
                text = text + cText
                self.setTargetText(text)
            else:
                text = text[:cursor] + cText + text[cursor:]
                self.setTargetText(text)   
            self.ui.txtTarget.setFocus()
            cursorPost = len(text[:cursor] + cText)
            cursor = self.ui.txtTarget.textCursor()
            cursor.setPosition(cursorPost)
            self.ui.txtTarget.setTextCursor(cursor)
        else:
#            for plurals
            currentTab = self.ui.tabWidgetTarget.currentIndex()
            text = self.targetBox[currentTab].toPlainText()
            if (replaceText == ""):
                cText =  self.sender().data().toString()
            else:
                cText = replaceText

            cursor = self.targetBox[currentTab].textCursor()
            cursor = cursor.position()

            if (len(text) <= 0):
                text = cText
                self.setTargetText(cText)
            elif (cursor == len(text)):
                text = text + cText
                self.setTargetText(text)
            else:
                text = text[:cursor] + cText + text[cursor:]
                self.setTargetText(text)
            self.targetBox[currentTab].setFocus()
            cursorPost = len(text[:cursor] + cText)
            cursor = self.targetBox[currentTab].textCursor()
            cursor.setPosition(cursorPost)
            self.targetBox[currentTab].setTextCursor(cursor)
            self.isSingle=False 
     
    def getSynonyms(self, cursorterm, eventobj):
        # gets synonyms of a word when right click the word
        if eventobj ==self.ui.txtSource:
            ThesFile = str(World.settings.value("sourceThesFile").toString())
        if eventobj ==self.ui.txtTarget:
            ThesFile = str(World.settings.value("targetThesFile").toString()) 
        if self.parent.operator.sourcelang=='':
            self.parent.operator.sourcelang="en_US"      
        if not ThesFile:   
            theslist=self.parent.preference.add_Thesaurus_in_table(purpose="searchThesFile")
            if theslist:
                for file in theslist:
                    if (eventobj ==self.ui.txtSource) and (self.parent.operator.sourcelang in file) :     
                        filewithourextnsion=file.replace(".dat", "")
                        ThesFile=filewithourextnsion.replace(".idx", "")
                        break
                    if (eventobj ==self.ui.txtTarget) and (self.parent.operator.langcur in file) :        
                        filewithourextnsion=file.replace(".dat", "")
                        ThesFile=filewithourextnsion.replace(".idx", "")
        if  ThesFile:   
            encod=myThesaurus.find_encoding(thesDirPath+ os.path.sep +ThesFile + ".idx", thesDirPath+ os.path.sep + ThesFile + ".dat")
            try:
                term1 = cursorterm.encode(encod)
                myThesaurus.program_init(thesDirPath+ os.path.sep + ThesFile + ".idx", thesDirPath+ os.path.sep + ThesFile + ".dat", term1)
                if myThesaurus.get_lastCount():
                    synlist=[]
                    for i in range(myThesaurus.get_lastCount()):
                        syn=myThesaurus.get_synonyms(i);
                        syn1=syn.decode(encod)
                        synlist.append(syn1)
                    return synlist
            except:
                pass        

            
    def emitTermRequest(self, pos):
        """
        Find word in txtSource from position pos, and emit lookupTerm signal.
                 
        if World.WF_debug:
            print "Entering emitTermRequest in class TUview"
              """
        menu = QtGui.QMenu()       
        length4 = 0
        if (self.eventObj == self.ui.txtSource or self.eventObj == self.ui.txtTarget or self.eventObj in self.pluralTextBoxList):
            text = self.eventObj.toPlainText()
#           start modification made for issue number 2818343
#            adding an extra n for strings having characters of World.regularExpressionCharacters
            if self.eventObj == self.ui.txtSource or self.eventObj in self.pluralTextBoxList:
                try:
                    if (str(text))[0] in World.regularExpressionCharacters:
                        text.prepend('n')
                    if (str(text))[-1] in World.regularExpressionCharacters:
                        text.append('n')
                except:
                    pass
#           end of  modification made for issue number 2818343
            if (not text):
                return
            cursor = self.eventObj.cursorForPosition(pos)
            position = cursor.position()
#my code. Added for showing glossary suggestions when more that two consecutivs words are highlighted and clicked on the txttarget            
#Issue number: 2817377 
            glossaryWords = self.sourceHighlighter.glossaryWords
# wordWithSpace
            termWithSpace = None
 #take the cursor position and check backwrd for getting the first index of the nearest highlighted portion in the txtSource 
            tempPos = position
            while (tempPos>=0 and (not termWithSpace)):
                try:
                    if globalClass.glsDict[str(text)][tempPos]:
                        #take the last index  and check whether it is bigger than the cursor position
                        lastIndex = globalClass.glsLastIndexDict[str(text)][tempPos]
                        if (position<= lastIndex):
                            termWithSpace = unicode(globalClass.glsDict[str(text)][tempPos])
                        else:
                            break
                except:
                    tempPos -= 1 
                    pass
#end of my code         
            # wordWithoutSpace
            withoutSpace = "\\b(\\w+)\\b"
            wordWithoutSpace = QtCore.QRegExp(withoutSpace)
            index = text.lastIndexOf(wordWithoutSpace, position)
            #the line below is added show suggestion for wrong words in target when right clicking over the wrong word. this will work for qt 4.5
            length4 = wordWithoutSpace.lastIndexIn(text, position)
            length = wordWithoutSpace.matchedLength()
            termWithoutSpace = unicode(wordWithoutSpace.capturedTexts()[0])
#            Adjustment for \n,\t and \r
            if text[length4] in ['n', 't', 'r'] and length4>0 and text[length4-1]=='\\':
                termWithoutSpace = termWithoutSpace[1:]
#            Adjustment for accelerators.
            if globalClass.endPuncDict:
                if globalClass.endPuncDict['accelerators'] != 'False':
                    accelerator = globalClass.endPuncDict[globalClass.endPuncDict['accelerators']]
    #                If accelerator is before the matched word.
                    if index != 0 and text[index-1] == accelerator:
                        if index-2 < 0 or text[index-2] in globalClass.targetReplaceChars:
                            termWithoutSpace = accelerator + termWithoutSpace
                            index -= 1
                            length += 1 
                        else:
                            index = text.lastIndexOf(wordWithoutSpace, index-2)
                            length4 = wordWithoutSpace.lastIndexIn(text, index)
                            length = wordWithoutSpace.matchedLength() + length + 1
                            termWithoutSpace = unicode(wordWithoutSpace.capturedTexts()[0]) + accelerator + termWithoutSpace
                            
    #                If accelerator is after the matched word.
                    elif (index+length) != len(text) and text[index+length] == accelerator:
                        index = text.lastIndexOf(wordWithoutSpace, index+length+1)
                        length4 = wordWithoutSpace.lastIndexIn(text, index)
                        length = wordWithoutSpace.matchedLength() + length + 1
                        termWithoutSpace = termWithoutSpace + accelerator + unicode(wordWithoutSpace.capturedTexts()[0])
    #                    End of accelerator adjustment
            self.cursorTerm=termWithoutSpace
            self.termWithoutSpaceLog = [index, length,  termWithoutSpace]
            if self.eventObj == self.ui.txtSource or self.eventObj in self.pluralTextBoxList:
                glossaryWords = self.sourceHighlighter.glossaryWords
                self.synonymsList=self.getSynonyms(termWithoutSpace, self.ui.txtSource)
                # first try wordWithSpace
                if (termWithSpace in glossaryWords):
                    self.emit(QtCore.SIGNAL("lookupTerm"), termWithSpace)
                else:
                    # then wordWithoutSpace
                    if (termWithoutSpace in glossaryWords):
                        self.emit(QtCore.SIGNAL("lookupTerm"), termWithoutSpace)
                    else:
                        menu = QtGui.QMenu()
                        #self.addThesaurusToPopup(menu)    
                        if self.synonymsList:
                            for i in range(len(self.synonymsList)):
                                synonyms = unicode(self.tr("%s")) % (unicode(self.synonymsList[i]))
                                menuAction = menu.addAction(synonyms)                        
                                menuAction.setData(QtCore.QVariant(self.synonymsList[i]))
                            menu.exec_(QtGui.QCursor.pos())
                        
            elif (self.eventObj == self.ui.txtTarget):
                 #thesaurus operation
                synonymsList=self.getSynonyms(termWithoutSpace, self.ui.txtTarget)
                if synonymsList:
                    for i in range(len(synonymsList)):
                        synonyms = unicode(self.tr("%s")) % (unicode(synonymsList[i]))
                        menuAction = menu.addAction(synonyms)                        
                        menuAction.setData(QtCore.QVariant(synonymsList[i]))
                    menu.exec_(QtGui.QCursor.pos())
                self.getSuggestedWords(termWithoutSpace)
                
    def setPattern(self, patternList):
        """
        call highlighter.setPattern()
        @param patternList: a list of pattern 
        if World.WF_debug:
            print "Entering setPattern in class TUview"
              """               
        self.sourceHighlighter.setPattern(patternList)

    def setSearchString(self, searchString, textField, foundPosition):
        """
        call highlighter.setSearchString()
        if World.WF_debug:
            print "Entering setSearchString in class TUview"
              """    
        if (textField == World.source):
            self.sourceHighlighter.setSearchString(searchString, foundPosition)
            self.targetHighlighter.setSearchString("", 0)
        elif (textField == World.target):
            self.targetHighlighter.setSearchString(searchString, foundPosition)
            self.sourceHighlighter.setSearchString("", 0)
        elif (textField == World.comment):
            self.sourceHighlighter.setSearchString("", 0)
            self.targetHighlighter.setSearchString("", 0)
    
    def setHighlightGlossary(self, bool):
        """
        call highlighter.setHighlightGlossary()

         
        if World.WF_debug:
            print "Entering setHighlightGlossary in class TUview"
              """
        self.sourceHighlighter.setHighlightGlossary(bool)
        self.sourceHighlighter.refresh()
        self.emitGlossaryWords()
    
    def closeEvent(self, event):
        """
        set text of action object to 'show Detail' before closing TUview.
        
        @param QCloseEvent Object: received close event when closing widget

         
        if World.WF_debug:
            print "Entering closeEvent in class TUview"
             """ 
        QtGui.QDockWidget.closeEvent(self, event)
        self.toggleViewAction().setChecked(False)
        
    
    def setScrollbarValue(self, value):
        """
        @param value: the new value for the scrollbar.

         
        if World.WF_debug:
            print "Entering setScrollbarValue in class TUview"
              """
        if (value < 0):
            value = 0
        self.disconnect(self.ui.fileScrollBar, QtCore.SIGNAL("valueChanged(int)"), self.emitCurrentIndex)
        self.ui.fileScrollBar.setValue(value)
        self.connect(self.ui.fileScrollBar, QtCore.SIGNAL("valueChanged(int)"), self.emitCurrentIndex)
        self.ui.fileScrollBar.setToolTip("%s / %s" % (value + 1,  self.ui.fileScrollBar.maximum() + 1))
    
    def filterChanged(self, filter, lenFilter):
        """
        Adjust the scrollbar maximum according to lenFilter.
        
        @param filter: helper constants for filtering
        @param lenFilter: len of filtered items.

         
        if World.WF_debug:
            print "Entering filterChanged in class TUview"
             """ 
        self.viewSetting(lenFilter)
        self.ui.fileScrollBar.setMaximum(max(lenFilter - 1, 0))
        self.ui.fileScrollBar.setEnabled(bool(lenFilter))
    
    @QtCore.pyqtSignature("int")
    
    def emitCurrentIndex(self, value):
        
        if World.WF_debug:
            print "Entering emitCurrentIndex in class TUview"
            
        """emit "scrollToRow" signal with value as row start from 0.
        @param value: current row."""
        self.emit(QtCore.SIGNAL("scrollToRow"), value)
    
    def replacespcecial(self, target, text=""):
        # replaces the non printing characters with special characters
        activestyle=World.settings.value("invisiblecharacater", QtCore.QVariant(True)).toBool()
        texttarget=unicode(target)
        self.nonprinttarget=unicode("")
        self.origtarget=texttarget
        for char in texttarget:
            unicodeno=ord(char)
            if unicodedata.category(unichr(unicodeno)) in World.catsCrl:
                for decimal in World.dict_nonprintable.keys():
                    if unicodeno==decimal:
                        name=World.dict_nonprintable[decimal]
                        replacechar=World.dict_invisibleSpecial[name]
                        if activestyle:
                            self.nonprinttarget=self.nonprinttarget+replacechar[0]
                        else:
                            self.nonprinttarget=self.nonprinttarget+replacechar[1]
                        
            else:    
                self.nonprinttarget=self.nonprinttarget+char
        if text=="source":        
            self.ui.txtSource.setPlainText(self.nonprinttarget)       
        if text=="target":     
            self.ui.txtTarget.setPlainText(self.nonprinttarget)
        self.prevtarget=self.nonprinttarget  
            
    def updateView(self, unit):
        """
        Update the text in source and target, set the scrollbar position,
        remove a value from scrollbar if the unit is not in filter.
        Then recalculate scrollbar maximum value.
        
        @param unit: unit class.

         
        if World.WF_debug:
            print "Entering updateView in class TUview"
                  """   
                
        if (not unit):
            return
        self.disconnect(self.ui.txtTarget, QtCore.SIGNAL("textChanged()"), self.emitTextChanged)
        self.ui.txtTarget.setReadOnly(False)
        self.ui.txtTarget.setFocus()

        self.emitTargetChanged()

        
        # set comment
        comment = unit.getcontext()
        comment += unit.getnotes("developer")
        if (comment == ""):
            self.ui.lblComment.hide()
        else:
            self.ui.lblComment.show()
            self.ui.lblComment.setText(unicode(comment))
        
        # set source and target, singular or plural.
        isPlural = unit.hasplural()
        self.ui.sourceStacked.setCurrentIndex(isPlural and 1 or 0)
        self.ui.targetStacked.setCurrentIndex(isPlural and 1 or 0)
        if (isPlural):
            # plural
            self.pluralTarget=True
            
            for i in range(self.ui.tabWidgetSource.count()):
                self.ui.tabWidgetSource.removeTab(0)
            for i in range(self.ui.tabWidgetTarget.count()):
                self.ui.tabWidgetTarget.removeTab(0)
                
            self.sourceStrings = unit.source.strings
            for i in range(len(unit.source.strings)):
                textbox = QtGui.QTextEdit()
                textbox.setReadOnly(True)
                if i==0:
                    glsExpression = self.sourceHighlighter.glsExpression 
                    self.sourceHighlighter = Highlighter(textbox)
                    self.sourceHighlighter.glsExpression = glsExpression
                    self.setHighlightGlossary(True)
                    self.pluralTextBoxList.insert(i, textbox)
                    textbox.installEventFilter(self)
                    textbox.contextMenuEvent = self.customContextMenuEvent
                    self.wasInPlural = True                
                textbox.setPlainText(unicode(self.sourceStrings[i]))
                self.ui.tabWidgetSource.addTab(textbox, "Plural %d" % (i + 1))
#            self.pluralTab=self.ui.tabWidgetSource.count()
            self.ui.tabWidgetSource.setCurrentIndex(0)
            self.tabIndex=0
            self.targetStrings = unit.target.strings
            self.targetBox = []
            pluralTab = ((self.nplurals > len(self.targetStrings)) and self.nplurals or len(self.targetStrings))
            for i in range(pluralTab):
                if (len(self.targetStrings) < pluralTab):
                    self.targetStrings.append("")
                self.textbox = QtGui.QTextEdit()
                self.textbox.setPlainText(self.targetStrings[i])
                self.ui.tabWidgetTarget.addTab(self.textbox, "Plural &%d" % (i + 1))
                self.connect(self.textbox, QtCore.SIGNAL("textChanged()"), self.pluralChanged)
                self.targetBox.append(self.textbox)
                
            self.ui.tabWidgetTarget.setCurrentIndex(0)
        else:
            if self.wasInPlural:
                glsExpression = self.sourceHighlighter.glsExpression 
                self.sourceHighlighter = Highlighter(self.ui.txtSource)
                self.sourceHighlighter.glsExpression = glsExpression
                self.setHighlightGlossary(True)
                self.wasInPlural = False
            # singular
            self.errGlsExpression=self.sourceHighlighter.glsExpression
            self.pluralTarget=False
            self.sourceStrings = []
            self.targetStrings = []
            self.targetBox = []
            currentRow = self.parent.dockOverview.ui.tableOverview.currentRow()
            if not self.parent.ui.actionShow_Nonprinting_Char.isChecked():                   
                if (unicode(unit.source) !=  unicode(self.ui.txtSource.toPlainText())):
                    self.ui.txtSource.setPlainText(unit.source)                    
                if (unicode(unit.target) !=  unicode(self.ui.txtTarget.toPlainText())):
                      
                    self.ui.txtTarget.setPlainText(unit.target or "")
                    self.setCursorToEnd(self.ui.txtTarget)
            else: 
                self.replacespcecial(unit.source, "source")
                self.replacespcecial(unit.target, "target")    
        # set the scrollbar position

        self.setScrollbarValue(unit.x_editor_row)
        self.connect(self.ui.txtTarget, QtCore.SIGNAL("textChanged()"), self.emitTextChanged)
        self.currentUnit = unit
        self.emitGlossaryWords()
        self.emitTargetChanged()
        self.foundPosition = -1
        self.unit = unit
        self.isSingle=False
        try:
            if (str(type(self.unit.source))=='<type \'unicode\'>'):    
                self.isSingle=True
        except:
            pass
        try:
            if(len(self.unit.source.strings) == 1):
                self.isSingle=True
        except:
            pass 

    def emitGlossaryWords(self):
        """
        emit term signal with word found in self.sourceHighlighter.glossaryWords

         
        if World.WF_debug:
            print "Entering emitGlossaryWordsin class TUview"
                    """
        glossaryWords = self.sourceHighlighter.glossaryWords
        if not glossaryWords:
            self.emit(QtCore.SIGNAL("clearTableGlossary"))
        self.emit(QtCore.SIGNAL("termList"), glossaryWords)
    
    def emitTextChanged(self):
        """
        @emit textchanged signal for widget that need to update text while typing.
        if World.WF_debug:
            print "Entering emitTextChanged in class TUview"
                    """          
        if (self.ui.txtTarget.document().isModified()):
            text = unicode(self.ui.txtTarget.toPlainText())
            self.emit(QtCore.SIGNAL("textChanged"), text)
            self.emit(QtCore.SIGNAL("contentChanged"), True)
            self.parent.fileaction.isSaved=False
            self.contentDirty = True
            
    def stopHighlighting(self,bool):
        if not bool:
            self.targetHighlighter.setMissSpelledList([])
            
    def emitTargetChanged(self):
        """
        @emit targetChanged signal if content is dirty 
        with target and currentUnit.
        """
        if World.WF_debug:
            print "Entering emitTargetChanged in class TUview"
            
#        try:
#            for i in range(len(self.targetStrings)):
#                # update each plural of target before emitting targetChanged
#                self.targetStrings[i] = unicode(self.targetBox[i].toPlainText())
#
#        except Exception:
#            pass
#        print "emitTargetChanged"
        if (self.contentDirty) and (hasattr(self, "currentUnit")):
            target = self.targetStrings or unicode(self.ui.txtTarget.toPlainText())
            self.emit(QtCore.SIGNAL("targetChanged"), target, self.currentUnit)
        self.contentDirty = False
        
    def pluralChanged(self):
        """
        changes the text of overview when a plural text is changed
        """
        if World.WF_debug:
            print "Entering pluralChanged in class TUview"
        self.contentDirty = True
#        self.emitTargetChanged()
        for i in range(len(self.targetStrings)):
            # update each plural of target before emitting targetChanged
            self.targetStrings[i] = unicode(self.targetBox[i].toPlainText())            
            if i==0:
                text = self.targetStrings[i]
                cell = self.parent.dockOverview.ui.tableOverview
                targetItem = cell.item(cell.currentRow(), 2)
                row = cell.row(targetItem)

                if (targetItem) and (not cell.isRowHidden(row)):
                    line = text.find("\n")
                    if (line >= 0):
                        text = text[:line] + "..."
                    targetItem.setText(text)

        if (self.contentDirty) and (hasattr(self, "currentUnit")):
            target = self.targetStrings or unicode(self.ui.txtTarget.toPlainText())
            self.emit(QtCore.SIGNAL("targetChanged"), target, self.currentUnit)
        self.contentDirty = False
                
    def source2target(self):
        """
        Copy the text from source to target.
        """
        if World.WF_debug:
            print "Entering source2target in class TUview"   
        if self.isSingle:
            text = self.ui.txtSource.toPlainText()
            self.setTargetText(text)
        else:
            currentTab = self.ui.tabWidgetTarget.currentIndex()
            text = self.unit.source.strings[currentTab]
            self.setTargetText(text)
            
        self.ui.txtTarget.setFocus()
        if (World.settings.value("cursorHome", QtCore.QVariant(True)).toBool()):
            self.setCursorToHome(self.ui.txtTarget)
        else:
            self.setCursorToEnd(self.ui.txtTarget)
        
    def translation2target(self, translation):
        """
        Copy the translation from TM to target.
        """
         
        if World.WF_debug:
            print "Entering translation2target in class TUview"
            
        self.setTargetText(translation)
        self.ui.txtTarget.setFocus()
        self.setCursorToEnd(self.ui.txtTarget)
    
    def replaceText(self, textField, position, length, replacedText):
        """
        replace the string (at position and length) with replacedText in txtTarget.
        
        @param textField: source or target text box.
        @param position: old string's start point.
        @param length: old string's length.
        @param replacedText: string to replace.
        """
        if World.WF_debug:
            print "Entering replaceText in class TUview"
            
        if (textField != World.target):
            return
        text = self.ui.txtTarget.toPlainText()
        text.replace(position, length, replacedText);
        self.ui.txtTarget.setPlainText(text)
    
    def applySettings(self):
        """
        Get settings from World and apply such as font, color, number of
        plural.
        """
         
        if World.WF_debug:
            print "Entering applySettings in class TUview"
            
        sourceColor = World.settings.value("tuSourceColor")
        if (sourceColor.isValid()):
            colorObj = QtGui.QColor(sourceColor.toString())
            palette = QtGui.QPalette(self.ui.txtSource.palette())
            palette.setColor(QtGui.QPalette.Active,QtGui.QPalette.ColorRole(QtGui.QPalette.Text), colorObj)
            palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(QtGui.QPalette.Text), colorObj)
            self.ui.txtSource.setPalette(palette)
        
        targetColor = World.settings.value("tuTargetColor")
        if (targetColor.isValid()):
            colorObj = QtGui.QColor(targetColor.toString())
            palette = QtGui.QPalette(self.ui.txtTarget.palette())
            palette.setColor(QtGui.QPalette.Active,QtGui.QPalette.ColorRole(QtGui.QPalette.Text), colorObj)
            palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(QtGui.QPalette.Text), colorObj)
            self.ui.txtTarget.setPalette(palette)
    
        fontObj = QtGui.QFont()
        sourcefont = World.settings.value("tuSourceFont")
        if (sourcefont.isValid() and fontObj.fromString(sourcefont.toString())):
            self.ui.txtSource.setFont(fontObj)
            self.ui.txtSource.setTabStopWidth(QtGui.QFontMetrics(fontObj).width("m"*8))
    
        targetfont = World.settings.value("tuTargetFont")
        if (targetfont.isValid() and fontObj.fromString(targetfont.toString())):
            self.ui.txtTarget.setFont(fontObj)
            self.ui.txtTarget.setTabStopWidth(QtGui.QFontMetrics(fontObj).width("m"*8))
        
        self.nplurals = World.settings.value("nPlural").toInt()[0]
        
    def viewSetting(self, arg = None):
        """
        Set the view to Detail view: scrollbar, 
        
        @param arg: type as list
        """
        if World.WF_debug:
            print "Entering viewSetting in class TUview"
            
        if (type(arg) is list):
            lenFilter = len(arg)
            self.ui.fileScrollBar.setMaximum(max(lenFilter - 1, 0))
            self.ui.fileScrollBar.setEnabled(bool(lenFilter))
        # value: a bool value just to set the widget visible or not.
        # value = True means there is at leaset a unit.
        value = (arg and True or False)

        if (value == False):
            self.ui.lblComment.clear()
            self.ui.txtSource.clear()
            self.ui.txtTarget.clear()
        self.ui.lblComment.setVisible(value)
        self.ui.sourceStacked.setEnabled(value)
        self.ui.targetStacked.setEnabled(value)
        self.contentDirty = False
        
    def setCursorToHome(self, obj):
        """
        move the obj cursor to the beginning of the first line.
        
        @param obj: QTextEdit object.
        """
         
        if World.WF_debug:
            print "Entering setCursorToHome in class TUview"
            
        cursor = obj.textCursor()
        cursor.setPosition(0)
        obj.setTextCursor(cursor)
        
    def setCursorToEnd(self, obj):
        """
        move the obj cursor to the end of text.
        
        @param obj: QTextEdit object.
        """
         
        if World.WF_debug:
            print "Entering setCursorToEnd in class TUview"
            
        cursor = obj.textCursor()
        cursor.setPosition(len(obj.toPlainText()))
        obj.setTextCursor(cursor)
    
    def setTargetText(self, text):
        """
        Set text of txtTarget while keep undo history.
        
        @param text: text for txtTarget.
        """
        if World.WF_debug:
            print "Entering setTargetText in class TUview"
        unit = self.parent.operator.getCurrentUnit()
        
        if self.isSingle:
            self.ui.txtTarget.selectAll()
            self.ui.txtTarget.insertPlainText(text)

        else:
            currentTab = self.ui.tabWidgetTarget.currentIndex()
            self.targetBox[currentTab].setText(text)
            if(currentTab== 0):
                text = QtGui.QTableWidgetItem(text)
                currentRow = self.parent.dockOverview.ui.tableOverview.currentRow()
                self.parent.dockOverview.ui.tableOverview.setItem(currentRow, 2, text)
        
    def setMissSpelledList(self, unitIndex, missSpelledList, suggestedList):
        """
        highlight wrong words in target view
       @param unitIndex: type as int, missSpelledList as a list of string, suggestedList as a list of string
        @param missSpelledList: type as a list of unicode strings
        @param suggestedList: type as a list of unicode strings
        """
         
        if World.WF_debug:
            print "Entering setMissSpelledList in class TUview"
            
            
        #to avoid Textchanged when just highlighting document
        self.disconnect(self.ui.txtTarget, QtCore.SIGNAL("textChanged()"), self.emitTextChanged)
        self.missSpelledList = missSpelledList
        self.suggestedList = suggestedList
        self.targetHighlighter.setMissSpelledList(self.missSpelledList)
        self.connect(self.ui.txtTarget, QtCore.SIGNAL("textChanged()"), self.emitTextChanged)
        
    def getSuggestedWords(self, missSpelledWord):
        """
        emit suggestedWords as a list according to missSpelledWord
        
        @param missSpelledWord: type as unicode string
        """
        if World.WF_debug:
            print "Entering getSuggestedWords in class TUview"
        
        if (self.missSpelledList):
            for i in range(len(self.missSpelledList)):
                if (missSpelledWord == self.missSpelledList[i]):
                    self.emit(QtCore.SIGNAL("suggestedWords"), self.suggestedList[i])
                    break
    
    def highlightAnWrongWord(self, missed):
        """
        highlight a wrong word
        @param missed: a wrong word to be highlighted
        """
         
        if World.WF_debug:
            print "Entering highlightAnWrongWord in class TUview"
            
        #to avoid Textchanged when just highlighting document
        self.disconnect(self.ui.txtTarget, QtCore.SIGNAL("textChanged()"), self.emitTextChanged)
        self.foundPosition = self.unit.target.find(missed, self.foundPosition + 1)
        if self.foundPosition == -1 :
            self.foundPosition = self.unit.target.find(missed, self.foundPosition + 1)
        self.targetHighlighter.highlightAnWrongWord(missed,self.foundPosition)
        self.connect(self.ui.txtTarget, QtCore.SIGNAL("textChanged()"), self.emitTextChanged)
        
    def clearHighlightWrongWord(self):
        """
        clear highlight of the wrong word
        """
        if World.WF_debug:
            print "Entering clearHighlightWrongWord in class TUview"
            
        self.foundPosition = -1
        self.targetHighlighter.highlightAnWrongWord("",self.foundPosition)
    
    def setContentDirty(self, bool):
        if World.WF_debug:
            print "Entering setContentDirty in class TUview"
            
        if (bool):            
            self.contentDirty = False
    
    def getModifiedWords(self):
#        This function gets the recently modified word/words and returns a list containing them.
        point = QtCore.QPoint(self.ui.txtTarget.cursorRect().left(), self.ui.txtTarget.cursorRect().top())
        cursorIndex = self.ui.txtTarget.cursorForPosition(point).position()
        target = unicode(self.ui.txtTarget.toPlainText())
#            Eliminating the unnecessary chars.
        for each in self.targetReplaceChars:
            text = target.replace(each,' ')
        rightSpace = text.find (' ', cursorIndex)
        if rightSpace == -1:    
            rightSpace = len(text)
        leftSpace = text.rfind(' ',0, cursorIndex-1)
        if leftSpace == -1 or cursorIndex == 0:    
            leftSpace = 0
        prunedWord = text[leftSpace:rightSpace]
        return prunedWord.split()
    
    def compareScapingtargets(self, target):
        # inserts scapes in the places of invisible characters while editing the target  
        if target==u'':
            self.prevtarget=u''
            return
        founduniflag=False
        founduni=0    
        newChar=u''
        newCharIndex=0
        foundmismatchcount=0
        foundmismatch=False
        newtarget=unicode(target)
        origtarget=self.prevtarget
        newtargetlan= len(newtarget)
        origtargetlan= len(origtarget)
        if origtarget!=u'':
            if newtargetlan>origtargetlan:
                origtargetlan=origtargetlan-1
                for Char in reversed(range(0,newtargetlan)):
                    if newtarget[Char]!=origtarget[origtargetlan]:
                        newChar=newChar+unicode(newtarget[Char])
                        newCharIndex=Char
                        foundmismatch=True
                        foundmismatchcount=foundmismatchcount+1
#                        if founduniflag:
#                            founduniflag=False
#                            continue
                    else:
                        if foundmismatch:
                            origtargetlan=origtargetlan-2
                            break
                        else:
                            origtargetlan=origtargetlan-1
            else:                
                newtargetlan=newtargetlan-1
                for oldChar in reversed(range(0,origtargetlan)):   
                    if origtarget[oldChar]!=newtarget[newtargetlan]:
                        newChar=newChar+unicode(origtarget[oldChar])
                        newCharIndex=oldChar
                        foundmismatch=True
                        foundmismatchcount=foundmismatchcount+1
                    else:
                        if foundmismatch:
                            newtargetlan=newtargetlan-2
                            break
                        else:
                            newtargetlan=newtargetlan-1  
#                    founduni=False                
                      
        speIndex=newCharIndex
        newChar=newChar[::-1]
        if origtarget==u'':
            newChar=unicode(target)
        founduni=0
        for c in range(0,len(newChar)):
                for index in World.dict_nonprintable.keys():
                    if ord(newChar[c])==index:
                        founduni=5
                        name=World.dict_nonprintable[index]
                        replacechar=World.dict_invisibleSpecial[name]
                        newChar=newChar.replace((newChar[c]), replacechar[1])

        if newCharIndex==0:
            return
            
        if newtargetlan>origtargetlan:
            nextindex=speIndex+(foundmismatchcount)
            specialtarget=newtarget[0:speIndex]+newChar +newtarget[nextindex:] 
        else:
            specialtarget=newtarget
        if origtarget==u'':
            specialtarget=newChar
        cursonlen= len(newtarget[0:speIndex]+newChar)
        self.cursor=self.cursor+founduni

        cursor = self.ui.txtTarget.textCursor()
        cursor = cursor.position()    
        self.ui.txtTarget.setPlainText(specialtarget)
        self.ui.txtTarget.setFocus()
        cursorPost = cursonlen
        cursor = self.ui.txtTarget.textCursor()
        cursor.setPosition(self.cursor)
        self.ui.txtTarget.setTextCursor(cursor)
        self.prevtarget=specialtarget    
        
    def comparetargets(self, target):
        # inserts word processing characters in the places of invisible characters while editing the target
        if target==u'':
            self.origtarget=u''
            return
        foundmismatchcount=0
        preventloop=0
        foundmismatch=0
        newCharIndex=0
        newChar=u''
        founduni=True
        newtarget=unicode(target)
        origtarget=self.origtarget 
        newtargetlan= len(newtarget)
        origtargetlan= len(origtarget)
        if origtarget!=u'':
            if newtargetlan>origtargetlan:
                origtargetlan=origtargetlan-1
                for Char in reversed(range(0,newtargetlan)):
                        if ord(origtarget[origtargetlan]) in World.dict_nonprintable.keys():
                            if newtarget[Char] in World.lst_invisibleSpecial:
                                origtargetlan=origtargetlan-1
                                continue
                            elif newtarget[Char]==origtarget[origtargetlan]:
                                newChar=newChar+unicode(newtarget[Char])
                                newCharIndex=Char
                                foundmismatch=True
                                foundmismatchcount=foundmismatchcount+1
                        if newtarget[Char]!=origtarget[origtargetlan]:
                            newChar=newChar+unicode(newtarget[Char])
                            newCharIndex=Char
                            foundmismatch=True
                            foundmismatchcount=foundmismatchcount+1
                        else:
                            if foundmismatch:
                                origtargetlan=origtargetlan-2
                                break
                            else:
                                origtargetlan=origtargetlan-1  
                        founduni=False
            else:
                newtargetlan=newtargetlan-1
                for oldChar in reversed(range(0,origtargetlan)):
                    if ord(origtarget[oldChar]) in World.dict_nonprintable.keys():
                        if newtarget[newtargetlan] in World.lst_invisibleSpecial:
                            newtargetlan=newtargetlan-1
                            continue      
                    if origtarget[oldChar]!=newtarget[newtargetlan]:
                        newChar=newChar+unicode(origtarget[oldChar])
                        newCharIndex=oldChar
                        foundmismatch=True
                        foundmismatchcount=foundmismatchcount+1
                    else:
                        if foundmismatch:
                            newtargetlan=newtargetlan-2
                            break
                        else:
                            newtargetlan=newtargetlan-1  
                    founduni=False
        if origtarget==u'':
            self.origtarget=unicode(target)
            newChar=unicode(target)
        else:        
            partialb4=origtarget[:newCharIndex]            
            partialafter=origtarget[newCharIndex:]
            newChar=newChar[::-1]
            if newtargetlan>origtargetlan:
                self.origtarget = partialb4 + newChar + partialafter
            else:
                partialafter=partialafter[len(newChar):]
                self.origtarget = partialb4 + partialafter  
        
        if newCharIndex==0:
            return    
        for c in range(0,len(newChar)):
            for index in World.dict_nonprintable.keys():
                if ord(newChar[c])==index:
                    name=World.dict_nonprintable[index]
                    replacechar=World.dict_invisibleSpecial[name]
                    newChar=newChar.replace((newChar[c]), replacechar[0])

        if newtargetlan>origtargetlan:
            nextindex=newCharIndex+(foundmismatchcount)
            specialtarget=newtarget[0:newCharIndex]+newChar +newtarget[nextindex:] 
        else:
            specialtarget=newtarget
        if origtarget==u'':
            specialtarget=newChar
        self.ui.txtTarget.setPlainText(specialtarget)
        self.ui.txtTarget.setFocus()
        cursor = self.ui.txtTarget.textCursor()
        cursor.setPosition(self.cursor)
        self.ui.txtTarget.setTextCursor(cursor)
                         
    def keyReleaseEvent(self, event):
#        This event is called automatically when a key release occurs.
        activestyle=World.settings.value("invisiblecharacater", QtCore.QVariant(True)).toBool()
        targetText=''
        sourceText=''
        source =''
        target='' 
        if globalClass.errorCheckEnableFlag and self.ui.txtTarget.hasFocus() or self.pluralTarget:
            source = self.ui.txtSource.toPlainText()
            target = self.ui.txtTarget.toPlainText()  
        if globalClass.errorCheckEnableFlag and self.pluralTarget:
            try:
                currentIndex=self.ui.tabWidgetSource.currentIndex()
                for i in range(len(self.targetStrings)):
                    if i==currentIndex:
                        targetText = self.targetStrings[i]
                        sourceText = self.sourceStrings[i] 
                        break
            except Exception:
                pass
        target = targetText or self.ui.txtTarget.toPlainText()
        self.curtarget=target
        source = sourceText or self.ui.txtSource.toPlainText()            
        if source == globalClass.currentSource:
            if not(target == globalClass.currentTarget):
                if globalClass.preComputeFlag == 1:
                    self.parent.callPreCompute()
                    globalClass.preComputeFlag = 0
#               Finding out the index of last modified word.
        self.recentModifiedWords = []
        key = event.key()
        if globalClass.spellCheckEnableFlag and self.ui.txtTarget.hasFocus() and key != QtCore.Qt.Key_Left and key != QtCore.Qt.Key_Right and key != QtCore.Qt.Key_Up and key != QtCore.Qt.Key_Down:
            self.allowNavigationKeys = True
            self.modifiedWords = self.getModifiedWords()
        elif self.allowNavigationKeys:   
            self.recentModifiedWords = self.getModifiedWords()
            for each in self.recentModifiedWords:
                if each not in self.modifiedWords:
                    self.allowNavigationKeys = False
                    self.modifiedWords = []
                    self.emitTargetChanged()
                    self.emit(QtCore.SIGNAL("breakerEncountered"))
                    break
        if self.prevtarget =="":
            self.prevtarget=target
            
        cursor = self.ui.txtTarget.textCursor()
        self.cursor = cursor.position()    
        
        if self.parent.ui.actionShow_Nonprinting_Char.isChecked():
            if activestyle:
                self.comparetargets(target)
            else:    
                self.compareScapingtargets(target)
            
if __name__ == "__main__":
    import sys, os
    # set the path for QT in order to find the icons
    QtCore.QDir.setCurrent(os.path.join(sys.path[0], "..", "ui"))
    app = QtGui.QApplication(sys.argv)
    Form = TUview(None)
    Form.show()
    sys.exit(app.exec_())
