#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#Copyright (c) 2006 - 2007 by The WordForge Foundation
#                       www.wordforgefoundation.org

#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details. 
#
# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)
#       Aditi Barua(aditi@ankur.org.bd)
#       Asheq Hamid(asheq@ankur.org.bd)
#       Md. Rezwan Shahid(rezwan@ankur.org.bd)
#       Goutom Roy(goutom@ankur.org.bd)
#This module is working on the display of TM in a talbe

from PyQt4 import QtCore, QtGui
import sys, os
import WordForge.Editor.modules.World as World
from WordForge.Editor.modules.globalClass import globalClass
from WordForge.Editor.modules.ErrorChecking import ErrorChecking
import string

class TableGlossary(QtGui.QDockWidget):
    def __init__(self, parent):

        if World.WF_debug:
            print "Enter __init__ in Class TableGlossary"
        
        QtGui.QDockWidget.__init__(self, parent)
        self.setObjectName("glossaryDock")
        self.setWindowTitle(self.tr("Glossary Suggestions"))
        self.form = QtGui.QWidget(self)
        from WordForge.Editor.ui.Ui_TableGlossary import Ui_Form
        self.ui = Ui_Form()
        self.ui.setupUi(self.form)
        self.setWidget(self.form)
        
        self.normalState = QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable
        self.toggleViewAction().setChecked(True)
        self.same = False
        self.applySettings()
        self.clipboard = QtGui.QApplication.clipboard()
        self.keySequence = []
#      Connect the signal to show popup definition   
        self.ui.tblGlossary.setMouseTracking(True)
#      If the current unit needs pos column, hen this flag is true.   
        self.needPosColumn = False
#      This flag is True if tableglossary table has a POS column.  
        self.hasPosColumn = False
        self.headerLabels = [self.tr("Term"),self.tr("Translation")]
        self.posIndex = 0
        self.glossaryCandidates = []
        self.connect(self.ui.tblGlossary, QtCore.SIGNAL("cellEntered (int,int)"), self.showDefinition)
        
    def showDefinition(self, row, col):
#        This function shows popup definition for term if it is available.
        definition =  ''
        if hasattr(self.glossaryCandidates[row],'entryDefinition'):
            if self.glossaryCandidates[row].entryDefinition:
                definition += 'Definition' + ' : '
                for eachdefinition in self.glossaryCandidates[row].entryDefinition:
                    definition += eachdefinition + '\n'
            if self.glossaryCandidates[row].subjectField:
                definition += 'Subject Field' + ' : '
                for each in self.glossaryCandidates[row].subjectField:
                    definition += each + '\n'
            if self.glossaryCandidates[row].context:
                definition += 'Context' + ' : '
                for each in self.glossaryCandidates[row].context:
                    definition += each + '\n'        
            for eachKey in self.glossaryCandidates[row].languageDependentDefinition.keys():
                if self.glossaryCandidates[row].languageDependentDefinition[eachKey]:
                    definition += eachKey + ' definition : '
                    for eachdefinition in self.glossaryCandidates[row].languageDependentDefinition[eachKey]:
                        definition += eachdefinition + '\n'
                
            definition = definition[:-1]
        self.ui.tblGlossary.item(row, 0).setToolTip(definition)
        
    def applySettings(self):
        """
        set color and font to the TM table.
        """
        if World.WF_debug:
            print "Enter applySettings in Class TableGlossary"
            
        glossaryColor = World.settings.value("glossaryColor")
        if (glossaryColor.isValid()):
            colorObj = QtGui.QColor(glossaryColor.toString())
            palette = self.ui.tblGlossary.palette()
            palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(QtGui.QPalette.Text), colorObj)
            palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(QtGui.QPalette.Text), colorObj)
            self.ui.tblGlossary.setPalette(palette)

        font = World.settings.value("glossaryFont")
        if (font.isValid()):
            fontObj = QtGui.QFont()
            if (fontObj.fromString(font.toString())):
              self.ui.tblGlossary.setFont(fontObj)
              
        self.ui.tblGlossary.resizeRowsToContents()

    def fillTable(self, candidates):
        """
        fill each found unit into table
        @param candidates:list of pofile object
        """
        if World.WF_debug:
            print "Enter fillTable in Class TableGlossary"
        # goutom glossary suggestion
        tempstr=string.lower(globalClass.currentSource)
        glist=ErrorChecking().hasXmlAttr(tempstr)
        splitedwords=[]
        for x in glist:
            splitedwords.append(self.splitingxmltag(x))
        for x in glist:
            t=tempstr.replace(str(x), "")
            tempstr=t
            
        self.viewSetting()
        table = self.ui.tblGlossary
        if (not candidates):
            return 
        for unit in candidates:
            c=0
            for x in glist:
                if unit.source in x and unit.source not in tempstr:
                    c=1
                    break
            if c==1:
                continue
#        goutom
            row = self.ui.tblGlossary.rowCount()
            for r in range(row):
                if (table.item(r,0).text() ==  unit.source and table.item(r,1).text() ==  unit.target):
                    self.same = True
                    break
            if self.same:
                self.same = False
                continue
            table.setRowCount(row + 1)
            item = QtGui.QTableWidgetItem(unit.source)
            item.setFlags(self.normalState)
            table.setItem(row, 0, item)
#          Add the POS value  
            if self.posIndex:
                if hasattr(unit, "targetPOSDict"):
                    itemValue=unit.targetPOSDict[unit.target]
                else:
                    itemValue = ''
                item = QtGui.QTableWidgetItem(itemValue )
                item.setFlags(self.normalState)
                table.setItem(row, 1, item)

            item = QtGui.QTableWidgetItem(unit.target)
            item.setFlags(self.normalState)
            table.setItem(row, self.posIndex + 1, item)
            
            keySequence = "Ctrl+" + str(row+1)
            if keySequence not in self.keySequence:
                self.keySequence.append(keySequence)
                shortcut = QtGui.QShortcut(QtGui.QKeySequence(keySequence), self.parent())
                shortcut.setEnabled(True)
                self.connect(shortcut, QtCore.SIGNAL("activated()"),  self.copyToClipboard)
                
    def copyToClipboard(self):
        if World.WF_debug:
            print "Enter copyToClipboard in Class TableGlossary"
        rowIndex = int(str(self.sender().key().toString())[-1])
        text = ""
        try:
            text = unicode(self.ui.tblGlossary.item(rowIndex -1, 1).text())
        except AttributeError:
            pass
        self.clipboard.setText(text)
        
    
    def closeEvent(self, event):
        """
        Unchecked the Glossary view action.
        @param QCloseEvent Object: received close event when closing widget
        """
        if World.WF_debug:
            print "Enter closeEvent in Class TableGlossary"
        QtGui.QDockWidget.closeEvent(self, event)
        self.toggleViewAction().setChecked(False)
        self.emit(QtCore.SIGNAL("closed"))
        
    def showEvent(self, event):
        """
        Checked the Glossary view action.
        @param QShowEvent Object: received show event when showing widget
        """
        if World.WF_debug:
            print "Enter showEvent in Class TableGlossary"
        QtGui.QDockWidget.showEvent(self, event)
        self.toggleViewAction().setChecked(True)
        self.emit(QtCore.SIGNAL("shown"))
    
    def viewSetting(self):
        if World.WF_debug:
            print "Enter viewSetting in Class TableGlossary"
         
        if hasattr(self, 'glossaryCandidates') and self.glossaryCandidates:
#      Code here for adding and deleting columns dynamically.
            self.needPosColumn = False
            for unit in self.glossaryCandidates:
#         Check if the unit has attribute to hold POS.  
                if hasattr(unit, 'targetPOSDict'):
#              Check if pos column is needed in the table or not.  
                    for eachkey in unit.targetPOSDict.keys():
                        if unit.targetPOSDict[eachkey] != '':
                            self.needPosColumn = True
                            break

        if self.needPosColumn and not  self.hasPosColumn:
            self.headerLabels = [self.tr("Term"),self.tr("Part of speech"), self.tr("Translation")]
            self.hasPosColumn = True
            self.needPosColumn = False
            self.posIndex = 1
        elif not self.needPosColumn and self.hasPosColumn:
            self.headerLabels = [self.tr("Term"),self.tr("Translation")]
            self.hasPosColumn = False
            self.posIndex = 0
            
        self.ui.tblGlossary.clear()
        self.ui.tblGlossary.setColumnCount(len(self.headerLabels))
        self.ui.tblGlossary.setHorizontalHeaderLabels(self.headerLabels)
        for i in range(len(self.headerLabels)):
            self.ui.tblGlossary.resizeColumnToContents(i)
            self.ui.tblGlossary.horizontalHeader().setResizeMode(i, QtGui.QHeaderView.Stretch)
        self.ui.tblGlossary.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        self.ui.tblGlossary.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        self.ui.tblGlossary.verticalHeader().hide()
        self.ui.tblGlossary.setRowCount(0)
        self.ui.tblGlossary.setEnabled(True)
    
if __name__ == "__main__":
    if World.WF_debug:
        print "Enter __name__ == __main__ in Class TableGlossary"
    app = QtGui.QApplication(sys.argv)
    table = TableGlossary(None)
    table.show()
    sys.exit(app.exec_())

