#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#Copyright (c) 2006 - 2007 by The WordForge Foundation
#                       www.wordforgefoundation.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details. 
#
# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)
#This module is working on the display of TM in a table

from PyQt4 import QtCore, QtGui
import sys, os
import WordForge.Editor.modules.World as World
from WordForge.Editor.ui.Ui_TableTM import Ui_Form
import WordForge.Editor.ui.images_rc
import difflib

class TableTM(QtGui.QDockWidget):
    """
    This class works with the display of translation memory in a table
    
    @signal itemDoubleClicked(QTableWidgetItem *): emitted when doubled clicking on tableWidgetItem
    @signal translation2target: emitted with target as string when itemDoubleClicked(QTableWidgetItem *) is emitted.
    @signal openFile: emitted with filename together with the findUnit signal
    @signal findUnit: emitted with the current source.
    @signal visible: emitted with the bool value to toggleViewAction
    
    """
    def __init__(self, parent = None):
        QtGui.QDockWidget.__init__(self, parent)
        self.setObjectName("miscDock")
        self.setWindowTitle(self.tr("Translation Memory Suggestions"))
        self.form = QtGui.QWidget(self)
        self.ui = Ui_Form()
        self.ui.setupUi(self.form)
        self.setWidget(self.form)
        self.parent = parent        
        
        self.normalState = QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable 
        self.qIcon = QtGui.QIcon
        self.infoIcon = self.qIcon(":/TM_info.png")
#        self.connect(self.ui.tblTM, QtCore.SIGNAL("itemDoubleClicked(QTableWidgetItem *)"), self.emitTarget)
#        self.connect(self.ui.tblTM, QtCore.SIGNAL("cellDoubleClicked(int, int)"), self.copySuggestionToTarget)
#        self.createContextMenu()
        self.allowUpdate = True
        self.applySettings()
        self.candidates = []
        self.rowNumber = 0
        self.keySequence = []
        self.targetList = []

    def applySettings(self):
        """
        set color and font to the TM table.
        """
        TMColor = World.settings.value("TMColor")
        if (TMColor.isValid()):
            colorObj = QtGui.QColor(TMColor.toString())
            palette = self.ui.listWidget.palette()
            palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(QtGui.QPalette.Text), colorObj)
            palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(QtGui.QPalette.Text), colorObj)
            self.ui.listWidget.setPalette(palette)

        font = World.settings.value("TMFont")
        if (font.isValid()):
            fontObj = QtGui.QFont()
            if (fontObj.fromString(font.toString())):
              self.ui.listWidget.setFont(fontObj)
              
    def createContextMenu(self):
        """
        context menu of items
        """
        self.menu = QtGui.QMenu()
        actionCopyResult = self.menu.addAction(self.qIcon(":/source.png"), self.tr("Copy Translation To Target"))
#        actionAddAsAltsTrans = self.menu.addAction(self.qIcon(":/add.png"), self.tr("Add as an alternative translation"))
#        actionRemoveAsAltsTrans = self.menu.addAction(self.qIcon(":/minus.png"), self.tr("Remove from alternative translations"))
#        actionEditFile = self.menu.addAction(self.qIcon(":/open.png"),self.tr("Edit File"))

        self.connect(actionCopyResult, QtCore.SIGNAL("triggered()"), self.copySuggestionToTarget)
#        self.connect(actionAddAsAltsTrans, QtCore.SIGNAL("triggered()"), self.emitAddAsAltTrans)
#        self.connect(actionRemoveAsAltsTrans, QtCore.SIGNAL("triggered()"), self.emitRemoveAltTran)
#        self.connect(actionEditFile, QtCore.SIGNAL("triggered()"), self.emitOpenFile)        
        
    def contextMenuEvent(self, e):
        row = self.ui.tblTM.rowCount()
        if (row >0):
            self.menu.exec_(e.globalPos())
        
    def InstantTMFillTable(self, candidates):
        """
        Fill table with candidates source and target for instantTM.
        @param candidates: list of unit.
        """
        self.ui.listWidget.clear()
        self.rowNumber = 0
        
        source1 = self.parent.dockTUview.ui.txtSource.toPlainText()
        source1 = str(source1)

        TMList = candidates
        ListText = []
        self.targetList = []
        
        for unit in TMList:
            match = str(unit.matchquality) +"%"
            source2 = str(unit.source)
            target = str(unit.target)
            self.targetList.append(target)
            self.rowNumber += 1
            text = self.matchWords(match, source1, source2, target)
            ListText.append(text)

        for each in range(len(ListText)):
            self.setListText(each, ListText[each])

    def fillTable(self, source, candidates):
        """
        Fill table with candidates source and target.
        @param source: source of the unit.
        @param candidates: list of unit.
        """
        self.ui.listWidget.clear()
        source1 = str(source)
        TMList = candidates[:]
        ListText = []
        self.targetList = []
        self.rowNumber = 0
        
        for each in range(len(TMList)):
            if (each%3) == 0:
                match = TMList[each]+"%"
            if (each%3) == 1:
                source2 = TMList[each]
            if (each%3) == 2:
                target = TMList[each]
                self.targetList.append(target)
#                increasing the row number of the listWidget
                self.rowNumber += 1
#                sending source1 and source2 to matchWords() to find out the differences between them
                text = self.matchWords(match, source1, source2, target)
#                appending those differences in ListText
                ListText.append(text)
                
#            sending for filling the TM ListWidget
        for each in range(len(ListText)):
            self.setListText(each, ListText[each])


    def matchWords(self, match, source1, source2, target):
        """
        find out the differences between srouce1 and source2
        @param match: match quality of candidate
        @param source1: current source
        @param source2: source of TM candidate
        @param target: target of TM candidate
        """
        text1 = source1
        text2 = source2
        source1 = []
        source2 = []
        tmp = ''
        tabChar = "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"

#        defining tags for different opcodes
        colorMark = "<span style=\"font-weight:600; color:#2e7a21;\">"
        colorDelete = "<span style=\" color:#9a0c32;\">"
        colorDeleteSpace = "<span style=\" background-color:#9a0c32;\">"
        colorReplace = "<span style=\" color:#934ab7;\">"
        colorReplaceSpace = "<span style=\" background-color:rgb(100, 50, 230);\">"
        colorInsert = "<span style=\" color:#2427fa;\">"
        colorInsertSpace = "<span style=\" background-color:rgb(156, 212, 255);\">"
        common = "</span>"

        for i in text1:
#            splitting text1, so that consecutive alpha-numeric letters stay together and other characters get split

            if ( ((ord(i) > 47) and (ord(i) < 58)) or ((ord(i) > 64) and (ord(i) < 91)) or ((ord(i) > 96) and (ord(i) < 123)) ):
                tmp = tmp + i
            
            else:
                if tmp is not '':
                    source1.append(tmp)
                tmp = ''
                source1.append(i)

#        making sure that last word gets included if the last character is not other than alfa-numeric
        if tmp is not '':
            source1.append(tmp)

        tmp = ''
        for i in text2:
#            splitting text2, so that consecutive alpha-numeric letters stay together and other characters get split

            if ( ((ord(i) > 47) and (ord(i) < 58)) or ((ord(i) > 64) and (ord(i) < 91)) or ((ord(i) > 96) and (ord(i) < 123)) ):
                tmp = tmp + i

            else:
                if tmp is not '':
                    source2.append(tmp)
                tmp = ''
                source2.append(i)

#        making sure that last word gets included if the last character is not other than alfa-numeric
        if tmp is not '':
            source2.append(tmp)
            
#        sending to SequenceMatcher to find out the difference
        matcher = difflib.SequenceMatcher(None, source1, source2)
        sourceMatch = matcher.get_opcodes()
#        for tag, i1, i2, j1, j2 in matcher.get_opcodes():
#            print ("%7s s1[%d:%d] (%s) s2[%d:%d] (%s)" % 
#                (tag, i1, i2, source1[i1:i2], j1, j2, source2[j1:j2])) 

        for i in range(len(source1)):
#            replacing space,  <, > etc with their corresponding html codes in source1
            if source1[i] == " ":
                source1[i] = "&nbsp;"

            if source1[i] == "<":
                source1[i] = "&lt;"

            if source1[i] == ">":
                source1[i] = "&gt;"

        for i in range(len(source2)):
#            replacing space,  <, > etc with their corresponding html codes in source2
            if source2[i] == " ":
                source2[i] = "&nbsp;"

            if source2[i] == "<":
                source2[i] = "&lt;"

            if source2[i] == ">":
                source2[i] = "&gt;"

#            replacing space,  <, > etc with their corresponding html codes in target
        target = target.replace(" ", "&nbsp;")
        target = target.replace("<", "&lt;")
        target = target.replace(">", "&gt;")
        target = target.replace("\t", tabChar)

        text1 = []
        text2= []

#        setting different html codes for different opcodes:
        for tag, i1, i2, j1, j2 in sourceMatch:
            if tag == "equal":
                for word in range(i1, i2):
                    if source1[word] == "\t":
                        text1.append(tabChar)
                    else:
                        text1.append(source1[word])
                
                for word in range(j1, j2):
                    if source2[word] == "\t":
                        text2.append(tabChar)
                    else:
                        text2.append(source2[word])
                
            if tag == "delete":
                text1.append(colorDelete) 
                for word in range(i1, i2):
                    if source1[word] == "&nbsp;":
                        text1.append(colorDeleteSpace)
                        text1.append(source1[word])
                        text1.append(common)
                    elif source1[word] == "\t":
                        text1.append(colorDeleteSpace)
                        text1.append(tabChar)
                        text1.append(common)
                    else:
                        text1.append(source1[word])
                text1.append(common)
                
#                nothing will have to be deleted from text2,  so the following code is not necessary
#                text2.append(colorDelete)                
#                for word in range(j1, j2):
#                    text2.append(source2[word])
#                text2.append(common)


            if tag == "insert":
#                nothing will have to be inserted in text1,  so the following code is not necessary
#                text1.append(colorInsert) 
#                for word in range(i1, i2):
#                    text1.append(source1[word])
#                text1.append(common)
                    
                text2.append(colorInsert)                
                for word in range(j1, j2):
                    if source2[word] == "&nbsp;":
                        text2.append(colorInsertSpace)
                        text2.append(source2[word])
                        text2.append(common)
                    elif source2[word] == "\t":
                        text2.append(colorInsertSpace)
                        text2.append(tabChar)
                        text2.append(common)
                    else:
                        text2.append(source2[word])
                text2.append(common)
                    
                    
            if tag == "replace":
                text1.append(colorReplace) 
                for word in range(i1, i2):
                    if source1[word] == "\t":
#                        text1.append(colorReplaceSpace)
                        text1.append(tabChar)
#                        text1.append(common)
                    else:
                        text1.append(source1[word])
                text1.append(common)
                    
                text2.append(colorReplace)                
                for word in range(j1, j2):
                    if source2[word] == "\t":
#                        text2.append(colorReplaceSpace)
                        text2.append(tabChar)
#                        text2.append(common)
                    else:
                        text2.append(source2[word])
                text2.append(common)


        text1 = "".join(text1)
        text2 = "".join(text2)
        
        if self.rowNumber < 10:
            KeySequence = "[Alt+" + str(self.rowNumber) + "]"
        elif self.rowNumber == 10:
            KeySequence = "[Alt+0]"
        else:
            KeySequence = ""

#        if the match if 100%, then only one source will be shown in the TM window:
        if match == "100%":
            text = "<b>" + match + tabChar + KeySequence +"</b><br/>"+colorMark + "Source: " + common + text1 + "<br/>" + colorMark + "Target: " + common + target

#        constructing the whole text if match is not 100%
        else:
            text = "<b>" + match + tabChar + KeySequence +"</b><br/>"+colorMark + "Source 1: " + common + text1 + "<br/>" + colorMark + "Source 2: " + common +  text2 + "<br/>" + colorMark + "Target: " + common + target

        return text
        
        
    def setListText(self, row, text):
        """
        Fill TM ListWidget with candidates
        @param text: text to be inserted in a row of the ListWidget
        """
        text = unicode(text)
        widget = QtGui.QWidget()
        layout = QtGui.QVBoxLayout(widget)
        textEdit = QtGui.QTextEdit()
        textEdit.setBackgroundRole(QtGui.QPalette.Dark)
        textEdit.setAutoFillBackground(True)
        textEdit.setReadOnly(True)
        textEdit.setHtml(text)
#        textEdit.setResizeMode(QtGui.QListWidget.Adjust)
        textEdit.adjustSize ()
        layout.addWidget(textEdit)

        layout.setSizeConstraint(QtGui.QLayout.SetMinAndMaxSize)
        item =QtGui.QListWidgetItem()
        item.setSizeHint(QtCore.QSize(2, 90))
        self.ui.listWidget.addItem(item)
        self.ui.listWidget.setItemWidget(item, widget)
        
        row = row + 1
        keySequence = None
        if row< 10:
            keySequence = "Alt+" + str(row)

        elif row == 10:
            keySequence = "Alt+0"
            
        if keySequence and (keySequence not in self.keySequence):
            self.keySequence.append(keySequence)
            shortcut = QtGui.QShortcut(QtGui.QKeySequence(keySequence), self.parent)
            shortcut.setEnabled(True)
            self.connect(shortcut, QtCore.SIGNAL("activated()"),  self.copySuggestionToTarget)


    def copySuggestionToTarget(self):
        """
        Copy TM suggestion to target field
        """
        rowIndex = int(str(self.sender().key().toString())[-1])

        try:
            text = unicode(self.targetList[rowIndex-1])
            self.parent.dockTUview.setTargetText(text)
        except:
            pass
#        unit = self.parent.operator.getCurrentUnit()
#        text = self.ui.tblTM.item(self.ui.tblTM.currentRow(), 2).text()
#        text = QtGui.QApplication.translate(None, str(text), None, QtGui.QApplication.UnicodeUTF8)
        
#        if len(unit.source.strings)==1:
#            self.parent.ui.actionSave.setEnabled(True)
#            self.parent.operator.modified = True
#
#        else:
#            currentTab = self.parent.dockTUview.ui.tabWidgetTarget.currentIndex()
#            self.parent.dockTUview.targetBox[currentTab].setText(text)
#            if(currentTab== 0):
#                text = QtGui.QTableWidgetItem(text)
#                currentRow = self.parent.dockOverview.ui.tableOverview.currentRow()
#                self.parent.dockOverview.ui.tableOverview.setItem(currentRow, 2, text)
#            self.parent.ui.actionSave.setEnabled(True)
#            self.parent.operator.modified = True


    def emitTarget(self):
        """
        Send "targetChanged" signal with target as string.
        """
        # don't allow fill table since the unit's target is same from here.
        self.allowUpdate = False
        row = self.ui.tblTM.currentRow()
        item = self.ui.tblTM.item(row, 2)
        if (item):
            target = item.data(QtCore.Qt.UserRole).toString()
            self.emit(QtCore.SIGNAL("translation2target"), unicode(target))
    
#    def emitAddAsAltTrans(self):
#        """
#        Send "AddAsAltTrans" signal with candidate to add as alternative translations
#        """
#        
#        candidate = self.candidates[self.ui.tblTM.currentRow()]
#        self.emit(QtCore.SIGNAL("addAsAltTrans"), candidate)
#
#    def emitRemoveAltTran(self):
#        """
#        Send "RemoveFromAltTrans" signal with candidate to remove from alternative translation
#        """
#        candidate = self.candidates[self.ui.tblTM.currentRow()]
#        self.emit(QtCore.SIGNAL("RemoveAltTran"), candidate)
#
#    def emitOpenFile(self):
#        """
#        Send "openFile" signal with filename together with the findUnit signal
#        with the current source.
#        """
#        row = self.ui.tblTM.currentRow()
#        item = self.ui.tblTM.item(row, 0)
#        itemSource = self.ui.tblTM.item(row, 1)
#        source = None
#        if (item and itemSource):
#            source = unicode(itemSource.data(QtCore.Qt.UserRole).toString())
#            filepath = unicode(item.data(QtCore.Qt.UserRole).toString())
#            self.emit(QtCore.SIGNAL("openFile"), filepath)
#            self.emit(QtCore.SIGNAL("findUnit"), source)
        
    def setToolTip(self, index = None, col = 0, tooltips = "", icon = QtGui.QIcon()):
        """
        mark icon indicate unit has more info and add tooltips.
        
        @param index: row in table.
        @param col: column to set tooltips.
        @param tooltips: more info about candidates such as which file the 
            candidate locates, who is the translator and when.
        @param icon: icon to set to col.
        """
        # get the current row
        if (not index):
            item = self.ui.tblTM.item(0, col)
        else:
            item = self.ui.tblTM.item(index, col)
        
        if (not item):
            return
        if (tooltips):
            item.setIcon(icon)
            item.setToolTip(unicode(tooltips))

    def closeEvent(self, event):
        """
        Unchecked the TM Lookup view action.
        
        @param QCloseEvent Object: received close event when closing widget
        """
        QtGui.QDockWidget.closeEvent(self, event)
        self.toggleViewAction().setChecked(False)
        self.emit(QtCore.SIGNAL("visible"), False)
        
    def showEvent(self, event):
        """
        Checked the TM Lookup view action.
        
        @param QShowEvent Object: received show event when showing widget
        """
        QtGui.QDockWidget.showEvent(self, event)
        self.toggleViewAction().setChecked(True)
        self.emit(QtCore.SIGNAL("visible"), True)
    
#    def newUnit(self):
#        """
#        Clear table to be filled by a new unit.
#        """
#        self.ui.tblTM.clear()
#        self.ui.tblTM.setEnabled(True)
#        self.ui.tblTM.setHorizontalHeaderLabels(self.headerLabels)
#        self.ui.tblTM.setSortingEnabled(False)
#        self.ui.tblTM.setRowCount(0)
    
    def shorten(self, text):
        """
        Return the first part of text, seperated by new line and filled with three dots.
        """
        line = text.find("\n")
        if (line >= 0):
            text = text[:line] + "..."
        return text
    
    def viewSetting(self):
        self.ui.listWidget.clear()
        self.targetList = []
#        self.ui.tblTM.clear()
#        self.headerLabels = [self.tr("Match"), self.tr("Source"), self.tr("Translation")]
#        self.ui.tblTM.setColumnCount(len(self.headerLabels))
#        self.ui.tblTM.setHorizontalHeaderLabels(self.headerLabels)
#        self.ui.tblTM.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
#        self.ui.tblTM.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
#        self.ui.tblTM.resizeColumnToContents(2)
#        self.ui.tblTM.horizontalHeader().setResizeMode(2, QtGui.QHeaderView.Stretch)
#        self.ui.tblTM.verticalHeader().hide()
#        self.ui.tblTM.setRowCount(0)
#        self.ui.tblTM.setEnabled(True)


if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    table = TableTM(None)
    table.show()
    sys.exit(app.exec_())

