#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007 by KhmerOS
# www.khmeros.info
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details.
#
# Developed by:
#       Hok Kakada      (hokkakada@khmeros.info)
#       San Titvirak    (titvirak@khmeros.info)
#       Seth Chanratha  (sethchanratha@khmeros.info)
#       Keo Sophon      (keosophon@khmeros.info)
# 
# This module is working on updating the xliff file.


from WordForge.Editor.modules import InternalGlossary
import unittest
import tempfile
import os

class testInternalGlossary(unittest.TestCase):
    
    def setUp(self):
        self.glossary = '''<?xml version='1.0' encoding='utf-8'?>
<xliff xmlns="urn:oasis:names:tc:xliff:document:1.1" version="1.1">
  <file original="NoName" source-language="en" datatype="plaintext">
    <body>
      <trans-unit approved="yes" id="accessiblestrings.src#RID_STR_ACC_NAME_BROWSEBUTTON.string.text">
        <source>Applications</source>
        <target>កម្មវិធី</target></trans-unit>
    </body>
  </file>
</xliff>'''
    
        self.input = '''<?xml version="1.0" encoding="utf-8"?>
    <xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
            <file datatype="po" original="doc.txt" source-language="en-US">
                    <body>
                            <trans-unit id="1" xml:space="preserve">
                                    <source>Applications</source>
                                    <target />
                            </trans-unit>
                            <trans-unit id="2" xml:space="preserve">
                                    <source>open</source>
                                    <target>បើក</target>
                            </trans-unit>
                    </body>
            </file>
    </xliff>'''
    
        self.output = '''<?xml version='1.0' encoding='utf-8'?>
<xliff xmlns="urn:oasis:names:tc:xliff:document:1.1" version="1.1">
            <file datatype="po" original="doc.txt" source-language="en-US">
                    <header><glossary><internal-file><xliff version="1.1">
  <file original="NoName" source-language="en" datatype="plaintext">
    <body>
      <trans-unit id="1" xml:space="preserve">
        <source>Applications</source>
        <target>កម្មវិធី</target>
      </trans-unit>
    </body>
  </file>
</xliff></internal-file></glossary></header><body>
                            <trans-unit id="1" xml:space="preserve">
                                    <source>Applications</source>
                                    <target/>
                            </trans-unit>
                            <trans-unit id="2" xml:space="preserve">
                                    <source>open</source>
                                    <target>បើក</target>
                            </trans-unit>
                    </body>
            </file>
    </xliff>'''
    
    def testEmbedGlosaryIntoFile(self):
        handledGlossary, glossary = tempfile.mkstemp(".xlf")
        glossaryObj = open(glossary, 'w')
        glossaryObj.write(self.glossary)
        glossaryObj.close()
        
        handleInput, input = tempfile.mkstemp(".xlf")
        inputObj = open(input, 'w')
        inputObj.write(self.input)
        inputObj.close()
        
        InternalGlossary.embedGlossary(glossary, input)
        
        head, tail = os.path.splitext(input)
        targetFile = head +'-embedded.xlf'
        
        self.assert_(os.path.exists(targetFile))
        
        from xml.dom import minidom
        self.assertEqual(minidom.parse(targetFile).toxml(), minidom.parseString(self.output).toxml())
        
        os.remove(glossary)
        os.remove(input)
        os.remove(targetFile)
        
    def testEmbedGlossaryIntoDir(self):
        handledGlossary, glossary = tempfile.mkstemp(".xlf")
        glossaryObj = open(glossary, 'w')
        glossaryObj.write(self.glossary)
        glossaryObj.close()
        
        inputDir = tempfile.mkdtemp()
        handleInput, input = tempfile.mkstemp(".xlf", "", inputDir )
        inputObj = open(input, 'w')
        inputObj.write(self.input)
        inputObj.close()
                
        #Folder in subfolder        
        subFolder = tempfile.mkdtemp("", "", inputDir)
        handleInputInSub, inputInSub = tempfile.mkstemp(".xlf", "", subFolder)
        inputObjInSub = open(inputInSub, 'w')
        inputObjInSub.write(self.input)
        inputObjInSub.close()
        
        # test insert input as a dir
        InternalGlossary.embedGlossary(glossary, inputDir)        
        targetDir = inputDir + '-embedded'        
        targetFile = targetDir + os.path.sep + os.path.basename(input)
        
        # check whether root folder and file under it exist or not
        self.assert_(os.path.exists(targetDir))           
        self.assert_(os.path.exists(targetFile))        
        
        # check whether sub folder and file under it exit or not
        subFolderInTarget = targetDir + os.path.sep + os.path.basename(subFolder)
        fileInSubFolderInTarget = subFolderInTarget + os.path.sep + os.path.basename(inputInSub)
        self.assert_(os.path.exists(subFolderInTarget))           
        self.assert_(os.path.exists(fileInSubFolderInTarget))        
        
if __name__ == '__main__':
    unittest.main()
        
