#!/usr/bin/python
# -*- coding: utf-8 -*-
#Copyright (c) 2006 - 2007 by The WordForge Foundation
#                       www.wordforge.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
# 
# See the LICENSE file for more details. 
#
# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)
# 
# This module is working on how to display Third Languages of xliff file on Main Editor.

import os,  sys
from PyQt4 import QtGui, QtCore
from WordForge.translate.storage import lisa

#class ThirdLanguages(QtGui.QDialog):
#    """
#    The Catalog Manager which holds the toolviews.
#    """
#    def __init__(self, parent):
#        QtGui.QDialog.__init__(self, parent)
#        self.ui = None
#        self.unit = None
#        # from list setXMLlang
#        self.XMLlang = '{http://www.w3.org/XML/1998/namespace}lang'
#        
#    def showDialog(self):
#        if (not self.ui):
#            from WordForge.Editor.ui.Ui_ThirdLanguage import Ui_Dialog
#            self.setWindowTitle(self.tr("Third Languages"))
#            self.ui = Ui_Dialog()
#            self.ui.setupUi(self)
#            
#            # set appearance
#            self.normalState = QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable
#            
#         # get language for each local
#            import WordForge.Editor.modules.World as World
#            dict_languages = World.dict_languages
#            self.locale=[]
#            for k, v in dict_languages.iteritems():
#                if (len(v)==3):
#                    self.locale.append(v[2])
#            self.locale.sort()
#            self.ui.cmbLanguage.addItems(self.locale)
#            self.connect(self.ui.cmdAdd, QtCore.SIGNAL("clicked()"), self.addThirdLanguage)
#            self.connect(self.ui.cmdDelete, QtCore.SIGNAL("clicked()"), self.removeThirdLanguage)
#            self.connect(self.ui.cmdOK, QtCore.SIGNAL("clicked()"), QtCore.SLOT("close()"))
#        self.viewSetting()
#        self.show()

class ThirdLanguages(QtGui.QDockWidget):
    """
    The Catalog Manager which holds the toolviews.
    """
    def __init__(self, parent):
        from WordForge.Editor.ui.Ui_ThirdLanguage import Ui_Form
        QtGui.QDockWidget.__init__(self, parent)
        self.setObjectName("thirdLanguage")
        self.setWindowTitle(self.tr("Third Languages"))
        self.form = QtGui.QWidget(self)
        self.ui = Ui_Form()
        self.ui.setupUi(self.form)
        self.setWidget(self.form)
        
        # set appearance
        self.normalState = QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable
            
        # get language for each local
#        import WordForge.Editor.modules.World as World
#        dict_languages = World.dict_languages
#        self.locale=[]
#        for k, v in dict_languages.iteritems():
#            if (len(v)==3):
#                self.locale.append(v[2])
#        self.locale.sort()
#        self.ui.cmbLanguage.addItems(self.locale)
#        self.connect(self.ui.cmdAdd, QtCore.SIGNAL("clicked()"), self.addThirdLanguage)
#        self.connect(self.ui.cmdDelete, QtCore.SIGNAL("clicked()"), self.removeThirdLanguage)
#        self.connect(self.ui.cmdOK, QtCore.SIGNAL("clicked()"), QtCore.SLOT("close()"))
#        self.viewSetting()

    def viewSetting(self):
        self.ui.tableWidget.clear()
        self.headerLabels = [self.tr("Index"), self.tr("Language"), self.tr("Alternative translation")]
        self.ui.tableWidget.setColumnCount(len(self.headerLabels))
        self.ui.tableWidget.setHorizontalHeaderLabels(self.headerLabels)
        self.ui.tableWidget.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        self.ui.tableWidget.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        self.ui.tableWidget.resizeColumnToContents(2)
        self.ui.tableWidget.horizontalHeader().setResizeMode(2, QtGui.QHeaderView.Stretch)
        self.ui.tableWidget.verticalHeader().hide()
        self.ui.tableWidget.setRowCount(0)
        self.ui.tableWidget.setEnabled(True)

    def updateView(self, unit):
        """
        update table of items
        @type unit: Translation Unit
        @param unit: Translation Unit contains alternative translations
        """
        self.viewSetting()
        translist = []
        try:
            translist = unit.getalttrans()
        except AttributeError:
            pass
        
        if (translist):
            for list in translist:
                if (list.alttranstype == "reference"):
                    try:
                        targetTag = None
                        
                        for entry in list.xmlelement:
                            if entry.tag.rfind("target") != -1:
                                targetTag = entry 
                        xmlLang = lisa.getXMLlang(targetTag)
                        if (list.target and xmlLang):
                            self.addItemThirdLang(list.target, xmlLang)
                    except:
                        continue
    
    def addItemThirdLang(self, text, lang):
        """
        add Item into table of items
        @type text: String
        @param text: Alternative translation of the source text.
        
        @type lang: String
        @param lang: third language code
        """
        row = self.ui.tableWidget.rowCount()
        self.ui.tableWidget.setRowCount(row + 1)
        
        # index
        item = QtGui.QTableWidgetItem(str(row + 1))
        item.setTextAlignment(QtCore.Qt.AlignRight + QtCore.Qt.AlignVCenter)
        item.setFlags(self.normalState)
        self.ui.tableWidget.setItem(row, 0, item)
        
        # language
        item = QtGui.QTableWidgetItem(lang)
        item.setFlags(self.normalState)
        self.ui.tableWidget.setItem(row, 1, item)
        
        # thirdlanuage
        item = QtGui.QTableWidgetItem(text)
        self.ui.tableWidget.setItem(row, 2, item)

    
#    def addThirdLanguage(self):matchquality
#        """
#        add alternative translation as Third Language
#        """
#        text = unicode(self.ui.txtNewThirdLanguage.toPlainText())
#        language = str(self.ui.cmbLanguage.currentText())
#        if (text and language and self.unit):
#            self.emit(QtCore.SIGNAL("addThirdlanguage"), text, language, None)
#            self.addItemThirdLang(text, language)
#        self.ui.txtNewThirdLanguage.clear()
#            
#    def removeThirdLanguage(self):
#        """
#        remove third language
#        """
#        currentRow = self.ui.tableWidget.currentRow()
#        item = self.ui.tableWidget.item(currentRow, 2)
#        if (item and self.unit):
#            text = unicode(item.text())
#            tranlist = self.unit.getalttrans()
#            for list in tranlist:
#                if (list.target == text):
#                    self.emit(QtCore.SIGNAL("removeThirdLanguage"), list)
#                    self.ui.tableWidget.removeRow(currentRow)
#                    break
        
if __name__ == "__main__":
    import sys, os
    # set the path for QT in order to find the icons
    QtCore.QDir.setCurrent(os.path.join(sys.path[0], "..", "ui"))
    app = QtGui.QApplication(sys.argv)
    thirdLanguage = ThirdLanguages(None)
#    thirdLanguage.showDialog()
    thirdLanguage.show()
    sys.exit(app.exec_())
    
