#!/usr/bin/env python
# -*- coding: utf-8 -*-
# script revision: 992813

# 'xliffmerge.py' is part of Lokalize
# (c) 2007-2009 Nick Shaforostoff
# Revision 992813
# http://websvn.kde.org/trunk/KDE/kdesdk/lokalize/scripts/xliffmerge.py
# License: GNU General Public License Version 2 
# python scripts/xliffmerge.py -i tests/testxliffmerge_tr.xlf -t tests/testxliffmerge_en.xlf -o test_merged.xlf

#TODO: mark as 'needs adaptation' when only punctuation is changed
#check id's change after document update
# Later modified by :
#       Aditi Barua(aditi@ankur.org.bd)
#       Asheq Hamid(asheq@ankur.org.bd)
#       Md. Rezwan Shahid(rezwan@ankur.org.bd)

from PyQt4.QtCore import *
from PyQt4.QtXml import *

import itertools
import math

import sys
sys.setrecursionlimit(10000)

from optparse import OptionParser

class XliffMerge:
    def __init__(self, oldFile, outFile, templateFIle):
        
        self.templateFile = templateFIle
        self.oldFile = oldFile
        self.stickyIds = False
        self.outFile = outFile

        self.strings={}
        
        (self.templateDoc, self.templateUnits, self.templateUnitsList)=self.getDocUnitsList(self.templateFile)
        (self.oldDoc, self.oldUnits, self.oldUnitsList)=self.getDocUnitsList(self.oldFile)

        self.freezedOldUnits=[]
        for i in range(self.oldUnits.size()):
            self.freezedOldUnits.append(self.oldUnits.at(i))
        
        self.removedUnits=[]
        
        self.INLINE_MARKUP_ELEMENTS=['g', 'x', 'bx', 'ex', 'bpt' , 'ept', 'ph', 'it'] #, 'mrk' -- doesn't have id attribute

        globals()['recentlyRemoved']=[]
        globals()['lastCommon']=-1
        
        self.phaseName=self.addPhase()

        self.C = self.LCS(self.oldUnitsList, self.templateUnitsList)
        
        for eachlist in self.C:
            eachlist.reverse()
        self.C.reverse()
        self.recordRemoved(self.C, self.oldUnitsList, self.templateUnitsList, len(self.oldUnitsList), len(self.templateUnitsList))
        self.merge(self.C, self.oldUnitsList, self.templateUnitsList, len(self.oldUnitsList), len(self.templateUnitsList))

        for remNode in [self.freezedOldUnits[x] for x in self.removedUnits]:
            remNode.parentNode().removeChild(remNode)


        self.containers=["source", "seg-source","target","g","bpt","ept","ph","it","mrk"] #immediate containers allowing markup
        for container in self.containers:
            self.fixWhiteSpaceInList(self.oldDoc.elementsByTagName(container))

        #if __name__ == "__main__":
        #print __name__
        file=QFile(self.outFile)
        file.open(QIODevice.WriteOnly)
        stream=QTextStream(file)
        self.oldDoc.save(stream,2)
        stream.flush()
        file.close()
        file.close()



    def saveElement(self, elem):
        contents=QString()
        stream=QTextStream(contents)
        elem.save(stream,0)
        return contents

    def elementText(self, parent):
        contents=QString()
        n=parent.firstChild()
        while not n.isNull():
            if n.isCharacterData():
                contents+=n.toCharacterData().data()
            elif n.isElement():
                contents+=self.elementText(n)
            n=n.nextSibling()
        return contents



    def getDocUnitsList(self, path):
        doc=QDomDocument()
        file=QFile(path)
        file.open(QIODevice.ReadOnly)
        reader=QXmlSimpleReader()
        #reader.setFeature('http://qtsoftware.com/xml/features/report-whitespace-only-CharData',True)
        reader.setFeature('http://trolltech.com/xml/features/report-whitespace-only-CharData',True)
        reader.setFeature('http://xml.org/sax/features/namespaces',False)
        source=QXmlInputSource(file)
        doc.setContent(source,reader)
        file.close()

        units=doc.elementsByTagName("trans-unit")

        unitsList=[]
        for i in range(units.count()):
            unit=units.at(i)
            if unit.firstChildElement("source").text() not in self.strings:
                self.strings[unit.firstChildElement("source").text()]=len(self.strings)
            unitsList.append(self.strings[unit.firstChildElement("source").text()])

        return (doc, units, unitsList)



    def lcs_length(self, xs, ys):
        ny = len(ys)
        curr = list(itertools.repeat(0, 1 + ny))
        for x in xs:
            prev = list(curr)
            for i, y in enumerate(ys):
                if x == y:
                    curr[i+1] = prev[i] + 1
                else:
                    curr[i+1] = max(curr[i], prev[i+1])
        return curr[ny]


    def LCS(self, X, Y):
        m = len(X)
        n = len(Y)
        # An (m+1) times (n+1) matrix
        self.C = [[0] * (n+1) for i in range(m+1)]
        for i in range(1, m+1):
            for j in range(1, n+1):
                if X[i-1] == Y[j-1]: 
                    self.C[i][j] = self.C[i-1][j-1] + 1
                else:
                    self.C[i][j] = max(self.C[i][j-1], self.C[i-1][j])
        return self.C


    def recordRemoved(self, C, X, Y, lenX, lenY):
        i = 0
        j = 0
        while i <= lenX and j<= lenY:
            if i < lenX and j < lenY and X[i] == Y[j]:
                i += 1
                j += 1
            else:
                if i < lenX  and (j == lenY or C[i][j+1] <= C[i+1][j]):
                    self.removedUnits.append(i)
                    i += 1
                    
                elif j < lenY and (i == lenX or C[i][j+1] > C[i+1][j]):
                    j += 1
                    
                else:
                    break


    def inlineTags(self, parent):
        result=[]
        elem=parent.firstChildElement()
        while not elem.isNull():
            result.append(elem.tagName())
            elem=elem.nextSiblingElement()
        return result

    def getIdsMap(self, parent):
        result={}
        elem=parent.firstChildElement()
        while not elem.isNull():
            result[elem.attribute('id')]=elem
            elem=elem.nextSiblingElement()
        return result

    def removeAttributes(self, elem):
        for attrNode in [elem.attributes().item(i) for i in range(elem.attributes().size())]:
            elem.removeChild(attrNode)

    def cloneToAltTrans(self, unitAltToBe, newUnit):
        altUnit=unitAltToBe.cloneNode().toElement()
        altUnit.setTagName('alt-trans')
        altUnit.setAttribute('alttranstype','previous-version')
        altUnit.setAttribute('phase-name',self.phaseName)
        altUnit.removeAttribute('id')
        altUnit.removeAttribute('approved')
        refNode=newUnit.firstChildElement('alt-trans')
        if refNode.isNull(): altUnit=newUnit.appendChild(altUnit)
        else: altUnit=newUnit.insertBefore(altUnit,refNode)


        subAltUnits=[]
        subAltUnit=altUnit.firstChildElement('alt-trans')
        refNode=altUnit
        while not subAltUnit.isNull():
            refNode=altUnit.parentNode().insertAfter(subAltUnit.cloneNode(),refNode)
            subAltUnits.append(subAltUnit)
            subAltUnit=subAltUnit.nextSiblingElement('alt-trans')

        for subAltUnit in subAltUnits:
            altUnit.removeChild(subAltUnit)

        return altUnit


    def merge(self, C, X, Y, lenX, lenY):

        i = 0
        j = 0
        
        while i <= lenX and j<= lenY:
            if i < lenX and j < lenY and X[i] == Y[j]:

                globals()['recentlyRemoved']=[]
                globals()['lastCommon']=i
                templateUnit=self.templateUnits.at(j).toElement()
                templateSource=templateUnit.firstChildElement("source")
                commonUnit=self.freezedOldUnits[i].toElement()
                commonTarget=commonUnit.firstChildElement("target")
                commonSource=commonUnit.firstChildElement("source")
                targetIdsMap=self.getIdsMap(commonTarget)
                equalIds=False
                
#                print "commonTarget: ", commonTarget.text()
#                print "commonSource: ", commonSource.text()

                # [only] inline markup differs?
                completelyEqual=self.saveElement(commonSource)==self.saveElement(templateSource)
                if not completelyEqual:
                    altUnit=self.cloneToAltTrans(commonUnit,commonUnit)
                    commonTarget.setAttribute('state','needs-review-adaptation')
                    commonUnit.setAttribute('phase-name',self.phaseName)
                    commonTarget.setAttribute('phase-name',self.phaseName)

                #print ' '+templateSource.text()
                # update inline markup attributes in target
                for markupElement in self.INLINE_MARKUP_ELEMENTS:

                    templateElem=templateSource.firstChildElement(markupElement)
                    commonSourceElem=commonSource.firstChildElement(markupElement)
                    while not templateElem.isNull() and not commonSourceElem.isNull():
                        equalIds=equalIds and commonSourceElem.attribute('id')==templateElem.attribute('id')
                        if targetIdsMap.has_key(commonSourceElem.attribute('id')):
                            commonTargetElem=targetIdsMap[commonSourceElem.attribute('id')]
                            self.removeAttributes(commonTargetElem)
                            for attrNode in [templateElem.attributes().item(i).toAttr() for i in range(templateElem.attributes().size())]:
                                commonTargetElem.setAttribute(attrNode.name(), attrNode.value())

                            del targetIdsMap[commonSourceElem.attribute('id')]

                        templateElem=templateElem.nextSiblingElement(markupElement)
                        commonSourceElem=commonSourceElem.nextSiblingElement(markupElement)

                # remove inline markup in target which doesn't have corresponding markup in source
                for orphan in targetIdsMap.itervalues():
                    if orphan.tagName()=='mrk': continue
                    self.removeAttributes(orphan)
                    child=orphan.firstChild()
                    while not child.isNull():
                        newChild=child.cloneNode()
                        orphan.parentNode().insertAfter(newChild,orphan.previousSibling())
                        child=child.nextSibling()
                    orphan.parentNode().removeChild(orphan)

                #copy templates source entirely
                commonUnit.insertAfter(self.oldDoc.importNode(templateSource.cloneNode(),True), commonSource)
                commonUnit.removeChild(commonSource)
                
                #now go over context items and see if they are different
                commonContextGroup=commonUnit.firstChildElement("context-group")
                templateContextGroup=templateUnit.firstChildElement("context-group")
                contextsDiffer=False
                while not commonContextGroup.isNull() and not templateContextGroup.isNull() and not contextsDiffer:
                    commonContext=commonContextGroup.firstChildElement("context")
                    templateContext=templateContextGroup.firstChildElement("context")
                    while not commonContext.isNull() and not templateContext.isNull() and not contextsDiffer:
                        contextsDiffer=(self.saveElement(templateContext)!=self.saveElement(commonContext) )
                        templateContext=templateContext.nextSiblingElement("context")
                        commonContext=commonContext.nextSiblingElement("context")
                    # check of the number os items in the lists are different
                    contextsDiffer=contextsDiffer or not commonContext.isNull() or not templateContext.isNull()
                    templateContextGroup=templateContextGroup.nextSiblingElement("context-group")
                    commonContextGroup=commonContextGroup.nextSiblingElement("context-group")
                # check of the number os items in the lists are different
                contextsDiffer=contextsDiffer or not commonContextGroup.isNull() or not templateContextGroup.isNull()
                if contextsDiffer:
                    commonContextGroup=commonUnit.firstChildElement("context-group")
                    insertMarker=commonContextGroup.previousSibling()
                    #newUnit=oldDoc.importNode(templateUnit, True).toElement()
                    while not commonContextGroup.isNull():
                        commonUnit.removeChild(commonContextGroup)
                        commonContextGroup=commonContextGroup.nextSiblingElement("context-group")

                    templateContextGroup=templateUnit.firstChildElement("context-group")
                    while not templateContextGroup.isNull():
                        newContextGroup=insertMarker.ownerDocument().importNode(templateContextGroup, True).toElement()
                        insertMarker=commonUnit.insertAfter(newContextGroup, insertMarker)
                        templateContextGroup=templateContextGroup.nextSiblingElement("context-group")

                #ids
                equalIds=equalIds and commonUnit.attribute('id')==templateUnit.attribute('id')
                if not equalIds and self.stickyIds:
                    commonUnit.removeAttribute('approved')

                    #if not commonTarget.attribute('state').contains('review'):
                    commonTarget.setAttribute('state','needs-review-l10n')
                    if not commonTarget.hasChildNodes(): commonTarget.parentNode().removeChild(commonTarget)
                #if equalIds and completelyEqual:
                    #commonTarget.setAttribute('state-qualifier','id-match')
                commonUnit.setAttribute('id',templateUnit.attribute('id'))
            
                i += 1
                j += 1
                
            else:
                
                if i < lenX  and (j == lenY or C[i][j+1] <= C[i+1][j]):
                    globals()['recentlyRemoved'].append(i)
                    i += 1
                    
                elif j < lenY and (i == lenX or C[i][j+1] > C[i+1][j]):
                    templateUnit=self.templateUnits.at(j)

                    newUnit=self.oldDoc.importNode(templateUnit, True).toElement()
                    newUnit.setAttribute('phase-name',self.phaseName)
                    
                    if globals()['lastCommon']==-1:
                        refNode=self.freezedOldUnits[0]
                        refNode.parentNode().insertBefore(newUnit, refNode)
                    else:
                        refNode=self.freezedOldUnits[ globals()['lastCommon'] ]
                        refNode.parentNode().insertAfter(newUnit, refNode)
                    globals()['lastCommon']=i-1 #to preserve order
#                    print 'global: ', globals()['lastCommon'], i, j

                    #look for alternate translations, neighbourhood first

                    #nonRecentlyRemoved=[x for x in self.removedUnits if x not in globals()['recentlyRemoved']]
                    maxUnits=[]
                    newUnitText=newUnit.firstChildElement("source").text()
                    newUnitWords=newUnitText.split(' ')

                    maxScore=0
                    scores={}
                    for x in self.removedUnits:
                        remNode=self.freezedOldUnits[x]
                        remNodeText=remNode.firstChildElement("source").text()
                        commonWordLen=self.lcs_length(newUnitWords,remNodeText.split(' '))
                        if (commonWordLen+1)<0.5*len(newUnitWords):
                            scores[x]=0
                            continue
                        commonLen=self.lcs_length(newUnitText,remNodeText)
                        remLen=newUnitText.size()-commonLen
                        addLen=remNodeText.size()-commonLen

                        if commonLen==0: score=0
                        else: score=99*math.exp(0.2*math.log(1.0*commonLen/newUnitText.size())) / (math.exp(0.015*addLen)*math.exp(0.01*remLen))
                        scores[x]=score
                        if maxScore<score:maxScore=score
                    if maxScore<80:
                        j+=1
                        continue

                    for x in self.removedUnits:
                        if scores[x]==maxScore:
                            remNode=self.freezedOldUnits[x]
                            maxUnits.append((score+1*(x in globals()['recentlyRemoved']), remNode))

                    def count_compare_inverted(x, y): return int(y[0]-x[0])
                    maxUnits.sort(count_compare_inverted)

                    for maxUnit in maxUnits:
                        #print maxUnit[0],
                        #print newUnitText,
                        #print '------------',
                        #print maxUnit[1].firstChildElement("source").text()
                        self.cloneToAltTrans(maxUnit[1],newUnit)
                        
                    j += 1
                    

                else:
                    break



    def addPhase(self):
        VERSION='0.1'

        file=self.oldDoc.elementsByTagName("file").at(0).toElement()
        header=file.firstChildElement("header")
        phasegroup=header.firstChildElement("phase-group")
        if phasegroup.isNull():
            phasegroup=self.oldDoc.createElement("phase-group")
            #order following XLIFF spec
            skl=header.firstChildElement("skl")
            if not skl.isNull(): header.insertAfter(phasegroup, skl)
            else: header.insertBefore(phasegroup, header.firstChildElement())
        phaseNames={}
        phaseElem=phasegroup.firstChildElement("phase")
        while not phaseElem.isNull():
            phaseNames[phaseElem.attribute("phase-name")]=True
            phaseElem=phaseElem.nextSiblingElement("phase")
        i=1
        while 'update-from-template-%d' % i in phaseNames:
            i+=1

        phaseElem=phasegroup.appendChild(self.oldDoc.createElement("phase")).toElement()
        phaseElem.setAttribute("phase-name",'update-from-template-%d' % i)

        phaseElem.setAttribute("process-name", 'update-from-template')
        phaseElem.setAttribute("tool-id",      'xliffmerge-%s' % VERSION)
        phaseElem.setAttribute("date",         QDate.currentDate().toString(Qt.ISODate))


        toolElem=header.firstChildElement("tool")
        while not toolElem.isNull() and toolElem.attribute("tool-id")!='xliffmerge-%s' % VERSION:
            toolElem=toolElem.nextSiblingElement("tool")

        if toolElem.isNull():
            toolElem=header.appendChild(self.oldDoc.createElement("tool")).toElement()
            toolElem.setAttribute("tool-id",'xliffmerge-%s' % VERSION)
            toolElem.setAttribute("tool-name","xliffmerge.py")
            toolElem.setAttribute("tool-version",VERSION)

        return 'update-from-template-%d' % i



    def fixWhiteSpace(self, elem):
        first=elem.firstChildElement()
        if not first.previousSibling().isCharacterData():
            elem.insertBefore(self.oldDoc.createTextNode(''),first)
        n=first
        while not n.isNull():
            if not n.nextSibling().isCharacterData():
                elem.insertAfter(self.oldDoc.createTextNode(''),n)
            n=n.nextSiblingElement()


    def fixWhiteSpaceInList(self, nodeList):
        for node in [nodeList.at(x) for x in range(nodeList.size())]:
            self.fixWhiteSpace(node)
