#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys, os, locale
from PyQt4 import QtCore, QtGui
from WordForge.Editor.ui.Ui_XliffMerger import Ui_Merger
from WordForge.Editor.modules.XliffMerge import XliffMerge
import WordForge.Editor.modules.World as World

class XLIFFMerger(QtGui.QDialog):
    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self.ui = None
        self.parent = parent
        self.lazyInit()
#        self.show()

    def selInput(self):
        '''
        Seleccionamos o input
        '''
        directory = World.settings.value("workingDir").toString()
        if (self.ui.comboBox.currentIndex()==0):
            filename = QtGui.QFileDialog.getOpenFileName(self, self.tr('Select input file'), directory, self.tr("xliff file (*.xlf *.xliff);;All Files (*)"))
        else:
            filename = QtGui.QFileDialog.getExistingDirectory(self, self.tr('Select input directory'), directory)
        if filename:
            self.ui.txtInput.setText(unicode(filename))

    def selOutput(self):
        '''
        Seleccionamos o output
        '''
        directory = World.settings.value("workingDir").toString()
        if (self.ui.comboBox.currentIndex()==0):            
            filename = QtGui.QFileDialog.getSaveFileName(self, self.tr('Select output file'), directory, self.tr("xliff file (*.xlf *.xliff);;All Files (*)"))
        else:            
            filename = QtGui.QFileDialog.getExistingDirectory(self, self.tr('Select output directory'), directory)			
        if filename:
            self.ui.txtOutput.setText(unicode(filename))

    def selTemplate(self):
        '''
        Seleccionamos o template
        '''
        directory = World.settings.value("workingDir").toString()
        if (self.ui.comboBox.currentIndex()==0):
            filename = QtGui.QFileDialog.getOpenFileName(self, self.tr('Select template file'), directory, self.tr("xliff file (*.xlf *.xliff);;All Files (*)"))
        else:
            filename = QtGui.QFileDialog.getExistingDirectory(self, self.tr('Select template directory'), directory)									
        if filename:
            self.ui.txtTemplate.setText(unicode(filename))
            
    def addItem(self):
        ''' Engadimos os ítems...'''
        input = unicode(self.ui.txtInput.text())
        output = unicode(self.ui.txtOutput.text())
        template = unicode(self.ui.txtTemplate.text())
        '''
        input = u'/home/javi/xliffmerger/ficheiros_gnomeñ/traduzidos'
        output = u'/home/javi/xliffmerger/ficheiros_gnomeñ/'
        template = u'/home/javi/xliffmerger/ficheiros_gnomeñ/templates'
        '''
        extensionsValidas = ['xlf','xliff']
        
        if len(input) == 0 or len(output) == 0 or len(template) == 0:
            #TODO: Check if all are valid
            QtGui.QMessageBox.warning(self, u"Incomplete!", u"You must fill input, template and output",  QtGui.QMessageBox.Ok)

            return
            
        if os.path.isdir(input):
            listado = self.Walk(input, self.ui.checkBox.isChecked(),  extensionsValidas)
            listadoDestino = self.Walk(template, self.ui.checkBox.isChecked(),  extensionsValidas) #os.listdir(template)
            for item in listado:
                coincidencias = 2
                trozoinicial = 3
                inicio = item[len(input):len(input)+trozoinicial]
                while coincidencias > 1:
                    coincidencias = 0
                    itemEngadir = ''
                    for itemDestino in listadoDestino:
                        if itemDestino[len(template):].startswith(inicio):
                            coincidencias+=1
                            itemEngadir = itemDestino
                    trozoinicial+=1
                    inicio = item[len(input):len(input)+trozoinicial]
                if coincidencias == 1:# and item.startswith(inicio):
                    i = QtGui.QTreeWidgetItem()
                    i.setText(0,item)
                    i.setText(1,itemEngadir)
                    destino = os.path.join(output, itemEngadir[len(template)+1:])
                    destino = destino[:-len(os.path.splitext(destino)[1]) ]+'.updated.xlf'
                    i.setText(2,destino)
                    self.ui.treeWidget.addTopLevelItem(i)
        else:
            i = QtGui.QTreeWidgetItem()
            i.setText(0,input)
            i.setText(1,template)
            i.setText(2,output)
            
            self.ui.treeWidget.addTopLevelItem(i)
            
    def removeItem(self):
        #Remove selected items from the treeWidget.
        selectedRows = self.ui.treeWidget.selectedItems()
        for row in selectedRows:
            for columnNumber in (0,row.columnCount()):
                self.ui.treeWidget.removeItemWidget(row,columnNumber)  


    def lazyInit(self):
        ''' Interface configuration '''
        if (self.ui):
            # already has ui
            return
        self.ui = Ui_Merger()
        self.ui.setupUi(self)
        self.setModal(True)
        self.connect(self.ui.btnInput, QtCore.SIGNAL("clicked(bool)"), self.selInput)
        self.connect(self.ui.btnOutput, QtCore.SIGNAL("clicked(bool)"), self.selOutput)
        self.connect(self.ui.btnTemplate, QtCore.SIGNAL("clicked(bool)"), self.selTemplate)
        self.connect(self.ui.buttonOK, QtCore.SIGNAL("clicked(bool)"), self.main)
        self.connect(self.ui.buttonCancel, QtCore.SIGNAL("clicked(bool)"), self.close)
        '''
        self.connect(self.ui.btnInput_2, QtCore.SIGNAL("clicked(bool)"), self.selInputDir)
        self.connect(self.ui.btnOutput_2, QtCore.SIGNAL("clicked(bool)"), self.selOutputDir)
        self.connect(self.ui.btnTemplate_2, QtCore.SIGNAL("clicked(bool)"), self.selTemplateDir)
        '''
        
        self.connect(self.ui.btnAdd, QtCore.SIGNAL("clicked(bool)"), self.addItem)
        self.connect(self.ui.btnRemove, QtCore.SIGNAL("clicked(bool)"), self.removeItem)
        self.connect(self.ui.comboBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.switchCombo)
        self.items = []

        headerLabels = [self.tr('Input'), \
            self.tr('Template'),\
            self.tr('Output')]
        headerItem = self.ui.treeWidget.headerItem()
        headerItem.setTextAlignment(0, QtCore.Qt.AlignHCenter)
        headerItem.setTextAlignment(1, QtCore.Qt.AlignHCenter)
        headerItem.setTextAlignment(2, QtCore.Qt.AlignHCenter)
        self.ui.treeWidget.setColumnCount(len(headerLabels))
        self.ui.treeWidget.setHeaderLabels(headerLabels)
        self.ui.treeWidget.setSortingEnabled(True)
        self.ui.treeWidget.sortItems(0, QtCore.Qt.AscendingOrder)
        self.ui.treeWidget.header().setResizeMode(QtGui.QHeaderView.Interactive)



        
    def switchCombo(self):
        self.ui.checkBox.setEnabled(not self.ui.checkBox.isEnabled())

    def Walk(self, root, recurse=False, pattern=[], return_folders=0 ):
        import fnmatch, string

        # initialize
        result = []
        # must have at least root folder
        try:
            names = os.listdir(root)
        except os.error:
            return result

        # expand pattern
        pat_list = pattern or []
        #pat_list = string.splitfields( pattern , ';' )
        # check each file
        for name in names:
            fullname =os.path.join(root, name)
            # grab if it matches our pattern and entry type
            for pat in pat_list:
                if fnmatch.fnmatch(fullname.lower(), "*"+pat):
                    if os.path.isfile(fullname) or (return_folders and os.path.isdir(fullname)):
                        result.append(unicode(fullname))
                    continue
                    
            # recursively scan other folders, appending results
            if recurse:
                if os.path.isdir(fullname) and not os.path.islink(fullname):
                    result = result + self.Walk( fullname, recurse, pattern, return_folders )
        return result


    def main(self):
        totalitems = self.ui.treeWidget.topLevelItemCount()

        if totalitems > 0:
            self.setCursor(QtCore.Qt.WaitCursor)
            while totalitems > 0:
                it = self.ui.treeWidget.takeTopLevelItem(0)
                #TODO: modularize xliffmerge.py to manage it internally
                if it:
                    input = it.text(0)
                    template = it.text(1)
                    output = it.text(2)
                    if not os.path.exists(os.path.split(unicode(output))[0]):
                        os.makedirs(os.path.split(unicode(output))[0])
                    options = XliffMerge(input, output, template)
                    totalitems = self.ui.treeWidget.topLevelItemCount()
            self.setCursor(QtCore.Qt.ArrowCursor)
            QtGui.QMessageBox.information(self, u"Complete!", u"Process completed successfully",  QtGui.QMessageBox.Ok)

            self.close()
        
        else:
            QtGui.QMessageBox.critical(self, u"Oops!", u"Nothing to process...",  QtGui.QMessageBox.Ok)


#if __name__ == "__main__":
#    main(None)


if __name__ == "__main__":
    # set the path for QT in order to find the icons
    reload(sys)
    sys.setdefaultencoding(locale.getdefaultlocale()[1])
    QtCore.QDir.setCurrent(os.path.join(sys.path[0], "..", "ui"))
    app = QtGui.QApplication(sys.argv)
    merger = XLIFFMerger(None)
    merger.show()
    sys.exit(app.exec_())
