#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007 by KhmerOS
# www.khmeros.info
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details.
#
# Developed by:
#       Hok Kakada      (hokkakada@khmeros.info)
#       San Titvirak    (titvirak@khmeros.info)
#       Seth Chanratha  (sethchanratha@khmeros.info)
#       Keo Sophon      (keosophon@khmeros.info)
# 
# This module is working on the conversion of php file to xliff file.

from optparse import OptionParser
from WordForge.translate.storage import xliff
import WordForge.translate.storage.moodle
import sys, os

class php2xlf:
    def __init__(self):
        self.strVersion = "xlf2php 0.1"
        self.usage = """%prog [-i INPUT] [-o OUTPUT] [-t TEMPLATE]\n
A tool for converting Moodle (.php) to Xliff (.xlf).
Supported input/output/template: 
  - File
  - Directory"""
        self.fileCount = 0
        self.unitCount = 0
        self.mergeCount = 0
        self.verbos = True
        
    def autoConvert(self, iFileName, oFileName = None, tFileName = None):
        """
        Convert a file or directory contains valid moodle .php to .xlf file.
        @param iFileName: file or directory.
        @param oFileName: file or directory. If not specified, it will generate
        a name automaticaly.
        @param tFileName: file or directory to merge with iFileName.
        """
        if (os.path.isfile(iFileName)):
            if (not oFileName):
                oFileName = self.getAutoName(iFileName, ".xlf")
            ret = self.convertFile(iFileName, oFileName, tFileName)
            if (self.verbos and ret):
                print "   ", oFileName, ",", self.numOfString
                self.numOfString = 0
        elif (os.path.isdir(iFileName)):
            if (not oFileName):
                oFileName = self.getAutoName(iFileName, "-xlf")
            
            for root, dirs, files in os.walk(iFileName):
                outputRoot = oFileName
                templateRoot = tFileName
                # file
                for file in files:
                    iFileName = os.path.join(root + os.path.sep + file)
                    oFileName = os.path.join(outputRoot + os.path.sep + self.getAutoName(file, ".xlf"))
                    if (tFileName):
                        tFileName = os.path.join(templateRoot + os.path.sep + file)
                    self.autoConvert(iFileName, oFileName, tFileName)
                # folder
                for folder in dirs:
                    iFileName = os.path.join(root + os.path.sep + folder)
                    oFileName = os.path.join(outputRoot + os.path.sep + folder)
                    if (tFileName):
                        tFileName = os.path.join(templateRoot + os.path.sep + folder)
                    self.autoConvert(iFileName, oFileName, tFileName)
                break
    
    def convertFile(self, iFileName, oFileName, tFileName):
        """
        Convert .php file to .xlf file, return bool indicates success process.
        """
        inputFile = open(iFileName, "r")
        try:
            mooFile = moodle.moodleFile(inputFile)
        except:
            return False
        inputFile.close()
        
        if (not mooFile.isValid()):
            return False
        
        if (tFileName):
            try:
                tFile = open(tFileName, "r")
                mFile = moodle.moodleFile(tFile)
                tempUnits = mFile.units
                tFile.close()
            except IOError:
                print ("WARNING: file \"%s\" not found, continue without merging." % tFileName)
                tempUnits = None
        else:
            tempUnits = None
        xlfFile = xliff.xlifffile(sourcelanguage = mooFile.sourceLanguage())
        # convert unit
        for unit in mooFile.units:
            self.convertUnit(xlfFile, unit, os.path.basename(iFileName), tempUnits)
        # add header
        xlfFile.createheadertool(os.path.basename(iFileName), "khmeros.info", "php2xlf", "0.1")
        xlfFile.createheadernote(os.path.basename(iFileName), mooFile.header)
        
        self.fileCount += 1
        self.unitCount += len(mooFile.units)
        self.numOfString = len(mooFile.units)
        
        # create directory if not exists.
        self.createDir(oFileName)
        
        outputFile = open(oFileName, "w")
        outputFile.write(str(xlfFile))
        outputFile.close()
        return True
    
    def convertUnit(self, xlfFile, mooUnit, filename = "NoName", tmpUnits = None):
        """
        Creates a transunit node.
        """
        fuzzy = False
        source = mooUnit.source
        if (tmpUnits):
            for tmpUnit in tmpUnits:
                if (tmpUnit.name == mooUnit.name): # and (tmpUnit.source != mooUnit.source):
                    target = tmpUnit.source
##                    fuzzy = True
                    self.mergeCount += 1
                    break
            else:
                target = mooUnit.target
        else:
            target = mooUnit.target
        
        unit = xlfFile.addsourceunit(source, filename, True)
        unit.target = target
        unit.addnote(mooUnit.comment, "translator")
        # Explicetely marking the fuzzy state will ensure that normal (translated)
        unit.markfuzzy(fuzzy)
        # moodle string references
        reference = mooUnit.getlocations()[0]
        if (reference):
            unit.createcontextgroup("reference", [("name", reference)])
        return unit
    
    def getAutoName(self, inputFile, extention):
        """
        Get a filename from input file by changing/adding extention.
        """
        indexDot = inputFile.rfind(".")
        indexSep = inputFile.rfind(os.path.sep)
        if (indexDot >= 0) and (indexDot > indexSep):
            outputFile = inputFile[:indexDot] + extention
        else:
            outputFile = inputFile + extention
        return outputFile
    
    def createDir(self, path):
        """
        Create directory if not exists.
        """
        path = os.path.dirname(os.path.abspath(path))
        if (not os.path.exists(path)):
            i = path.find(os.path.sep)
            while (i >= 0):
                dirName = path[:i]
                if (dirName) and (not os.path.exists(dirName)):
                    os.mkdir(dirName)
                i = path.find(os.path.sep, i + 1)
            os.mkdir(path)
   
def main(argv=None):
    from optparse import OptionParser
    import sys, os
    converter = php2xlf()
    
    if (sys.argv[0].endswith("py")):
        py = "python "
    else:
        py = ""
    
    parser = OptionParser(usage = py + os.path.basename(sys.argv[0]) + " " + converter.usage, version = converter.strVersion)
    parser.add_option("-q", action="store_true", dest="quiet", default=False, 
            help="do not print converted files and total strings count.")
    parser.add_option("-t", dest="tempFile", action="store", type="string",
            help="file name or directory to merge with INPUT, must have same structure as INPUT.", metavar="TEMPLATE", default=None)
    parser.add_option("-i", dest="inputFile", action="store", type="string",
            help="read from INPUT in .php format (file or directory).", metavar="INPUT", default=None)
    parser.add_option("-o", dest="outputFile", action="store", type="string",
            help="write to OUTPUT in .xlf format (file or directory). If not specified, it will create the name according to INPUT by changing to .xlf for file, or adding -xlf for directory.", metavar="OUTPUT", default=None)
    options, args = parser.parse_args()
    
    if (options.quiet):
        converter.verbos = False
    else:
        converter.verbos = True
        print "Start conversion..."
    
    if (options.inputFile):
        converter.autoConvert(options.inputFile, options.outputFile, options.tempFile)
    
    if (converter.verbos):
        print "Summary:"
        print "  %d file%s processed." % (converter.fileCount, ((converter.fileCount > 1) and "s") or "")
        print "  %d string%s converted." % (converter.unitCount, ((converter.unitCount > 1) and "s") or "")
        if (options.tempFile):
            print "  %d string%s merged." % (converter.mergeCount, ((converter.mergeCount > 1) and "s") or "")
    
if __name__ == "__main__":
    main()
