#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007 by KhmerOS
# www.khmeros.info
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details.
#
# Developed by:
#       Hok Kakada      (hokkakada@khmeros.info)
#       San Titvirak    (titvirak@khmeros.info)
#       Seth Chanratha  (sethchanratha@khmeros.info)
#       Keo Sophon      (keosophon@khmeros.info)
# 
# This module is working on the conversion of php file to xliff file.

from optparse import OptionParser
from WordForge.translate.storage import xliff
from WordForge.translate.storage import moodle
import sys, os

class xlf2php:
    def __init__(self):
        self.strVersion = "xlf2php 0.1"
        self.usage = """%prog [-i INPUT] [-o OUTPUT] [-t TEMPLATE]\n
A tool for converting Xliff (.xlf) to Moodle (.php).
Supported input/output/template: 
  - File
  - Directory"""
        self.fileCount = 0
        self.unitCount = 0
        self.convertedCount = 0
        self.verbos = True
    
    def copyTemplateToOutput(self, tFileName, iFileName, oFileName = None):
        if (os.path.isfile(tFileName)):
            if (not tFileName.endswith(".php")) and (oFileName):
                # create directory if not exists.
                self.createDir(oFileName)
                # do the copy
                tFile = open(os.path.abspath(tFileName), "r")
                oFile = open(os.path.abspath(oFileName), "w")
                oFile.write(tFile.read())
                oFile.close()
                tFile.close()
            
        elif (os.path.isdir(tFileName)):
            if (not oFileName):
                oFileName = self.getAutoName(iFileName, "-php")
            
            for root, dirs, files in os.walk(tFileName):
                outputRoot = oFileName
                # file
                for file in files:
                    tFileName = os.path.join(root + os.path.sep + file)
                    oFileName = os.path.join(outputRoot + os.path.sep + file)
                    self.copyTemplateToOutput(tFileName, iFileName, oFileName)
                # folder
                for folder in dirs:
                    tFileName = os.path.join(root + os.path.sep + folder)
                    oFileName = os.path.join(outputRoot + os.path.sep + folder)
                    self.copyTemplateToOutput(tFileName, iFileName, oFileName)
                break
    
    def autoConvert(self, iFileName, oFileName = None):
        """
        Convert a file or directory contains valid moodle .php to .xlf file.
        @param iFileName: file or directory.
        @param oFileName: file or directory. If not specified, it will generate
        a name automaticaly.
        @param tFileName: template file or directory, for oFileName to be based.
        """
        if (os.path.isfile(iFileName)):
            if (not oFileName):
                oFileName = self.getAutoName(iFileName, ".php")
            ret = self.convertFile(iFileName, oFileName)
            if (self.verbos and ret):
                print "   ", oFileName
        elif (os.path.isdir(iFileName)):
            if (not oFileName):
                oFileName = self.getAutoName(iFileName, "-php")
            
            for root, dirs, files in os.walk(iFileName):
                outputRoot = oFileName
                # file
                for file in files:
                    iFileName = os.path.join(root + os.path.sep + file)
                    oFileName = os.path.join(outputRoot + os.path.sep + self.getAutoName(file, ".php"))
                    self.autoConvert(iFileName, oFileName)
                # folder
                for folder in dirs:
                    iFileName = os.path.join(root + os.path.sep + folder)
                    oFileName = os.path.join(outputRoot + os.path.sep + folder)
                    self.autoConvert(iFileName, oFileName)
                break
    
    def convertFile(self, iFileName, oFileName):
        """
        Convert .xlf file to .php file, return bool indicates success process.
        """
        inputFile = open(iFileName, "r")
        
        try:
            xlfFile = xliff.xlifffile.parsefile(inputFile)
        except:
            return False
        
        inputFile.close()
        
        mooFile = moodle.moodleFile()
        headerNote = xlfFile.getheadernote()
        mooFile.addHeaderUnit(headerNote)
        for unit in xlfFile.units:
            self.convertUnit(mooFile, unit)
        
        self.fileCount += 1
        self.unitCount += len(xlfFile.units) - 1    # minus header unit
        
        if (not str(mooFile)):
            return False
        
        # create directory if not exists.
        self.createDir(oFileName)
        
        outputFile = open(oFileName, "w")
        outputFile.write(str(mooFile))
        outputFile.close()
        return True
    
    def convertUnit(self, mooFile, xlfUnit, filename="NoName"):
        """
        Creates a transunit node.
        """
        reference = self.getReference(xlfUnit)
##        if (reference == MOOHEADER):
##            mooFile.addHeaderUnit(xlfUnit.source)
##        elif (reference):
        if (reference):
##            source = xlfUnit.source
            target = xlfUnit.target
            if (target):
                mooFile.addSourceUnit(target, reference)
                self.convertedCount += 1
##            else:
##                mooFile.addSourceUnit(source, reference)
    
    def getReference(self, unit):
        """
        Return a reference as string.
        """
##        if (unit.getrestype() == MOOHEADER):
##            return MOOHEADER
        group = unit.getcontextgroups("reference")
        try:
            reference = group[0][0][1]
        except IndexError:
            return None
        return reference
    
    def getAutoName(self, inputFile, extention):
        """
        Get a filename from input file by changing/adding extention.
        """
        indexDot = inputFile.rfind(".")
        indexSep = inputFile.rfind(os.path.sep)
        if (indexDot >= 0) and (indexDot > indexSep):
            outputFile = inputFile[:indexDot] + extention
        else:
            outputFile = inputFile + extention
        return outputFile
    
    def createDir(self, path):
        """
        Create directory if not exists.
        """
        path = os.path.dirname(os.path.abspath(path))
        if (not os.path.exists(path)):
            i = path.find(os.path.sep)
            while (i >= 0):
                dirName = path[:i]
                if (dirName) and (not os.path.exists(dirName)):
                    os.mkdir(dirName)
                i = path.find(os.path.sep, i + 1)
            os.mkdir(path)
    
def main(argv=None):
    from optparse import OptionParser
    import sys, os
    converter = xlf2php()
    
    if (sys.argv[0].endswith("py")):
        py = "python "
    else:
        py = ""
    
    parser = OptionParser(usage = py + os.path.basename(sys.argv[0]) + " " + converter.usage, version = converter.strVersion)
    parser.add_option("-q", action="store_true", dest="quiet", default=False, 
            help="do not print converted files and total strings count.")
    parser.add_option("-t", dest="tempFile", action="store", type="string",
            help="template file name or directory, OUTPUT will be base on it.", metavar="TEMPLATE", default=None)
    parser.add_option("-i", dest="inputFile", action="store", type="string",
            help="read from INPUT in .xlf format (file or directory).", metavar="INPUT", default=None)
    parser.add_option("-o", dest="outputFile", action="store", type="string",
            help="write to OUTPUT in .php format (file or directory). If not specified, it will create the name according to INPUT by changing to .php for file, or adding -php for directory.", metavar="OUTPUT", default=None)
    options, args = parser.parse_args()
    
    if (options.quiet):
        converter.verbos = False
    else:
        converter.verbos = True
        print "Start conversion..."
    
    if (options.inputFile):
        if (options.tempFile):
            converter.copyTemplateToOutput(options.tempFile, options.inputFile, options.outputFile)
        converter.autoConvert(options.inputFile, options.outputFile)
    
    if (converter.verbos):
        print "Summary:"
        print "  %d file%s processed." % (converter.fileCount, ((converter.fileCount > 1) and "s") or "")
        print "  %d of %d string%s converted." % (converter.convertedCount, converter.unitCount, ((converter.unitCount > 1) and "s") or "")


if __name__ == '__main__':
    main()
