#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007 by KhmerOS
# www.khmeros.info
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details.
#
# Developed by:
#       Hok Kakada      (hokkakada@khmeros.info)
#       San Titvirak    (titvirak@khmeros.info)
#       Seth Chanratha  (sethchanratha@khmeros.info)
#       Keo Sophon      (keosophon@khmeros.info)
# 
# This module is working on updating the xliff file.

"""update XLIFF localization file"""

import sys, os

class xliffupdate:
    def __init__(self):
        self.strVersion = "xliffupdate 0.1"
        self.usage = """%prog [-i INPUT -t TRANSLATED] [-o OUTPUT]\n
A tool for updating Xliff (.xlf) file, create new file or directory by merging
the old one which is translated with the new one."""
        self.verbos = True
        self.fileCount = 0
        self.unitCount = 0
        self.updatedCount = 0
        self.excludeFuzzy = False
        self.includeInfo = ""
        self.cleanAll = "cleanall"
        self.clean = "clean"
    
    def autoUpdate(self, iFileName, tFileName, oFileName = None):
        """
        Update a file or directory.
        @param iFileName: untranslated file or directory.
        @param tFileName: translated file or directory.
        @param oFileName: file or directory. If not specified, it will generate
        a name automaticaly.
        """
        if (os.path.isfile(iFileName)):
            if (not oFileName):
                oFileName = self.getAutoName(iFileName, "-merged")
            self.updateFile(iFileName, tFileName, oFileName)
        elif (os.path.isdir(iFileName)):
            if (not oFileName):
                oFileName = self.getAutoName(iFileName, "-merged")
            
            for root, dirs, files in os.walk(iFileName):
                translatedRoot = tFileName
                outputRoot = oFileName
                # file
                for file in files:
                    iFileName = os.path.join(root + os.path.sep + file)
                    tFileName = os.path.join(translatedRoot + os.path.sep + file)
                    oFileName = os.path.join(outputRoot + os.path.sep + file)
                    self.autoUpdate(iFileName, tFileName, oFileName)
                # folder
                for folder in dirs:
                    iFileName = os.path.join(root + os.path.sep + folder)
                    tFileName = os.path.join(translatedRoot + os.path.sep + folder)
                    oFileName = os.path.join(outputRoot + os.path.sep + folder)
                    self.autoUpdate(iFileName, tFileName, oFileName)
                break
    
    def updateFile(self, iFileName, tFileName, oFileName):
        """
        Update .xlf file, the output file's content will be copied from template
        file and translated when exact strings are found in translatedFile.
        """
        
        from WordForge.translate.storage import factory
        
        try:
            istore = factory.getobject(iFileName)
            tstore = factory.getobject(tFileName)
        except Exception, e:
            print str(e)
            return
        
        for unit in tstore.units:            
            if (unit.isfuzzy() and self.excludeFuzzy):
                continue
            self.updateUnit(istore, unit)
        
        self.fileCount += 1
        self.unitCount += len(istore.units)
        # header unit is not count
        if (istore.units[0].isheader()):
            self.unitCount -= 1
        if (tstore.units[0].isheader()) and (tstore.units[0].target):
            self.updatedCount -= 1

        # create directory if not exists.
        self.createDir(oFileName)

        outputFile = open(oFileName, "w")
        outputFile.write(str(istore))
        outputFile.close()
        return True
    
    def updateUnit(self, istore, translatedUnit, filename="NoName"):
        """
        Update unit to newXlf.
        """
        source = translatedUnit.source
        target = translatedUnit.target     
            
        unit = istore.findunit(source)            
        if (unit):
            unit.target = target
            self.updatedCount += 1
            
            if (self.includeInfo == self.cleanAll) or ((self.includeInfo == self.clean) and translatedUnit.istranslated()):
                return               
                    
            id = translatedUnit.getid()                               
            unit.markapproved(translatedUnit.isapproved())
            if (id == unit.getid()):
                unit.addnote(unicode(translatedUnit.getnotes("translator")))                
            
            translist = translatedUnit.getalttrans()
            from WordForge.translate.storage import lisa
            if (translist):
                for list in translist:                                         
                    targetTag = list.xmlelement[0]
                    xmlLang = lisa.getXMLlang(targetTag)                                   
                    if (xmlLang):
                        unit.addalttrans(list.target, None, xmlLang)
                    else:
                        unit.addalttrans(list.target, "TM Search", None)                                
            return

    def getAutoName(self, inputFile, suffix):
        """
        Get a filename from input file by changing/adding extention.
        """
        indexDot = inputFile.rfind(".")
        indexSep = inputFile.rfind(os.path.sep)
        if (indexDot >= 0) and (indexDot > indexSep):
            outputFile = inputFile[:indexDot].rstrip(os.path.sep) + suffix + inputFile[indexDot:]
        else:
            outputFile = inputFile.rstrip(os.path.sep) + suffix
        return outputFile
    
    def createDir(self, path):
        """
        Create directory if not exists.
        """
        path = os.path.dirname(os.path.abspath(path))
        if (not os.path.exists(path)):
            i = path.find(os.path.sep)
            while (i >= 0):
                dirName = path[:i]
                if (dirName) and (not os.path.exists(dirName)):
                    os.mkdir(dirName)
                i = path.find(os.path.sep, i + 1)
            os.mkdir(path)

def main(argv=None):
    
    from optparse import OptionParser
    import sys, os
    tool = xliffupdate()
    
    if (sys.argv[0].endswith("py")):
        py = "python "
    else:
        py = ""
    
    parser = OptionParser(usage = py + tool.usage, version = tool.strVersion)
    parser.add_option("-q", action="store_true", dest="quiet", default=False, 
            help="do not print updated files and total strings count.")
    parser.add_option("-t", dest="tranlatedFile", action="store", type="string",
            help="translated .xlf file name or directory.", metavar="TRANSLATED/TEMPLATE", default=None)
    parser.add_option("-i", dest="inputFile", action="store", type="string",
            help="new .xlf file name or directory.", metavar="INPUT", default=None)
    parser.add_option("-o", dest="outputFile", action="store", type="string",
            help="write to OUTPUT (file or directory). If not specified, it will create the name base on INPUT by adding suffix -merged.", metavar="OUTPUT", default=None)
        
    parser.add_option("", "--nofuzzy", dest="excludefuzzy", action="store_true", default=False,  help="exclude units marked fuzzy")
    
    parser.add_option("", "--includeinfo", dest="includeinfo", default="all", type="choice",
            choices=["all", "clean", "cleanall"], 
            help="TYPE: all (copy information from WordForge.translated, fuzzy or untranslated units), clean (copy information from fuzzy or untranslated units), cleanall (information from all kinds of units is not copied)", metavar="TYPE")
 
    
    options, args = parser.parse_args()
    
    if (options.excludefuzzy):
        tool.excludefuzzy = True    
        
    tool.includeInfo = options.includeinfo
        
    if (options.inputFile and options.tranlatedFile):
        if (not os.path.exists(options.inputFile)):
            print "Input file or directory is not existed! Program exit."
            return
        elif (not os.path.exists(options.tranlatedFile)):
            print "Translated file or directory is not existed! Program exit."
            return
        
        tool.autoUpdate(options.inputFile, options.tranlatedFile, options.outputFile)
    
    if (options.quiet):
        tool.verbos = False
    else:
        tool.verbos = True
        print "Start update..."
    
    if (tool.verbos):
        print "Summary:"
        print "  %d file%s processed." % (tool.fileCount, ((tool.fileCount > 1) and "s") or "")
        print "  %d of %d string%s updated." % (tool.updatedCount, tool.unitCount, ((tool.unitCount > 1) and "s") or "")

if __name__ == '__main__':
    main()
