#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007 by KhmerOS
# www.khmeros.info
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details.
#
# Developed by:
#       Hok Kakada      (hokkakada@khmeros.info)
#       Keo Sophon      (keosophon@khmeros.info)
#       San Titvirak      (titvirak@khmeros.info)
#       Seth Chanratha  (sethchanratha@khmeros.info)
#       
# 

"""
Classes that hold units of .php files (moodleUnit) or entire files (moodleFile).
These files are used in translating Moodle software.
"""

from WordForge.translate.storage import base
##from translate.misc import quote
import sys

eol = "\n"

class moodleUnit(base.TranslationUnit):
    """
    An element of a moodle file i.e. a name and value, and any comments
    associated.
    """
    def __init__(self, source=""):
        """
        construct a blank moodleUnit
        """
        super(moodleUnit, self).__init__(source)
        self.name = ""
        self.source = source
        self.comment = ""
    
    def setsource(self, source):
        """
        Sets the source AND the target to be equal.
        """
        self.msgid = self.toMoodleString(source)
    
    def getsource(self):
        msgid = self.toPlainString(self.msgid)
        return msgid
    source = property(getsource, setsource)
    
    def settarget(self, target):
        """
        Note: monoligual file, source and target is the same!
        """
        self.source = self.toMoodleString(target)
    
    def gettarget(self):
        return ""
    target = property(gettarget, settarget)
    
    def __str__(self):
        """
        Convert to a string. double check that unicode is handled somehow here.
        """
        source = self.getoutput()
        if isinstance(source, unicode):
            return source.encode(getattr(self, "encoding", "UTF-8"))
        return source
    
    def getoutput(self):
        """
        Convert the element back into formatted lines for a .php file.
        """
        if self.isblank():
            return "\n"
        else:
            return "$string['%s'] = '%s';\\\\%s\n" % (self.name, self.msgid, self.comment)
    
    def getlocations(self):
        return [self.name]
    
    def addnote(self, note, origin=None):
        return
    
    def getnotes(self, origin=None):
        return ""
    
    def removenotes(self):
        return
    
    def isblank(self):
        """returns whether this is a blank element, containing only comments..."""
        return not (self.name or self.msgid)
    
    def toMoodleString(self, text):
        if (text):
            text = text.replace("'", "\\'")
            text = text.replace("\"", "\\\"")
        return text
    
    def toPlainString(self, text):
        if (text):
            text = text.replace("\\'", "'")
            text = text.replace("\\\"", "\"")
        return text
    
class moodleFile(base.TranslationStore):
    """
    This class represents a .php file, made up of moodleUnit.
    """
    UnitClass = moodleUnit
    def __init__(self, inputfile=None):
        """construct a moodleFile, optionally reading in from inputfile"""
        super(moodleFile, self).__init__(unitclass = self.UnitClass)
        self.units = []
        self.filename = getattr(inputfile, "name", "")
        self.header = ""
        self.validMoodle = False
        if inputfile is not None:
            phpSource = inputfile.read()
            inputfile.close()
            self.parse(phpSource)
    
    def addHeaderUnit(self, header):
        self.header = header
    
    def addSourceUnit(self, target, reference):
        unit = moodleUnit()
        unit.name = reference
        unit.source = target
        self.addunit(unit)
    
    def sourceLanguage(self):
        return ""
    
    def isValid(self):
        return self.validMoodle
    
    def parse(self, phpSource):
        """
        Read the source of a php file in and include them as units.
        """
        newunit = moodleUnit()
        isMultiLine = False
        lines = phpSource.split("\n")
        self.header = ""
        endOfHeader = False
        
        if (phpSource.upper()[:5] == ("<?PHP")) or (phpSource.upper()[3:8] == ("<?PHP")):
            self.validMoodle = True
        else:
            self.validMoodle = False
            return
        
        for line in lines:
            # handle multiline msgid if we're in one
            if (isMultiLine):
                newunit.msgid += "\n" + line.strip()
                # see if there's more
                sColonPos = line.find(";")
                if (sColonPos == -1):
                    isMultiLine = True
                else:
                    isMultiLine = False
                
                if (not isMultiLine):
                    # we're finished, add it to the list...
                    sColonPos = newunit.msgid.find(";")
                    if (sColonPos >= 0):
                        comment = newunit.msgid[sColonPos + 1:].strip()[2:]
                        newunit.comment = comment
                    
                    msgid = newunit.msgid[:sColonPos].strip()
                    if (msgid[-1] == "\"") or (msgid[-1] == "\'"):
                        msgid = msgid[:-1]
                    
                    newunit.msgid = msgid
                    self.units.append(newunit)
                    newunit = moodleUnit()
                    
            elif not line.strip():
                # this is a blank line...
                if str(newunit).strip():
                    self.units.append(newunit)
                    newunit = moodleUnit()
            elif (line[:7].lower() == "$string"):
                if (not endOfHeader):
                    endOfHeader = True
                equalspos = line.find("=")
                sColonPos = line.find(";")
                if (sColonPos == -1):
                    isMultiLine = True
                    sColonPos = len(line)
                else:
                    isMultiLine = False
                
                # if equal pos, this is a definition
                if (equalspos != -1):
                    # name = "$string['something'  ] " --> name = "something"
                    name = line[:equalspos].strip()
                    name = name[7:].strip(" [] ")[1:-1]
                    
                    msgid = line[equalspos + 1:sColonPos].strip()
                    if (msgid[0] == "\"") or (msgid[0] == "\'"):
                        msgid = msgid[1:]
                    if (msgid[-1] == "\"") or (msgid[-1] == "\'"):
                        msgid = msgid[:-1]
                    
                    comment = line[sColonPos + 1:].strip()[2:]
                    print "comment:", comment
                    newunit.name = name
                    newunit.msgid = msgid
                    newunit.comment = comment
                
                if (not isMultiLine):
                    self.units.append(newunit)
                    newunit = moodleUnit()
            
            if (not endOfHeader):
                # append the header until it find "$String"
                self.header += line + "\n"
        self.header = self.header.strip("<?PHPphp ")
        # see if there is a leftover one...
        if isMultiLine > 0:
            self.units.append(newunit)
        
        
    def __str__(self):
        """
        Convert the units back to lines.
        """
        if (not self.units):
            return ""
        
        lines = []
        for unit in self.units:
            lines.append(str(unit))
        return "<?PHP " + (self.header).encode("utf-8") + "".join(lines) + "\n?>"
    
    def parsestring(cls, storestring):
        """
        Parses the properties file contents in the storestring.
        """
        parsedfile = moodleFile()
        parsedfile.parse(storestring)
        return parsedfile
    parsestring = classmethod(parsestring)

##if __name__ == "__main__":
##    import sys
##    pf = moodleFile(sys.stdin)
##    sys.stdout.write(str(pf))
